<?php
defined('BASEPATH') or exit('No direct script access allowed');

class Advance extends CI_Controller
{

    function __construct()
    {
        parent::__construct();
        $this->load->model('AdvanceModal');
        $this->load->model('EmployeesModel');
        $this->load->model('UserModel');
        $this->load->library('Middleware');
    }

    public function index()
    {
    }

    // get all Advance
    public function getAdvances()
    {
        $token = $this->tokenmiddleware->main();
        $this->UserModel->checkPermission();
        $user = $this->jwtmiddleware->decode($token);
        $start_date = $this->input->get('start_date');
        $end_date = $this->input->get('end_date');
        $advances = $this->AdvanceModal->getAll($start_date, $end_date);
        echo json_encode(
            array(
                'status' => 200,
                'message' => 'عرض السلف',
                'token' => $token,
                'data' => $advances,
                'isAuth' => true,
                'total' => $this->AdvanceModal->count($start_date, $end_date)
            ),
            JSON_UNESCAPED_UNICODE
        );
        exit();
    }

    public function getHistoryOfAdvance()
    {
        $token = $this->tokenmiddleware->main();
        $this->UserModel->checkPermission();
        $user = $this->jwtmiddleware->decode($token);
        // hash from user to get his data
        $hash = $user['hash'];
        // check hash is exist
        if (!empty($hash)) {
            $employee = $this->EmployeesModel->getEmployee($hash);
            if (empty($employee)) {
                echo json_encode(
                    array(
                        'status' => 400,
                        'message' => 'لا يوجد موظف بهذا الرقم',
                        'token' => $token,
                        'data' => null,
                        'isAuth' => true
                    ),
                    JSON_UNESCAPED_UNICODE
                );
                exit();
            }
        }
        $advances = $this->AdvanceModal->getHistoryOfAdvance($hash);
        echo json_encode(
            array(
                'status' => 200,
                'message' => 'السلف السابقة للموظف',
                'token' => $token,
                'data' => $advances,
                'isAuth' => true
            ),
            JSON_UNESCAPED_UNICODE
        );
        exit();
    }

    public function getAdvance()
    {
        $token = $this->tokenmiddleware->main();
        $this->UserModel->checkPermission();
        $user = $this->jwtmiddleware->decode($token);

        $id = $this->input->get('id');
        $advance = $this->AdvanceModal->getAdvance($id);
        echo json_encode(
            array(
                'status' => 200,
                'message' => 'عرض السلفة',
                'token' => $token,
                'data' => $advance,
                'isAuth' => true
            ),
            JSON_UNESCAPED_UNICODE
        );
        exit();
    }

    public function getMyRequests()
    {
        $token = $this->tokenmiddleware->main();
        $user = $this->jwtmiddleware->decode($token);
        $user_hash = $user['hash'];
        $advances = $this->AdvanceModal->getMyAdvances($user_hash, "requests");
        echo json_encode(
            array(
                'status' => 200,
                'message' => 'عرض السلف',
                'token' => $token,
                'data' => $advances,
                'isAuth' => true
            ),
            JSON_UNESCAPED_UNICODE
        );
        exit();
    }

    public function getMyAgreements()
    {
        $token = $this->tokenmiddleware->main();
        $user = $this->jwtmiddleware->decode($token);
        $user_hash = $user['hash'];
        $advances = $this->AdvanceModal->getMyAdvances($user_hash, "agreements");
        echo json_encode(
            array(
                'status' => 200,
                'message' => 'عرض السلف',
                'token' => $token,
                'data' => $advances,
                'isAuth' => true
            ),
            JSON_UNESCAPED_UNICODE
        );
        exit();
    }

    public function changeStatus()
    {

        $token = $this->tokenmiddleware->main();
        $this->UserModel->checkPermission();
        $user = $this->jwtmiddleware->decode($token);
        $user_hash = $user['hash'];


        $id = $this->input->post('id');
        $status_id = $this->input->post('status_id');
        $notes = $this->input->post('notes');
        $user_hash = $this->input->post("user_hash");
        $user_admin = $this->input->post('user_admin'); // this reciever

        $advance = $this->AdvanceModal->changeStatus($id, [
            'status_id' => $status_id,
            'notes' => $notes,
            'user_hash' => $this->input->post('user_hash') ?? $user_hash,
            'user_admin' => $user_admin,
            'is_finish' => ($status_id == 1 || $status_id == 2) ? 1 : 0
        ]);



        if (is_string($advance)) {
            echo json_encode(
                array(
                    'status' => 400,
                    'message' => $advance,
                    'token' => $token,
                    'isAuth' => true,
                    'action' => false
                ),
                JSON_UNESCAPED_UNICODE
            );
            exit();
        }

        if ($_SERVER['SERVER_NAME'] !== 'localhost') {
            $this->jwtmiddleware->insertNotToFirebase([
                "type" => "advance",
                "status" => $status_id,
                "user" => $user_hash,
                "receiver" => $user_admin,
            ]);
        }


        echo json_encode(
            array(
                'status' => 200,
                'message' => 'تم تغيير حالة السلفة بنجاح',
                'token' => $token,
                'isAuth' => true,
                'action' => true
            ),
            JSON_UNESCAPED_UNICODE
        );
        exit();
    }

    public function getAdvanceStatus()
    {
        $token = $this->tokenmiddleware->main();
        $this->UserModel->checkPermission();
        $user = $this->jwtmiddleware->decode($token);
        $id = $this->input->get('id');

        $advances_type = $this->AdvanceModal->getStatus($id);
        echo json_encode(
            array(
                'status' => 200,
                'message' => 'عرض حالة السلفة',
                'token' => $token,
                'data' => $advances_type,
                'isAuth' => true
            ),
            JSON_UNESCAPED_UNICODE
        );
        exit();
    }

    public function createAdvance()
    {
        $token = $this->tokenmiddleware->main();
        $this->UserModel->checkPermission();
        $user = $this->jwtmiddleware->decode($token);
        $user_hash = $user['hash'];

        // check if advance created

        $data = array(
            'user_hash' => $this->input->post('user_hash') ?? $user_hash,
            'price' => $this->input->post('price'),
            'notes' => $this->input->post('notes'),
            'user_admin' => $this->input->post('user_admin'),
        );

        if ($this->input->post("date")) {
            $data['date'] = $this->input->post("date");
        }

        $advance = $this->AdvanceModal->create($data);


        // check if advance created
        if (!$advance) {
            echo json_encode(
                array(
                    'status' => 400,
                    'message' => 'حدث خطأ اثناء اضافة السلفة',
                    'token' => $token,
                    'isAuth' => true,
                    'action' => true
                ),
                JSON_UNESCAPED_UNICODE
            );
            exit();
        }

        if (is_string($advance)) {
            echo json_encode(
                array(
                    'status' => 400,
                    'message' => $advance,
                    'token' => $token,
                    'isAuth' => true,
                    'action' => false
                ),
                JSON_UNESCAPED_UNICODE
            );
            exit();
        }
        if ($_SERVER['SERVER_NAME'] !== 'localhost') {
            $this->jwtmiddleware->sendNot(
                "تم ارسال طلب سلفة اليك",
                json_encode(array('type' => 8)),
                $this->UserModel->getMobileToken($data['user_admin']),
                'تم ارسال طلب سلفة اليك'
            );

            $this->jwtmiddleware->insertNotToFirebase([
                "type" => "advance",
                "status" => "request",
                "user" => $data['user_hash'],
                "receiver" => $data['user_admin'],
            ]);
        }


        echo json_encode(
            array(
                'status' => 200,
                'message' => 'تم اضافة السلفة بنجاح',
                'token' => $token,
                'isAuth' => true,
                'action' => true
            ),
            JSON_UNESCAPED_UNICODE
        );
        exit();
    }


    public function deleteAdvance()
    {
        $token = $this->tokenmiddleware->main();
        $this->UserModel->checkPermission();
        $user = $this->jwtmiddleware->decode($token);

        $id = $this->input->post('id');
        $this->AdvanceModal->delete($id);
        echo json_encode(
            array(
                'status' => 200,
                'message' => 'تم حذف السلفة بنجاح',
                'token' => $token,
                'isAuth' => true,
                'action' => true
            ),
            JSON_UNESCAPED_UNICODE
        );
        exit();
    }
    public function getAdvanceSteps()
    {
        $token = $this->tokenmiddleware->main();
        $this->UserModel->checkPermission();
        $user = $this->jwtmiddleware->decode($token);

        $id = $this->input->get('id');
        $steps = $this->AdvanceModal->getSteps($id);
        echo json_encode(
            array(
                'status' => 200,
                'message' => 'عرض خطوات السلفة',
                'token' => $token,
                'data' => $steps,
                'isAuth' => true
            ),
            JSON_UNESCAPED_UNICODE
        );
        exit();
    }

    public function getStatus()
    {
        $token = $this->tokenmiddleware->main();
        $this->UserModel->checkPermission();
        $status = $this->AdvanceModal->getAdvanceStatus();
        echo json_encode(
            array(
                'status' => 200,
                'message' => 'عرض حالات السلفة',
                'token' => $token,
                'data' => $status,
                'isAuth' => true
            ),
            JSON_UNESCAPED_UNICODE
        );
        exit();
    }

    public function whoCanAgreement()
    {
        $token = $this->tokenmiddleware->main();
        $this->UserModel->checkPermission();
        $status = $this->AdvanceModal->whoCan("agreement");
        echo json_encode(
            array(
                'status' => 200,
                'message' => 'عرض من يمكنه الموافقة',
                'token' => $token,
                'data' => $status,
                'isAuth' => true
            ),
            JSON_UNESCAPED_UNICODE
        );
        exit();
    }

    public function whoCanSendRequest()
    {
        $token = $this->tokenmiddleware->main();
        $this->UserModel->checkPermission();
        $status = $this->AdvanceModal->whoCan("request");
        echo json_encode(
            array(
                'status' => 200,
                'message' => 'عرض من يمكنه ارسال طلب',
                'token' => $token,
                'data' => $status,
                'isAuth' => true
            ),
            JSON_UNESCAPED_UNICODE
        );
        exit();
    }

}