<?php
defined('BASEPATH') or exit('No direct script access allowed');

class Announce extends CI_Controller
{

    public function __construct()
    {
        parent::__construct();
        $this->load->model('AnnounceModel');
        $this->load->model('UserModel');
        $this->load->library('Middleware');
    }

    public function index()
    {
    }

    // get all Groups
    public function getAnnounces()
    {
        $token = $this->tokenmiddleware->main();
        $this->UserModel->checkPermission();
        $user = $this->jwtmiddleware->decode($token);

        $Announces = $this->AnnounceModel->getAnnounces();
        echo json_encode(
            array(
                'status' => 200,
                'message' => 'عرض كل التعاميم',
                'token' => $token,
                'data' => $Announces,
                'isAuth' => true,
                'total' => $this->AnnounceModel->count_records(),
            ),
            JSON_UNESCAPED_UNICODE
        );
        exit();
    }

    public function getAnnounce()
    {
        $token = $this->tokenmiddleware->main();
        $this->UserModel->checkPermission();
        $user = $this->jwtmiddleware->decode($token);

        $hash = $this->input->get('hash');
        $Announce = $this->AnnounceModel->getAnnounce($hash);
        echo json_encode(
            array(
                'status' => 200,
                'message' => 'عرض تعميم',
                'token' => $token,
                'data' => $Announce,
                'isAuth' => true,
            ),
            JSON_UNESCAPED_UNICODE
        );
        exit();
    }

    public function createAnnounce()
    {
        $token = $this->tokenmiddleware->main();
        $this->UserModel->checkPermission();
        $user = $this->jwtmiddleware->decode($token);

        $users = $this->input->post('users');
        $users = json_decode($users);
        $data = array(
            'hash' => $this->jwtmiddleware->createHash(),
            'title' => $this->input->post('title'),
            'details' => $this->input->post('details'),
            'number' => $this->input->post('number'),
            'date' => $this->input->post('date'),
            'attachment' => $this->input->post('attachment'),
            // title, details, number, date,attachment
        );
        if ($this->input->post('reward_id')) {
            $data['reward_id'] = $this->input->post('reward_id');
        }
        $this->AnnounceModel->addAnnounce($data);
        // loop on users and add them to announce_to_user table
        foreach ($users as $user) {
            $this->AnnounceModel->createAnnounceToUser(
                array(
                    'announce_hash' => $data['hash'],
                    'user_hash' => $user,
                )
            );
            var_dump($this->UserModel->getMobileToken($user));
            $this->jwtmiddleware->sendNot(
                "تعميم",
                json_encode(array('type' => 5)),
                $this->UserModel->getMobileToken($user),
                // message
                "تم استلام تعميم جديد بعنوان " . $data['title']
            );
        }
        echo json_encode(
            array(
                'status' => 200,
                'message' => 'تم اضافة التعميم بنجاح',
                'token' => $token,
                'isAuth' => true,
                'action' => true,
            ),
            JSON_UNESCAPED_UNICODE
        );
        exit();
    }

    public function updateAnnounce()
    {
        $token = $this->tokenmiddleware->main();
        $this->UserModel->checkPermission();
        $user = $this->jwtmiddleware->decode($token);

        $hash = $this->input->post('hash');
        $users = $this->input->post('users');
        $users = json_decode($users);
        $this->AnnounceModel->deleteAnnounceToUser($hash);
        $data = array(
            'title' => $this->input->post('title'),
            'details' => $this->input->post('details'),
            'number' => $this->input->post('number'),
            'date' => $this->input->post('date'),
            'attachment' => $this->input->post('attachment'),
        );
        $this->AnnounceModel->updateAnnounce($hash, $data);
        // loop on users and add them to announce_to_user table
        foreach ($users as $user) {
            $this->AnnounceModel->createAnnounceToUser(
                array(
                    'announce_hash' => $hash,
                    'user_hash' => $user,
                )
            );
        }
        echo json_encode(
            array(
                'status' => 200,
                'message' => 'تم تعديل التعميم بنجاح',
                'token' => $token,
                'isAuth' => true,
                'action' => true,
            ),
            JSON_UNESCAPED_UNICODE
        );
        exit();
    }

    public function deleteAnnounce()
    {
        $token = $this->tokenmiddleware->main();
        $this->UserModel->checkPermission();
        $user = $this->jwtmiddleware->decode($token);

        $hash = $this->input->post('hash');
        $this->AnnounceModel->deleteAnnounce($hash);
        $this->AnnounceModel->deleteAnnounceToUser($hash);
        echo json_encode(
            array(
                'status' => 200,
                'message' => 'تم حذف التعميم بنجاح',
                'token' => $token,
                'isAuth' => true,
                'action' => true,
            ),
            JSON_UNESCAPED_UNICODE
        );
        exit();
    }

    public function getMyAnnounces()
    {
        $token = $this->tokenmiddleware->main();
        $this->UserModel->checkPermission();
        $user = $this->jwtmiddleware->decode($token);

        $user_hash = $user['hash'];
        $Announces = $this->AnnounceModel->getMyAnnounces($user_hash);
        echo json_encode(
            array(
                'status' => 200,
                'message' => 'عرض التعاميم الخاصة بي',
                'token' => $token,
                'data' => $Announces,
                'isAuth' => true,
                'total' => $this->AnnounceModel->count_records(),
            ),
            JSON_UNESCAPED_UNICODE
        );
        exit();
    }
}
