<?php
defined('BASEPATH') or exit ('No direct script access allowed');


class AssignmentItemsController extends CI_Controller
{
    function __construct()
    {
        parent::__construct();
        $this->load->model('AssignmentItemsModel');
        $this->load->library('Middleware');
    }

    public function index()
    {
    }

    public function getAll()
    {
        $token = $this->tokenmiddleware->main();
        $this->UserModel->checkPermission();
        $user = $this->jwtmiddleware->decode($token);
        $start_date = $this->input->get('start_date');
        $end_date = $this->input->get('end_date');
        $advances = $this->AssignmentItemsModel->getAll($start_date, $end_date);
        echo json_encode(
            array(
                'status' => 200,
                'message' => "عرض الذمم",
                'token' => $token,
                'data' => $advances,
                'isAuth' => true,
                'total' => $this->AssignmentItemsModel->getCount($start_date, $end_date)
            ),
            JSON_UNESCAPED_UNICODE
        );
        exit();
    }

    public function get()
    {
        $token = $this->tokenmiddleware->main();
        $this->UserModel->checkPermission();
        $user = $this->jwtmiddleware->decode($token);

        $id = $this->input->get('id');
        $user_hash = $this->input->get('user_hash');
        $advance = $this->AssignmentItemsModel->get($user_hash);
        echo json_encode(
            array(
                'status' => 200,
                'message' => "عرض الذمة",
                'token' => $token,
                'data' => $advance,
                'isAuth' => true
            ),
            JSON_UNESCAPED_UNICODE
        );
        exit();
    }

    public function moveItem()
    {
        $token = $this->tokenmiddleware->main();
        $this->UserModel->checkPermission();

        $currentUser = $this->input->post('currentUser');
        $userHash = $this->input->post('userHash');
        $itemId = $this->input->post('itemId');

        if (!empty ($currentUser) || !empty ($userHash) || !empty ($itemId)) {
            $response = $this->AssignmentItemsModel->moveItem($currentUser, $userHash, $itemId);

            if ($response) {
                echo json_encode(
                    array(
                        'status' => 200,
                        'message' => "تم نقل الذمه بنجاح",
                        'token' => $token,
                        'isAuth' => true
                    ),
                    JSON_UNESCAPED_UNICODE
                );
                exit();
            }
        }
        echo json_encode(
            array(
                'status' => 400,
                'message' => "الرجاء ادخال جميع البيانات",
                'isAuth' => true
            ),
            JSON_UNESCAPED_UNICODE
        );
        exit();

    }

    public function getUserItemAssignments()
    {
        $token = $this->tokenmiddleware->main();
        $this->UserModel->checkPermission();
        $user = $this->jwtmiddleware->decode($token);

        $user_hash = $this->input->get("user_hash");
        $items = $this->AssignmentItemsModel->getUserItemAssignments($user_hash);
        echo json_encode(
            array(
                'status' => 200,
                'message' => "عرض ذمم الموظف",
                'token' => $token,
                'data' => $items,
                'isAuth' => true
            ),
            JSON_UNESCAPED_UNICODE
        );
        exit();

    }
    public function getUserItemAssignmentsRows()
    {
        $token = $this->tokenmiddleware->main();
        $this->UserModel->checkPermission();
        $user = $this->jwtmiddleware->decode($token);

        $user_hash = $this->input->get("user_hash");
        $items = $this->AssignmentItemsModel->getUserItemAssignmentsRows($user_hash);
        echo json_encode(
            array(
                'status' => 200,
                'message' => "عرض ذمم الموظف",
                'token' => $token,
                'data' => $items,
                'isAuth' => true
            ),
            JSON_UNESCAPED_UNICODE
        );
        exit();

    }

    public function create()
    {
        $token = $this->tokenmiddleware->main();
        $this->UserModel->checkPermission();
        $user = $this->jwtmiddleware->decode($token);
        $user_hash = $user['hash'];
        // check if advance created

        $data = array(
            'user_hash' => $this->input->post('user_hash'),
            'item_id' => $this->input->post('item_id'),
            'assignment_date' => $this->input->post('assignment_date'),
        );
        $advance = $this->AssignmentItemsModel->create($data);
        echo json_encode(
            array(
                'status' => 200,
                'message' => "تم اضافة الذمة",
                'token' => $token,
                'data' => $advance,
                'isAuth' => true
            ),
            JSON_UNESCAPED_UNICODE
        );
    }

    public function delete()
    {
        $token = $this->tokenmiddleware->main();
        $this->UserModel->checkPermission();
        $user = $this->jwtmiddleware->decode($token);
        $user_hash = $this->input->post('hash');
        $advance = $this->AssignmentItemsModel->delete($user_hash);

        echo json_encode(
            array(
                'status' => 200,
                'message' => "تم حذف الذمة",
                'token' => $token,
                'data' => $advance,
                'isAuth' => true
            ),
            JSON_UNESCAPED_UNICODE
        );
    }

    public function update()
    {
        $token = $this->tokenmiddleware->main();
        $this->UserModel->checkPermission();
        $user = $this->jwtmiddleware->decode($token);

        $id = $this->input->post('id');
        $data = array(
            'user_hash' => $this->input->post('user_hash'),
            'item_id' => $this->input->post('item_id'),
            'assignment_date' => $this->input->post('assignment_date'),
        );
        $advance = $this->AssignmentItemsModel->update($id, $data);
        echo json_encode(
            array(
                'status' => 200,
                'message' => "تم تعديل الذمة",
                'token' => $token,
                'data' => $advance,
                'isAuth' => true
            ),
            JSON_UNESCAPED_UNICODE
        );
    }

    public function getItems()
    {
        $token = $this->tokenmiddleware->main();
        $this->UserModel->checkPermission();
        $user = $this->jwtmiddleware->decode($token);

        $items = $this->AssignmentItemsModel->getItems();
        echo json_encode(
            array(
                'status' => 200,
                'message' => "عرض الاصناف",
                'token' => $token,
                'data' => $items,
                'isAuth' => true
            ),
            JSON_UNESCAPED_UNICODE
        );
        exit();
    }

}

