<?php
defined('BASEPATH') or exit('No direct script access allowed');

class Auth extends CI_Controller
{

    function __construct()
    {
        parent::__construct();
        $this->load->model('UserModel');
        $this->load->library('HeaderMiddleware');
        $this->load->library('JwtMiddleware');
    }

    public function index()
    {
    }


    public function login()
    {
        $username = $this->input->post('username');
        $password = $this->input->post('password');
    
        // Validate username and password
        if ($username == null || $password == null) {
            echo json_encode(
                array(
                    'status' => 400,
                    'message' => 'يجب ادخال اسم المستخدم وكلمة المرور',
                    'token' => null,
                    'data' => null,
                    'isAuth' => false
                ),
                JSON_UNESCAPED_UNICODE
            );
            exit();
        }
    
        $user = $this->UserModel->getUser($username, $password);
    
        if ($user) {
            $hash = $user['hash'];
            $user['username'] = $username;
            $user['password'] = $password;
            $role_ids = $user['role_ids']; // This will be a comma-separated string of role IDs
            $is_mobile = $this->input->post('mobile_token') ? true : false;
    
            // Convert role_ids to an array for easier manipulation
            $roles = explode(',', $role_ids);

            $cannotLogin = $this->UserModel->isUserHashDeleted($hash);
            
            // Check if the user has is_work = 2
            if ($cannotLogin) {
                echo json_encode(
                    array(
                        'status' => 400,
                        'message' => 'لا يمكن تسجيل الدخول، المستخدم غير نشط',
                        'token' => null,
                        'data' => null,
                        'isAuth' => false
                    ),
                    JSON_UNESCAPED_UNICODE
                );
                exit();
            }
    
            // Check if the user has a specific role that should not log in via web
            if (!$is_mobile && in_array('7', $roles)) {
                // Restrict login for specific role on web
                echo json_encode(
                    array(
                        'status' => 400,
                        'message' => 'الدخول عبر الويب ممنوع لهذه الفئة',
                        'token' => null,
                        'data' => null,
                        'isAuth' => false
                    ),
                    JSON_UNESCAPED_UNICODE
                );
                exit();
            }
    
            $mobile_token = $this->input->post('mobile_token');
            if ($mobile_token) {
                $this->UserModel->updateMobileToken($username, $password, $mobile_token);
                $token = $this->jwtmiddleware->encode($user, "99999999999");
            } else {
                $token = $this->jwtmiddleware->encode($user);
            }
            echo json_encode(
                array(
                    'status' => 200,
                    'message' => 'تم تسجيل الدخول بنجاح',
                    'token' => $token,
                    'data' => $user,
                    'isAuth' => true
                ),
                JSON_UNESCAPED_UNICODE
            );
            exit();
        } else {
            echo json_encode(
                array(
                    'status' => 400,
                    'message' => 'تأكد من اسم المستخدم وكلمة المرور',
                    'token' => null,
                    'data' => null,
                    'isAuth' => false
                ),
                JSON_UNESCAPED_UNICODE
            );
            exit();
        }
    }
    

    public function updateProfile()
    {
        $this->load->library('TokenMiddleware');
        $token = $this->tokenmiddleware->main();
        $user = $this->jwtmiddleware->decode($token);

        $hash = $user['hash'];
        $data = $this->input->post();
        $result = $this->UserModel->updateProfile($data, $hash);
        if ($result) {
            $user = $this->UserModel->getUser($data['username'], $data['password']);
            echo json_encode(
                array(
                    'status' => 200,
                    'message' => 'تم تحديث البيانات بنجاح',
                    'token' => null,
                    'data' => $user,
                    "action" => true,
                    'isAuth' => true
                ),
                JSON_UNESCAPED_UNICODE
            );
            exit();
        } else {
            echo json_encode(
                array(
                    'status' => 400,
                    'message' => 'نفس البيانات موجودة بالفعل',
                    'token' => null,
                    'data' => null,
                    'isAuth' => false
                ),
                JSON_UNESCAPED_UNICODE
            );
            exit();
        }
    }

    // get all employees
    public function getEmployees()
    {
        $this->load->library('TokenMiddleware');
        $token = $this->tokenmiddleware->main();
        $this->UserModel->checkPermission();
        $user = $this->jwtmiddleware->decode($token);

        $employees = $this->UserModel->getEmployees();
        echo json_encode(
            array(
                'status' => 200,
                'message' => 'تم تسجيل الدخول بنجاح',
                'token' => $user,
                'data' => $employees,
                'isAuth' => true
            ),
            JSON_UNESCAPED_UNICODE
        );
        exit();
    }

public function error_logs_fingerPrint() {
    $error_message = $this->input->post("message");
    $error_code = 400; // HTTP status code for Bad Request

    $this->UserModel->error_logs_fingerPrint($error_message, $error_code);
    echo json_encode(
        array(
            'status' => 200,
            'message' => 'تم تسجيل الخطأ بنجاح',
            'token' => null,
            'data' => null,
            'isAuth' => false
        ),
    );
}

    public function fingerPrint()
    {
        $this->load->library('TokenMiddleware');
        $token = $this->tokenmiddleware->main();
        $this->UserModel->checkPermission();
        $user = $this->jwtmiddleware->decode($token);

        $hash = $this->input->post('hash');
        $x = $this->input->post('x');
        $y = $this->input->post('y');
        $type = $this->input->post('type');
        $mobileInfo = $this->input->post('mobileInfo');
        $fingerPrintTime = date('Y-m-d H:i:s');
        // check if hash , x , y , type , mobileInfo is not null
        if ($hash == null || $x == null || $y == null || $type == null || $mobileInfo == null) {
            echo json_encode(
                array(
                    'status' => 400,
                    'message' => 'يجب ادخال جميع البيانات',
                    'data' => null,
                    'isAuth' => false
                ),
                JSON_UNESCAPED_UNICODE
            );
            exit();
        }
        $result = $this->UserModel->fingerPrint($hash, $x, $y, $type, $mobileInfo , $fingerPrintTime);
        // $this->jwtmiddleware->insertNotToFirebase([
        //     "type" => "fingerPrint",
        //     "status" => $type,
        //     "user" => $hash,
        // ]);

        if (is_string($result)) {
            echo json_encode(
                array(
                    'status' => 400,
                    'message' => $result,
                    'data' => null,
                    'isAuth' => false
                ),
                JSON_UNESCAPED_UNICODE
            );
            exit();
        } else {
            echo json_encode(
                array(
                    'status' => 200,
                    'message' => 'تم تسجيل البصمة بنجاح',
                    'data' => null,
                    'isAuth' => true
                ),
                JSON_UNESCAPED_UNICODE
            );
            exit();
        }

    }

    public function summary()
    {
        $this->load->library('TokenMiddleware');
        $token = $this->tokenmiddleware->main();
        $this->UserModel->checkPermission();
        $user = $this->jwtmiddleware->decode($token);


        $hash = $user['hash'];
        $summary = $this->UserModel->summary($hash);
        echo json_encode(
            array(
                'status' => 200,
                'message' => 'ملخص الموظف',
                'data' => $summary,
                'isAuth' => true
            ),
            JSON_UNESCAPED_UNICODE
        );
        exit();
    }

    public function updateUserPermission()
    {
        $this->load->library('TokenMiddleware');
        $token = $this->tokenmiddleware->main();
        $this->UserModel->checkPermission();
        $user = $this->jwtmiddleware->decode($token);

        // get all post data
        $data = $this->input->post();
        // remove hash from data
        $hash = $data['hash'];
        unset($data['hash']);
        $this->UserModel->updateUserPermission($hash, $data);
        echo json_encode(
            array(
                'status' => 200,
                'message' => 'تم تحديث الصلاحيات بنجاح',
                'data' => null,
                'isAuth' => true,
                'action' => true,
                'hash' => $hash
            ),
            JSON_UNESCAPED_UNICODE
        );
        exit();
    }

    public function getUserPermission()
    {
        $this->load->library('TokenMiddleware');
        $token = $this->tokenmiddleware->main();
        $this->UserModel->checkPermission();
        $user = $this->jwtmiddleware->decode($token);

        $hash = $this->input->get('hash');
        $permission = $this->UserModel->getUserPermission($hash);
        echo json_encode(
            array(
                'status' => 200,
                'message' => 'عرض الصلاحيات بنجاح',
                'data' => $permission,
                'isAuth' => true
            ),
            JSON_UNESCAPED_UNICODE
        );
        exit();
    }

    public function getUserActivePermission()
    {
        $token = $this->tokenmiddleware->main();
        $user = $this->jwtmiddleware->decode($token);

        $hash = $user['hash'];
        $permission = $this->UserModel->getUserActivePermission($hash);
        echo json_encode(
            array(
                'status' => 200,
                'message' => 'عرض الصلاحيات بنجاح',
                'data' => $permission,
                'isAuth' => true
            ),
            JSON_UNESCAPED_UNICODE
        );
        exit();
    }
}