<?php
defined('BASEPATH') or exit('No direct script access allowed');

class Book extends CI_Controller
{
    function __construct()
    {
        parent::__construct();
        $this->load->model('BookModel');
        $this->load->model('UserModel');
        $this->load->library('Middleware');
        $this->load->library('migration');
        if ($this->migration->latest() === FALSE) {
            show_error($this->migration->error_string());
        }
    }
    public function index()
    {
    }
    public function create_transaction()
    {
        $token = $this->tokenmiddleware->main();
        $this->UserModel->checkPermission();
        $user = $this->jwtmiddleware->decode($token);
        //
        $hash = $this->input->post('hash');
        $subject = $this->input->post('subject');
        $details = $this->input->post('details');
        $export_side = $this->input->post('export_side');
        $import_side = $this->input->post('import_side');
        $attachment = $this->input->post('attachment');
        $book_date = $this->input->post('book_date');
        $book_number = $this->input->post('book_number');
        $book_code = $this->input->post('book_code');
        $is_exported = $this->input->post('is_exported');
        $is_attachment = $this->input->post('is_attachment') ?? 0;
        $attached_book_id = $this->input->post('book_id');
        $count = $this->BookModel->get_count();

        $data = array(
            'hash' => $hash,
            'subject' => $subject,
            'details' => $details,
            'export_side' => $export_side,
            // 'import_side' => $import_side,
            'count' => $count + 1,
            'attachment' => $attachment,
            'book_date' => $book_date,
            'book_number' => $book_number,
            'is_exported ' => $is_exported,
            'is_attachment ' => $is_attachment,
            'main_book  ' => $attached_book_id ?? null,
            'book_code' => $book_code
        );

        $response = $this->BookModel->create_book_transaction($data);

        $import_side_as_object = array();

        $import_side_int = explode(",", $import_side);
        foreach ($import_side_int as $side) {
            $import_side_as_object[] = array('transaction_id' => $response, 'company_id' => $side);
        }
        // insert all import sides in the median table
        $import_sides_res = $this->BookModel->insert_import_sides($import_side_as_object);

        $this->BookModel->update_count();
        echo json_encode(
            array(
                'status' => 200,
                'message' => 'تم اضافة التحويل بنجاح',
                'token' => $token,
                'isAuth' => true,
                'action' => true,
                'response' => $response
            ),
            JSON_UNESCAPED_UNICODE
        );
        exit();
    }

    public function set_count()
    {
        $token = $this->tokenmiddleware->main();
        $this->UserModel->checkPermission();
        $user = $this->jwtmiddleware->decode($token);
        //

        $count = $this->input->post('count');
        $response = $this->BookModel->set_count($count);
        echo json_encode(
            array(
                'status' => 200,
                'message' => 'تم تعديل العداد بنجاح',
                'token' => $token,
                'isAuth' => true,
                'action' => true,
                'response' => $response
            ),
            JSON_UNESCAPED_UNICODE
        );
        exit();
    }

    public function get_transactions()
    {
        $token = $this->tokenmiddleware->main();
        $this->UserModel->checkPermission();
        $user = $this->jwtmiddleware->decode($token);

        $page = $this->input->post('page');
        if (!$page)
            $page = 0;

        $per_page = $this->input->post('perPage');
        if (!$per_page)
            $per_page = 10;

        $start_date = $this->input->post('start_date');
        $end_date = $this->input->post('end_date');
        $book_date = $this->input->post('book_date');
        $export_side = $this->input->post('export_side');
        $import_side = $this->input->post('import_side');
        $subject = $this->input->post('subject');
        $is_exported = $this->input->post('is_exported');
        $book_number = $this->input->post('book_number');
        $description = $this->input->post('description');
        $hash = $this->input->post('hash');
        $data = array(
            'page' => $page,
            'per_page' => $per_page,
            'start_date' => $start_date,
            'end_date' => $end_date,
            'export_side' => $export_side,
            'import_side' => $import_side,
            'subject' => $subject,
            'is_exported' => $is_exported,
            'book_date' => $book_date,
            'book_number' => $book_number,
            'hash' => $hash,
            'description' => $description
        );

        $response = $this->BookModel->get_transactions($data);
        $total = $this->BookModel->get_transactions_count($data);

        // get attachment of every book
        foreach ($response as &$transaction) {
            $id = $transaction->id;
            $attachments = $this->BookModel->get_attachments($id);
            $transaction->all = $attachments;
        }
        foreach ($response as &$attachment) {
            if ($attachment->is_attachment == 1 and $attachment->main_book != 0) {
                $id = $attachment->main_book;
                $parent_book = $this->BookModel->get_attachments($id);
                $attachment->main_book = $parent_book[0];
            }
        }
        echo json_encode(
            array(
                'status' => 200,
                'message' => 'البيانات',
                'token' => $token,
                'isAuth' => true,
                'action' => false,
                'data' => $response,
                'totalPages' => intval($total / $per_page) + 1,
                'total' => $total,
                'is_exported' => $is_exported
            ),
            JSON_UNESCAPED_UNICODE
        );
        exit();
    }

    public function add_side_company()
    {
        $token = $this->tokenmiddleware->main();
        $this->UserModel->checkPermission();
        $user = $this->jwtmiddleware->decode($token);

        $name = $this->input->post('name');
        $response = $this->BookModel->add_side_company($name);

        echo json_encode(
            array(
                'status' => 200,
                'message' => 'تم اضافة جهة جديدة بنجاح',
                'token' => $token,
                'isAuth' => true,
                'action' => true,
                'response' => $response
            ),
            JSON_UNESCAPED_UNICODE
        );
        exit();

    }

    public function get_side_companies()
    {
        $token = $this->tokenmiddleware->main();
        $this->UserModel->checkPermission();
        $user = $this->jwtmiddleware->decode($token);

        $response = $this->BookModel->get_sides();

        echo json_encode(
            array(
                'status' => 200,
                'message' => 'الجهات',
                'token' => $token,
                'isAuth' => true,
                'action' => false,
                'data' => $response,
                'total' => 10
            ),
            JSON_UNESCAPED_UNICODE
        );
        exit();
    }

    public function get_attachments()
    {
        $token = $this->tokenmiddleware->main();
        $this->UserModel->checkPermission();
        $user = $this->jwtmiddleware->decode($token);
        $book_id = $this->input->post('book_id');

        $response = $this->BookModel->get_attachments($book_id);

        echo json_encode(
            array(
                'status' => 200,
                'message' => 'المرفقات',
                'token' => $token,
                'isAuth' => true,
                'action' => false,
                'data' => $response
            ),
            JSON_UNESCAPED_UNICODE
        );
        exit();
    }

    //    public function create_attachments()
//    {
//        $token = $this->tokenmiddleware->main();
//        $user = $this->jwtmiddleware->decode($token);
//
//        $name = $this->input->post('name');
//        $attachment = $this->input->post('attachment');
//        $notes = $this->input->post('notes');
//        $book_id = $this->input->post('book_id');
//
//        $data = array(
//            'name' => $name,
//            'location' => $attachment,
//            'notes' => $notes,
//            'book_id' => $book_id,
//        );
//
//        $response = $this->BookModel->create_attachments($data);
//        echo json_encode(
//            array(
//                'status' => 200,
//                'message' => 'تم اضافة المرفقات بنجاح',
//                'token' => $token,
//                'isAuth' => true,
//                'action' => true,
//            ),
//            JSON_UNESCAPED_UNICODE
//        );
//        exit();
//    }

    public function delete_transaction()
    {
        $token = $this->tokenmiddleware->main();
        $this->UserModel->checkPermission();
        $user = $this->jwtmiddleware->decode($token);

        $transaction_id = $this->input->post('id');
        $attachments = $this->BookModel->get_attachments($transaction_id);

        if (!empty($attachments)) {
            echo json_encode(
                array(
                    'status' => 401,
                    'message' => 'لا يمكن حذف كتاب لديه ملحقات، يجب حذف ملحقاته اولا',
                    'token' => $token,
                    'isAuth' => true,
                    'action' => true,
                    'attachments' => $attachments

                ),
                JSON_UNESCAPED_UNICODE
            );
            exit();
        }

        $response = $this->BookModel->delete_attachments($transaction_id);
        echo json_encode(
            array(
                'status' => 200,
                'message' => 'تم حذف أرشفة الكتاب بنجاح',
                'token' => $token,
                'isAuth' => true,
                'action' => true,
                'response' => $response
            ),
            JSON_UNESCAPED_UNICODE
        );
        exit();
    }

    public function set_desc()
    {
        //        $token = $this->tokenmiddleware->main();
        $this->UserModel->checkPermission();
        //        $user = $this->jwtmiddleware->decode($token);

        $hash = $this->input->post('hash');
        $desc = $this->input->post('desc');
        $response = $this->BookModel->set_desc($hash, $desc);
        echo json_encode(
            array(
                'status' => 200,
                'message' => 'تم اضافة الوصف بنجاح',
                //                'token' => $token,
                'isAuth' => true,
                'action' => true,
                'response' => $response
            ),
            JSON_UNESCAPED_UNICODE
        );
        exit();
    }

    public function get_books()
    {
        $token = $this->tokenmiddleware->main();
        $this->UserModel->checkPermission();
        $user = $this->jwtmiddleware->decode($token);

        $response = $this->BookModel->get_books();
        echo json_encode(
            array(
                'status' => 200,
                'message' => 'الكتب',
                'token' => $token,
                'isAuth' => true,
                'action' => false,
                'data' => $response
            ),
            JSON_UNESCAPED_UNICODE
        );
        exit();
    }

    public function edit_transaction()
    {
        $token = $this->tokenmiddleware->main();
        $this->UserModel->checkPermission();
        $user = $this->jwtmiddleware->decode($token);
        $id = $this->input->post('id');
        $subject = $this->input->post('subject');
        $details = $this->input->post('details');
        $book_number = $this->input->post('book_number');
        $is_exported = $this->input->post('is_exported');
        $book_date = $this->input->post('book_date');
        $hash = $this->input->post('hash');
        $export_side = $this->input->post('export_side');
        $import_sides = $this->input->post('import_sides');
        $attachment = $this->input->post('attachment');
        if (isset($import_sides)) {
            $import_side_as_object = array();
            $import_side_int = explode(",", $import_sides);
            foreach ($import_side_int as $side) {
                $import_side_as_object[] = array('transaction_id' => $id, 'company_id' => $side);
            }
            // Delete old records in transaction_to_side_imports
            $affected_rows = $this->BookModel->delete_old_import_sides($id);
            // insert all import sides in the median table
            $import_sides_res = $this->BookModel->insert_import_sides($import_side_as_object);
        }

        $data = array(
            'subject' => $subject,
            'details' => $details,
            'book_number' => $book_number,
            'is_exported' => $is_exported,
            'id' => $id,
            'book_date' => $book_date,
            'hash' => $hash,
            'export_side' => $export_side,
            'attachment' => $attachment
        );

        $response = $this->BookModel->edit_book_transaction($data);

        echo json_encode(
            array(
                'status' => 200,
                'message' => 'تم تعديل التحويل بنجاح',
                'token' => $token,
                'isAuth' => true,
                'action' => true,
                'response' => $response,
                'export_side' => $data['export_side']
            ),
            JSON_UNESCAPED_UNICODE
        );
        exit();
    }

    public function get_book_code()
    {
        $token = $this->tokenmiddleware->main();
        $this->UserModel->checkPermission();
        $user = $this->jwtmiddleware->decode($token);
        $id = $this->input->post('book_id');
        $response = $this->BookModel->get_book_code($id);
        echo json_encode(
            array(
                'status' => 200,
                'message' => 'كود الكتاب',
                'token' => $token,
                'isAuth' => true,
                'action' => false,
                'data' => $response
            ),
            JSON_UNESCAPED_UNICODE
        );
        exit();
    }

    public function delete_sides()
    {
        $token = $this->tokenmiddleware->main();
        $this->UserModel->checkPermission();
        $user = $this->jwtmiddleware->decode($token);

        $side_id = $this->input->post('id');
        $response = $this->BookModel->delete_side($side_id);
        echo json_encode(
            array(
                'status' => 200,
                'message' => 'تم حذف الجهة بنجاح',
                'token' => $token,
                'isAuth' => true,
                'action' => true,
                'response' => $response
            ),
            JSON_UNESCAPED_UNICODE
        );
        exit();
    }
}