<?php

defined('BASEPATH') or exit('No direct script access allowed');

class Document extends CI_Controller
{

    private $officeServerUrl = 'http://office.native-code-iq.com';

    public function __construct()
    {
        parent::__construct();
        $this->load->model('Document_model');
        $this->load->model('UserModel');
        $this->load->model('DocManager');
        $this->load->model('HistoryManger');
        $this->load->model('DocManager');
        $this->load->model('DocumentJwtManager');
        $this->load->model('DocumentTrackerManger');
        $this->load->library('upload');


        // $this->load->model('DocumentUsers');
        $this->load->library('Middleware');

        $this->load->helper('url');

    }

    public function createDocument()
    {
        $token = $this->tokenmiddleware->main();
        $this->UserModel->checkPermission();
        $user = $this->jwtmiddleware->decode($token)['hash'];


        $apiUrl = $this->officeServerUrl . '/create-document';
        $fileType = $this->input->post('fileType');
        $useSample = $this->input->post('useSample') ?? false;

        $workSpaceId = $this->input->post('document_workspace_id') ?? NULL;
        $title = $this->input->post('title') ?? NULL;


        if (empty($workSpaceId)) {
            echo json_encode(array(
                'status' => 400,
                'message' => 'document_workspace_id is required'
            ));
            exit();
        }

        if (empty($fileType)) {
            echo json_encode(array(
                'status' => 400,
                'message' => 'fileType is required'
            ));
            exit();
        }

        if (empty($title)) {
            echo json_encode(array(
                'status' => 400,
                'message' => 'title is required'
            ));
            exit();
        }




        $postData = json_encode([
            'fileType' => $fileType,
            'useSample' => $useSample
        ]);
        $ch = curl_init();
        curl_setopt($ch, CURLOPT_URL, $apiUrl);
        curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
        curl_setopt($ch, CURLOPT_POST, true);
        curl_setopt($ch, CURLOPT_POSTFIELDS, $postData);
        curl_setopt($ch, CURLOPT_HTTPHEADER, [
            'Content-Type: application/json',
        ]);
        $response = curl_exec($ch);
        $httpCode = curl_getinfo($ch, CURLINFO_HTTP_CODE);
        curl_close($ch);

        if ($httpCode === 200) {
            $resData = json_decode($response, true); // The second parameter true returns an associative array

            $data = [
                'document_workspace_id' => $workSpaceId,
                'file_url' => $this->officeServerUrl . $resData['url'],
                'title' => $title,
                'file_type' => $fileType,
                'unique_key' => $resData['file'],
                'user_id' => $user,
                'created_at' => date('Y-m-d H:i:s'),
                'updated_at' => date('Y-m-d H:i:s')

            ];

            $documentId = $this->Document_model->createDocument($data);

            echo json_encode(array(
                "status" => 200,
                "data" => [
                    "message" => "File created successfully",
                    "filename" => $title,
                    "documentId" => $documentId,

                ],
            ));

        } else {
            echo json_encode(array(
                'status' => 400,
                'message' => $response . " " . $httpCode
            ));

            exit();
        }
    }

    /**
     * Handle document file upload.
     * Expected POST parameter: 'document' file.
     */
    public function uploadDocument()
    {
        // if (empty($_FILES['file']['name'])) {
        //     return $this->output
        //         ->set_content_type('application/json')
        //         ->set_output(json_encode(value: ['error' => 'No file uploaded.']));
        // }

        $filePath = $_FILES['file']['tmp_name'];
        $fileName = $_FILES['file']['name'];
        $fileMime = $_FILES['file']['type'];

        // Send the file to the Express API
        $response = $this->sendToExpress($filePath, $fileName, $fileMime);

        return $this->output
            ->set_content_type('application/json')
            ->set_output(json_encode($response));
    }

    private function sendToExpress($filePath, $fileName, $fileMime)
    {
        $apiUrl = $this->officeServerUrl . '/upload';

        $postData = [
            'file' => new CURLFile($filePath, $fileMime, $fileName)
        ];

        $ch = curl_init();
        curl_setopt($ch, CURLOPT_URL, $apiUrl);
        curl_setopt($ch, CURLOPT_POST, true);
        curl_setopt($ch, CURLOPT_POSTFIELDS, $postData);
        curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
        curl_setopt($ch, CURLOPT_HTTPHEADER, [
            'Content-Type: multipart/form-data'
        ]);

        $response = curl_exec($ch);
        $error = curl_error($ch);
        curl_close($ch);

        if ($error) {
            return ['error' => $error];
        }

        return json_decode($response, true);
    }

    public function getWorkSpaces()
    {
        $this->UserModel->checkPermission();

        $res = $this->Document_model->getWorkSpaces();

        echo json_encode(array(
            'status' => 200,
            'data' => $res
        ));
        exit();
    }

    public function createWorkSpace()
    {
        $this->UserModel->checkPermission();

        $title = $this->input->post('title');
        if (empty($title)) {
            echo json_encode(array(
                'status' => 400,
                'message' => 'Title is required'
            ));
            exit();
        }

        $data = ['title' => $title];
        $res = $this->Document_model->createWorkSpace($data);
        if ($res) {
            echo json_encode(array(
                'status' => 200,
                'data' => [
                    'message' => 'WorkSpace Created Successfully',
                    'id' => $res
                ],
            ));
            exit();
        }

        echo json_encode(array(
            'status' => 400,
            'data' => [
                'message' => 'WorkSpace Creation Failed',
            ],
        ));
        exit();
    }

    public function getDocs()
    {
        $this->UserModel->checkPermission();
        $token = $this->tokenmiddleware->main();
        $user = $this->jwtmiddleware->decode($token)['hash'];
        try {

            $data = $this->Document_model->getDocs($user);

            if (!empty($data)) {
                echo json_encode(array(
                    'status' => 200,
                    'data' => $data
                ));
                exit();
            } else {
                echo json_encode(array(
                    'status' => 400,
                    'message' => 'something went wrong !'
                ));
                exit();
            }


        } catch (Exception $e) {
            $response['error'] = $e->getMessage();
            echo json_encode($response);
        }
    }

    public function createDocumentOnlyOffice() {
        $this->UserModel->checkPermission();
    
        try {
            $token = $this->tokenmiddleware->main();
            $user = $this->jwtmiddleware->decode($token)['hash'];
            $document_workspace_id = $this->Document_model->getWorkSpaceForUser($user);
    
            // OnlyOffice parameters
            $template_id = $this->input->post('template_id') ?? NULL;
            $fileType = $this->input->post('fileType') ?? NULL;
    
            if(!empty($template_id)) {
                $template = $this->Document_model->getDocumentTemplate($template_id);
                if(empty($template)) {
                    echo json_encode([
                        'status' => 400,
                        'message' => 'Template not found'
                    ]);
                    exit();
                }
                $fileType = pathinfo($template['file_name'], PATHINFO_EXTENSION);
                $template_id = $template['file_name'];
            }

            // Data for storing in DB
            $title = $this->input->post('title') ?? NULL;
    
            // Call OnlyOffice API
            $responseData = $this->sendOnylOfficeCreateRequest($fileType, $template_id);
    
            // If API request failed
            if ($responseData['statusCode'] !== 200) {
                echo json_encode([
                    'status' => $responseData['statusCode'],
                    'message' => 'Something went wrong!',
                    'error' => $responseData['body']
                ]);
                exit();
            }
    
            // Decode response body
            $bodyData = json_decode($responseData['body'], true);
    
            if (!isset($bodyData['fileName'], $bodyData['documentType'], $bodyData['key'])) {
                echo json_encode([
                    'status' => 400,
                    'message' => 'Invalid response from OnlyOffice API'
                ]);
                exit();
            }
    
            // Save document to database
            $fileName = $bodyData['fileName'];
            $documentType = $bodyData['documentType'];
            $file_key = $bodyData['key'];
    
            $documentData = [
                'document_workspace_id' => $document_workspace_id,
                'file_name' => $fileName,
                'user_id' => $user,
                'file_key' => $file_key,
                'title' => $title,
                'file_type' => $fileName,
                'doc_type' => $documentType,
                'created_at' => date('Y-m-d H:i:s'),
                'updated_at' => date('Y-m-d H:i:s')
            ];
    
            $documentId = $this->Document_model->createDocument($documentData);
    
            // Success response
            echo json_encode([
                'status' => 200,
                'message' => 'Document created successfully!',
                'filename' => $fileName,
                'documentType' => $documentType,
                'documentId' => $documentId
            ]);
        } catch (Exception $e) {
            // Handle unexpected errors
            echo json_encode([
                'status' => 500,
                'message' => 'Internal Server Error',
                'error' => $e->getMessage()
            ]);
        }
        exit();
    }

    

    private function sendOnylOfficeCreateRequest($fileType, $template_id = NULL) {
        $apiUrl = $this->officeServerUrl . '/create?fileType=' . urlencode($fileType);

        if ($template_id !== NULL) {
            $apiUrl .= '&sample=' . urlencode($template_id);
        }


        $ch = curl_init($apiUrl);
        
        curl_setopt($ch, CURLOPT_POST, true);
        curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
        curl_setopt($ch, CURLOPT_HEADER, true); // Include headers in response
        curl_setopt($ch, CURLOPT_HTTPHEADER, [
            'Content-Type: multipart/form-data'
        ]);

        // Execute request
        $response = curl_exec($ch);
        
        // Check for cURL errors
        if (curl_errno($ch)) {
            curl_close($ch);
            return ['statusCode' => 500, 'body' => json_encode(['error' => curl_error($ch)])];
        }

        // Get HTTP status code
        $httpCode = curl_getinfo($ch, CURLINFO_HTTP_CODE);

        // Separate headers from body
        $headerSize = curl_getinfo($ch, CURLINFO_HEADER_SIZE);
        $body = substr($response, $headerSize);

        // Close cURL
        curl_close($ch);

        return ['statusCode' => $httpCode, 'body' => $body];
    }

    public function createSampleDocumentOnlyOffice() {
        $this->UserModel->checkPermission();

        try {
            $token = $this->tokenmiddleware->main();
            $user = $this->jwtmiddleware->decode($token)['hash'];
            $title = $this->input->post('title') ?? NULL;
            $is_template = $this->input->post('is_template') ?? false;
            if(empty($title)) {
                echo json_encode([
                    'status' => 400,
                    'message' => 'Title is required'
                ]);
                exit();
            }


            $responseData = $this->uploadDocumentToOnlyOffice($is_template);
        
        
            if ($responseData['statusCode'] !== 200) {
                echo json_encode([
                    'status' => $responseData['statusCode'],
                    'message' => 'Something went wrong!',
                    'error' => $responseData['body']
                ]);
                exit();
            }

            $bodyData = json_decode($responseData['body'], true);
            $documentType = $bodyData['documentType'];
            $fileName = $bodyData['filename'];
            $file_key = $bodyData['key'] ?? NULL;

            $documentTemplateData = [
                'title' => $title,
                'file_name' => $fileName,
                'doc_type' => $documentType,
                'file_key' => $file_key,
                'owner_id' => $user,
                'created_at' => date('Y-m-d H:i:s'),
                'updated_at' => date('Y-m-d H:i:s')
            ];
    
    
    
           $docTemplateId =  $this->Document_model->createDocumentTemplate($documentTemplateData);

            echo json_encode([
                'status' => 200,
                'message' => 'Document tempalte created successfully!',
                'filename' => $fileName,
                'documentType' => $documentType,
                'documentId' => $docTemplateId
            ]);
            
        } catch (Exception $e) {
            echo json_encode([
                'status' => 500,
                'message' => 'Internal Server Error',
                'error' => $e->getMessage()
            ]);
        }
       

    }

    private function uploadDocumentToOnlyOffice($is_template = false) {
        $apiUrl = $this->officeServerUrl . '/upload';

        if (isset($_FILES['uploadedFile'])) {
            $fileTmpPath = $_FILES['uploadedFile']['tmp_name'];
            $fileName = $_FILES['uploadedFile']['name'];
        
            $postFields = [
                'uploadedFile' => new CURLFile($fileTmpPath, $_FILES['uploadedFile']['type'], $fileName), // ← Add this comma
                'is_template' => $is_template ? 1 : 0
            ];
        
            $ch = curl_init($apiUrl);
            if (!$ch) {
                die("cURL initialization failed!");
            }
            curl_setopt($ch, CURLOPT_POST, true);
            curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
            curl_setopt($ch, CURLOPT_HTTPHEADER, [
                'Content-Type: multipart/form-data'
            ]);

            curl_setopt($ch, CURLOPT_POSTFIELDS, $postFields);
        
            $response = curl_exec($ch);
            if ($response === false) {
                $error = curl_error($ch);
                curl_close($ch);
                return ['statusCode' => 500, 'body' => json_encode(['error' => $error])];
            }
            
            // Get HTTP status code
            $httpCode = curl_getinfo($ch, CURLINFO_HTTP_CODE);
            
            // Close cURL
            curl_close($ch);
            return ['statusCode' => $httpCode, 'body' => $response];
                
        } else {
            echo "No file uploaded.";
        }

    }

    public function getDocumentsTemplates() {
        $this->UserModel->checkPermission();

        $templates = $this->Document_model->getDocumentsTemplate();

        echo json_encode(array(
            'status' => 200,
            'data' => $templates
        ));

    }


    public function getConfigDocumentOnlyOffice($documentId) {
        $this->UserModel->checkPermission();

        $token = $this->tokenmiddleware->main();
        $userHash = $this->jwtmiddleware->decode($token)['hash'];
        $userName = $this->jwtmiddleware->decode($token)['name'];
        $userImage = $this->jwtmiddleware->decode($token)['image_profile'];

        $document = $this->Document_model->getDocument($docId, $userHash);


        if (empty($document)) {
            echo json_encode(array(
                'status' => 400,
                'message' => 'Document not found'
            ));
            exit();
        }
        $type = $this->input->post('type') ?? 'Desktop';
    }

    public function getDocument($docId) {
        $this->UserModel->checkPermission();
        $token = $this->tokenmiddleware->main();
        $userHash = $this->jwtmiddleware->decode($token)['hash'];

        $doc = $this->Document_model->getDocument($docId, $userHash);

        if($doc) {
            echo json_encode([
                'status' => 200,
                'data' => $doc
            ]);
        } else {
            echo json_encode([
                'status' => 400,
                'message' => 'Document not found'
            ]);
        }
        return $doc;
    }

    public function getDocConfigOffice($docId) {
        $this->UserModel->checkPermission();

        $token = $this->tokenmiddleware->main();
        $userHash = $this->jwtmiddleware->decode($token)['hash'];
        $userName = $this->jwtmiddleware->decode($token)['name'];
        $userImage = $this->jwtmiddleware->decode($token)['image_profile'];

        $document = $this->Document_model->getDocument($docId, $userHash);

        if (empty($document)) {
            echo json_encode(array(
                'status' => 400,
                'message' => 'Document not found'
            ));
            exit();
        }

        $type = $this->input->post('type') ?? 'Desktop';
        $docType = $document['doc_type'];

        $fileName = $document['file_name'];
        $key = $document['file_key'];

        $doc = array(
            'type' => $type,
            'documentType' => $docType,
            'document' => array(
                'title' => $fileName,
                'url' => "http://proxy/download?fileName=" . $fileName,
                'directUrl' => "",
                'fileType' => pathinfo($document['file_name'], PATHINFO_EXTENSION),
                'key' => $key,
                'info' => array(
                    'owner' => $userName,
                    'uploaded' => date('Y-m-d H:i:s'),
                    'favorite' => null
                ),
                'permissions' => array(
                    'chat' => true,
                    'comment' => true,
                    'copy' => true,
                    'download' => true,
                    'edit' => true,
                    'fillForms' => true,
                    'modifyContentControl' => true,
                    'modifyFilter' => true,
                    'print' => true,
                    'review' => true,
                    'reviewGroups' => null,
                    'commentGroups' => array(),
                    'userInfoGroups' => null,
                    'protect' => true
                ),
                'referenceData' => array(
                    'instanceId' => "http://office.native-code-iq.com",
                    'fileKey' => json_encode(['fileName' => $fileName], JSON_PRETTY_PRINT),
                ),
            ),
              "actionLink" => null,
              "mode" => "edit",
              "lang" => "en",
              "callbackUrl" => "http://proxy/track?filename=" . $fileName,
              "createUrl" => "http://office.native-code-iq.com/create?fileType=desktop",
              "templates" => [
            [
            "image" => "",
            "title" => "Blank",
            "url" => "http://office.native-code-iq.com/create?fileType=desktop"
            ],
            [
            "image" => "http://office.native-code-iq.com/static/images/file_docx.svg",
            "title" => "With sample content",
            "url" => "http://office.native-code-iq.com/create?fileType=desktop&sample=true"
            ]
        ],
    "user" => [
        "id" => "uid-1",
        "name" => "John Smith",
        "group" => "",
        "image" => "http://office.native-code-iq.com/static/images/uid-1.jpg"
    ],
    "embedded" => [
        "saveUrl" => "http://office.native-code-iq.com/download?fileName=" . $fileName,
        "embedUrl" => "http://office.native-code-iq.com/download?fileName=" . $fileName,
        "shareUrl" => "http://office.native-code-iq.com/download?fileName=" . $fileName,
        "toolbarDocked" => "top"
    ],
    "customization" => [
        "about" => true,
        "comments" => true,
        "feedback" => true,
        "forcesave" => false,
        "submitForm" => true,
        "goback" => [
            "blank" => false,
            "url" => "http://office.native-code-iq.com"
        ]
    ]
        );

        $jwtDocument = $this->jwtmiddleware->genereate_token_document($doc);

        $doc['token'] = $jwtDocument;
        echo json_encode(array(
            'status' => 200,
            'data' => $doc
        ));
        exit();
    }

    public function createNew()
    {
        $this->UserModel->checkPermission();

        $token = $this->tokenmiddleware->main();

        $user = $this->jwtmiddleware->decode($token)['hash'];

        $response = array();

        try {
            $sample = $this->input->get('sample', FALSE);

            $title = $this->input->post('name') ?? NULL;
            $fileType = $this->input->post('file_type') ?? NULL;
            $document_workspace_id = $this->input->post('document_workspace_id') ?? NULL;

            if (empty($document_workspace_id)) {
                echo json_encode(array(
                    'status' => 400,
                    'message' => 'document_workspace_id is required'
                ));
                exit();
            }

            if (empty($fileType)) {
                echo json_encode(array(
                    'status' => 400,
                    'message' => 'file_type is required'
                ));
                exit();
            }

            if (empty($title)) {
                $title = 'Document_' . time(); // Example: Document_1700000000
            }

            if (!$fileType) {
                throw new Exception("fileType is required");
            }

            // Pass the title to createSample
            $filename = $this->DocManager->createSample($fileType, $sample, $title);



            $documentData = array(
                'document_workspace_id' => $document_workspace_id,
                'file_url' => "/" . $filename,
                'title' => $title,
                'file_type' => $fileType,
                'unique_key' => time(),
                'user_id' => $user,
                'created_at' => date('Y-m-d H:i:s'),
                'updated_at' => date('Y-m-d H:i:s')
            );

            $documentId = $this->Document_model->createDocument($documentData);
            echo json_encode(array(
                "status" => 200,
                "data" => [
                    "message" => "File created successfully",
                    "filename" => $filename,
                    "documentId" => $documentId,

                ],
            ));

            exit();
        } catch (Exception $e) {
            $response['error'] = $e->getMessage();
            echo json_encode($response);
        }
    }

    public function getDocConfig($docId)
    {
        $this->UserModel->checkPermission();

        $token = $this->tokenmiddleware->main();
        $userHash = $this->jwtmiddleware->decode($token)['hash'];
        $userName = $this->jwtmiddleware->decode($token)['name'];
        $userImage = $this->jwtmiddleware->decode($token)['image_profile'];

        $document = $this->Document_model->getDocument($docId, $userHash);


        if (empty($document)) {
            echo json_encode(array(
                'status' => 400,
                'message' => 'Document not found'
            ));
            exit();
        }




        $type = $this->input->post('type');

        // $fileName = $this->input->post('filename');
        $fileName = $document['title'];

        $editorMode = $this->input->get('editorMode') ?? NULL;
        $documentType = $this->DocManager->getFileType($fileName);
        $docExtenstion = $this->DocManager->getFileExtenstion($fileName);

        $canFill = $this->DocManager->isCanFillForms($documentType);
        if (empty($editorMode) && $canFill) {
            $editorMode = "fillForms";
        } else if (empty($editorMode) && !$canFill) {
            $editorMode = "edit";
        }

        // TODO: validation ensure all inserted

        $downloadUrl = $this->DocManager->getDownloadUrl($fileName);


        $types = array('desktop', 'mobile', 'embedded');
        $edType = in_array($type, $types) ? $type : 'desktop';  // Default to 'desktop' if not valid
        $docKey = $this->DocManager->generateFileKey($fileName);
        $storagePath = $this->DocManager->getStoragePath($fileName);


        // $user = $this->DocumentUsers->getUserFromReq($userHash);  # get user


        $meta = $this->HistoryManger->getMetaPath($storagePath);
        $infoObj = NULL;




        if (isset($meta) && !empty($meta)) {
            // If meta data exists, use the provided owner and creation time
            $infoObj = [
                'owner' => $meta['uname'] ?? 'ME',
                'uploaded' => $meta['created'] ?? date('d.m.Y H:i:s')
            ];
        } else {
            // Otherwise, use default values: 'Me' as the owner and the current date/time
            $infoObj = [
                'owner' => 'Me',
                'uploaded' => date('d.m.Y H:i:s')
            ];
        }

   
        $doc = array(
            "document" => array(
                "directUrl" => "",
                "fileType" => "docx",
                "info" => array(
                    "owner" => "John Smith",
                    "uploaded" => "2025-03-24 20:16:49",
                    "favorite" => null
                ),
                "key" => "-5887718913756390027",
                "permissions" => array(
                    "chat" => true,
                    "comment" => true,
                    "copy" => true,
                    "download" => true,
                    "edit" => true,
                    "fillForms" => true,
                    "modifyContentControl" => true,
                    "modifyFilter" => true,
                    "print" => true,
                    "review" => true,
                    "reviewGroups" => null,
                    "commentGroups" => array(),
                    "userInfoGroups" => null,
                    "protect" => true
                ),
                "referenceData" => array(
                    "fileKey" => "{\"fileName\":\"new.docx\",\"userAddress\":\"197.132.184.246__172.18.0.2\"}",
                    "instanceId" => "https://office.native-code-iq.com/example"
                ),
                "title" => "new.docx",
                "url" => "https://office.native-code-iq.com/example/download?fileName=new.docx&useraddress=197.132.184.246__172.18.0.2",
                "isForm" => false
            ),
            "documentType" => "word",
            "editorConfig" => array(
                "actionLink" => null,
                "callbackUrl" => "https://office.native-code-iq.com/example/track?filename=new.docx&useraddress=197.132.184.246__172.18.0.2",
                "coEditing" => null,
                "createUrl" => "https://office.native-code-iq.com/example/editor?fileExt=docx&userid=uid-1&type=desktop&lang=en",
                "customization" => array(
                    "about" => true,
                    "comments" => true,
                    "close" => array(
                        "visible" => false
                    ),
                    "feedback" => true,
                    "forcesave" => false,
                    "goback" => array(
                        "blank" => false,
                        "url" => "https://office.native-code-iq.com/example"
                    ),
                    "submitForm" => true
                ),
                "embedded" => array(
                    "embedUrl" => "https://office.native-code-iq.com/example/download?fileName=new.docx",
                    "saveUrl" => "https://office.native-code-iq.com/example/download?fileName=new.docx",
                    "shareUrl" => "https://office.native-code-iq.com/example/download?fileName=new.docx",
                    "toolbarDocked" => "top"
                ),
                "fileChoiceUrl" => "",
                "lang" => "en",
                "mode" => "edit",
                "plugins" => array(
                    "pluginsData" => array()
                ),
                "templates" => array(
                    array(
                        "image" => "",
                        "title" => "Blank",
                        "url" => "https://office.native-code-iq.com/example/editor?fileExt=docx&userid=uid-1&type=desktop&lang=en"
                    ),
                    array(
                        "image" => "https://office.native-code-iq.com/example/images/file_docx.svg",
                        "title" => "With sample content",
                        "url" => "https://office.native-code-iq.com/example/editor?fileExt=docx&userid=uid-1&type=desktop&lang=en&sample=true"
                    )
                ),
                "user" => array(
                    "group" => "",
                    "id" => "uid-1",
                    "image" => "https://office.native-code-iq.com/example/images/uid-1.png",
                    "name" => "John Smith"
                ),
                "canCoAuthoring" => true,
                "canUseHistory" => true,
                "canHistoryClose" => true,
                "canHistoryRestore" => true,
                "canSendEmailAddresses" => false,
                "canRequestEditRights" => true,
                "canRequestClose" => true,
                "canRename" => true,
                "canMakeActionLink" => true,
                "canRequestUsers" => true,
                "canRequestSendNotify" => true,
                "canRequestSaveAs" => true,
                "canRequestInsertImage" => true,
                "canRequestMailMergeRecipients" => false,
                "canRequestCompareFile" => false,
                "canRequestSharingSettings" => false,
                "canRequestCreateNew" => false,
                "canRequestReferenceData" => true,
                "canRequestOpen" => true,
                "canRequestSelectDocument" => true,
                "canRequestSelectSpreadsheet" => true,
                "canRequestReferenceSource" => true,
                "canSaveDocumentToBinary" => false,
                "canStartFilling" => false,
                "canRequestRefreshFile" => true,
                "canUpdateVersion" => true
            ),
            "height" => "100%",

            "type" => "desktop",
            "width" => "100%",
            "events" => [],
            "frameEditorId" => "iframeEditor",
            "parentOrigin" => "https://office.native-code-iq.com"
        );
        $jwtDocument = $this->jwtmiddleware->genereate_token_document($doc);



        $doc['token'] = $jwtDocument;


        echo json_encode(array(
            'status' => 200,
            'data' => $doc
        ));
        exit();

    }



    // this for callback function that will handle save and update document
    public function track()
    {


        echo json_encode(array('error' => 0));
        exit();
        $res = [];

        try {
            $json = file_get_contents('php://input');
            $data = $this->readData(json_decode($json));
            var_dump($data);
        } catch (Exception $e) {
            $res['error'] = $e->getMessage();
        }
    }

    private function readData($data)
    {
        header("Access-Control-Allow-Origin: *");

        // Allow specific methods
        header("Access-Control-Allow-Methods: GET, POST, OPTIONS, PUT, DELETE");

        // Allow specific headers (e.g., Authorization, Content-Type)
        header("Access-Control-Allow-Headers: Content-Type, Authorization");

        if ($this->DocumentJwtManager->isEnabled() && $this->DocumentJwtManager->useForRequest()) {
            // Attempt to retrieve the token from the request body
            var_dump($data);
            return;
            $token = isset($data['token']) ? $data['token'] : null;

            // If token is not provided in the body, check the Authorization header
            if (!$token) {
                $headerName = $this->Configuration_model->jwt_header();
                $token = $this->input->get_request_header($headerName, TRUE);

                // If the token begins with "Bearer ", remove that prefix
                if ($token && stripos($token, 'Bearer ') === 0) {
                    $token = substr($token, strlen('Bearer '));
                }
            }

            // If no token is found, handle the error (e.g., show an error message or throw an exception)
            if (!$token) {
                throw new Exception('No JWT token provided');
            }

            // Decode the JWT token
            $decoded = $this->DocumentJwtManager->decode($token);

            // If the decoded token contains a "payload", replace $data with that payload
            if (isset($decoded->payload)) {
                $data = $decoded->payload;
            }
        }
        return $data;
    }





    public function callback($documentId)
    {
        // OnlyOffice sends a JSON payload in the request body.
        $json = file_get_contents('php://input');
        $data = json_decode($json);

        // Update the document status or other metadata based on the callback data.
        // $this->Document_model->updateDocumentStatus($documentId, $data);

        echo json_encode(array('error' => 0));
    }
}