<?php

defined('BASEPATH') or exit('No direct script access allowed');

class DocumentController extends CI_Controller
{

    public function __construct()
    {
        parent::__construct();
        $this->load->model('Document_model');
        $this->load->model('UserModel');
        $this->load->model('DocManager');
        $this->load->library('Middleware');
        $this->load->helper('url');
    }

    /**
     * Handle document file upload.
     * Expected POST parameter: 'document' file.
     */
    public function upload()
    {
        // Ensure the user is logged in.
        $token = $this->tokenmiddleware->main();
        $this->UserModel->checkPermission();
        $user = $this->jwtmiddleware->decode($token);
        if (!$user) {
            show_error('Unauthorized', 401);
        }

        $config['upload_path'] = './uploads/';
        $config['allowed_types'] = 'pdf|doc|docx|xls|xlsx|ppt|pptx|txt|odt';
        $this->load->library('upload', $config);

        if (!$this->upload->do_upload('document')) {
            $error = array('error' => $this->upload->display_errors());
            echo json_encode($error);
            return;
        }
        $uploadData = $this->upload->data();

        // Generate a unique key for OnlyOffice (important for caching/versions).
        $documentKey = md5($uploadData['file_name'] . time());

        // Assume each user has one workspace.
        $workspace_id = $user["hash"];

        $documentData = array(
            'workspace_id' => $workspace_id,
            'file_path' => $uploadData['file_name'],
            'title' => $uploadData['orig_name'],
            'document_key' => $documentKey,
            'created_at' => date('Y-m-d H:i:s'),
            'updated_at' => date('Y-m-d H:i:s')
        );
        // $documentId = $this->Document_model->createDocument($documentData);

        echo json_encode(array(
            'status' => 200,
            'data' => [
                'hi' => $documentData,
                'documentId' => 'it just a test now',
                'message' => 'Upload successful'
            ]
        ));
    }

    public function getWorkSpaces()
    {
        $this->UserModel->checkPermission();

        $res = $this->Document_model->getWorkSpaces();

        echo json_encode(array(
            'status' => 200,
            'data' => $res
        ));
        exit();
    }

    public function createWorkSpace()
    {
        $this->UserModel->checkPermission();

        $title = $this->input->post('title');
        if (empty($title)) {
            echo json_encode(array(
                'status' => 400,
                'message' => 'Title is required'
            ));
            exit();
        }

        $data = ['title' => $title];
        $res = $this->Document_model->createWorkSpace($data);
        if ($res) {
            echo json_encode(array(
                'status' => 200,
                'data' => [
                    'message' => 'WorkSpace Created Successfully',
                    'id' => $res
                ],
            ));
            exit();
        }

        echo json_encode(array(
            'status' => 400,
            'data' => [
                'message' => 'WorkSpace Creation Failed',
            ],
        ));
        exit();
    }

    public function getDocs()
    {
        $this->UserModel->checkPermission();
        $token = $this->tokenmiddleware->main();
        $user = $this->jwtmiddleware->decode($token)['hash'];
        try {

            $data = $this->Document_model->getDocs($user);

            if (!empty($data)) {
                echo json_encode(array(
                    'status' => 200,
                    'data' => $data
                ));
                exit();
            } else {
                echo json_encode(array(
                    'status' => 400,
                    'message' => 'something went wrong !'
                ));
                exit();
            }


        } catch (Exception $e) {
            $response['error'] = $e->getMessage();
            echo json_encode($response);
        }
    }

    public function createNew()
    {
        $this->UserModel->checkPermission();

        $token = $this->tokenmiddleware->main();

        $user = $this->jwtmiddleware->decode($token)['hash'];

        $response = array();

        try {
            $sample = $this->input->get('sample', FALSE);

            $title = $this->input->post('name') ?? NULL;
            $fileType = $this->input->post('file_type') ?? NULL;
            $document_workspace_id = $this->input->post('document_workspace_id') ?? NULL;

            if (empty($document_workspace_id)) {
                echo json_encode(array(
                    'status' => 400,
                    'message' => 'document_workspace_id is required'
                ));
                exit();
            }

            if (empty($fileType)) {
                echo json_encode(array(
                    'status' => 400,
                    'message' => 'file_type is required'
                ));
                exit();
            }

            if (empty($title)) {
                $title = 'Document_' . time(); // Example: Document_1700000000
            }

            if (!$fileType) {
                throw new Exception("fileType is required");
            }

            // Pass the title to createSample
            $filename = $this->DocManager->createSample($fileType, $sample, $title);



            $documentData = array(
                'document_workspace_id' => $document_workspace_id,
                'file_url' => "/" . $filename,
                'title' => $title,
                'file_type' => $fileType,
                'unique_key' => time(),
                'user_id' => $user,
                'created_at' => date('Y-m-d H:i:s'),
                'updated_at' => date('Y-m-d H:i:s')
            );

            $documentId = $this->Document_model->createDocument($documentData);
            echo json_encode(array(
                "status" => 200,
                "data" => [
                    "message" => "File created successfully",
                    "filename" => $filename,
                    "documentId" => $documentId,

                ],
            ));

            exit();
        } catch (Exception $e) {
            $response['error'] = $e->getMessage();
            echo json_encode($response);
        }
    }



    public function callback($documentId)
    {
        // OnlyOffice sends a JSON payload in the request body.
        $json = file_get_contents('php://input');
        $data = json_decode($json);

        // Update the document status or other metadata based on the callback data.
        // $this->Document_model->updateDocumentStatus($documentId, $data);

        echo json_encode(array('error' => 0));
    }
}