<?php
defined('BASEPATH') or exit('No direct script access allowed');

class EmpCalcWorkedHoursController extends CI_Controller
{

    public function __construct()
    {
        parent::__construct();
        $this->load->model('EmpCalcWorkedHoursConfigModel');
        $this->load->model('EmployeesWorkedHoursModel');
        $this->load->model('UserModel');
        $this->load->model('Scheduled_tasks_model');
        $this->load->library('Middleware');
    }

    public function index()
    {
        $token = $this->tokenmiddleware->main();
        $this->UserModel->checkPermission();

        $result = $this->EmpCalcWorkedHoursConfigModel->getConfiguration();

        echo json_encode(
            array(
                'status' => 200,
                'message' => 'عرض جميع البيانات',
                'token' => $token,
                'data' => $result,
                'isAuth' => true,
            ),
            JSON_UNESCAPED_UNICODE
        );
        exit();
    }

    public function getConfigration()
    {
        $token = $this->tokenmiddleware->main();
        $this->UserModel->checkPermission();
        $id = $this->input->get('id');

        $result = $this->EmpCalcWorkedHoursConfigModel->getConfigurationById($id);
        echo json_encode(
            array(
                'status' => 200,
                'message' => 'عرض البيانات',
                'token' => $token,
                'data' => $result,
                'isAuth' => true,
            ),
            JSON_UNESCAPED_UNICODE
        );
        exit();

    }

    public function addConfigration()
    {

        $token = $this->tokenmiddleware->main();
        $this->UserModel->checkPermission();
        $data = array(
            'name' => $this->input->post('name'),
            'calculate_full_late' => $this->input->post('calculate_full_late') ?? 1,
            'calc_by_missing' => $this->input->post('calc_by_missing') ?? 0,
            'penalty_multiplier' => $this->input->post('penalty_multiplier') ?? 1.00,
            'is_penalty_based_on_lateArrival' => $this->input->post('is_penalty_based_on_lateArrival') ?? 1,
            'over_time_allowance' => $this->input->post('over_time_allowance') ?? 0,
        );

        $result = $this->EmpCalcWorkedHoursConfigModel->addConfiguration($data);
        echo json_encode(
            array(
                'status' => 200,
                'message' => 'تمت العملية بنجاح',
                'token' => $token,
                'data' => $result,
                'isAuth' => true,
            ),
            JSON_UNESCAPED_UNICODE
        );
        exit();
    }

    public function editConfigration()
    {

        $token = $this->tokenmiddleware->main();
        $this->UserModel->checkPermission();
        $id = $this->input->post('id');

        $data = array(
            'name' => $this->input->post('name'),
            'calculate_full_late' => $this->input->post('calculate_full_late') ?? 1,
            'calc_by_missing' => $this->input->post('calc_by_missing') ?? 0,
            'penalty_multiplier' => $this->input->post('penalty_multiplier') ?? 1.00,
            'is_penalty_based_on_lateArrival' => $this->input->post('is_penalty_based_on_lateArrival') ?? 1,
            'over_time_allowance' => $this->input->post('over_time_allowance') ?? 0,
        );

        $result = $this->EmpCalcWorkedHoursConfigModel->editConfigration($id, $data);

        echo json_encode(
            array(
                'status' => 200,
                'message' => 'تم تعديل البيانات بنجاح',
                'token' => $token,
                'data' => $result,
                'isAuth' => true,
                'action' => true,
            ),
            JSON_UNESCAPED_UNICODE
        );
        exit();
    }

    public function deleteConfiguration()
    {
        $token = $this->tokenmiddleware->main();
        $this->UserModel->checkPermission();
        $id = $this->input->get('id');

        $result = $this->EmpCalcWorkedHoursConfigModel->deleteConfiguration($id);
        $message = '';
        if ($result['status'] === 'success') {
            $message = 'تم حذف البيانات بنجاح';
            echo json_encode(
                array(
                    'status' => 200,
                    'message' => $message,
                    'token' => $token,
                    'data' => $result,
                    'isAuth' => true,
                    'action' => true,
                ),
                JSON_UNESCAPED_UNICODE
            );
            exit();
        } else {
            $message = $result['message'];
            echo json_encode(
                array(
                    'status' => 400,
                    'message' => $message,
                    'token' => $token,
                    'data' => $result,
                    'isAuth' => true,
                    'action' => false,
                ),
                JSON_UNESCAPED_UNICODE
            );
            exit();
        }

    }

    public function makeConfigIsTheMain()
    {
        $token = $this->tokenmiddleware->main();
        $this->UserModel->checkPermission();

        $id = $this->input->post('id');

        if (empty($id)) {
            $response = [
                'status' => 400,
                'message' => 'ID is required',
                'token' => $token,
                'isAuth' => true,
                'action' => false,
            ];
            echo json_encode($response, JSON_UNESCAPED_UNICODE);
            exit();
        }
        $result = $this->EmpCalcWorkedHoursConfigModel->makeConfigIsTheMain($id);

        $response = [
            'status' => 200,
            'message' => 'تمت العملية بنجاح',
            'token' => $token,
            'data' => $result,
            'isAuth' => true,
            'action' => true,
        ];

        if ($result['status'] == 'info') {
            $response['status'] = 200;
            $response['message'] = $result['message'];
        } elseif ($result['status'] == 'success') {
            $response['status'] = 200;
            $response['message'] = $result['message'];
        } elseif ($result['status'] == 'error') {
            $response['status'] = 400;
            $response['message'] = $result['message'];
        }

        echo json_encode($response, JSON_UNESCAPED_UNICODE);
        exit();
    }

    // TODO: this works now with only one employee make employees array and use it
    public function recalculateWorkedHours()
    {
        // Get input data
        $date_start = $this->input->post('date_start');
        $date_end = $this->input->post('date_end');
        $user_hashes = $this->input->post('user_hashes'); // Expecting a JSON string of user hashes
        $configId = $this->input->post('configId');

        // Initialize response array
        $response = ['status' => 'error', 'message' => ''];

        // Validate required fields
        if (empty($date_start) || empty($date_end) || empty($user_hashes)) {
            $response['message'] = 'Start Date, End Date, and User Hashes are required.';
            echo json_encode($response, JSON_UNESCAPED_UNICODE);
            exit();
        }

        // Validate date format
        if (!$this->valid_date($date_start) || !$this->valid_date($date_end)) {
            $response['message'] = 'Invalid date format. Expected format: YYYY-MM-DD.';
            echo json_encode($response, JSON_UNESCAPED_UNICODE);
            exit();
        }

        // Decode user_hashes JSON string to array
        $user_hashes_array = json_decode($user_hashes, true);

        // Validate user_hashes_array
        if (!is_array($user_hashes_array) || empty($user_hashes_array)) {
            $response['message'] = 'User Hashes must be a non-empty JSON array.';
            echo json_encode($response, JSON_UNESCAPED_UNICODE);
            exit();
        }

        // Validate each user_hash
        foreach ($user_hashes_array as $user_hash) {
            if (!ctype_alnum($user_hash)) {
                $response['message'] = 'Each User Hash must contain only alphanumeric characters.';
                echo json_encode($response, JSON_UNESCAPED_UNICODE);
                exit();
            }
        }

        // Validate configId if provided
        if (!empty($configId) && !ctype_digit($configId)) {
            $response['message'] = 'Config ID must be an integer.';
            echo json_encode($response, JSON_UNESCAPED_UNICODE);
            exit();
        }

        // Proceed with your logic for all user_hashes
        // $result = $this->EmployeesWorkedHoursModel->processEmployeeData($date_start, $date_end, $user_hashes_array, $configId);

        $jobData = array(
            'date_start' => $date_start,
            'date_end' => $date_end,
            'employees_user_hash_array' => $user_hashes,
            'configId' => $configId,
        );

        $this->Scheduled_tasks_model->insert_task('calculate_specific_employees', $jobData, date('Y-m-d H:i:s'));

        $response['status'] = 'success';
        $response['message'] = 'Worked hours recalculated successfully for all employees.';

        echo json_encode($response, JSON_UNESCAPED_UNICODE);
        exit();
    }

    public function recalculateWorkedHoursForAll()
    {
        // Get input data for date range and optional configId
        $date_start = $this->input->post('date_start');
        $date_end = $this->input->post('date_end');
        $configId = $this->input->post('configId');

        // Initialize response array
        $response = ['status' => 'error', 'message' => ''];

        // Validate required fields
        if (empty($date_start) || empty($date_end)) {
            $response['message'] = 'Start Date and End Date are required.';
            echo json_encode($response, JSON_UNESCAPED_UNICODE);
            exit();
        }

        // Validate date format
        if (!$this->valid_date($date_start) || !$this->valid_date($date_end)) {
            $response['message'] = 'Invalid date format. Expected format: YYYY-MM-DD.';
            echo json_encode($response, JSON_UNESCAPED_UNICODE);
            exit();
        }

        // Validate configId if provided
        if (!empty($configId) && !ctype_digit($configId)) {
            $response['message'] = 'Config ID must be an integer.';
            echo json_encode($response, JSON_UNESCAPED_UNICODE);
            exit();
        }

        // Fetch all employee user_hashes
        $all_employees = $this->EmployeesWorkedHoursModel->getAllEmployeeUserHashes();

        // If no employees found
        if (empty($all_employees)) {
            $response['message'] = 'No employees found.';
            echo json_encode($response, JSON_UNESCAPED_UNICODE);
            exit();
        }

        $jobData = array(
            'date_start' => $date_start,
            'date_end' => $date_end,
            'configId' => $configId,
        );
        $this->Scheduled_tasks_model->insert_task('calculate_all_employees', $jobData, date('Y-m-d H:i:s'));

        // Check if processing was successful
        $response['status'] = 'success';
        $response['message'] = 'Worked hours recalculated successfully for all employees.';

        echo json_encode($response, JSON_UNESCAPED_UNICODE);
        exit();
    }

    // Custom date validation function
    private function valid_date($date)
    {
        $d = DateTime::createFromFormat('Y-m-d', $date);
        return $d && $d->format('Y-m-d') === $date;
    }
}
