<?php
class EmpPayroll extends CI_Controller
{
    public function __construct()
    {
        parent::__construct();
        $this->load->model('empPayrollModel');
        $this->load->library('JwtMiddleware');
        $this->load->library('Middleware');
        $this->load->model('UserModel');
        $this->load->model('NotificationPayrollModel');

    }

    public function create()
    {

        $this->UserModel->checkPermission();
        if ($this->input->post()) {

            $postedData = $this->input->post();
            $allowedColumns = $this->db->list_fields('emp_payroll');

            $data = array();
            foreach ($postedData as $field => $value) {
                if (in_array($field, $allowedColumns)) {
                    $data[$field] = $value;
                }
            }

            $inserted_id = $this->empPayrollModel->insertEmpPayroll($data);
            if(is_string($inserted_id)) {
                echo json_encode(
                    array(
                        'status' => 400,
                        'message' => $inserted_id,
                        'action' => true
                    ),
                    JSON_UNESCAPED_UNICODE
                );
                exit();
            }

            if ($inserted_id) {
                // Insertion successful
                echo json_encode(
                    array(
                        'status' => 200,
                        'message' => 'تم إضافة الموظف بنجاح',
                        'data' => $inserted_id,
                        'action' => true
                    ),
                    JSON_UNESCAPED_UNICODE
                );
                exit();
            } else {
                // Insertion failed
                echo json_encode(
                    array(
                        'status' => 400,
                        'message' => 'فشل إضافة الموظف',
                        'action' => false
                    ),
                    JSON_UNESCAPED_UNICODE
                );
                exit();
            }
        } else {
            echo json_encode(
                array(
                    'status' => 400,
                    'message' => 'فشل إضافة الموظف: النموذج لم يتم إرساله أو البيانات غير صحيحة',
                    'action' => false
                ),
                JSON_UNESCAPED_UNICODE
            );
            exit();
        }
    }


    // this simillar to create() but can send with emp as array and loop on ever emp to assign
    public function insertEmpsPayroll()
    {

        $this->UserModel->checkPermission();
        if ($this->input->post()) {
            $postedData = $this->input->post();
            $allowedColumns = $this->db->list_fields('emp_payroll');
            $jsonString = $this->input->post("data");

            $hash = $this->jwtmiddleware->createHash();
            $data = json_decode($jsonString, true);
            foreach ($data as $item) {
                $itemToInsert = array(); // Initialize an array to hold the data for each item
                $itemToInsert['hash'] = $hash;


                foreach ($item as $field => $value) {
                    if (in_array($field, $allowedColumns)) {
                        $itemToInsert[$field] = $value; // Add only allowed fields to the item to insert
                    }
                }


                // Now, insert the item into the database
                $inserted_id = $this->empPayrollModel->insertEmpPayroll($itemToInsert);
            }

            // // Assuming $postedData is an array of arrays or objects
            // foreach ($postedData as $item) {
            //     $data = array();
            //     foreach ($item as $field => $value) {
            //         if (in_array($field, $allowedColumns)) {
            //             $data[$field] = $value;
            //         }
            //     }

            //     // Call the model method to insert each item
            //     $inserted_id = $this->empPayrollModel->insertEmpPayroll($data);
            //     // Handle the result of the insertion as needed
            // }

            // Send a response indicating success or failure
            echo json_encode(
                array(
                    'status' => 200,
                    'message' => 'تم إضافة الموظف بنجاح',
                    'data' => $inserted_id, // Adjust this as needed
                    'action' => true
                ),
                JSON_UNESCAPED_UNICODE
            );
            exit();
        } else {
            // Handle the case where no data was posted
            echo json_encode(
                array(
                    'status' => 400,
                    'message' => 'فشل إضافة الموظف: النموذج لم يتم إرساله أو البيانات غير صحيحة',
                    'action' => false
                ),
                JSON_UNESCAPED_UNICODE
            );
            exit();
        }
    }
    public function update()
    {
        $this->UserModel->checkPermission();

        // Check if the form is submitted
        if ($this->input->post()) {
            $postedData = $this->input->post();
            $allowedColumns = $this->db->list_fields('emp_payroll');

            $data = array();
            foreach ($postedData as $field => $value) {
                if (in_array($field, $allowedColumns)) {
                    $data[$field] = $value;
                }
            }
            $emp_id = $data['emp_id'];

            $affected_rows = $this->empPayrollModel->updateEmpPayroll($data, $emp_id);

            if ($affected_rows > 0) {
                // Update successful
                echo json_encode(
                    array(
                        'status' => 200,
                        'message' => 'تم تحديث بيانات الموظف بنجاح',
                        'data' => $affected_rows,
                        'action' => true
                    ),
                    JSON_UNESCAPED_UNICODE
                );
                exit();
            } else {
                // Update failed or no changes made
                echo json_encode(
                    array(
                        'status' => 400,
                        'message' => 'فشل تحديث بيانات الموظف: لم يتم تحديث أي بيانات أو حدث خطأ',
                        'action' => false
                    ),
                    JSON_UNESCAPED_UNICODE
                );
                exit();
            }
        } else {
            echo json_encode(
                array(
                    'status' => 400,
                    'message' => 'فشل تحديث بيانات الموظف: النموذج لم يتم إرساله أو البيانات غير صحيحة',
                    'action' => false,
                ),
                JSON_UNESCAPED_UNICODE
            );
            exit();
        }
    }
    public function delete()
    {
        $this->UserModel->checkPermission();

        if ($this->input->post()) {
            $id = $this->input->post('id');

            $affected_rows = $this->empPayrollModel->deleteEmpPayroll($id);

            if ($affected_rows > 0) {
                echo json_encode(
                    array(
                        'status' => 200,
                        'message' => 'تم حذف الموظف بنجاح',
                        'data' => $affected_rows,
                        'action' => true
                    ),
                    JSON_UNESCAPED_UNICODE
                );
                exit();
            } else {
                // Deletion failed or record not found
                echo json_encode(
                    array(
                        'status' => 400,
                        'message' => 'فشل حذف الموظف: لم يتم حذف أي بيانات أو حدث خطأ',
                        'action' => false
                    ),
                    JSON_UNESCAPED_UNICODE
                );
                exit();
            }
        } else {
            // If not submitted or validation fails, provide an appropriate message
            echo json_encode(
                array(
                    'status' => 400,
                    'message' => 'فشل حذف الموظف: النموذج لم يتم إرساله أو البيانات غير صحيحة',
                    'action' => false
                ),
                JSON_UNESCAPED_UNICODE
            );
            exit();
        }
    }

    public function isEmpPayRollExist()
    {
        $this->UserModel->checkPermission();

        if ($this->input->get()) {
            $user_hash = $this->input->get("emp_id");
            $year_month = $this->input->get('year_months');

            $data = $this->empPayrollModel->isEmpPayRollExist($user_hash, $year_month);

            if ($data) {
                echo json_encode(
                    array(
                        'status' => 200,
                        'message' => 'data is founded',
                        'data' => $data,
                    ),
                    JSON_UNESCAPED_UNICODE
                );
                exit();
            } else {
                echo json_encode(
                    array(
                        'status' => 500,
                        'message' => 'حدث خطأ',
                    ),
                    JSON_UNESCAPED_UNICODE
                );
                exit();
            }


        }
    }

    public function updateApproveEmpPayroll()
    {
        $this->UserModel->checkPermission();

        if ($this->input->post()) {
            $container = $this->input->post('container');

            // Decode the JSON data
            $containerArray = json_decode($container, true);

            // Check if decoding was successful
            if ($containerArray !== null) {
                // Start a database transaction
                $this->db->trans_start();

                try {
                    foreach ($containerArray as $record) {
                        if (isset ($record['user_hash']) && isset ($record['approved'])) {
                            $user_hash = $record['user_hash'];
                            $approved = $record['approved'];
                            $payrollData = $this->empPayrollModel->getDataByHash($record['id']);
                            if($record['approved'] == "1") {
                                $date_parts = explode('-', $payrollData->from);
                                $month = (int)$date_parts[1]; 
                                $message = "تم صرف مبلغ {$payrollData->flat_salary} عن مستحقات الشهر {$month} للمزيد من التفاصيل متابعة الحسابات الادارية";
                                
                                $this->NotificationPayrollModel->insertNotPayroll($payrollData, $record['approved'], $message);
                            

                             $this->jwtmiddleware->sendNot(  
                                "تم الصرف",
                                json_encode(array('type' => 9)),
                                $this->UserModel->getMobileToken($record['user_hash']),
                                $message
                            );

                            }
                           
                            // Call the update function from your model
                            $this->empPayrollModel->updateApproveEmpPayroll($user_hash, $approved);

                        }
                    }

                    // Commit the transaction if all updates are successful
                    $this->db->trans_commit();

                    // Additional response for success, if needed
                    echo json_encode(array('status' => 200, 'message' => 'تم عمل مصادقه بنجاح', 'action' => true), JSON_UNESCAPED_UNICODE);
                } catch (Exception $e) {
                    // Rollback the transaction if any update fails
                    $this->db->trans_rollback();

                    // Handle the exception, log it, or return an error response
                    echo json_encode(array('status' => 500, 'message' => 'Internal Server Error'), JSON_UNESCAPED_UNICODE);
                }
            } else {
                // Handle JSON decoding error
                echo json_encode(array('status' => 400, 'message' => 'Bad Request'), JSON_UNESCAPED_UNICODE);
            }
        } else {
            // Handle error if the request method is not POST
            echo json_encode(array('status' => 405, 'message' => 'Method Not Allowed'), JSON_UNESCAPED_UNICODE);
        }
    }

    public function getNotificationPayrollByUserHash() {
        $user_hash = $this->input->get('user_hash');
        
        if(empty($user_hash)) {
            echo json_encode(array('status' => 400, 'message' => 'الرجاء ادخال جميع البيانات'), JSON_UNESCAPED_UNICODE);
        } else {
            $data = $this->NotificationPayrollModel->getNotificationPayrollByUserHash($user_hash);
        
            if(!empty($data)) {
                echo json_encode(array('status' => 200, 'message' => 'نجاح', 'data' => $data['data'], 'count' => $data['count']), JSON_UNESCAPED_UNICODE);
            } else {
                echo json_encode(array('status' => 404, 'message' => 'لا يوجد', 'data' => [], 'count' => 0), JSON_UNESCAPED_UNICODE);
            }
    
        }

    }





}