<?php
defined('BASEPATH') or exit('No direct script access allowed');

class Finance extends CI_Controller
{

    function __construct()
    {
        parent::__construct();
        $this->load->model('FinanceModel');
        $this->load->model('UserModel');
        $this->load->library('Middleware');
    }

    public function index()
    {
    }

    // get all Groups
    public function getALl()
    {
        $token = $this->tokenmiddleware->main();
        $this->UserModel->checkPermission();
        $user = $this->jwtmiddleware->decode($token);

        $headers = apache_request_headers();
        $page = $headers['page'] ?? 1;
        $perPage = $headers['perPage'] ?? 10;
        $search = $headers['q'] ?? '';
        $result = $this->FinanceModel->get_all($page, $perPage, $search);
        echo json_encode(
            array(
                'status' => 200,
                'message' => ' عرض المصاريف بنجاح',
                'token' => $token,
                'data' => $result,
                'isAuth' => true,
                'total' => $this->FinanceModel->count_records($search),
            ),
            JSON_UNESCAPED_UNICODE
        );
        exit();
    }

    public function get()
    {
        $token = $this->tokenmiddleware->main();
        $this->UserModel->checkPermission();
        $user = $this->jwtmiddleware->decode($token);

        $hash = $this->input->get('hash');
        $result = $this->FinanceModel->get($hash);
        if ($result) {
            echo json_encode(
                array(
                    'status' => 200,
                    'message' => 'عرض المصروف',
                    'token' => $token,
                    'data' => $result,
                    'isAuth' => true
                ),
                JSON_UNESCAPED_UNICODE
            );
            exit();
        } else {
            echo json_encode(
                array(
                    'status' => 400,
                    'message' => 'حدث خطأ أثناء عرض المصروف',
                    'token' => $token,
                    'isAuth' => true,
                    'hash' => $hash
                ),
                JSON_UNESCAPED_UNICODE
            );
            exit();
        }
        exit();
    }

    public function create()
    {
        $token = $this->tokenmiddleware->main();
        $this->UserModel->checkPermission();
        $user = $this->jwtmiddleware->decode($token);

        // Get data from the user input
        // name, address, id, hash, created_at, updated_at
        $data = array(
            'price' => $this->input->post('price'),
            'type' => $this->input->post('type'),
            'note' => $this->input->post('note'),
            'name' => $this->input->post('name'),
            'created_date' => date('Y-m-d H:i:s'),
            'updated_date' => date('Y-m-d H:i:s')
        );

        // Call the create client function in the model
        $result = $this->FinanceModel->add($data);

        // Check if the client was successfully created
        if ($result) {
            echo json_encode(
                array(
                    'status' => 200,
                    'message' => 'تم إنشاء المصروف بنجاح',
                    'token' => $token,
                    'data' => $result,
                    'isAuth' => true
                ),
                JSON_UNESCAPED_UNICODE
            );
            exit();
        } else {
            echo json_encode(
                array(
                    'status' => 400,
                    'message' => 'حدث خطأ أثناء إنشاء المصروف',
                    'token' => $token,
                    'isAuth' => true,
                    'action' => true
                ),
                JSON_UNESCAPED_UNICODE
            );
            exit();
        }
    }

    public function update()
    {
        $token = $this->tokenmiddleware->main();
        $this->UserModel->checkPermission();
        $user = $this->jwtmiddleware->decode($token);

        $hash = $this->input->post('hash');
        $data = array(
            'price' => $this->input->post('price'),
            'type' => $this->input->post('type'),
            'note' => $this->input->post('note'),
            'name' => $this->input->post('name'),
            'updated_date' => date('Y-m-d H:i:s')
        );
        $this->FinanceModel->update($hash, $data);
        echo json_encode(
            array(
                'status' => 200,
                'message' => 'تم تعديل المصروف بنجاح',
                'token' => $token,
                'isAuth' => true,
                'action' => true
            ),
            JSON_UNESCAPED_UNICODE
        );
        exit();
    }

    public function delete()
    {
        $token = $this->tokenmiddleware->main();
        $this->UserModel->checkPermission();
        $user = $this->jwtmiddleware->decode($token);

        $hash = $this->input->post('hash');
        $this->FinanceModel->delete($hash);
        echo json_encode(
            array(
                'status' => 200,
                'message' => 'تم حذف المصروف بنجاح',
                'token' => $token,
                'isAuth' => true,
                'action' => true
            ),
            JSON_UNESCAPED_UNICODE
        );
        exit();
    }
}