<?php

class JobTitlesController extends CI_Controller
{
    public function __construct()
    {
        parent::__construct();
        $this->load->model('JobTitlesModel');
        $this->load->library('Middleware');
    }

    public function index()
    {
        $this->UserModel->checkPermission();
        $jobTitles = $this->JobTitlesModel->get_records();
        echo json_encode(
            array(
                'status' => 200,
                'message' => "get records",
                'token' => null,
                'data' => $jobTitles,
                'isAuth' => true,
                'total' => $this->JobTitlesModel->count_records(),
            ),
            JSON_UNESCAPED_UNICODE
        );
        exit();
    }
    public function getNamesForSelect()
    {
        $this->UserModel->checkPermission();
        $jobTitles = $this->JobTitlesModel->get_select_all_records();
        echo json_encode(
            array(
                'status' => 200,
                'message' => "get records",
                'token' => null,
                'data' => $jobTitles,
                'isAuth' => true,
               
            ),
            JSON_UNESCAPED_UNICODE
        );
        exit();
    }

    public function get()
    {
        $this->UserModel->checkPermission();
        $id = $this->input->get('id');
        if (empty($id)) {
            echo json_encode(
                array(
                    'status' => 400,
                    'message' => 'الرجاء ادخال جميع البيانات',
                    'token' => null,
                    'data' => null,
                    'isAuth' => true,
                ),
                JSON_UNESCAPED_UNICODE
            );
            exit();
        }
        $jobTitle = $this->JobTitlesModel->get_records_by_id($id);
        echo json_encode(
            array(
                'status' => 200,
                'message' => 'عرض الوظيفة',
                'token' => null,
                'data' => $jobTitle,
                'isAuth' => true,
            ),
            JSON_UNESCAPED_UNICODE
        );
        exit();
    }

    public function insert()
    {
        $this->UserModel->checkPermission();
        $name = $this->input->post('name');
        $maximum_salary = $this->input->post('maximum_salary');
        if (empty($name) || empty($maximum_salary)) {
            echo json_encode(
                array(
                    'status' => 400,
                    'message' => 'الرجاء ادخال جميع البيانات',
                    'token' => null,
                    'data' => null,
                    'isAuth' => true,
                ),
                JSON_UNESCAPED_UNICODE
            );
            exit();
        }

        $data = array(
            'name' => $name,
            'maximum_salary' => $maximum_salary,
        );

        $jobTitle = $this->JobTitlesModel->insert_record($data);
        if (is_string($jobTitle)) {
            echo json_encode(
                array(
                    'status' => 400,
                    'message' => $jobTitle,
                    'token' => null,
                    'data' => null,
                    'isAuth' => true,
                ),
                JSON_UNESCAPED_UNICODE
            );
            exit();
        }

        echo json_encode(
            array(
                'status' => 200,
                'message' => 'تم اضافه المسمي الوظيفة',
                'token' => null,
                'data' => $jobTitle,
                'isAuth' => true,
            ),
            JSON_UNESCAPED_UNICODE
        );
        exit();
    }

    public function update()
    {
        $this->UserModel->checkPermission();
        $id = $this->input->post('id');
        $name = $this->input->post('name');
        $maximum_salary = $this->input->post('maximum_salary');

        if (empty($id) || empty($maximum_salary)) {
            echo json_encode(
                array(
                    'status' => 400,
                    'message' => 'الرجاء ادخال جميع البيانات',
                    'token' => null,
                    'data' => null,
                    'isAuth' => true,
                ),
                JSON_UNESCAPED_UNICODE
            );
            exit();
        }

        $data = array(
            'name' => $name,
            'maximum_salary' => $maximum_salary,
        );

        $jobTitle = $this->JobTitlesModel->update_record($id, $data);
        if (is_string($jobTitle)) {
            echo json_encode(
                array(
                    'status' => 400,
                    'message' => $jobTitle,
                    'token' => null,
                    'data' => null,
                    'isAuth' => true,
                ),
                JSON_UNESCAPED_UNICODE
            );
            exit();
        }

        echo json_encode(
            array(
                'status' => 200,
                'message' => 'تم تعديل المسمي الوظيفة',
                'token' => null,
                'data' => $jobTitle,
                'isAuth' => true,
            ),
            JSON_UNESCAPED_UNICODE
        );
        exit();

    }

    public function delete()
    {
        $this->UserModel->checkPermission();
        $id = $this->input->post('id');
        if (empty($id)) {
            echo json_encode(
                array(
                    'status' => 400,
                    'message' => 'الرجاء ادخال جميع البيانات',
                    'token' => null,
                    'data' => null,
                    'isAuth' => true,
                ),
                JSON_UNESCAPED_UNICODE
            );
            exit();
        }
        $jobTitle = $this->JobTitlesModel->delete_record($id);
        if (is_string($jobTitle)) {
            echo json_encode(
                array(
                    'status' => 400,
                    'message' => $jobTitle,
                    'token' => null,
                    'data' => null,
                    'isAuth' => true,
                ),
                JSON_UNESCAPED_UNICODE
            );
            exit();
        }

        echo json_encode(
            array(
                'status' => 200,
                'message' => 'تم حذف المسمي الوظيفة',
                'token' => null,
                'data' => null,
                'isAuth' => true,
            ),
            JSON_UNESCAPED_UNICODE
        );
        exit();
    }

}
