<?php
defined('BASEPATH') or exit('No direct script access allowed');

class Order extends CI_Controller
{

    function __construct()
    {
        parent::__construct();
        $this->load->model('OrderModel');
        $this->load->model('UserModel');
        $this->load->library('Middleware');
    }

    public function index()
    {
    }

    // get all Groups
    public function getOrders()
    {
        $token = $this->tokenmiddleware->main();
        $this->UserModel->checkPermission();
        $user = $this->jwtmiddleware->decode($token);

        $Orders = $this->OrderModel->getOrders();
        echo json_encode(
            array(
                'status' => 200,
                'message' => 'عرض كل الاوامر',
                'token' => $token,
                'data' => $Orders,
                'isAuth' => true,
                'total' => $this->OrderModel->count_records()
            ),
            JSON_UNESCAPED_UNICODE
        );
        exit();
    }

    public function getOrder()
    {
        $token = $this->tokenmiddleware->main();
        $this->UserModel->checkPermission();
        $user = $this->jwtmiddleware->decode($token);

        $hash = $this->input->get('hash');
        $Order = $this->OrderModel->getOrder($hash);
        echo json_encode(
            array(
                'status' => 200,
                'message' => 'عرض أمر',
                'token' => $token,
                'data' => $Order,
                'isAuth' => true
            ),
            JSON_UNESCAPED_UNICODE
        );
        exit();
    }

    public function createOrder()
    {
        $token = $this->tokenmiddleware->main();
        $this->UserModel->checkPermission();
        $user = $this->jwtmiddleware->decode($token);

        $user_hash = $this->input->post('user_hash');
        $data = array(

            'order_title_hash' => $this->input->post('order_title_hash'),
            'number' => $this->input->post('number'),
            'details' => $this->input->post('details'),
            'date' => $this->input->post('date'),
            'attachment' => $this->input->post('attachment')
        );
        // user hash is "['1','2','3']" or '1'
        if (strpos($user_hash, '[') !== false) {
            $user_hash = json_decode($user_hash);
            foreach ($user_hash as $key => $value) {
                $hash = $this->jwtmiddleware->createHash();
                $data['hash'] = $hash;
                $data['user_hash'] = $value;
                $this->OrderModel->addOrder($data);
                $this->jwtmiddleware->sendNot(
                    "امر اداري",
                    json_encode(array('type' => 6)),
                    $this->UserModel->getMobileToken($value),
                    // message
                    "تم استلام امر اداري رقم " . $data['number'],
                );
            }
        } else {
            $hash = $this->jwtmiddleware->createHash();
            $data['hash'] = $hash;
            $data['user_hash'] = $user_hash;
            $this->OrderModel->addOrder($data);
        }
        echo json_encode(
            array(
                'status' => 200,
                'message' => 'تم اضافة الامر بنجاح',
                'token' => $token,
                'isAuth' => true,
                'action' => true
            ),
            JSON_UNESCAPED_UNICODE
        );
        exit();
    }

    public function updateOrder()
    {
        $token = $this->tokenmiddleware->main();
        $this->UserModel->checkPermission();
        $user = $this->jwtmiddleware->decode($token);

        $hash = $this->input->post('hash');
        $data = array(
            'order_title_hash' => $this->input->post('order_title_hash'),
            'user_hash' => $this->input->post('user_hash'),
            'number' => $this->input->post('number'),
            'details' => $this->input->post('details'),
            'date' => $this->input->post('date'),
            'attachment' => $this->input->post('attachment')
        );
        $this->OrderModel->updateOrder($hash, $data);
        echo json_encode(
            array(
                'status' => 200,
                'message' => 'تم تعديل الامر بنجاح',
                'token' => $token,
                'isAuth' => true,
                'action' => true
            ),
            JSON_UNESCAPED_UNICODE
        );
        exit();
    }

    public function deleteOrder()
    {
        $token = $this->tokenmiddleware->main();
        $this->UserModel->checkPermission();
        $user = $this->jwtmiddleware->decode($token);

        $hash = $this->input->post('hash');
        $this->OrderModel->deleteOrder($hash);
        echo json_encode(
            array(
                'status' => 200,
                'message' => 'تم حذف الامر بنجاح',
                'token' => $token,
                'isAuth' => true,
                'action' => true
            ),
            JSON_UNESCAPED_UNICODE
        );
        exit();
    }

    public function getMyOrder()
    {
        $token = $this->tokenmiddleware->main();
        $this->UserModel->checkPermission();
        $user = $this->jwtmiddleware->decode($token);

        $user_hash = $user['hash'];

        $data = $this->OrderModel->getMyOrder($user_hash);
        echo json_encode(
            array(
                'status' => 200,
                'message' => 'عرض الاوامر الخاصة بي',
                'token' => $token,
                'isAuth' => true,
                'action' => true,
                'data' => $data
            ),
            JSON_UNESCAPED_UNICODE
        );
        exit();
    }
}