<?php
defined('BASEPATH') OR exit('No direct script access allowed');

class PlacesMarkers extends CI_Controller {

    public function __construct() {
        parent::__construct();
        $this->load->model('PlacesMarkers_model');
        $this->load->model('UserModel');
        $this->load->library('Middleware');
        $this->output->set_content_type('application/json'); // Set the default response type to JSON
        $this->checkIsAuth();
    }



    // Create a new marker
    public function create() {
        $data = $this->getInputData();
        
        if ($this->validateMarkerData($data)) {
            $result = $this->PlacesMarkers_model->create_marker($data);
            $data = $result['status'] ? $result['data'] : null;
            $this->sendResponse($result['status'] ? 201 : 500, $result['message'], $data);
        }
    }


    public function index() {
        $result = $this->PlacesMarkers_model->get_markers();
        $statusCode = $result['status'] ? 200 : 500;
        $message = $result['status'] ? 'Markers retrieved successfully.' : $result['message'];
        $data = $result['status'] ? $result['data'] : null;
        $total = $result['total'];
    
        $this->sendResponse($statusCode, $message, $data, $total);
    }

    public function view($id) {
        $result = $this->PlacesMarkers_model->get_marker_by_id($id);
        $statusCode = $result['status'] ? 200 : 404;
        $message = $result['status'] ? 'Marker retrieved successfully.' : $result['message'];
        $data = $result['status'] ? $result['data'] : null;
    
        $this->sendResponse($statusCode, $message, $data);
    }

    public function update($id) {
        $data = $this->getInputData();
        if ($this->validateMarkerData($data)) {
            $result = $this->PlacesMarkers_model->update_marker($id, $data);
            $this->sendResponse($result['status'] ? 200 : 500, $result['message']);
        }
    }

    public function delete($id) {

        if ($this->validateId($id)) {
            $result = $this->PlacesMarkers_model->delete_marker($id);
            $this->sendResponse($result['status'] ? 200 : 500, $result['message']);
        }
    }

    private function checkIsAuth() {
        $token = $this->tokenmiddleware->main();
        $this->UserModel->checkPermission();        
    }

    private function getInputData() {
        return array(
            'lat' => $this->input->post('lat'),
            'lng' => $this->input->post('lng'),
            'name' => $this->input->post('name')
        );
    }

    private function validateMarkerData($data) {
        $this->load->library('form_validation');
        $this->form_validation->set_rules('lat', 'Latitude', 'required|decimal');
        $this->form_validation->set_rules('lng', 'Longitude', 'required|decimal');
        $this->form_validation->set_rules('name', 'Name', 'required|max_length[255]');

        if ($this->form_validation->run() == FALSE) {
            $this->sendResponse(400, validation_errors());
            return false;
        }
        return true;
    }

    private function validateId($id) {
        if (empty($id) || !is_numeric($id)) {
            $this->sendResponse(400, 'Invalid marker ID.');
            return false;
        }
        return true;
    }
    private function sendResponse($status, $message, $data = null, $total = null) {
        $response = [
            'status' => $status,
            'message' => $message,
            'isAuth' => true
        ];
    
        if ($data !== null) {
            $response['data'] = $data;
        }
    
        if ($total !== null) {
            $response['total'] = $total;
        }
    
        $this->output
             ->set_status_header($status)
             ->set_output(json_encode($response, JSON_UNESCAPED_UNICODE));
    }
}