<?php
defined('BASEPATH') or exit('No direct script access allowed');
class PositionTreeController extends CI_Controller
{
    function __construct()
    {
        parent::__construct();
        $this->load->model('PositionTreeModel');
        $this->load->model('UserModel');
        $this->load->library('Middleware');
    }



    public function getPositionTypeSelect()
    {
        $token = $this->tokenmiddleware->main();
        $this->UserModel->checkPermission();


        $data = $this->PositionTreeModel->getPositionTypeSelect();


        if (!empty($data)) {
            $response = array(
                'status' => 200,
                'message' => 'positonTypes succefully',
                'data' => $data
            );
            $this->output->set_output(json_encode($response));
        } else {
            $response = array(
                'status' => 404,
                'message' => 'noPosition types found',
            );
            $this->output->set_output(json_encode($response));
        }
    }

    public function getPositionNameSelect()
    {
        $token = $this->tokenmiddleware->main();
        $this->UserModel->checkPermission();


        $data = $this->PositionTreeModel->getPositionNameSelect();

        if (!empty($data)) {
            $response = array(
                'status' => 200,
                'message' => 'position name succefully',
                'data' => $data
            );
            $this->output->set_output(json_encode($response));
        } else {
            $response = array(
                'status' => 200,
                'message' => 'no Position name found',
                'data' => [],
            );
            $this->output->set_output(json_encode($response));
        }
    }

    public function getAllPositions()
    {
        $token = $this->tokenmiddleware->main();
        $this->UserModel->checkPermission();

        $data = $this->PositionTreeModel->getAllPositions();

        if (!empty($data)) {
            $response = array(
                'status' => 200,
                'message' => 'positions succefully found it',
                'data' => $data,
                'total' => $this->PositionTreeModel->count_records()
            );
            $this->output->set_output(json_encode($response));
        } else {
            $response = array(
                'status' => 200,
                'message' => 'No positions found',
                'data' => []
            );
            $this->output->set_output(json_encode($response));
        }
    }

    public function getPositionById()
    {
        $token = $this->tokenmiddleware->main();
        $this->UserModel->checkPermission();

        $id = $this->input->get('hash');

        $data = $this->PositionTreeModel->getPositionById($id);

        if (!empty($data)) {
            $response = array(
                'status' => 200,
                'message' => 'positions succefully found it',
                'data' => $data
            );
            $this->output->set_output(json_encode($response));
        } else {
            $response = array(
                'status' => 404,
                'message' => 'positions not found',
                'data' => []
            );
            $this->output->set_output(json_encode($response));
        }

    }

    public function insertPosition()
    {
        $token = $this->tokenmiddleware->main();
        $this->UserModel->checkPermission();

        $positionTypeID = $this->input->post('PositionTypeID');
        $name = $this->input->post("Name");
        $parentId = $this->input->post("ParentID");
        $permissions = $this->input->post("Permissions");
        $percentage = $this->input->post("Percentage");
        $workedWithWhoId = $this->input->post('workedWithWhoId'); // this should be array
        $item = $this->input->post("Item");
        $todos = $this->input->post("Todos");
        $KPI = $this->input->post("KPI");
        $Specifications = $this->input->post("Specifications");
        $user_hash = $this->input->post('user_hash');


        $data = array(
            'PositionTypeID' => $positionTypeID,
            'Name' => $name,
            "ParentID" => $parentId,
            'Permissions' => $permissions,
            'Percentage' => $percentage,
            'KPI' => $KPI,
            'Specifications' => $Specifications,
            'Item' => $item,
            'Todos' => $todos,
            'lead_hash' => $user_hash
        );

        $this->db->trans_start();

        try {

            $positionId = $this->PositionTreeModel->insertPosition($data);

            if ($positionId) {

                // here first should check if $workedWithWhoId is not empty array
                if (!empty($workedWithWhoId)) {
                    foreach ($workedWithWhoId as $whoId) {
                        $relationData = [
                            'position_id' => $positionId,
                            'work_with_who_id' => $whoId
                        ];
                        $this->PositionTreeModel->insertPositionRelation($relationData);
                    }

                }

                $this->db->trans_complete();

                if ($this->db->trans_status() === FALSE) {
                    $this->sendResponse(500, 'Transaction failed');
                } else {
                    $this->sendResponse(200, 'تم انشاء المنصب بنجاح');
                }

            } else {
                $this->db->trans_complete();
                $this->sendResponse(500, 'Failed to insert position');
            }
        } catch (Exception $e) {
            $this->db->trans_rollback();
            $this->sendResponse(500, 'Exception: ' . $e->getMessage());
        }
    }

    public function updatePosition()
    {
        $token = $this->tokenmiddleware->main();
        $this->UserModel->checkPermission();

        // Assuming you have similar input validation for the update as well
        $positionId = $this->input->get("hash");

        $positionTypeID = $this->input->post('PositionTypeID');
        $name = $this->input->post("Name");
        $parentId = $this->input->post("ParentID");
        $permissions = $this->input->post("Permissions");
        $KPI = $this->input->post("KPI");
        $percentage = $this->input->post("Percentage");
        $workedWithWhoId = $this->input->post('workedWithWhoId'); // this should be array
        $item = $this->input->post("Item");
        $todos = $this->input->post("Todos");
        $user_hash = $this->input->post('user_hash');

        $data = array(
            'PositionTypeID' => $positionTypeID,
            'Name' => $name,
            "ParentID" => $parentId,
            'Permissions' => $permissions,
            'Percentage' => $percentage,
            'Item' => $item,
            'Todos' => $todos,
            'KPI' => $KPI,
            'lead_hash' => $user_hash
        );

        $this->db->trans_start();

        try {
            // Update the position
            $isUpdated = $this->PositionTreeModel->updatePosition($positionId, $data);

            // Check if the position update was successful
            if ($isUpdated) {


                // Get current relations for the position
                $currentRelations = $this->PositionTreeModel->getCurrentRelations($positionId);

                // Ensure that $currentRelations and $workedWithWhoId are arrays
                $currentRelations = is_array($currentRelations) ? $currentRelations : [];
                $workedWithWhoId = is_array($workedWithWhoId) ? $workedWithWhoId : [];

                // Identify relations to update and insert
                $toUpdate = array_intersect($currentRelations, $workedWithWhoId);
                $toInsert = array_diff($workedWithWhoId, $currentRelations);

                // Update existing relations
                foreach ($toUpdate as $whoId) {
                    $relationData = [
                        'position_id' => $positionId,
                        'work_with_who_id' => $whoId
                    ];
                    $this->PositionTreeModel->updatePositionRelation($positionId, $whoId, $relationData);
                }

                // Insert new relations
                foreach ($toInsert as $whoId) {
                    $relationData = [
                        'position_id' => $positionId,
                        'work_with_who_id' => $whoId
                    ];
                    $this->PositionTreeModel->insertPositionRelation($relationData);
                }

                // Delete relations not mentioned in the current set
                $toDelete = array_diff($currentRelations, $workedWithWhoId);
                foreach ($toDelete as $whoId) {
                    $this->PositionTreeModel->deletePositionRelation($positionId, $whoId);
                }

                $this->db->trans_complete();

                // Check the status of the transaction
                if ($this->db->trans_status() === FALSE) {
                    $this->sendResponse(500, 'Transaction failed');
                } else {
                    $this->sendResponse(200, 'تم تعديل المنصب بنجاح');
                }
            } else {
                $this->db->trans_complete();
                $this->sendResponse(500, 'Failed to update position');
            }
        } catch (Exception $e) {
            $this->db->trans_rollback();
            $this->sendResponse(500, 'Exception: ' . $e->getMessage());
        }
    }

    public function deletePosition()
    {
        $token = $this->tokenmiddleware->main();
        $this->UserModel->checkPermission();

        $positionId = $this->input->get('hash');

        // Check if the position ID is provided
        if (!$positionId) {
            $this->sendResponse(400, 'Position ID not provided');
        }

        // Assuming you have a deletePosition method in your PositionTreeModel
        $isDeleted = $this->PositionTreeModel->deletePosition($positionId);


        if ($isDeleted) {
            $this->sendResponse(200, 'تم مسح المنصب بنجاح');
        } else {
            $this->sendResponse(500, 'فشل مسح المنصب');
        }


    }

    private function sendResponse($status, $message, $action = true, $total = null)
    {
        $response = array('status' => $status, 'message' => $message, 'action' => $action, 'total' => $total);
        $this->output->set_output(json_encode($response));
    }

}