<?php
defined('BASEPATH') or exit('No direct script access allowed');

class Project extends CI_Controller
{

    public function __construct()
    {
        parent::__construct();
        $this->load->model('ProjectModel');
        $this->load->model('UserModel');
        $this->load->library('Middleware');
    }

    public function index()
    {
    }

    public function getEmployeeDetailsProject()
    {

        $token = $this->tokenmiddleware->main();
        $this->UserModel->checkPermission();
        $user = $this->jwtmiddleware->decode($token);

        $employeeHash = $this->input->get('hash');

        $result = $this->ProjectModel->getEmployeeDetailsProject($employeeHash);

        if ($result) {
            echo json_encode(
                array(
                    'status' => 200,
                    'message' => 'عرض الموظفين',
                    'token' => $token,
                    'data' => $result,
                    'isAuth' => true,
                ),
                JSON_UNESCAPED_UNICODE
            );
            exit();
        } else {
            echo json_encode(
                array(
                    'status' => 400,
                    'message' => 'حدث خطاء اثناء عرض الموظفين',
                    'token' => $token,
                    'isAuth' => true,
                ),
                JSON_UNESCAPED_UNICODE
            );
            exit();

        }
    }

    // get all Groups
    public function getALl()
    {
        $token = $this->tokenmiddleware->main();
        $this->UserModel->checkPermission();
        $user = $this->jwtmiddleware->decode($token);

        $headers = apache_request_headers();
        $page = $headers['page'] ?? 1;
        $perPage = $headers['perPage'] ?? 10;
        $search = $this->input->get('q') ?? '';
        $result = $this->ProjectModel->get_all($page, $perPage, $search);
        echo json_encode(
            array(
                'status' => 200,
                'message' => ' عرض المشاريع بنجاح',
                'token' => $token,
                'data' => $result,
                'isAuth' => true,
                'total' => $this->ProjectModel->count_records($search),
            ),
            JSON_UNESCAPED_UNICODE
        );
        exit();
    }

    public function getMobileALl()
    {
        $token = $this->tokenmiddleware->main();
        $this->UserModel->checkPermission();
        $user = $this->jwtmiddleware->decode($token);

        $result = $this->ProjectModel->mobile_all();
        echo json_encode(
            array(
                'status' => 200,
                'message' => ' عرض المشاريع بنجاح',
                'token' => $token,
                'data' => $result,
                'isAuth' => true,
            ),
            JSON_UNESCAPED_UNICODE
        );
        exit();
    }

    public function get()
    {
        $token = $this->tokenmiddleware->main();
        $this->UserModel->checkPermission();
        $user = $this->jwtmiddleware->decode($token);

        $hash = $this->input->get('hash');
        $result = $this->ProjectModel->get($hash);

        $employees = $result['employees'];
        // Role IDs mapping
        $roleIds = [
            "managers" => "1",
            "officials" => "2",
            "subordinate" => "3",
            "admin" => "4",
            "employees_workers" => "5",
        ];

        // Loop through employees and categorize them by roles
        foreach ($employees as &$employee) {
            $roleId = $employee["role_id"];
            $employeeId = $employee["employee_id"];
            $employeeName = $employee["name"]; // Assuming employee_name is available in $employee

            $roleName = array_search($roleId, $roleIds); // Get role name from role ID

            // If role name is found, add employee ID to the corresponding role array in $result
            if ($roleName !== false) {
                if (!isset($result[$roleName])) {
                    $result[$roleName] = []; // Initialize if not already set
                }
                $result[$roleName][] = ["value" => $employeeId, "label" => $employeeName];
            }
        }
        if ($result) {
            echo json_encode(
                array(
                    'status' => 200,
                    'message' => 'عرض المشروع',
                    'token' => $token,
                    'data' => $result,
                    'isAuth' => true,
                ),
                JSON_UNESCAPED_UNICODE
            );
            exit();
        } else {
            echo json_encode(
                array(
                    'status' => 400,
                    'message' => 'حدث خطأ أثناء عرض المشروع',
                    'token' => $token,
                    'isAuth' => true,
                    'hash' => $hash,
                ),
                JSON_UNESCAPED_UNICODE
            );
            exit();
        }
    }

    public function checkUserHaveActionsToAnyProject()
    {
        $token = $this->tokenmiddleware->main();
        $this->UserModel->checkPermission();
        $user = $this->jwtmiddleware->decode($token);

        $result = $this->ProjectModel->checkUserHaveActionsToAnyProject($user['hash']);

        if ($result) {
            echo json_encode(
                array(
                    'status' => 200,
                    'message' => 'عرض المشروع',
                    'token' => $token,
                    'data' => $result,
                    'isAuth' => true,
                ),
                JSON_UNESCAPED_UNICODE
            );
            exit();
        } else {
            echo json_encode(
                array(
                    'status' => 400,
                    'message' => 'حدث خطأ أثناء عرض المشروع',
                    'token' => $token,
                    'isAuth' => true,
                ),
                JSON_UNESCAPED_UNICODE
            );
            exit();
        }
    }
    public function getProjectsOptionsForUser()
    {
        $token = $this->tokenmiddleware->main();
        $this->UserModel->checkPermission();
        $user = $this->jwtmiddleware->decode($token);

        $result = $this->ProjectModel->getProjectsOptionsForUser($user['hash']);

        if ($result) {
            echo json_encode(
                array(
                    'status' => 200,
                    'message' => 'عرض خيارات المشاريع',
                    'token' => $token,
                    'data' => $result,
                    'isAuth' => true,
                ),
                JSON_UNESCAPED_UNICODE
            );
            exit();
        } else {
            echo json_encode(
                array(
                    'status' => 404,
                    'message' => 'حدث خطأ أثناء عرض المشروع',
                    'isAuth' => true,
                    'data' => [],
                    'action' => false
                ),
                JSON_UNESCAPED_UNICODE,
            );
            exit();
        }
    }

    public function create()
    {
        $token = $this->tokenmiddleware->main();
        $this->UserModel->checkPermission();
        $user = $this->jwtmiddleware->decode($token)['hash'];

        // Get data from the user input
        $data = array(
            'hash' => $this->jwtmiddleware->createHash(),
            'title' => $this->input->post('title'),
            'project_type' => $this->input->post('project_type'),
            'client_hash' => $this->input->post('client_hash'),
            'description' => $this->input->post('description'),
            'start_date' => $this->input->post('start_date'),
            'deadline' => $this->input->post('deadline'),
            'price' => $this->input->post('price'),
            'created_at' => date('Y-m-d H:i:s'),
            'updated_at' => date('Y-m-d H:i:s'),
        );

        // Call the create project function in the model
        $result = $this->ProjectModel->add($data);

        // Check if the project was successfully created
        if ($result) {
            echo json_encode(
                array(
                    'status' => 200,
                    'message' => 'تم إنشاء المشروع بنجاح',
                    'token' => $token,
                    'data' => $result,
                    'isAuth' => true,
                    'action' => true,
                ),
                JSON_UNESCAPED_UNICODE
            );
            exit();
        } else {
            echo json_encode(
                array(
                    'status' => 400,
                    'message' => 'حدث خطأ أثناء إنشاء المشروع',
                    'token' => $token,
                    'isAuth' => true,
                ),
                JSON_UNESCAPED_UNICODE
            );
            exit();
        }
    }

    public function createProjectWithEmployees()
    {
        $token = $this->tokenmiddleware->main();
        $user = $this->jwtmiddleware->decode($token)['hash'];

        $this->UserModel->checkPermission();
        // Get the form data
        $title = $this->input->post('title');
        $admin = json_decode($this->input->post('admin'), true);
        $subordinate = json_decode($this->input->post('subordinate'), true);
        $employees = json_decode($this->input->post('employees'), true);
        $officials = json_decode($this->input->post('officials'), true);
        $managers = json_decode($this->input->post('managers'), true);

        // Insert project and employees within a transaction
        $success = $this->ProjectModel->insert_project_with_employees($title, $admin, $subordinate, $employees, $officials, $managers, $user);

        if (is_string($success)) {
            echo json_encode(
                array(
                    'status' => 400,
                    'message' => $success,
                    'token' => $token,
                    'isAuth' => true,
                    'action' => true,
                ),
                JSON_UNESCAPED_UNICODE
            );
            exit();
        }

        if ($success) {
            echo json_encode(
                array(
                    'status' => 200,
                    'message' => 'عرض المشروع',
                    'token' => $token,
                    'data' => "تم انشاء المشروع بنجاح",
                    'isAuth' => true,
                ),
                JSON_UNESCAPED_UNICODE
            );
            exit();
        } else {
            echo json_encode(
                array(
                    'status' => 400,
                    'message' => 'حدث خطأ أثناء عرض المشروع',
                    'token' => $token,
                    'isAuth' => true,
                ),
                JSON_UNESCAPED_UNICODE
            );
            exit();
        }
    }
    public function update_project_with_employees()
    {
        $token = $this->tokenmiddleware->main();
        $this->UserModel->checkPermission();
        $user = $this->jwtmiddleware->decode($token)['hash'];
        // Get the form data
        $title = $this->input->post('title');
        $admin = json_decode($this->input->post('admin'), true);
        $subordinate = json_decode($this->input->post('subordinate'), true);
        $employees = json_decode($this->input->post('employees'), true);
        $officials = json_decode($this->input->post('officials'), true);
        $managers = json_decode($this->input->post('managers'), true);
        $projectId = $this->input->post('hash');

        // Insert project and employees within a transaction
        $success = $this->ProjectModel->update_project_with_employees($projectId, $title, $admin, $subordinate, $employees, $officials, $managers, $user);

        if ($success) {
            echo json_encode(
                array(
                    'status' => 200,
                    'message' => 'عرض المشروع',
                    'token' => $token,
                    'data' => "تم تحديث بيانات المشروع بنجاح",
                    'isAuth' => true,
                ),
                JSON_UNESCAPED_UNICODE
            );
            exit();
        } else {
            echo json_encode(
                array(
                    'status' => 400,
                    'message' => 'حدث خطأ أثناء تحديث بيانات المشروع',
                    'token' => $token,
                    'isAuth' => true,
                ),
                JSON_UNESCAPED_UNICODE
            );
            exit();
        }
    }

    public function update()
    {
        $token = $this->tokenmiddleware->main();
        $this->UserModel->checkPermission();
        $user = $this->jwtmiddleware->decode($token);

        $hash = $this->input->post('hash');
        $data = array(
            'title' => $this->input->post('title'),
            'project_type' => $this->input->post('project_type'),
            'client_hash' => $this->input->post('client_hash'),
            'description' => $this->input->post('description'),
            'start_date' => $this->input->post('start_date'),
            'deadline' => $this->input->post('deadline'),
            'price' => $this->input->post('price'),
            'updated_at' => date('Y-m-d H:i:s'),
        );
        $this->ProjectModel->update($hash, $data);
        echo json_encode(
            array(
                'status' => 200,
                'message' => 'تم تعديل الامر بنجاح',
                'token' => $token,
                'isAuth' => true,
                'action' => true,
            ),
            JSON_UNESCAPED_UNICODE
        );
        exit();
    }

    public function getEmployeesByProject()
    {
        $token = $this->tokenmiddleware->main();
        $this->UserModel->checkPermission();
        $user = $this->jwtmiddleware->decode($token);

        $projectHash = $this->input->get('hash');
        $result = $this->ProjectModel->getEmployeesByProject($projectHash);

        if ($result) {
            echo json_encode(
                array(
                    'status' => 200,
                    'message' => 'عرض الموظفين',
                    'token' => $token,
                    'data' => $result,
                    'isAuth' => true,
                ),
                JSON_UNESCAPED_UNICODE
            );
            exit();
        } else {
            echo json_encode(
                array(
                    'status' => 400,
                    'message' => 'حدث خطأ أثناء عرض الموظفين',
                    'token' => $token,
                    'isAuth' => true,
                    'hash' => $projectHash,
                ),
                JSON_UNESCAPED_UNICODE
            );
            exit();
        }
    }

    public function delete()
    {
        $token = $this->tokenmiddleware->main();
        $this->UserModel->checkPermission();
        $user = $this->jwtmiddleware->decode($token)['hash'];

        $hash = $this->input->post('hash');
        $this->ProjectModel->delete($hash, $user);
        echo json_encode(
            array(
                'status' => 200,
                'message' => 'تم حذف الامر بنجاح',
                'token' => $token,
                'isAuth' => true,
                'action' => true,
            ),
            JSON_UNESCAPED_UNICODE
        );
        exit();
    }

    // detail of project
    public function detail()
    {
        $token = $this->tokenmiddleware->main();
        $this->UserModel->checkPermission();
        $user = $this->jwtmiddleware->decode($token);
        $hash = $this->input->get('hash');

        $result = $this->ProjectModel->detail($hash);

        if ($result) {
            echo json_encode(
                array(
                    'status' => 200,
                    'message' => 'عرض تفاصيل المشروع',
                    'token' => $token,
                    'data' => $result,
                    'isAuth' => true,
                ),
                JSON_UNESCAPED_UNICODE
            );
            exit();
        } else {
            echo json_encode(
                array(
                    'status' => 400,
                    'message' => 'حدث خطأ أثناء عرض تفاصيل المشروع',
                    'token' => $token,
                    'isAuth' => true,
                    'hash' => $hash,
                    'data' => $result,
                ),
                JSON_UNESCAPED_UNICODE
            );
            exit();
        }
        exit();
    }
}
