<?php
defined('BASEPATH') or exit ('No direct script access allowed');

class RewardsAndPenalties extends CI_Controller
{

    function __construct()
    {
        parent::__construct();
        $this->load->model('RewardsAndPenaltiesModel');
        $this->load->model('UserModel');
        $this->load->library('Middleware');
    }

    public function get()
    {
        $token = $this->tokenmiddleware->main();
        $this->UserModel->checkPermission();
        $user = $this->jwtmiddleware->decode($token);

        $entredId = $this->input->get('id');

        if (empty ($entredId)) {
            echo json_encode(
                array(
                    'status' => 400,
                    'message' => 'الرجاء ادخال جميع البيانات المطلوبه',
                    'token' => $token,
                    'isAuth' => true,
                ),
                JSON_UNESCAPED_UNICODE
            );
            exit();
        }

        $data = $this->RewardsAndPenaltiesModel->get($entredId);

        if (is_string($data)) {
            echo json_encode(
                array(
                    'status' => 400,
                    'message' => 'تم بنجاح',
                    'data' => $data,
                    'token' => $token,
                    'isAuth' => true,
                ),
                JSON_UNESCAPED_UNICODE
            );
            exit();
        }
        echo json_encode(
            array(
                'status' => 200,
                'message' => 'بنجاح',
                'data' => $data,
                'token' => $token,
                'isAuth' => true,
            ),
            JSON_UNESCAPED_UNICODE
        );
        exit();

    }
    public function getAll()
    {
        $token = $this->tokenmiddleware->main();
        $this->UserModel->checkPermission();
        $user = $this->jwtmiddleware->decode($token);

        // Directly call the model's getAll method which handles pagination
        $data = $this->RewardsAndPenaltiesModel->getAll();

        if (is_string($data)) {
            echo json_encode(
                array(
                    'status' => 400,
                    'message' => $data,
                    'token' => $token,
                    'isAuth' => true,
                ),
                JSON_UNESCAPED_UNICODE
            );
            exit();
        }
        echo json_encode(
            array(
                'status' => 200,
                'message' => 'تم بنجاح',
                'data' => $data,
                'token' => $token,
                'isAuth' => true,
            ),
            JSON_UNESCAPED_UNICODE
        );
        exit();
    }
    public function getTypes()
    {
        $token = $this->tokenmiddleware->main();
        $this->UserModel->checkPermission();
        $user = $this->jwtmiddleware->decode($token);

        $data = $this->RewardsAndPenaltiesModel->getTypes();

        if (is_string($data)) {
            echo json_encode(
                array(
                    'status' => 400,
                    'message' => $data,
                    'token' => $token,
                    'isAuth' => true,
                ),
                JSON_UNESCAPED_UNICODE
            );
            exit();
        }
        echo json_encode(
            array(
                'status' => 200,
                'message' => 'تم بنجاح',
                'data' => $data,
                'token' => $token,
                'isAuth' => true,
            ),
            JSON_UNESCAPED_UNICODE
        );
        exit();
    }


    // get for employees
    public function getEmployee()
    {
        $token = $this->tokenmiddleware->main();
        $this->UserModel->checkPermission();
        $user = $this->jwtmiddleware->decode($token);

        $id = $this->input->get('id');

        if (empty ($id)) {
            echo json_encode(
                array(
                    'status' => 400,
                    'message' => 'الرجاء ادخال جميع البيانات المطلوبه',
                    'token' => $token,
                    'isAuth' => true,
                ),
                JSON_UNESCAPED_UNICODE
            );
            exit();
        }

        $data = $this->RewardsAndPenaltiesModel->getEmployee($id);

        if (is_string($data)) {
            echo json_encode(
                array(
                    'status' => 400,
                    'message' => $data,
                    'token' => $token,
                    'isAuth' => true,
                ),
                JSON_UNESCAPED_UNICODE
            );
            exit();
        }
        echo json_encode(
            array(
                'status' => 200,
                'message' => 'تم بنجاح',
                'data' => $data,
                'token' => $token,
                'isAuth' => true,
            ),
            JSON_UNESCAPED_UNICODE
        );
        exit();
    }

    public function getEmployees()
    {
        $token = $this->tokenmiddleware->main();
        $this->UserModel->checkPermission();
        $user = $this->jwtmiddleware->decode($token);

        $data = $this->RewardsAndPenaltiesModel->getEmployees();

        if (is_string($data)) {
            echo json_encode(
                array(
                    'status' => 400,
                    'message' => $data,
                    'token' => $token,
                    'isAuth' => true,
                ),
                JSON_UNESCAPED_UNICODE
            );
            exit();
        }
        echo json_encode(
            array(
                'status' => 200,
                'message' => 'تم بنجاح',
                'data' => $data,
                'total' => $this->RewardsAndPenaltiesModel->countRewardsEmployees()->total,
                'token' => $token,
                'isAuth' => true,
            ),
            JSON_UNESCAPED_UNICODE
        );
        exit();
    }
    public function insert()
    {
        $token = $this->tokenmiddleware->main();
        $this->UserModel->checkPermission();
        $user = $this->jwtmiddleware->decode($token);

        // Check if the request method is POST and if the required data is present
        if ($this->input->post()) {
            $penalties_rewards_types_id = $this->input->post('penalties_rewards_types_id');
            $amount = $this->input->post('amount');
            $date_added = $this->input->post('date_added');
            $description = $this->input->post('description');

            // Validate the input data
            if (

                empty ($penalties_rewards_types_id) ||
                empty ($amount) ||
                empty ($date_added) ||
                empty ($description)
            ) {
                echo json_encode(
                    array(
                        'status' => 400,
                        'message' => "الرجاء ادخال جميع البيانات",
                        'token' => $token,
                        'isAuth' => true,
                    ),
                    JSON_UNESCAPED_UNICODE
                );
                exit();
            }

            // Prepare the data array for insertion
            $data = [
                'penalties_rewards_types_id' => $penalties_rewards_types_id,
                'amount' => $amount,
                'date_added' => $date_added,
                'description' => $description
            ];


            if (!empty ($this->input->post('notes'))) {
                $data['notes'] = $this->input->post('notes');
            }

            // Assuming you have a method in your model to insert the data
            $result = $this->RewardsAndPenaltiesModel->insert($data);
            if (is_string($result)) {
                echo json_encode(
                    array(
                        'status' => 400,
                        'message' => $data,
                        'token' => $token,
                        'isAuth' => true,
                    ),
                    JSON_UNESCAPED_UNICODE
                );
                exit();
            }

            echo json_encode(
                array(
                    'status' => 200,
                    'message' => "تم الإضافة بنجاح",
                    'token' => $token,
                    'isAuth' => true,
                ),
                JSON_UNESCAPED_UNICODE
            );
            exit();


        } else {
            echo json_encode(
                array(
                    'status' => 400,
                    'message' => "الرجاء ادخال بيانات",
                    'token' => $token,
                    'isAuth' => true,
                ),
                JSON_UNESCAPED_UNICODE
            );
            exit();
        }
    }

    public function update()
    {
        $token = $this->tokenmiddleware->main();
        $this->UserModel->checkPermission();
        $user = $this->jwtmiddleware->decode($token);

        // Check if the request method is POST and if the required data is present
        if ($this->input->post()) {
            $id = $this->input->post('id');
            $penalties_rewards_types_id = $this->input->post('penalties_rewards_types_id');
            $amount = $this->input->post('amount');
            $date_added = $this->input->post('date_added');
            $description = $this->input->post('description');

            // Validate the input data
            if (
                empty ($id) ||
                empty ($penalties_rewards_types_id) ||
                empty ($amount) ||
                empty ($date_added) ||
                empty ($description)
            ) {
                echo json_encode(
                    array(
                        'status' => 400,
                        'message' => "الرجاء ادخال جميع البيانات المطلوبه",
                        'token' => $token,
                        'isAuth' => true,
                    ),
                    JSON_UNESCAPED_UNICODE
                );
                exit();
            }

            // Prepare the data array for update
            $data = [
                'penalties_rewards_types_id' => $penalties_rewards_types_id,
                'amount' => $amount,
                'date_added' => $date_added,
                'description' => $description

            ];


            if (!empty ($this->input->post('notes'))) {
                $data['notes'] = $this->input->post('notes');
            }

            // Assuming you have a method in your model to update the data
            $result = $this->RewardsAndPenaltiesModel->update($id, $data);
            if (is_string($result)) {
                echo json_encode(
                    array(
                        'status' => 400,
                        'message' => $result,
                        'token' => $token,
                        'isAuth' => true,
                    ),
                    JSON_UNESCAPED_UNICODE
                );
                exit();
            }

            echo json_encode(
                array(
                    'status' => 200,
                    'message' => "تم التحديث بنجاح",
                    'token' => $token,
                    'isAuth' => true,
                ),
                JSON_UNESCAPED_UNICODE
            );
            exit();

        } else {
            echo json_encode(
                array(
                    'status' => 400,
                    'message' => "الرجاء ادخال بيانات",
                    'token' => $token,
                    'isAuth' => true,
                ),
                JSON_UNESCAPED_UNICODE
            );
            exit();
        }
    }
    public function delete()
    {
        $token = $this->tokenmiddleware->main();
        $this->UserModel->checkPermission();
        $user = $this->jwtmiddleware->decode($token);

        // Check if the request method is POST and if the ID is present
        if ($this->input->post()) {
            $id = $this->input->post('id');

            // Validate the input data
            if (empty ($id)) {
                echo json_encode(
                    array(
                        'status' => 400,
                        'message' => "الرجاء ادخال الرقم التعريفي",
                        'token' => $token,
                        'isAuth' => true,
                    ),
                    JSON_UNESCAPED_UNICODE
                );
                exit();
            }

            // Assuming you have a method in your model to delete the data
            $result = $this->RewardsAndPenaltiesModel->delete($id);
            if (is_string($result)) {
                echo json_encode(
                    array(
                        'status' => 400,
                        'message' => $result,
                        'token' => $token,
                        'isAuth' => true,
                    ),
                    JSON_UNESCAPED_UNICODE
                );
                exit();
            }

            echo json_encode(
                array(
                    'status' => 200,
                    'message' => "تم الحذف بنجاح",
                    'token' => $token,
                    'isAuth' => true,
                ),
                JSON_UNESCAPED_UNICODE
            );
            exit();

        } else {
            echo json_encode(
                array(
                    'status' => 400,
                    'message' => "الرجاء ادخال الرقم التعريفي",
                    'token' => $token,
                    'isAuth' => true,
                ),
                JSON_UNESCAPED_UNICODE
            );
            exit();
        }
    }


    public function addNewEmployeeRewardOrPenalty()
    {
        $token = $this->tokenmiddleware->main();
        $this->UserModel->checkPermission();
        $user = $this->jwtmiddleware->decode($token);

        // Get emp_id array from user input
        $emp_id_input = $this->input->post("emp_id");
        // $penalties_rewards_id = $this->input->post("penalties_rewards_id");
        $amount = $this->input->post("amount");
        $date_added = $this->input->post("date_added");
        $penalties_rewards_types_id = $this->input->post("penalties_rewards_types_id");
        $notes = $this->input->post("notes");

        // Validate required fields
        if (empty ($amount) || empty ($date_added) || empty ($penalties_rewards_types_id) || empty ($notes)) {
            echo json_encode(
                array(
                    'status' => 400,
                    'message' => "Please provide all required data",
                    'token' => $token,
                    'isAuth' => true,
                ),
                JSON_UNESCAPED_UNICODE
            );
            exit();
        }

        // Parse the input to extract the array
        $emp_id_array = json_decode($emp_id_input, true);

        // Check if emp_id_array is not empty and is valid
        if (!empty ($emp_id_array) && is_array($emp_id_array)) {
            // Iterate through each emp_id in the array
            foreach ($emp_id_array as $emp_id) {
                // Check if emp_id is numeric
                if (is_numeric($emp_id)) {
                    $data = [
                        // 'penalties_rewards_id' => $penalties_rewards_id,
                        'amount' => $amount,
                        'date_added' => $date_added,
                        'penalties_rewards_types_id' => $penalties_rewards_types_id,
                        'notes' => $notes,
                        'emp_id' => $emp_id
                    ];
                    $result = $this->RewardsAndPenaltiesModel->addNewEmployeeRewardOrPenalty($data);
                    if ($result) {
                        echo json_encode(
                            array(
                                'status' => 200,
                                'message' => "تم انشاء المكافئه او العقوبه بنجاح",
                                'data' => $result,
                                'token' => $token,
                                'isAuth' => true,
                                'action' => true,
                            ),
                            JSON_UNESCAPED_UNICODE
                        );
                        exit();
                    } else {
                        echo "Failed to process for emp_id: $emp_id <br>";
                    }
                } else {
                    echo "Invalid emp_id found in array.";
                }
            }
        } else {
            echo "Invalid emp_id array provided.";
        }
    }


    public function updateEmployeeRewardOrPenalty()
    {
        $token = $this->tokenmiddleware->main();
        $this->UserModel->checkPermission();
        $user = $this->jwtmiddleware->decode($token);

        // Check if the request method is POST and if the required data is present
        if ($this->input->post()) {
            $id = $this->input->post('id');
            $penalties_rewards_types_id = $this->input->post('penalties_rewards_types_id');
            $amount = $this->input->post('amount');
            $date_added = $this->input->post('date_added');

            // Validate the input data
            if (
                empty ($id) ||
                empty ($penalties_rewards_types_id) ||
                empty ($amount) ||
                empty ($date_added)
            ) {
                echo json_encode(
                    array(
                        'status' => 400,
                        'message' => "الرجاء ادخال جميع البيانات المطلوبه",
                        'token' => $token,
                        'isAuth' => true,
                    ),
                    JSON_UNESCAPED_UNICODE
                );
                exit();
            }

            // Prepare the data array for update
            $data = [
                'penalties_rewards_types_id' => $penalties_rewards_types_id,
                'amount' => $amount,
                'date_added' => $date_added,
            ];

            if (!empty ($this->input->post('notes'))) {
                $data['notes'] = $this->input->post('notes');
            }

            // Assuming you have a method in your model to update the data
            $result = $this->RewardsAndPenaltiesModel->updateEmployeeRewardOrPenalty($id, $data);
            if (is_string($result)) {
                echo json_encode(
                    array(
                        'status' => 400,
                        'message' => $result,
                        'token' => $token,
                        'isAuth' => true,
                    ),
                    JSON_UNESCAPED_UNICODE
                );
                exit();
            }

            echo json_encode(
                array(
                    'status' => 200,
                    'message' => "تم التحديث بنجاح",
                    'token' => $token,
                    'isAuth' => true,
                ),
                JSON_UNESCAPED_UNICODE
            );
            exit();

        } else {
            echo json_encode(
                array(
                    'status' => 400,
                    'message' => "الرجاء ادخال بيانات",
                    'token' => $token,
                    'isAuth' => true,
                ),
                JSON_UNESCAPED_UNICODE
            );
            exit();
        }
    }
    public function deleteEmployeeRewardOrPenalty()
    {
        $token = $this->tokenmiddleware->main();
        $this->UserModel->checkPermission();
        $user = $this->jwtmiddleware->decode($token);

        // Check if the request method is POST and if the ID is present
        if ($this->input->post()) {
            $id = $this->input->post('id');

            // Validate the input data
            if (empty ($id)) {
                echo json_encode(
                    array(
                        'status' => 400,
                        'message' => "الرجاء ادخال الرقم التعريفي",
                        'token' => $token,
                        'isAuth' => true,
                    ),
                    JSON_UNESCAPED_UNICODE
                );
                exit();
            }

            // Assuming you have a method in your model to delete the data
            $result = $this->RewardsAndPenaltiesModel->deleteEmployeeRewardOrPenalty($id);
            if (is_string($result)) {
                echo json_encode(
                    array(
                        'status' => 400,
                        'message' => $result,
                        'token' => $token,
                        'isAuth' => true,
                    ),
                    JSON_UNESCAPED_UNICODE
                );
                exit();
            }

            echo json_encode(
                array(
                    'status' => 200,
                    'message' => "تم الحذف بنجاح",
                    'token' => $token,
                    'isAuth' => true,
                ),
                JSON_UNESCAPED_UNICODE
            );
            exit();

        } else {
            echo json_encode(
                array(
                    'status' => 400,
                    'message' => "الرجاء ادخال الرقم التعريفي",
                    'token' => $token,
                    'isAuth' => true,
                ),
                JSON_UNESCAPED_UNICODE
            );
            exit();
        }
    }


}