<?php
defined('BASEPATH') or exit('No direct script access allowed');

class RolesPermissionsController extends CI_Controller
{

    function __construct()
    {
        parent::__construct();
        $this->load->model('RolesPermissionsModel');
        $this->load->model('UserModel');
        $this->load->library('Middleware');
    }


    public function add_permission()
    {
        $this->UserModel->checkPermission();
        $json = json_decode($this->input->raw_input_stream, true);
        $data = $json;


        if (
            isset($data['role_id']) && isset($data['permissionIds']) && isset($data['is_allowed'])
            && is_string($data['role_id']) && is_array($data['permissionIds']) && is_bool($data['is_allowed'])
        ) {

            // Valid data, proceed with inserting into the database
            $insert = $this->RolesPermissionsModel->insert_permission($data);
            if ($insert) {
                $this->output->set_content_type('application/json')
                    ->set_output(json_encode(['message' => 'Permission added successfully', 'status' => 200]));
            } else {
                $this->output->set_content_type('application/json')
                    ->set_output(json_encode(['message' => 'Failed to add permission', 'status' => 400]));
            }

        } else {
            // Invalid data
            $this->output->set_content_type('application/json')
                ->set_output(json_encode(['message' => 'Invalid data', 'status' => 400]));
        }
    }
    public function getAllRoles()
    {
        $this->UserModel->checkPermission();
        $roles = $this->RolesPermissionsModel->getAllRoles();
        echo json_encode(
            array(
                'status' => 200,
                'message' => "data for role",
                'token' => null,
                'data' => $roles,
                'isAuth' => true,
            ),
            JSON_UNESCAPED_UNICODE
        );
        exit();
    }

    public function getRoles()
    {
        $this->UserModel->checkPermission();
        $roles = $this->RolesPermissionsModel->getRoles();
        echo json_encode(
            array(
                'status' => 200,
                'message' => "data for role",
                'token' => null,
                'data' => $roles,
                'total' => $this->RolesPermissionsModel->countAllRoles(),
                'isAuth' => true,
            ),
            JSON_UNESCAPED_UNICODE
        );
        exit();
    }

    public function getRolesForSelect()
    {
        $roleId = $this->input->get('role_id');

        $this->UserModel->checkPermission();
        $roles = $this->RolesPermissionsModel->get_permissions_label_by_role($roleId);
        echo json_encode(
            array(
                'status' => 200,
                'message' => "data for role",
                'token' => null,
                'data' => $roles,
                'isAuth' => true,
            ),
            JSON_UNESCAPED_UNICODE
        );
        exit();
    }

    public function get_permission_labels() {
        $this->UserModel->checkPermission();
        $roles = $this->RolesPermissionsModel->get_permission_labels();
        echo json_encode(
            array(
                'status' => 200,
                'message' => "data for role",
                'token' => null,
                'data' => $roles,
                'isAuth' => true,
            ),
            JSON_UNESCAPED_UNICODE
        );
        exit();
    }


    

    public function getUserRoles()
    {
        $this->UserModel->checkPermission();
        $user_id = $this->input->get('user_id');
        if (empty($user_id)) {
            echo json_encode(
                array(
                    'status' => 400,
                    'message' => "userId not proivded",
                    'token' => null,
                ),
                JSON_UNESCAPED_UNICODE
            );
            exit();
        }

        $userRoles = $this->RolesPermissionsModel->getUserRoles($user_id);

        if (empty($userRoles)) {
            echo json_encode(
                array(
                    'status' => 404,
                    'message' => "user roles not found",
                    'token' => null,
                    'data' => null,
                    'isAuth' => true,
                ),
                JSON_UNESCAPED_UNICODE
            );
            exit();

        }
        echo json_encode(
            array(
                'status' => 200,
                'message' => "user roles",
                'token' => null,
                'data' => $userRoles,
                'isAuth' => true,
            ),
            JSON_UNESCAPED_UNICODE
        );
        exit();
    }

    public function insertRolesForUser()
    {
        $this->UserModel->checkPermission();

        $user_hash = $this->input->post('user_hash');
        $rolesStr = $this->input->post('roles');

        if (empty($user_hash) || empty($rolesStr)) {
            $response = [
                'status' => 400,
                'message' => 'Data not provided correctly',
                'token' => null,
            ];
        } else {
            $rolesStr = str_replace("'", '"', $rolesStr);

            $rolesArr = json_decode($rolesStr, true);

            if (!empty($rolesArr)) {
                $result = $this->RolesPermissionsModel->insertUserRole($user_hash, $rolesArr);
                $response = [
                    'status' => $result ? 200 : 400,
                    'message' => $result ? 'Success' : 'Something went wrong',
                    'token' => null,
                    'data' => $result ? $result : [],
                ];
            } else {
                $response = [
                    'status' => 400,
                    'message' => 'Invalid roles format',
                    'token' => null,
                ];
            }
        }

        echo json_encode($response, JSON_UNESCAPED_UNICODE);
        exit();
    }



}