<?php
defined('BASEPATH') or exit('No direct script access allowed');

class Roles_permissions extends CI_Controller
{

    // Constructor to load the model
    public function __construct()
    {
        parent::__construct();
        $this->load->model('Roles_permissions_model');
        $this->load->helper('url');
        $this->load->library('form_validation');
        $this->load->model('UserModel');
    }

    // GET: Retrieve permissions for a specific role
    public function get_permissions($role_id)
    {
        $permissions = $this->Roles_permissions_model->get_permissions_by_role($role_id);
        if ($permissions) {
            $this->output->set_content_type('application/json')
                ->set_output(json_encode($permissions));
        } else {
            $this->output->set_content_type('application/json')
                ->set_output(json_encode(['message' => 'No permissions found for this role']));
        }
    }



    // GET: Retrieve permissions for a specific role in a format suitable for a select box
    public function get_permissions_label($role_id)
    {
        $permissions = $this->Roles_permissions_model->get_permissions_label_by_role($role_id);
        if ($permissions) {
            $this->output->set_content_type('application/json')
                ->set_output(json_encode($permissions));
        } else {
            $this->output->set_content_type('application/json')
                ->set_output(json_encode(['message' => 'No permissions found for this role']));
        }
    }

    // POST: Add a new permission for a role
    public function add_permission()
    {
        $this->UserModel->checkPermission();
        $json = json_decode($this->input->raw_input_stream, true);
        $data = $json;


        if (
            isset($data['role_id']) && isset($data['permissionIds']) && isset($data['is_allowed'])
            && is_string($data['role_id']) && is_array($data['permissionIds']) && is_bool($data['is_allowed'])
        ) {

            // Valid data, proceed with inserting into the database
            $insert = $this->Roles_permissions_model->insert_permission($data);
            if ($insert) {
                $this->output->set_content_type('application/json')
                    ->set_output(json_encode(['message' => 'Permission added successfully', 'status' => 200]));
            } else {
                $this->output->set_content_type('application/json')
                    ->set_output(json_encode(['message' => 'Failed to add permission', 'status' => 400]));
            }

        } else {
            // Invalid data
            $this->output->set_content_type('application/json')
                ->set_output(json_encode(['message' => 'Invalid data', 'status' => 400]));
        }
    }


    // PUT: Update permission for a role
    public function update_permission($role_id, $permission_id)
    {
        $data = json_decode(file_get_contents('php://input'), true);

        // Check if 'is_allowed' is set and is a boolean
        if (!isset($data['is_allowed']) || !is_bool($data['is_allowed'])) {
            $this->output->set_content_type('application/json')
                ->set_output(json_encode(['message' => 'Invalid data for is_allowed. It should be a boolean.']));
            return;
        }

        // Proceed with the update if validation passes
        $update = $this->Roles_permissions_model->update_permission($role_id, $permission_id, $data);

        if ($update) {
            $this->output->set_content_type('application/json')
                ->set_output(json_encode(['message' => 'Permission updated successfully']));
        } else {
            $this->output->set_content_type('application/json')
                ->set_output(json_encode(['message' => 'Failed to update permission']));
        }
    }


    // DELETE: Remove permission for a role
    public function delete_permission($role_id, $permission_id)
    {
        $delete = $this->Roles_permissions_model->delete_permission($role_id, $permission_id);
        if ($delete) {
            $this->output->set_content_type('application/json')
                ->set_output(json_encode(['message' => 'Permission deleted successfully']));
        } else {
            $this->output->set_content_type('application/json')
                ->set_output(json_encode(['message' => 'Failed to delete permission']));
        }
    }
}
