<?php
defined('BASEPATH') or exit('No direct script access allowed');

class Suggestion extends CI_Controller
{

    public function __construct()
    {
        parent::__construct();
        $this->load->model('SuggestionModel');
        $this->load->model('UserModel');
        $this->load->library('Middleware');
    }

    public function index()
    {
    }

    // get all Groups
    public function getSuggestions()
    {
        $token = $this->tokenmiddleware->main();
        $this->UserModel->checkPermission();
        $user = $this->jwtmiddleware->decode($token);

        $Suggestions = $this->SuggestionModel->getSuggestions($user['user_type'], $user['hash']);
        echo json_encode(
            array(
                'status' => 200,
                'message' => 'عرض كل البصمات',
                'token' => $token,
                'data' => $Suggestions,
                'isAuth' => true,
                'total' => $this->SuggestionModel->count_records($user['user_type'], $user['hash']),
                'user' => $user,
            ),
            JSON_UNESCAPED_UNICODE
        );
        exit();
    }

    public function getSuggestion()
    {
        $token = $this->tokenmiddleware->main();
        $this->UserModel->checkPermission();
        $user = $this->jwtmiddleware->decode($token);

        $hash = $this->input->get('hash');
        $Suggestion = $this->SuggestionModel->getSuggestion($hash);
        echo json_encode(
            array(
                'status' => 200,
                'message' => 'عرض بصمة',
                'token' => $token,
                'data' => $Suggestion,
                'isAuth' => true,
            ),
            JSON_UNESCAPED_UNICODE
        );
        exit();
    }

    public function createSuggestion()
    {
        $token = $this->tokenmiddleware->main();
        $this->UserModel->checkPermission();
        $user = $this->jwtmiddleware->decode($token);

        $data = array(
            'hash' => $this->jwtmiddleware->createHash(),
            'text' => $this->input->post('text'),
            'user_hash' => $this->input->post('user_hash'),
            'sender_hash' => $this->input->post('sender_hash') ?? $user['hash'],
            'type' => $this->input->post('type') ?? '1',
        );
        $this->SuggestionModel->addSuggestion($data);

        $employessShouldNotifiy = $this->SuggestionModel->getEmployeeShouldBeNotifiedWhenAddingSuggestion();
        for ($i = 0; $i < count($employessShouldNotifiy); $i++) {
            $this->jwtmiddleware->sendNot(
                "تم اضافه اقتراح جديد",
                json_encode(array('type' => 13)),
                $this->UserModel->getMobileToken($employessShouldNotifiy[$i]['user_hash']),
                'تم اضافه اقتراح جديد'
            );
        }

        $notification_data = [
            'type' => 'suggestion',
            'user' => $user['hash'],
            'status' => 'create',
            'sender' => $user['hash'],
            'receiver' => $data['user_hash'],
        ];
        $this->jwtmiddleware->insertNotToFirebase($notification_data);

        echo json_encode(
            array(
                'status' => 200,
                'message' => 'تم اضافة الاقتراح بنجاح',
                'token' => $token,
                'isAuth' => true,
                'action' => true,
            ),
            JSON_UNESCAPED_UNICODE
        );
        exit();
    }



    public function updateSuggestion()
    {
        $token = $this->tokenmiddleware->main();
        $this->UserModel->checkPermission();
        $user = $this->jwtmiddleware->decode($token);

        $hash = $this->input->post('hash');
        $data = array(
            'text' => $this->input->post('text'),
            'user_hash' => $this->input->post('user_hash'),
            'type' => $this->input->post('type') ?? '1',
        );
        $this->SuggestionModel->updateSuggestion($hash, $data);
        echo json_encode(
            array(
                'status' => 200,
                'message' => 'تم تعديل الاقتراح بنجاح',
                'token' => $token,
                'isAuth' => true,
                'action' => true,
            ),
            JSON_UNESCAPED_UNICODE
        );
        exit();
    }

    public function deleteSuggestion()
    {
        $token = $this->tokenmiddleware->main();
        $this->UserModel->checkPermission();
        $user = $this->jwtmiddleware->decode($token);

        $hash = $this->input->post('hash');
        $this->SuggestionModel->deleteSuggestion($hash);
        echo json_encode(
            array(
                'status' => 200,
                'message' => 'تم حذف الاقتراح بنجاح',
                'token' => $token,
                'isAuth' => true,
                'action' => true,
            ),
            JSON_UNESCAPED_UNICODE
        );
        exit();
    }
}
