<?php
defined('BASEPATH') or exit('No direct script access allowed');

class UserAttachment extends CI_Controller
{

    function __construct()
    {
        parent::__construct();
        $this->load->model('UserAttachmentModel');
        $this->load->model('UserModel');
        $this->load->library('Middleware');
    }

    public function index()
    {
    }

    public function get()
    {
        $token = $this->tokenmiddleware->main();
        $this->UserModel->checkPermission();
        $user = $this->jwtmiddleware->decode($token);

        $hash = $this->input->get('hash');
        $result = $this->UserAttachmentModel->get($hash);
        if ($result) {
            echo json_encode(
                array(
                    'status' => 200,
                    'message' => 'عرض الملف',
                    'token' => $token,
                    'data' => $result,
                    'isAuth' => true
                ),
                JSON_UNESCAPED_UNICODE
            );
            exit();
        } else {
            echo json_encode(
                array(
                    'status' => 400,
                    'message' => 'حدث خطأ أثناء عرض الملف',
                    'token' => $token,
                    'isAuth' => true,
                    'hash' => $hash
                ),
                JSON_UNESCAPED_UNICODE
            );
            exit();
        }
        exit();
    }

    public function create()
    {
        $token = $this->tokenmiddleware->main();
        $this->UserModel->checkPermission();
        $user = $this->jwtmiddleware->decode($token);

        // Get data from the user input
        $data = array(
            'hash' => $this->jwtmiddleware->createHash(),
            'title_attachment' => $this->input->post('title_attachment'),
            'attachment_type' => $this->input->post('attachment_type'),
            'attachment' => $this->input->post('attachment'),
            'user_hash' => $this->input->post('user_hash'),
        );

        // Call the create userAttachment function in the model
        $result = $this->UserAttachmentModel->add($data);

        // Check if the userAttachment was successfully created
        if ($result) {
            echo json_encode(
                array(
                    'status' => 200,
                    'message' => 'تم إنشاء الملف بنجاح',
                    'token' => $token,
                    'data' => $result,
                    'isAuth' => true,
                    'action' => true
                ),
                JSON_UNESCAPED_UNICODE
            );
            exit();
        } else {
            echo json_encode(
                array(
                    'status' => 400,
                    'message' => 'حدث خطأ أثناء إنشاء الملف',
                    'token' => $token,
                    'isAuth' => true
                ),
                JSON_UNESCAPED_UNICODE
            );
            exit();
        }
    }

    public function update()
    {
        $token = $this->tokenmiddleware->main();
        $this->UserModel->checkPermission();
        $user = $this->jwtmiddleware->decode($token);

        $hash = $this->input->post('hash');
        $data = array(
            'title_attachment' => $this->input->post('title_attachment'),
            'attachment_type' => $this->input->post('attachment_type'),
            'attachment' => $this->input->post('attachment')
        );
        $this->UserAttachmentModel->update($hash, $data);
        echo json_encode(
            array(
                'status' => 200,
                'message' => 'تم تعديل الملف بنجاح',
                'token' => $token,
                'isAuth' => true,
                'action' => true
            ),
            JSON_UNESCAPED_UNICODE
        );
        exit();
    }

    public function delete()
    {
        $token = $this->tokenmiddleware->main();
        $this->UserModel->checkPermission();
        $user = $this->jwtmiddleware->decode($token);

        $hash = $this->input->post('hash');
        $this->UserAttachmentModel->delete($hash);
        echo json_encode(
            array(
                'status' => 200,
                'message' => 'تم حذف الملف بنجاح',
                'token' => $token,
                'isAuth' => true,
                'action' => true
            ),
            JSON_UNESCAPED_UNICODE
        );
        exit();
    }
}