<?php

class WhatsApp extends CI_Controller
{

    public function __construct()
    {
        parent::__construct();
        $this->load->model('UserModel');
        $this->load->model('WhatsAppModel');
        $this->load->library('Middleware');

    }

    public function getMessages()
    {
        $token = $this->tokenmiddleware->main();
        $this->UserModel->checkPermission();

        $messages = $this->WhatsAppModel->getMessages();

        echo json_encode(
            array(
                'status' => 200,
                'message' => 'عرض كل الموظفين',
                'token' => $token,
                'data' => $messages,
                'isAuth' => true,
                'total' => $this->WhatsAppModel->counts_records(),
            ),
            JSON_UNESCAPED_UNICODE
        );
        exit();

    }

    // whats app template



    public function getTemplates()
    {
        $token = $this->tokenmiddleware->main();
        $this->UserModel->checkPermission();

        $templates = $this->WhatsAppModel->getTemplates();
        $data = $templates['data'];
        $total = $templates['total'];

        echo json_encode(
            array(
                'status' => 200,
                'message' => 'عرض كل القالبات',
                'token' => $token,
                'data' => $data,
                'isAuth' => true,
                'total' => $total,
            ),
            JSON_UNESCAPED_UNICODE
        );
        exit();
    }

    public function getTemplate()
    {
        $token = $this->tokenmiddleware->main();
        $this->UserModel->checkPermission();

        $id = $this->input->get('id');

        if (empty($id)) {
            echo json_encode(
                array(
                    'status' => 400,
                    'message' => 'الرجاء ادخال جميع البيانات',
                    'token' => $token,
                    'data' => null,
                    'isAuth' => true,
                ),
                JSON_UNESCAPED_UNICODE
            );
            exit();

        }

        $template = $this->WhatsAppModel->getTemplate($id);

        echo json_encode(
            array(
                'status' => 200,
                'message' => 'عرض القالب',
                'token' => $token,
                'data' => $template,
                'isAuth' => true,
            ),
            JSON_UNESCAPED_UNICODE
        );
        exit();
    }

    public function createTemplate()
    {
        $token = $this->tokenmiddleware->main();
        $this->UserModel->checkPermission();

        $name = $this->input->post('name');
        $message = $this->input->post('message');
        $notifiy_type = $this->input->post('notify_whatsapp_type');

        if (empty($name) || empty($message)) {
            echo json_encode(
                array(
                    'status' => 400,
                    'message' => 'الرجاء ملء جميع الحقول',
                    'token' => $token,
                    'data' => null,
                    'isAuth' => true,
                ),
                JSON_UNESCAPED_UNICODE
            );
            exit();
        }
        $data = array(
            'name' => $name,
            'message' => $message,
            'notify_whatsapp_type' => $notifiy_type
        );


        $result = $this->WhatsAppModel->createTemplate($data);

        if ($result) {
            echo json_encode(
                array(
                    'status' => 200,
                    'message' => 'تم إضافة القالب بنجاح',
                    'token' => $token,
                    'data' => $result,
                    'isAuth' => true,
                ),
                JSON_UNESCAPED_UNICODE
            );
            exit();
        }

        echo json_encode(
            array(
                'status' => 400,
                'message' => 'فشل إضافة القالب',
                'token' => $token,
                'data' => null,
                'isAuth' => true,
            ),
            JSON_UNESCAPED_UNICODE
        );
        exit();
    }

    public function updateTemplate()
    {
        $token = $this->tokenmiddleware->main();
        $this->UserModel->checkPermission();

        $id = $this->input->post('id');
        $name = $this->input->post('name');
        $message = $this->input->post('message');
        $notifiy_type = $this->input->post('notify_whatsapp_type');

        if (empty($id) || empty($name) || empty($message)) {
            echo json_encode(
                array(
                    'status' => 400,
                    'message' => 'الرجاء ملء جميع الحقول',
                    'token' => $token,
                    'data' => null,
                    'isAuth' => true,
                ),
                JSON_UNESCAPED_UNICODE
            );
            exit();
        }

        $data = array(
            'name' => $name,
            'message' => $message,
        );

        if ($notifiy_type !== NULL && $notifiy_type !== '') {
            $data['notify_whatsapp_type'] = $notifiy_type;
        }

        $result = $this->WhatsAppModel->updateTemplate($id, $data);

        if (!empty($result)) {
            echo json_encode(
                array(
                    'status' => 200,
                    'message' => 'تم تعديل القالب بنجاح',
                    'token' => $token,
                    'data' => $result,
                    'isAuth' => true,
                ),
                JSON_UNESCAPED_UNICODE
            );
            exit();
        }

        echo json_encode(
            array(
                'status' => 400,
                'message' => 'فشل تعديل القالب',
                'token' => $token,
                'data' => null,
                'isAuth' => true,
            ),
            JSON_UNESCAPED_UNICODE
        );
        exit();
    }

    public function deleteTemplate()
    {
        $token = $this->tokenmiddleware->main();
        $this->UserModel->checkPermission();

        $id = $this->input->post('id');

        if (empty($id)) {
            echo json_encode(
                array(
                    'status' => 400,
                    'message' => 'الرجاء ادخال جميع البيانات',
                    'token' => $token,
                    'data' => null,
                    'isAuth' => true,
                ),
                JSON_UNESCAPED_UNICODE
            );
            exit();

        }

        $result = $this->WhatsAppModel->deleteTemplate($id);

        if (!empty($result)) {
            echo json_encode(
                array(
                    'status' => 200,
                    'message' => 'تم حذف القالب بنجاح',
                    'token' => $token,
                    'data' => $result,
                    'isAuth' => true,
                ),
                JSON_UNESCAPED_UNICODE
            );
            exit();
        }

        echo json_encode(
            array(
                'status' => 400,
                'message' => 'فشل حذف القالب',
                'token' => $token,
                'data' => null,
                'isAuth' => true,
            ),
            JSON_UNESCAPED_UNICODE
        );
        exit();
    }

    public function getWhatsAppNotifiesTypeOptions()
    {
        $token = $this->tokenmiddleware->main();
        $this->UserModel->checkPermission();

        $options = $this->WhatsAppModel->getWhatsAppNotifiesTypeOptions();


        return $this->jsonResponse(200, '', $token, $options);
    }
    public function getWhatsAppNotifiesAvailable()
    {
        $token = $this->tokenmiddleware->main();
        $this->UserModel->checkPermission();

        $availableOptions = $this->WhatsAppModel->getWhatsAppNotifiesAvailable();

        return $this->jsonResponse(200, '', $token, $availableOptions);
    }


    public function sendWhatsAppMessage()
    {
        $token = $this->tokenmiddleware->main();
        $this->UserModel->checkPermission();
        $customers = $this->input->post('customers') ?? '[]';

        $customersArray = json_decode($customers, true);
        if (json_last_error() !== JSON_ERROR_NONE || !is_array($customersArray)) {
            return $this->jsonResponse(400, 'الرجاء ادخال بيانات صحيحه', $token);
        }

        if (count($customersArray) == 0) {
            return $this->jsonResponse(400, 'الرجاء ادخال بيانات صحي', $token);
        }

        $results = [];
        foreach ($customersArray as $customer) {
            if (!isset($customer['mobile']) || !isset($customer['message'])) {
                $results[] = [
                    'mobile' => $customer['mobile'] ?? null,
                    'status' => 'failed',
                    'error' => 'Missing mobile or message',
                ];
                continue;
            }

            $formattedMobile = '+964' . preg_replace('/^0/', '', $customer['mobile']);
            $resJson = $this->jwtmiddleware->sendWhatsMessageForMultiUsers([
                'body' => $customer['message'],
                'numbers' => [$formattedMobile],
            ]);

            $res = json_decode($resJson, true);
            if ($res && isset($res['sent']) && $res['sent'] === "true" && $res['message'] === "ok") {
                $results[] = [
                    'mobile' => $formattedMobile,
                    'status' => 'success',
                ];
            } else {
                $results[] = [
                    'mobile' => $formattedMobile,
                    'status' => 'failed',
                    'error' => $res['message'] ?? 'Unknown error',
                ];
            }
        }

        return $this->jsonResponse(200, 'Processing completed', $token, [
            'token' => $token,
            'data' => $results,
        ]);
    }


    // Helper function for consistent JSON responses
    private function jsonResponse($status, $message, $token, $data = null)
    {
        echo json_encode(
            array(
                'status' => $status,
                'message' => $message,
                'token' => $token,
                'data' => $data,
                'isAuth' => true,
            ),
            JSON_UNESCAPED_UNICODE
        );
        exit();
    }




}
