<?php
defined('BASEPATH') or exit('No direct script access allowed');

class Zk_devicesController extends CI_Controller
{
    public function __construct()
    {
        parent::__construct();
        $this->load->model('Zk_devicesModel', 'zk_devices');
        $this->load->model('UserModel', 'user');
        $this->load->helper('url');
        $this->load->library('form_validation');
        $this->output->set_content_type('application/json');
    }

    /**
     * Helper method to send JSON responses.
     */
    protected function jsonResponse($status, $message, $data = [], $httpCode = 200)
    {
        $this->output
            ->set_output(json_encode([
                'status' => $status,
                'message' => $message,
                'data' => $data
            ]));
    }

    /**
     * List all records.
     */
    public function index()
    {
        try {
            $records = $this->zk_devices->getAll();
            $this->jsonResponse(200, 'Records retrieved successfully', $records);
        } catch (Exception $e) {
            $this->jsonResponse('500', $e->getMessage(), [], 500);
        }
    }



    /**
     * Get a single record by ID.
     */
    public function show($id)
    {
        try {
            $record = $this->zk_devices->getById($id);
            if ($record) {
                $this->jsonResponse(200, 'Record retrieved successfully', $record);
            } else {
                $this->jsonResponse('500', 'Record not found', [], 404);
            }
        } catch (Exception $e) {
            $this->jsonResponse('500', $e->getMessage(), [], 500);
        }
    }

    /**
     * Create a new record.
     */
    public function create()
    {
        try {
            // Set validation rules
            $this->form_validation->set_rules('ip', 'ip is required', 'required');
            $this->form_validation->set_rules('port', 'port is required', 'required');
            $this->form_validation->set_rules('name', 'name is required', 'required');
            $this->form_validation->set_rules('lat', 'lat is required', 'required');
            $this->form_validation->set_rules('lng', 'lng is required', 'required');
            // Add more validation rules as needed

            if ($this->form_validation->run() == FALSE) {
                $this->jsonResponse('500', validation_errors(), [], 400);
                return;
            }

            // Prepare data
            $data = [
                'ip' => $this->input->post('ip', TRUE), // XSS filtering
                'port' => $this->input->post('port', true),
                'name' => $this->input->post('name', true),
                'lat' => $this->input->post('lat', true),
                'lng' => $this->input->post('lng', true),
            ];

            // Create record
            $id = $this->zk_devices->create($data);
            if ($id) {
                $this->jsonResponse(200, 'Record created successfully', ['id' => $id], 201);
            } else {
                $this->jsonResponse('500', 'Failed to create record', [], 500);
            }
        } catch (Exception $e) {
            $this->jsonResponse('500', $e->getMessage(), [], 500);
        }
    }

    /**
     * Update an existing record.
     */
    public function update($id)
    {
        try {
            // Set validation rules
            $this->form_validation->set_rules('ip', 'ip is required', 'required');
            $this->form_validation->set_rules('port', 'port is required', 'required');
            $this->form_validation->set_rules('name', 'name is required', 'required');            // Add more validation rules as needed

            if ($this->form_validation->run() == FALSE) {
                $this->jsonResponse('500', validation_errors(), [], 400);
                return;
            }

            // Prepare data
            $data = [
                'ip' => $this->input->post('ip', TRUE), // XSS filtering
                'port' => $this->input->post('port', true),
                'name' => $this->input->post('name', true)
            ];

            // Update record
            $result = $this->zk_devices->update($id, $data);
            if ($result) {
                $this->jsonResponse(200, 'Record updated successfully', $data);
            } else {
                $this->jsonResponse('500', 'Failed to update record', [], 500);
            }
        } catch (Exception $e) {
            $this->jsonResponse('500', $e->getMessage(), [], 500);
        }
    }


    /**
     * Delete a record.
     */
    public function delete($id)
    {
        try {
            $this->zk_devices->delete($id);
            $this->jsonResponse(200, 'Record deleted successfully');

        } catch (Exception $e) {
            $this->jsonResponse('500', $e->getMessage(), [], 500);
        }
    }
}