<?php
defined('BASEPATH') or exit('No direct script access allowed');

class Holiday extends CI_Controller
{

    function __construct()
    {
        parent::__construct();
        $this->load->library('Middleware');
        $this->load->model('HolidayModel');
        $this->load->model('UserModel');
    }

    public function index()
    {
    }

    // get all Holidays
    public function getHolidays()
    {
        $token = $this->tokenmiddleware->main();
        $this->UserModel->checkPermission();
        $user = $this->jwtmiddleware->decode($token);

        $holidays = $this->HolidayModel->getHolidays();
        echo json_encode(
            array(
                'status' => 200,
                'message' => 'عرض كل العطل',
                'token' => $token,
                'data' => $holidays,
                'isAuth' => true,
                'total' => $this->HolidayModel->count_records()
            ),
            JSON_UNESCAPED_UNICODE
        );
        exit();
    }

    public function getHoliday()
    {
        $token = $this->tokenmiddleware->main();
        $this->UserModel->checkPermission();
        $user = $this->jwtmiddleware->decode($token);

        $hash = $this->input->get('hash');
        $holiday = $this->HolidayModel->getHoliday($hash);
        echo json_encode(
            array(
                'status' => 200,
                'message' => 'عرض العطل',
                'token' => $token,
                'data' => $holiday,
                'isAuth' => true
            ),
            JSON_UNESCAPED_UNICODE
        );
        exit();
    }

    public function createHoliday()
    {
        $token = $this->tokenmiddleware->main();
        $this->UserModel->checkPermission();
        $user = $this->jwtmiddleware->decode($token);
        $startDate = $this->input->post('start');
        $endDate = $this->input->post('end');
        while (strtotime($startDate) <= strtotime($endDate)) {
            $data = array(
                'hash' => $this->jwtmiddleware->createHash(),
                'type' => $this->input->post('type'),
                'date' => $startDate,
            );
            $this->HolidayModel->addHoliday($data);
            $startDate = date("Y-m-d", strtotime("+1 day", strtotime($startDate)));
        }
        echo json_encode(
            array(
                'status' => 200,
                'message' => 'تم اضافة العطل بنجاح',
                'token' => $token,
                'isAuth' => true,
                'action' => true
            ),
            JSON_UNESCAPED_UNICODE
        );
        exit();
    }

    public function updateHoliday()
    {
        $token = $this->tokenmiddleware->main();
        $this->UserModel->checkPermission();
        $user = $this->jwtmiddleware->decode($token);

        $hash = $this->input->post('hash');
        $data = array(
            'type' => $this->input->post('type'),
            'date' => $this->input->post('date'),
        );
        $this->HolidayModel->updateHoliday($hash, $data);
        echo json_encode(
            array(
                'status' => 200,
                'message' => 'تم تعديل العطل بنجاح',
                'token' => $token,
                'isAuth' => true,
                'action' => true
            ),
            JSON_UNESCAPED_UNICODE
        );
        exit();
    }

    public function deleteHoliday()
    {
        $token = $this->tokenmiddleware->main();
        $this->UserModel->checkPermission();
        $user = $this->jwtmiddleware->decode($token);

        $hash = $this->input->post('hash');
        $result = $this->HolidayModel->deleteHoliday($hash);
        echo json_encode(
            array(
                'status' => 200,
                'message' => 'تم حذف العطل بنجاح',
                'token' => $token,
                'isAuth' => true,
                'data' => $result,
                'action' => true
            ),
            JSON_UNESCAPED_UNICODE
        );
        exit();
    }
}