<?php

class TokenMiddleware
{
    public function __construct()
    {
        $this->CI = &get_instance();
        $this->CI->load->model('UserModel');
        $this->CI->load->library('JwtMiddleware');
        $this->CI->load->database();

    }

    public function getBearerToken()
    {
        $headers = apache_request_headers();
        $authorization = $headers['Authorization'] ?? $headers['authorization'] ?? null;
        if (isset($authorization)) {
            if (preg_match('/Bearer\s(\S+)/', $authorization, $matches)) {
                return $matches[1];
            }
        }
        return 0;
    }

    public function checkToken($token)
    {
        if ($token) {
            $jwt = new JwtMiddleware(); // Ensure JwtMiddleware class exists and is included
            $decoded = $jwt->decode($token);
            if ($decoded) {
                return $decoded;
            } else {
                return 0;
            }
        } else {
            return 0;
        }
    }

    public function main()
    {
        // Load necessary libraries and models
        // $this->load->library('JwtMiddleware');
        // $this->load->model('UserModel');

        $jwt = new JwtMiddleware(); // Ensure JwtMiddleware class exists and is included

        // Get token from header
        $token = $this->getBearerToken();

        // Check token
        if (!$this->checkToken($token)) {
            echo json_encode(
                array(
                    "status" => 401,
                    "message" => "يجب تسجيل الدخول للوصول لهذه الصفحة",
                    "isAuth" => false,
                    "data" => null,
                    "token" => null,
                )
            );
            exit();
        }

        // Decode the token
        $tokenData = $jwt->decode($token);
        $user_hash = $tokenData['hash'];

        $user = $this->CI->UserModel->getUser($tokenData['username'], $tokenData['password']);

        if (!$user) {
            // If user doesn't exist
            echo json_encode(
                array(
                    "status" => 401,
                    "message" => "الحساب غير موجود أو تم حذفه",
                    "isAuth" => false,
                    "data" => null,
                    "token" => null,
                )
            );
            exit();
        }

        if ($user['user_type_name'] !== 'admin') {
            // Check if user_hash is deleted or user credentials are invalid
            if ($this->CI->UserModel->isUserHashDeleted($user_hash)) {
                echo json_encode(
                    array(
                        "status" => 401,
                        "message" => "الحساب غير موجود أو تم حذفه",
                        "isAuth" => false,
                        "data" => null,
                        "token" => null,
                    )
                );
                exit();
            }
        }

        // $this->logMobileDetection($user_hash);

        // $isMobile = $this->isMobileRequest(); // Function to determine the platform
        $userAgent = '';

        // Try different server variables
        if (!empty($_SERVER['HTTP_USER_AGENT'])) {
            $userAgent = $_SERVER['HTTP_USER_AGENT'];
        } elseif (function_exists('apache_request_headers')) {
            $headers = apache_request_headers();
            $userAgent = $headers['User-Agent'] ?? $headers['user-agent'] ?? '';
        }

        $isDart = $this->isDartUserAgent($userAgent);

        if ($isDart) {

            if ($user['token'] !== $token) {
                echo json_encode(
                    array(
                        "status" => 401,
                        "message" => "تم تسجيل الخروج من الجهاز الآخر",
                        "isAuth" => false,
                        "data" => null,
                        "token" => null,
                    )
                );
                exit();
            }
        }


        // If token is valid, update token
        $newToken = $jwt->encode($tokenData);
        return $newToken;
    }
    public function isMobileRequest()
    {
        $is_mobile = $this->CI->input->post('mobile_token') ? true : false;

        if (!$is_mobile) {
            return false;
        }

        // Get User-Agent from multiple sources
        $userAgent = '';

        // Try different server variables
        if (!empty($_SERVER['HTTP_USER_AGENT'])) {
            $userAgent = $_SERVER['HTTP_USER_AGENT'];
        } elseif (function_exists('apache_request_headers')) {
            $headers = apache_request_headers();
            $userAgent = $headers['User-Agent'] ?? $headers['user-agent'] ?? '';
        }

        // Comprehensive mobile device detection
        $mobileKeywords = [
            'android',
            'iphone',
            'ipad',
            'ipod',
            'windows phone',
            'blackberry',
            'opera mini',
            'mobile',
            'kindle',
            'silk',
            'playbook',
            'bb10',
            'webos',
            'mobi',
            'touch',
            'phone',
            'symbian',
            'nintendo',
            'tablet',
            'mobile safari',
            'iemobile',
            'fennec',
            'wap',
        ];

        // Additional regex patterns for mobile detection
        $mobilePatterns = [
            '/android/i',
            '/webos/i',
            '/iphone/i',
            '/ipad/i',
            '/ipod/i',
            '/blackberry/i',
            '/windows phone/i',
            '/opera mini/i',
            '/iemobile/i',
            '/dart/i',
        ];

        // Check for mobile keywords
        $keywordMatch = false;
        foreach ($mobileKeywords as $keyword) {
            if (stripos($userAgent, $keyword) !== false) {
                $keywordMatch = true;
                break;
            }
        }

        // Check regex patterns
        $patternMatch = false;
        foreach ($mobilePatterns as $pattern) {
            if (preg_match($pattern, $userAgent)) {
                $patternMatch = true;
                break;
            }
        }

        // Additional checks for screen size and touch capabilities
        $screenSizeIndicators = [
            'mobile',
            'phone',
            'touch',
            'tablet',
            'smaller screen',
        ];

        $screenSizeMatch = false;
        foreach ($screenSizeIndicators as $indicator) {
            if (stripos($userAgent, $indicator) !== false) {
                $screenSizeMatch = true;
                break;
            }
        }

        // Determine if it's a mobile device

        return $keywordMatch || $patternMatch || $screenSizeMatch;
    }

    public function logMobileDetection($user_hash = null)
    {
        // Get User-Agent
        $userAgent = $_SERVER['HTTP_USER_AGENT'] ?? '';

        // Perform mobile detection
        $isMobile = $this->isMobileRequest();

        // Prepare detection details
        $detectionDetails = [
            'keywords_matched' => $this->getMatchedKeywords($userAgent),
            'patterns_matched' => $this->getMatchedPatterns($userAgent),
        ];

        // Prepare SQL
        $sql = "INSERT INTO mobile_detection_logs (
            user_hash,
            user_agent,
            is_mobile,
            detection_result,
            ip_address,
            additional_info
        ) VALUES (?, ?, ?, ?, ?, ?)";

        //     // Execute query
        $this->CI->db->query($sql, [
            $user_hash,
            $userAgent,
            $isMobile ? 1 : 0,
            json_encode($detectionDetails),
            '321414124124124',
            serialize($detectionDetails),
        ]);

        return $isMobile;
    }

    private function isDartUserAgent($userAgent)
    {
        return stripos($userAgent, 'dart') !== false;
    }

    // Helper methods to get matched keywords and patterns
    private function getMatchedKeywords($userAgent)
    {
        $mobileKeywords = [
            'android',
            'iphone',
            'ipad',
            'ipod',
            'windows phone',
            'blackberry',
            'opera mini',
            'mobile',
            'kindle',
        ];

        $matched = [];
        foreach ($mobileKeywords as $keyword) {
            if (stripos($userAgent, $keyword) !== false) {
                $matched[] = $keyword;
            }
        }

        return $matched;
    }

    private function getMatchedPatterns($userAgent)
    {
        $mobilePatterns = [
            '/android/i',
            '/webos/i',
            '/iphone/i',
            '/ipad/i',
            '/ipod/i',
            '/blackberry/i',
            '/windows phone/i',
        ];

        $matched = [];
        foreach ($mobilePatterns as $pattern) {
            if (preg_match($pattern, $userAgent)) {
                $matched[] = $pattern;
            }
        }

        return $matched;
    }

}
