<?php
class AdvancePaymentModel extends CI_Model
{
    public function __construct()
    {
        parent::__construct();
        $this->load->database();
    }



    // get all Groups
    public function createAdvanceReq($price, $date, $goverment, $projectId, $notes, $userId)
    {

        $data = array(
            'price' => $price,
            'date' => $date,
            'userId' => $userId,
            'goverment' => $goverment,

        );

        if ($notes !== null) {
            $data['notes'] = $notes;
        }
        if ($projectId !== null) {
            $data['projectId'] = $projectId;
        }


        return $this->db->insert('RequestPayment', $data);
        // just for testing here
    }



    public function isInvoiceUnique($invoice_num)
    {
        $existingInvoice = $this->db->get_where('advance_payment', array('invoice_number' => $invoice_num))->row();
        if ($existingInvoice) {
            return false;
        }
        return true;
    }

    public function paymentAdvance(
        $price,
        $date,
        $categoryId,
        $invoice_num,
        $invoice_date,
        $notes,
        $projectId,
        $userId,
        $reqPaymentId,
        $attachmentPaths
    ) {
        $isUnique = $this->isInvoiceUnique($invoice_num);

        if (!$isUnique) {
            return false;
        }

        $data = array(
            'price' => $price,
            'date' => $date,
            'invoice_number' => $invoice_num,
            'invoice_date' => $invoice_date,
            'projectId' => $projectId,
            'userId' => $userId,
            'requestPaymentId' => $reqPaymentId,
            'categoryId' => $categoryId,
        );

        // Check if attachment paths are provided
        if (!empty($attachmentPaths)) {
            // Serialize the array of attachment paths into a string and store it
            $data['attachment'] = $attachmentPaths;
        }

        if ($notes !== null) {
            $data['notes'] = $notes;
        }

        $TotalAndRemaining = $this->getTotalAndRemaining($reqPaymentId);


        if (floatval($price) > $TotalAndRemaining['remaining']) {
            return "سعر التسديد اكبر من المتبقي";
        }


        return $this->db->insert('advance_payment', $data);
    }
    public function updatePaymentAdvance(
        $id,
        $price,
        $date,
        $categoryId,
        $invoice_num,
        $invoice_date,
        $notes,
        $projectId,
        $userId,
        $reqPaymentId,
        $attachmentPaths
    ) {
        $data = array(
            'id' => $id,
            'price' => $price,
            'date' => $date,
            'invoice_number' => $invoice_num,
            'invoice_date' => $invoice_date,
            'projectId' => $projectId,
            'userId' => $userId,
            'requestPaymentId' => $reqPaymentId,
            'categoryId' => $categoryId,

        );

        // Check if attachment paths are provided
        if (!empty($attachmentPaths)) {
            // Serialize the array of attachment paths into a string and store it
            $data['attachment'] = $attachmentPaths;
        }

        if ($notes !== null) {
            $data['notes'] = $notes;
        }

        // Specify the condition for the update
        $where = array('id' => $id);


        // Update the advance_payment table where invoice_number matches
        return $this->db->update('advance_payment', $data, $where);
    }




    public function getPayment($hash, $page = 1, $perPage = 10)
    {
        $this->db->select('*');
        $this->db->from('advance');
        $this->db->where('user_hash', $hash);
        $this->db->where('status_id', '1');
        $this->db->where('is_deleted = 0');

        $offset = ($page - 1) * $perPage;

        $this->db->limit($perPage, $offset);


        $query = $this->db->get();

        if ($query->num_rows() > 0) {
            return $query->result_array();
        } else {
            return array();
        }
    }

    public function getPaymentById($id)
    {
        $this->db->select('*');
        $this->db->from('advance');
        $this->db->where('id', $id);

        $query = $this->db->get();

        if ($query->num_rows() > 0) {
            return $query->row(); // Assuming you expect a single row for a unique payment ID
        } else {
            return null; // Return null if no payment data is found for the provided ID
        }
    }


    // here page and perPage parameters don't work correclty



    public function getAdvancePaymentRemaining($reqPaymentId, $page = 1, $perPage = 10)
    {
        $this->db->select_sum('price');
        $this->db->select("(select price from advance where id = '$reqPaymentId') as reqPaymentPrice");
        $this->db->from('advance_payment');
        $this->db->where('requestPaymentId', $reqPaymentId);

        $offset = ($page - 1) * $perPage;
        $this->db->limit($perPage, $offset);

        $query = $this->db->get();

        if ($query->num_rows() > 0) {
            return $this->getTotalAndRemaining($reqPaymentId);           // return $result->price;
        } else {
            return 0;
        }
    }

    private function getTotalAndRemaining($reqPaymentId)
    {
        $this->db->select_sum('price');
        $this->db->select("(select price from advance where id = '$reqPaymentId') as reqPaymentPrice");
        $this->db->where('requestPaymentId', $reqPaymentId);
        $this->db->from('advance_payment');


        $result = $this->db->get()->row();

        return array(
            'total' => floatval($result->price),
            'remaining' => floatval($result->reqPaymentPrice) - floatval($result->price)
        );
    }
    public function getAdvancePayment($reqPaymentId, $page = 1, $perPage = 10)
    {
        $this->db->select('*');
        $this->db->from('advance_payment');
        $this->db->select('(select title from project WHERE  hash=advance_payment.projectId) as ProjectTitle');
        $this->db->select('(select name from accounting_tree WHERE  hash=advance_payment.categoryId) as categoryTitle');

        $this->db->where('requestPaymentId', $reqPaymentId);
        $offset = ($page - 1) * $perPage;
        $this->db->limit($perPage, $offset);

        $query = $this->db->get();

        if ($query->num_rows() > 0) {
            return $query->result_array();
        } else {
            return array();
        }
    }
    public function getAdvancePaymentInfo($reqPaymentId, $page = 1, $perPage = 10)
    {
        $this->db->select('*');
        $this->db->select('(select title from project WHERE  hash=advance_payment.projectId) as ProjectTitle');
        $this->db->select('(select name from accounting_tree WHERE  hash=advance_payment.categoryId) as categoryTitle');
        $this->db->from('advance_payment');
        $this->db->where('requestPaymentId', $reqPaymentId);

        $offset = ($page - 1) * $perPage;
        $this->db->limit($perPage, $offset);

        $query = $this->db->get();

        if ($query->num_rows() > 0) {
            return $query->result_array();
        } else {
            return array();
        }
    }
    // public function getAdvancePaymentById($paymentId)
    // {
    //     $this->db->select('advance_payment.*, project.title as projectLabel, project.hash as projectValue');
    //     $this->db->from('advance_payment');
    //     $this->db->join('project', 'advance_payment.projectId = project.hash', 'left');
    //     $this->db->where('advance_payment.id', $paymentId);

    //     $query = $this->db->get();

    //     if ($query->num_rows() > 0) {
    //         return $query->row_array();
    //     } else {
    //         return array();
    //     }
    // }
    public function getAdvancePaymentById($paymentId)
    {
        $this->db->select('advance_payment.*, project.title as projectLabel, project.hash as projectValue, accounting_tree.name as categoryLabel, accounting_tree.hash as categoryValue');
        $this->db->from('advance_payment');
        $this->db->join('project', 'advance_payment.projectId = project.hash', 'left');
        $this->db->join('accounting_tree', 'advance_payment.categoryId = accounting_tree.hash', 'left');
        $this->db->where('advance_payment.id', $paymentId);

        $query = $this->db->get();

        if ($query->num_rows() > 0) {
            return $query->row_array();
        } else {
            return array();
        }
    }

    public function getCustomRequestPayments($hash, $startDate, $endDate, $page = 1, $perPage = 10)
    {
        $this->db->select('*');
        $this->db->where('userId', $hash);
        $this->db->where('date >=', $startDate);
        $this->db->where('date <=', $endDate);
        $this->db->from("RequestPayment");
        $offset = ($page - 1) * $perPage;
        $this->db->limit($perPage, $offset);
        $query = $this->db->get();

        if ($query->num_rows() > 0) {
            return $query->result_array();
        } else {
            return array();
        }

    }


    public function getCustomAdvancePayments($startDate, $endDate, $reqPaymentId, $page = 1, $perPage = 10)
    {
        $this->db->select('*');
        $this->db->from('advance_payment');
        $this->db->where('requestPaymentId', $reqPaymentId);
        $this->db->where('invoice_date >=', $startDate);
        $this->db->where('invoice_date <=', $endDate);
        $this->db->where('requestPaymentId', $reqPaymentId);

        $offset = ($page - 1) * $perPage;
        $this->db->limit($perPage, $offset);

        $query = $this->db->get();

        if ($query->num_rows() > 0) {
            return $query->result_array();
        } else {
            return array();
        }
    }
    public function getCustomAdvancePaymentReports($startDate, $endDate, $reqPaymentId)
    {
        $this->db->select('tree.name category,  COALESCE(SUM(price), 0) AS price, invoice_date');
        $this->db->from('advance_payment advance');
        $this->db->join('accounting_tree tree', 'advance.categoryId = tree.hash');

        if (!empty($startDate) && !empty($endDate)) {
            $this->db->where('invoice_date >=', $startDate);
            $this->db->where('invoice_date <=', $endDate);
        }


        $this->db->where('requestPaymentId', $reqPaymentId);


        $this->db->group_by(['advance.categoryId', 'tree.name', 'invoice_date']);


        $query = $this->db->get();

        return($query->num_rows() > 0) ? $query->result_array() : [];
    }






    public function deleteRequestPayment($id)
    {
        $table = 'RequestPayment';

        $existingRecord = $this->db->get_where($table, array('PaymentID' => $id))->row();

        if ($existingRecord) {
            $this->db->where('PaymentID', $id);
            $this->db->delete($table);
            return true;
        } else {
            return false;
        }
    }

    public function deleteAdvancePaymentById($paymentId)
    {
        $table = 'advance_payment';

        $existingRecord = $this->db->get_where($table, array('id' => $paymentId))->row();

        if ($existingRecord) {
            $this->db->where('id', $paymentId);
            $this->db->delete($table);
            return true;
        } else {
            return false;
        }
    }


}