<?php
class AnnounceModel extends CI_Model
{
    public function __construct()
    {
        parent::__construct();
        $this->load->database();
    }

    // get all Groups
    public function count_records()
    {
        $headers = apache_request_headers();
        $search = $this->input->get('q') ?? '';
        $this->db->select('count(*) as count');
        $this->db->from('announce');
        if ($search) {
            $this->db->like('hash', $search);
        }
        $query = $this->db->get();
        return $query->result_array()[0]['count'];
    }

    public function getAnnounces()
    {
        // get page and limit and search from header
        $headers = apache_request_headers();
        $page = $headers['page'] ?? $this->input->get('page') ?? 1;
        $perPage = $headers['perPage'] ?? $this->input->get('perPage') ?? 10;
        $search = $this->input->get('q') ?? '';
        $offset = ($page - 1) * $perPage;

        $this->db->select(
            'title, details, number, date,attachment, hash '
        );
        $this->db->from('announce');

        // search
        if ($search) {
            $this->db->like('title', $search);
        }

        // order by
        $this->db->order_by('id', 'DESC');
        // limit
        $this->db->limit($perPage, $offset);
        $query = $this->db->get();

        return $query->result_array();
    }

    function getAnnounce($hash)
    {
        $this->db->select(
            'title, details, number, date,attachment, announce.hash'
        );
        // get all announce_to_user as array of objects {value: 1, label: 'name'}
        $this->db->select('concat("[",GROUP_CONCAT(announce_to_user.user_hash),"]") as users');
        $this->db->from('announce');
        $this->db->join('announce_to_user', 'announce.hash = announce_to_user.announce_hash', 'left');
        $this->db->where('announce.hash =', $hash);
        $query = $this->db->get();

        $result = $query->result_array()[0];
        $users = $result['users'];
        $users = str_replace(array('[', ']'), '', $users); // remove brackets
        $users = explode(',', $users); // convert to array
        $users = array_map('trim', $users); // remove whitespace
        $result['users'] = $users;
        return $result;
    }

    function addAnnounce($data)
    {
        $this->db->insert('announce', $data);
        return $this->db->insert_id();
    }

    function updateAnnounce($hash, $data)
    {
        $this->db->where('hash', $hash);
        $this->db->update('announce', $data);
        return $this->db->affected_rows();
    }

    function deleteAnnounce($hash)
    {
        $this->db->where('hash', $hash);
        $this->db->delete('announce');
        return $this->db->affected_rows();
    }

    public function createAnnounceToUser($data)
    {
        $this->db->insert('announce_to_user', $data);
        return $this->db->insert_id();
    }

    public function deleteAnnounceToUser($announce_hash)
    {
        $this->db->where('announce_hash', $announce_hash);
        $this->db->delete('announce_to_user');
        return $this->db->affected_rows();
    }

    public function getMyAnnounces($user_hash)
    {
        // get page and limit and search from header
        $headers = apache_request_headers();
        $page = $headers['page'] ?? $this->input->get('page') ?? 1;
        $perPage = $headers['perPage'] ?? $this->input->get('perPage') ?? 10;
        $search = $this->input->get('q') ?? '';
        $offset = ($page - 1) * $perPage;

        $this->db->select(
            'title, details, number, date,attachment, announce.hash '
        );
        $this->db->from('announce');
        // inner join with announce_to_user
        $this->db->join('announce_to_user', 'announce.hash = announce_to_user.announce_hash', 'inner');
        $this->db->where('announce_to_user.user_hash', $user_hash);
        // search
        if ($search) {
            $this->db->like('title', $search);
        }

        // order by
        $this->db->order_by('announce.id', 'DESC');
        // limit
        $this->db->limit($perPage, $offset);
        $query = $this->db->get();

        return $query->result_array();
    }
}
