<?php
class AssignmentItemsModel extends CI_Model
{
    public function __construct()
    {
        parent::__construct();
        $this->load->database();
        $this->load->library('Middleware');
    }

    public function getCount($start, $end)
    {
        $headers = apache_request_headers();
        $search = $headers['q'] ?? '';
        $department = $this->input->get('department') ?? "";

        $this->db->select('count(DISTINCT item_assignments.user_hash) as count');
        $this->db->from('item_assignments');
        $this->db->join('emp_information', 'emp_information.user_hash = item_assignments.user_hash', 'left');

        // where
        if (isset($start) && $start != "" && $start != 'null') {
            $this->db->where('DATE(assignment_date) >=', $start);
        }
        if (isset($end) && $end != "" && $end != 'null') {
            $this->db->where('DATE(assignment_date) <=', $end);
        }
        if (isset($search) && $search != "" && $search != 'null') {
            $this->db->where('item_assignments.user_hash', $search);
        }
        if (isset($department) && $department != "" && $department != 'null') {
            $this->db->where('emp_information.partment', $department);
        }

        $query = $this->db->get();
        return $query->result_array()[0]['count'];

    }

    public function getAll($start, $end)
    {
        $headers = apache_request_headers();
        $department = $this->input->get('department') ?? "";

        $page = $this->input->get('page') ?? 1;
        $perPage = $this->input->get('perPage') ?? 10;
        $search = $this->input->get('q') ?? '';
        $offset = ($page - 1) * $perPage;

        $this->db->select("system_users.name as emp, system_users.image_profile as image");
        $this->db->select("partments.name as department");
        $this->db->select("system_group_name.name as job");
        $this->db->select("item_assignments.*,item_name");
        $this->db->select("(select Name from Positions where Positions.ID=emp_information.position) as position");

        $this->db->from('item_assignments');
        $this->db->join('system_users', 'system_users.hash = item_assignments.user_hash', 'left');
        $this->db->join('emp_information', 'emp_information.user_hash = item_assignments.user_hash', 'left');
        $this->db->join('partments', 'partments.hash = emp_information.partment', 'left');
        $this->db->join('system_group_name', 'system_group_name.hash = system_users.user_type', 'left');
        $this->db->join('inventory_items', 'item_assignments.item_id = inventory_items.item_id', 'left');

        $this->db->group_by('system_users.name');

        // Where
        if (isset($start) && $start != "" && $start != 'null') {
            $this->db->where('DATE(item_assignments.assignment_date) >=', $start);
        }
        if (isset($end) && $end != "" && $end != 'null') {
            $this->db->where('DATE(item_assignments.assignment_date) <=', $end);
        }
        // Search
        if (isset($search) && $search != "" && $search != 'null') {
            $this->db->like('system_users.name', $search);
            // or item name
            $this->db->or_like('item_name', $search);
        }
        if (isset($department) && $department != "" && $department != 'null') {
            $this->db->where('emp_information.partment', $department);
        }
        // Order by
        $this->db->order_by('item_assignments.assignment_id DESC');
        // Limit
        $this->db->limit($perPage, $offset);
        $query = $this->db->get();
        return $query->result_array();
    }

    public function moveItem($currentUser, $userHash, $itemId)
    {
        // Get count of old item assignments for the old user hash and item ID
        $this->db->where('user_hash', $currentUser);
        $this->db->where('item_id', $itemId);
        $deleted_count = $this->db->count_all_results('item_assignments');

        // Begin transaction
        $this->db->trans_begin();

        // Insert new item assignments for the new user hash based on the count of deleted items
        for ($i = 0; $i < $deleted_count; $i++) {
            $insert_data = array(
                'user_hash' => $userHash,
                'item_id' => $itemId,
                'assignment_date' => date('Y-m-d'), // Assuming today's date for assignment_date
            );
            $this->db->insert('item_assignments', $insert_data);
        }

        // Delete old item assignments for the old user hash
        $this->db->where('user_hash', $currentUser);
        $this->db->where('item_id', $itemId);
        $this->db->delete('item_assignments');

        // Check if transaction is successful
        if ($this->db->trans_status() === false) {
            // Rollback transaction if failed
            $this->db->trans_rollback();
            return 'Transaction failed. Rollback executed.';
        } else {
            // Commit transaction if successful
            $this->db->trans_commit();
            return true; // Return count of deleted item assignments
        }
    }

    public function get($user_hash)
    {
        $this->db->select("ia.user_hash, GROUP_CONCAT(i.item_id) as item_ids, GROUP_CONCAT(i.item_name) as item_names, ia.assignment_date");
        $this->db->from('item_assignments ia');
        $this->db->join('inventory_items i', 'ia.item_id = i.item_id', 'left');
        $this->db->where('ia.user_hash', $user_hash);
        $this->db->group_by('ia.user_hash, ia.assignment_date');
        $query = $this->db->get();

        $results = $query->result_array();
        $groupedResults = [];

        foreach ($results as $row) {
            $item_ids = explode(',', $row['item_ids']);
            $item_names = explode(',', $row['item_names']);
            $items = [];

            // Combine item_ids and item_names into an array of objects
            foreach ($item_ids as $index => $item_id) {
                $items[] = [
                    'value' => $item_id,
                    'label' => $item_names[$index]
                ];
            }

            $groupedResults[] = [
                'user_hash' => $row['user_hash'],
                'items' => $items,
                'assignment_date' => $row['assignment_date']
            ];
        }

        return $groupedResults;

    }
    public function getUserItemAssignments($user_hash)
    {
        $this->db->select('ii.item_id,ia.user_hash, ii.item_name, ii.item_type, iit.name, COUNT(ia.item_id) AS qty');
        $this->db->from('item_assignments AS ia');
        $this->db->join('inventory_items AS ii', 'ia.item_id = ii.item_id', 'inner');
        $this->db->join('inventory_items_types AS iit', 'ii.item_type = iit.id', 'inner');
        $this->db->where('ia.user_hash', $user_hash);
        $this->db->group_by('ia.user_hash, ii.item_id');
        $this->db->order_by('ia.user_hash, ii.item_id');
        $query = $this->db->get();
        return $query->result_array();

    }

    // With out count
    public function getUserItemAssignmentsRows($user_hash)
    {
        $this->db->select('ii.item_id, ia.user_hash, ii.item_name, ii.item_type, iit.name, ia.assignment_id');
        $this->db->from('item_assignments AS ia');
        $this->db->join('inventory_items AS ii', 'ia.item_id = ii.item_id', 'inner');
        $this->db->join('inventory_items_types AS iit', 'ii.item_type = iit.id', 'inner');
        $this->db->where('ia.user_hash', $user_hash);
        $this->db->order_by('ia.user_hash, ii.item_id');
        $query = $this->db->get();
        return $query->result_array();

    }

    public function create($data)
    {
        $itemIds = json_decode($data['item_id'], true);
        $users_hashs = json_decode($data['user_hash'], true);

        // Initialize an array to store the number of affected rows for each insertion
        $affectedRows = [];

        // Loop through each user_hash
        foreach ($users_hashs as $user_hash) {
            // Loop through each itemId
            foreach ($itemIds as $itemId) {
                // Prepare the data for insertion
                $insertData = [
                    'user_hash' => $user_hash,
                    'item_id' => $itemId,
                    'assignment_date' => $data['assignment_date']
                ];

                // Insert the data into the database table
                $this->db->insert('item_assignments', $insertData);

                // Record the number of affected rows for this insertion
                $affectedRows[] = $this->db->affected_rows();
            }
        }

        // Return the array of affected rows
        return $affectedRows;
    }

    public function delete($user_hash)
    {
        $this->db->where('user_hash', $user_hash);
        $this->db->delete('item_assignments');
        return $this->db->affected_rows();

    }

    public function update($user_hash, $data)
    {
        $itemIds = json_decode($data['item_id'], true);
        $assignment_date = $data['assignment_date'];

        // Start a database transaction (if supported by your framework)
        $this->db->trans_start();
        // Step 1: Delete rows that do not match any itemId in $itemIds
        $this->db->where_in('user_hash', $user_hash);
        $this->db->delete('item_assignments');

        // Step 2: Insert new rows for each itemId in $itemIds
        $success = true;
        foreach ($itemIds as $newItemId) {
            $insertData = [
                'user_hash' => $user_hash,
                'assignment_date' => $assignment_date,
                'item_id' => $newItemId,
            ];

            $result = $this->db->insert('item_assignments', $insertData);
            if (!$result) {
                $success = false; // Set success flag to false if any insert fails
            }
        }

        // Commit the transaction
        $this->db->trans_complete();

        // Return the success status
        return $success && $this->db->trans_status();
    }


    public function getItems()
    {
        $this->db->select("item_name as label, item_id as value");
        $this->db->from('inventory_items');
        $query = $this->db->get();
        return $query->result_array();
    }
}
