<?php
class AttendModel extends CI_Model
{
    public function __construct()
    {
        parent::__construct();
        $this->load->database();
        $this->load->model('UserModel');
    }

    // get all Groups
    public function count_records()
    {
        $headers = apache_request_headers();
        $search = $headers['q'] ?? '';
        $department = $this->input->post('department');
        $this->db->select('count(*) as count');
        $this->db->from('system_users');
        // left join with emp_fingerprint
        $this->db->join('emp_information', 'emp_information.user_hash = system_users.hash', 'inner');
        // where is_work = 1
        $this->db->where('is_work', 1);
        if ($search) {
            $this->db->like('system_users.hash', $search);
        }
        if (isset($department) && $department != "") {
            $this->db->where('emp_information.partment', $department);
        }
        $query = $this->db->get();
        return $query->result_array()[0]['count'];
    }

    public function getAttends($date_start, $date_end)
    {
        // get page and limit and search from header
        $page = $this->input->get('page') ?? 1;
        $perPage = $this->input->get('perPage') ?? 10;
        $search = $this->input->get('q') ?? '';
        $offset = ($page - 1) * $perPage;
        $department = $this->input->post('department');

        $this->db->select('
            system_users.name,
            system_users.hash as user_hash,
            image_profile as image,
            (select name from system_group_name where system_group_name.hash=user_type) as job_name,
            partments.name as Positions
         ');
        $this->db->from('system_users');
        // inner join emp_information
        $this->db->join('emp_information', 'emp_information.user_hash = system_users.hash', 'inner');
        $this->db->join('partments', 'emp_information.partment = partments.hash', 'left');
        // where is_work = 1
        $this->db->where('is_work', 1);
        // $this->db->where('emp_fingerprint.fingerprint_time IS NOT NULL');
        //  $this->db->where('emp_fingerprint.fingerprint_time !=', '0000-00-00 00:00:00');

        // search
        if (isset($search) && $search != "") {
            $this->db->where('system_users.hash', $search);
        }
        if (isset($department) && $department != "") {
            $this->db->like('emp_information.position', $department);
        }
        // order by
        $this->db->order_by("partments.name, system_users.id", "asc");
        // limit
        $this->db->limit($perPage, $offset);
        $query = $this->db->get();
        return $query->result_array();
    }

    public function getFingerPrints($hash, $start_time, $end_time)
    {
        $this->db->select('day_print');
        $this->db->from('emp_fingerprint');
        $this->db->where('user_hash', $hash);
        $this->db->where('date(fingerprint_time) >=', $start_time);
        $this->db->where('date(fingerprint_time) <=', $end_time);
        $query = $this->db->get();
        return $query->result_array();
    }

    public function getAttend($day, $user_hash)
    {
        $query = $this->db->query("
            WITH NumberedRecords AS (
                SELECT
                    fingerprint_time,
                    user_hash as user_hash,
                    is_flexible,
                    in_out,
                    hash,
                    ST_X(location) AS lat,
                    ST_Y(location) AS lng,
                    ROW_NUMBER() OVER (
                        PARTITION BY in_out
                        ORDER BY fingerprint_time
                    ) as rn
                FROM emp_fingerprint
                WHERE DATE(fingerprint_time) = '$day'
                AND user_hash = '$user_hash'
            )
            SELECT
                DATE(in_records.fingerprint_time) AS date,
                in_records.is_flexible,
                TIME_FORMAT(in_records.fingerprint_time, '%h:%i %p') AS inTime,
                TIME_FORMAT(out_records.fingerprint_time, '%h:%i %p') AS outTime,
                CASE
                    WHEN out_records.fingerprint_time IS NOT NULL
                    THEN TIMESTAMPDIFF(minute, in_records.fingerprint_time, out_records.fingerprint_time)/60
                    ELSE NULL
                END AS time_difference,
                in_records.hash as in_hash,
                out_records.hash as out_hash,
                CASE
                    WHEN out_records.fingerprint_time IS NULL THEN 'لم يخرج بعد'
                    WHEN DATE_FORMAT(
                        SEC_TO_TIME(
                            FLOOR(
                                (JSON_EXTRACT(acs.schedule_data, '$[0].start') - (acs.allowance_time / 60)) * 3600
                            )
                        ),
                        '%H:%i:%s'
                    ) <= TIME(in_records.fingerprint_time) THEN 'متاخر عن العمل'
                    ELSE 'حاضر'
                END as status,
                in_records.lat as in_lat,
                in_records.lng as in_lng,
                out_records.lat as out_lat,
                out_records.lng as out_lng
            FROM (
                SELECT * FROM NumberedRecords
                WHERE in_out = 'in'
            ) in_records
            LEFT JOIN (
                SELECT * FROM NumberedRecords
                WHERE in_out = 'out'
            ) out_records ON in_records.rn = out_records.rn
            LEFT JOIN emp_information ei
                ON ei.user_hash = '$user_hash'
            LEFT JOIN actual_schedules acs
                ON acs.user_hash = '$user_hash'
                AND DATE(acs.date) = '$day'
            ORDER BY in_records.fingerprint_time
        ");

        return $query->result_array();
    }
    public function getDayStatus($start_date, $end_date, $user_hash)
    {
        $query = $this->db->query(
            "
           SELECT

TIME(MIN(emp_fingerprint.fingerprint_time)),
    CASE
        WHEN COUNT(*) = 0 THEN 'غياب'
        WHEN COUNT(*) = 1 AND MIN(emp_fingerprint.fingerprint_time) IS NULL THEN 'غياب'
        WHEN COUNT(*) = 1 AND in_out = 'in' AND  DATE_FORMAT(SEC_TO_TIME(FLOOR((JSON_EXTRACT(schedule_data, '$[0].start') - (allowance_time / 60) ) * 3600)), '%H:%i:%s') < TIME(MIN(emp_fingerprint.fingerprint_time)) THEN 'متأخر عن العمل'
        WHEN COUNT(*) = 1 AND in_out = 'out' THEN 'بصم خروج بدون دخول'
        WHEN COUNT(*) = 1
             AND DATE_FORMAT(SEC_TO_TIME(FLOOR((JSON_EXTRACT(schedule_data, '$[0].start') - (allowance_time / 60) ) * 3600)), '%H:%i:%s') <= TIME(MIN(emp_fingerprint.fingerprint_time)) THEN 'متأخر عن العمل'
        WHEN COUNT(*) = 1
            AND  DATE_FORMAT(SEC_TO_TIME(FLOOR((JSON_EXTRACT(schedule_data, '$[0].start') - (allowance_time / 60) ) * 3600)), '%H:%i:%s') >= TIME(MIN(emp_fingerprint.fingerprint_time)) THEN 'مازال يداوم'
        WHEN COUNT(*) = 2 AND user_create_record = 5555 THEN 'السيستم بصم عنه بصمة الخروج'
        WHEN COUNT(*) = 2
            AND  DATE_FORMAT(SEC_TO_TIME(FLOOR((JSON_EXTRACT(schedule_data, '$[0].start') - (allowance_time / 60) ) * 3600)), '%H:%i:%s') < TIME(MIN(emp_fingerprint.fingerprint_time)) THEN 'متأخر عن العمل صباحا وخرج من الدوام'
        ELSE 'حضر وانتهي دوامه'
    END AS status,
    DATE(fingerprint_time) AS date,
    TIME_FORMAT(MIN(fingerprint_time), '%h:%i %p') AS inTime,
    CASE
    WHEN COUNT(*) = 1 THEN 'لم يخرج بعد'
    ELSE TIME_FORMAT(MAX(fingerprint_time), '%h:%i %p')

    -- ELSE (
        -- SELECT TIME_FORMAT(fingerprint_time, '%h:%i %p')
        -- FROM emp_fingerprint out_fp
        -- WHERE out_fp.in_out_reference = emp_fingerprint.id
        -- LIMIT 1
    -- )
END AS outTime
FROM
    emp_fingerprint
LEFT JOIN emp_information ON emp_information.user_hash = emp_fingerprint.user_hash
LEFT JOIN actual_schedules ON actual_schedules.user_hash = emp_fingerprint.user_hash
    AND DATE(actual_schedules.date) = DATE(emp_fingerprint.fingerprint_time)
WHERE
    DATE(fingerprint_time) >= '$start_date'
    AND DATE(fingerprint_time) <= '$end_date'
     AND emp_fingerprint.user_hash = '$user_hash'
GROUP BY
    emp_fingerprint.user_hash,
    DATE(fingerprint_time);
            "
        );
        return $query->result_array();
    }

    public function getCurrentMonthAttendForEmp($user_hash)
    {
        $firstDayOfMonth = date('Y-m-01 00:00:00');
        $firstDayOfNextMonth = date('Y-m-d H:i:s', strtotime('+1 month', strtotime($firstDayOfMonth)));
        $this->db->select('(select name from system_users where hash=emp_fingerprint.user_hash) as name,user_hash');
        $this->db->select("GROUP_CONCAT( DATE(fingerprint_time)) AS  attend");
        $this->db->from('emp_fingerprint');
        // check user
        $this->db->where('user_hash', $user_hash);
        // check data span
        $this->db->where('fingerprint_time >=', $firstDayOfMonth);
        $this->db->where('fingerprint_time <', $firstDayOfNextMonth);

        // return result
        $query = $this->db->get();
        $result = $query->result_array();
        // add start and end date as date(YYYY-MM-DD)
        $result[0]['start'] = date('Y-m-01', strtotime($firstDayOfMonth));
        $result[0]['end'] = date('Y-m-d', strtotime($firstDayOfNextMonth . ' -1 day'));
        return $result[0];
    }

    //    public function saveCurrentLocation($hash, $note, $lat, $lng)
    //    {
    //        $query = $this->db->query(
    //            'insert into check_in(user_hash,note,location) values($hash,$note,POINT($lat, $lng))'
    //        );
    //        return $query->result_array();
    //    }

    public function getTodayAttends($page, $perPage)
    {
        $offset = ($page - 1) * $perPage;
        $query = $this->db->query("
        SELECT
        name,
        system_users.hash as hash,
        (select name from system_group_name where system_group_name.hash=user_type) as job_name,
        (select name from partments where partments.hash=partment) as department_name,
        (select Name from Positions where Positions.ID=emp_information.position) as position,
        image_profile,
        CASE
            WHEN (SELECT type FROM holiday WHERE date = CURDATE()) IS NULL THEN 'يوم عمل'
            ELSE (SELECT type FROM holiday WHERE date = CURDATE())
        END AS status,
                CASE
                    -- WHEN TIME(NOW()) < work_from OR TIME(NOW()) > work_to THEN 'خارج وقت الدوام'
                    WHEN EXISTS (
                        SELECT 1
                        FROM vacations
                        WHERE vacations.user_hash = system_users.hash
                        AND CURDATE() BETWEEN DATE(date_start) AND DATE(date_end - INTERVAL 1 DAY)
                        AND vacations.status = '111'
                        -- AND vacations.vication_type_hash IN ('111', '222', '333', '444', '555')
                    ) THEN 'مجاز'
                    WHEN COUNT(*) = 0 THEN 'لم يحضر بعد'
                    WHEN COUNT(*) = 1 AND MIN(emp_fingerprint.fingerprint_time) IS NULL THEN 'لم يحضر بعد'
          WHEN COUNT(*) = 1 AND
     TIME(MIN(emp_fingerprint.fingerprint_time)) > DATE_FORMAT(SEC_TO_TIME(FLOOR((JSON_EXTRACT(schedule_data, '$[0].start') * 3600))), '%H:%i:%s')
     AND DATE_FORMAT(SEC_TO_TIME(FLOOR((JSON_EXTRACT(schedule_data, '$[0].start') - (allowance_time / 60)) * 3600)), '%H:%i:%s') <= TIME(MIN(emp_fingerprint.fingerprint_time))
THEN 'متاخر عن العمل'

                    -- here its else of upper this comment
					WHEN COUNT(CASE WHEN in_out = 'in' THEN 1 END) >= 1 THEN 'حاضر'
                    WHEN COUNT(*) = 1 AND in_out = 'out' THEN 'بصم خروج بدون دخول'
                     -- WHEN COUNT(*) = 1 AND allow_time <= TIME(MIN(emp_fingerprint.fingerprint_time)) THEN 'متاخر عن العمل'
                     -- WHEN COUNT(*) = 1 AND allow_time >= TIME(MIN(emp_fingerprint.fingerprint_time)) THEN 'حاضر'
                     -- WHEN COUNT(*) = 2 AND allow_time < TIME(MIN(emp_fingerprint.fingerprint_time)) THEN 'متاخر عن العمل'
                    ELSE 'حضر وانتهي دوامه'
                END AS title,
        CASE
            WHEN COUNT(*)  < 2 THEN ''
            ELSE TIME_FORMAT(MAX(emp_fingerprint.fingerprint_time), '%h:%i %p')
        END AS outTime,
        CASE
            WHEN COUNT(*) > 0   THEN TIME_FORMAT(min(emp_fingerprint.fingerprint_time), '%h:%i %p')
            ELSE ''
        END AS inTime,
        case
            when count(*) = 1 then TIMESTAMPDIFF(minute,MIN(fingerprint_time),curtime())/60
            when count(*) = 2 then TIMESTAMPDIFF(minute,MIN(fingerprint_time),MAX(fingerprint_time))/60
            else ''
        end as time_difference
    FROM
        system_users
            LEFT JOIN
        emp_fingerprint ON user_hash = system_users.hash
            AND DATE(emp_fingerprint.fingerprint_time) = DATE(CURDATE())
            JOIN
        emp_information ON system_users.hash = emp_information.user_hash
            AND is_work = 1
        LEFT JOIN vacations ON vacations.user_hash = system_users.hash
            AND vacations.date_start <= CURDATE()
            AND vacations.date_end >= CURDATE()
        LEFT JOIN actual_schedules
        ON actual_schedules.user_hash = system_users.hash
        AND actual_schedules.date = DATE(emp_fingerprint.fingerprint_time)
    GROUP BY system_users.hash
    order by inTime desc
    limit $perPage offset $offset;");

        $result = $query->result_array();
        // total emp_informations where is_work = 1
        $this->db->select('count(*) as count');
        $this->db->from('emp_information');
        $this->db->where('is_work', 1);
        $query = $this->db->get();
        $total = $query->result_array();

        return array(
            'data' => $result,
            'total' => $total[0]['count'],
        );
    }

    public function getCurrentSchedule($user_hash, $date, $fingerprint_time)
    {
        $this->db->select('schedule_data')->from('actual_schedules')->where('user_hash', $user_hash)->where('date', $date);
        $data = $this->db->get()->row();

        if ($data) {
            $schedule_array = json_decode($data->schedule_data, true);

            // Find the schedule that matches the fingerprint time
            foreach ($schedule_array as $schedule) {
                if ($fingerprint_time >= $schedule['start'] - 1 && $fingerprint_time <= $schedule['end']) {
                    return $schedule;
                }
            }
        }
        return null; // Return null if no matching schedule is found
    }

    public function hasFingerprint($data, $in_out)
    {
        $today = date('Y-m-d');
        $user_hash = $data['user_hash'];
        $this->db->where('user_hash', $user_hash)
            ->where('fingerprint_time >=', $today . ' 00:00:00')
            ->where('fingerprint_time <=', $today . ' 23:59:59')
            ->where('in_out', $in_out)
            ->where('data_entry !=', '');

        $query = $this->db->get('emp_fingerprint');
        return $query->num_rows() > 0;
    }

    public function countFingerprint($data, $in_out)
    {
        $today = date('Y-m-d');
        $user_hash = $data['user_hash'];
        $this->db->where('user_hash', $user_hash)
            ->where('fingerprint_time >=', $today . ' 00:00:00')
            ->where('fingerprint_time <=', $today . ' 23:59:59')
            ->where('in_out', $in_out);

        $query = $this->db->get('emp_fingerprint');
        return $query->num_rows();
    }

    public function getPreviousFIngerIn($user_hash)
    {
        $today = date('Y-m-d');

        $query = $this->db->select('*')
            ->from('emp_fingerprint')
            ->where('in_out', 'in')
            ->where('user_hash', $user_hash)
            ->where('fingerprint_time >=', "$today 00:00:00")
            ->where('fingerprint_time <=', "$today 23:59:59")
            ->order_by('fingerprint_time', 'desc')
            ->limit(1)
            ->get();

        if ($query->num_rows() > 0) {
            return $query->row(); // Fetch the result
        } else {
            return [];
        }
    }

    private function convertDecimalToDatetime($decimalTime)
    {
        // Extract hours and minutes from the decimal time
        $hours = floor($decimalTime);
        $minutes = ($decimalTime - $hours) * 60;

        // Format the hours and minutes into H:i:s format
        $time = sprintf('%02d:%02d:00', $hours, $minutes);

        // Assume the date is the current date
        $currentDate = date('Y-m-d');

        // Combine the date with the end time
        $fingerprintTime = $currentDate . ' ' . $time;

        return $fingerprintTime;
    }

    public function handleForgetToFingerOut($data)
    {
        $hash = $data['hash'];
        $checkIn = $data['fingerprint_time'];
        unset($data['hash']);

        $prevoiusFingerIn = $this->getPreviousFIngerIn($data['user_hash']);

        $currentSchedule = $this->getCurrentSchedule($data['user_hash'], date('Y-m-d'), date('H:i:s', strtotime($prevoiusFingerIn->fingerprint_time)));

        if ($currentSchedule) {
            $fingerprint_time = date('H:i:s', strtotime($data['fingerprint_time']));

            list($hours, $minutes) = explode(':', $fingerprint_time);
            $fingerprintTimeFloat = (float) $hours + ((float) $minutes / 60.0);

            // Check if the current time falls within the schedule
            if ($fingerprintTimeFloat >= $currentSchedule['start'] - 1 && $fingerprintTimeFloat <= $currentSchedule['end']) {
                return "لقد بصمت دخول لا يمكن بصم دخول مرتين لنفس الشيفت";
            } else {

                // Handle forget to fingerOut (clock out)
                $data['fingerprint_time'] = $this->convertDecimalToDatetime($currentSchedule['end']);
                $data['in_out'] = 'out';
                $this->createAttend($data);
                $data['hash'] = $hash;
                $data['fingerprint_time'] = $checkIn;
                $data['in_out'] = 'in';
                $this->db->insert('emp_fingerprint', $data);
                return $this->db->insert_id();
            }

        } else {
            return "عفوا لا يوجد لديك جدول فى هذا الوقت";
        }

    }

    // public function createAttend($data)
    // {
    //     $countFingerIn = $this->countFingerprint($data, 'in');

    //     if ($data['in_out'] == 'in' && $countFingerIn > 0) {
    //         $countFingerOut = $this->countFingerprint($data, 'out');
    //         if ($countFingerIn > $countFingerOut) {
    //             return $this->handleForgetToFingerOut($data);
    //         }
    //     }

    //     if ($data['in_out'] == 'in') {
    //         $currentSchedule = $this->getCurrentSchedule($data['user_hash'], date('Y-m-d'), date('H:i:s', strtotime($data['fingerprint_time'])));
    //         if ($currentSchedule) {
    //             $fingerprint_time = date('H:i:s', strtotime($data['fingerprint_time']));

    //             list($hours, $minutes) = explode(':', $fingerprint_time);
    //             $fingerprintTimeFloat = (float) $hours + ((float) $minutes / 60.0);

    //             // Check if the current time falls within the schedule
    //             if ($fingerprintTimeFloat >= $currentSchedule['start'] - 1 && $fingerprintTimeFloat <= $currentSchedule['end']) {
    //                 $this->db->insert('emp_fingerprint', $data);
    //                 return $this->db->insert_id();
    //             } else {
    //                 return "عفوا لا يوجد لديك جدول فى هذا الوقت";
    //             }
    //         }
    //         return "عفوا لا يوجد لديك جدول فى هذا الوقت";
    //     }

    //     $this->db->insert('emp_fingerprint', $data);
    //     return $this->db->insert_id();
    // }

    // this will for scheduleOnly
    public function createAttend($data)
    {
        $date = date('Y-m-d', strtotime($data['fingerprint_time']));
        $result = $this->hasActualSchedule($data, $date);

        $user_hash = $data['user_hash'];
        $in_out = $data['in_out'];

        if ($result['status'] !== 'success') {
            return $result;
        }
        $scheduleData = $result['data'];

        $moreOneSchedule = $this->isMoreOneSchedule($scheduleData);

        if ($moreOneSchedule) {

            // handle here
            $scheduleId = $this->input->post('schedule_id');
            if (empty($scheduleId || !$scheduleId)) {
                return $this->sendErrorResponse("الجدول غير موجود");
            }
            $data['schedule_id'] = $scheduleId;

            $scheduleExits = $this->checkIfScheduleIdAttendNotExits($data, $date);
            if ($scheduleExits) {
                $in_out == "in" ? $errorMessage = "الموظف لديه بصمه دخول بالفعل" : $errorMessage = "الموظف لديه بصمه خروج بالفعل";
                return $this->sendErrorResponse($errorMessage);
            }

            return $this->insertAttend($data);
        }

        $scheduleExits = $this->checkIfScheduleInAttendNotExists($user_hash, $date, $in_out);

        if ($scheduleExits) {
            $in_out == "in" ? $errorMessage = "الموظف لديه بصمه دخول بالفعل" : $errorMessage = "الموظف لديه بصمه خروج بالفعل";
            return $this->sendErrorResponse($errorMessage);
        }
        $scheduleId = $scheduleData[0]['id'];
        // if ($in_out == 'out') {
        //     $inExists = $this->checkIfInExists($user_hash);
        //     if (!$inExists) {
        //         return $this->sendErrorResponse("لا يمكن تسجيل بصمة خروج قبل بصمة دخول");
        //     }
        // }

        if (!$scheduleId) {
            return $this->sendErrorResponse("الجدول غير موجود");
        }

        $data['schedule_id'] = $scheduleId;

        return $this->insertAttend($data);
    }

    public function createFlexibleAttend($data)
    {
        $date = date('Y-m-d', strtotime($data['fingerprint_time']));
        $result = $this->hasDailyFlexible($data, $date);
        if ($result['status'] !== 'success') {
            return $result;
        }

        $validationResult = $this->validateInOut($data);
        if ($validationResult['status'] !== 'success') {
            return $validationResult;
        }

        return $this->insertFlexibleAttend($data);

    }

    private function hasDailyFlexible($data, $date)
    {
        $user_hash = $data['user_hash'];
        $scheduleData = $this->db->select('*')->from('daily_flexible_schedule_targets')
            ->where('user_hash', $user_hash)->where('date', $date)
            ->get()->row_array();

        if ($this->db->affected_rows() > 0) {

            if (!empty($data) && count($data) > 0) {
                return [
                    'status' => 'success',
                    'data' => $scheduleData,
                ];
            }

            return [
                'status' => 'error',
                'message' => 'الموظف ليس لديه جدول مرن فى هذا التاريخ',
            ];

        }

        return [
            'status' => 'error',
            'message' => 'الموظف ليس لديه جدول مرن فى هذا التاريخ',
        ];
    }

    private function validateInOut($data)
    {
        $user_hash = $data['user_hash'];
        $fingerprint_time = $data['fingerprint_time'];
        $in_out = $data['in_out'];

        // Get the last attendance record for this user within 24 hours
        $last24Hours = date('Y-m-d H:i:s', strtotime('-24 hours', strtotime($fingerprint_time)));

        $lastRecord = $this->db->select('*')
            ->from('emp_fingerprint')
            ->where('user_hash', $user_hash)
            ->where('fingerprint_time >=', $last24Hours)
            ->where('fingerprint_time <=', $fingerprint_time)
            ->where('is_flexible', '1')
            ->order_by('fingerprint_time', 'DESC')
            ->limit(1)
            ->get()
            ->row_array();

        // Case 1: If this is an "in" record
        if ($in_out == "in") {
            // Check if there's already an "in" without an "out" within 24 hours

            if (!empty($lastRecord) && $lastRecord['in_out'] == 'out') {
                return ['status' => 'success'];
            }

            if (!empty($lastRecord) && $lastRecord['in_out'] == "in") {
                return [
                    'status' => 'error',
                    'message' => 'لا يمكن تسجيل دخول جديد قبل تسجيل الخروج',
                ];
            }

        }

        // Case 2: If this is an "out" record
        if ($in_out == "out") {
            // Check if there's a corresponding "in" record within 24 hours

            if (empty($lastRecord) || $lastRecord['in_out'] !== "in") {
                return [
                    'status' => 'error',
                    'message' => 'لا يمكن تسجيل الخروج بدون تسجيل الدخول خلال 24 ساعة',
                ];
            }
        }

        return [
            'status' => 'success',
        ];
    }

    private function insertFlexibleAttend($data)
    {

        $data['is_flexible'] = '1';
        $this->db->insert('emp_fingerprint', $data);
        $data['fingerprint_id'] = $this->db->insert_id();

        if ($data['in_out'] == "out") {
            $updatedId = $this->UserModel->handleFingerprintScheduling($data);
            return ['status' => 'success', 'data' => $updatedId];
        }
        $this->UserModel->handleFingerprintScheduling($data);
        $insertedId = $this->db->insert_id();

        return ['status' => 'success', 'data' => $insertedId];

    }

    private function checkIfScheduleInAttendNotExists($user_hash, $date, $in_out)
    {

        $this->db->select('id')
            ->from('emp_fingerprint')
            ->where('user_hash', $user_hash)
            ->where('DATE(fingerprint_time)', $date)
            ->where('is_flexible', '0');

        if ($in_out == "in") {
            $this->db->where('in_out', 'in');
        } elseif ($in_out == "out") {
            $this->db->where('in_out', 'out');
        }

        // Get result
        $result = $this->db->get()->row_array();

        if (!empty($result)) {
            return true;
        }

        return false;
    }

    private function checkIfInExists($user_hash)
    {
        $this->db->where('user_hash', $user_hash);
        $this->db->order_by('fingerprint_time', 'DESC');
        $this->db->limit(1);
        $query = $this->db->get('emp_fingerprint');

        $lastRecord = $query->row();

        if ($lastRecord && $lastRecord->in_out === 'in') {
            return true;
        }

        return false;
    }

    private function checkIfScheduleIdAttendNotExits($data, $date)
    {

        $user_hash = $data['user_hash'];
        $schedule_id = $data['schedule_id'];
        $in_out = $data['in_out'];

        $this->db->select('id')
            ->from('emp_fingerprint')
            ->where('user_hash', $user_hash)
            ->where('DATE(fingerprint_time)', $date)
            ->where('is_flexible', '0')
            ->where('schedule_id', $schedule_id);

        if ($in_out == "in") {
            $this->db->where('in_out', 'in');
        } elseif ($in_out == "out") {
            $this->db->where('in_out', 'out');
        }

        $result = $this->db->get()->row_array();

        if (!empty($result)) {
            return true;
        }

        return false;

    }

    private function insertAttend($data)
    {

        $scheduleId = $data['schedule_id'];

        $this->db->where('user_hash', $data['user_hash']);
        $this->db->where('fingerprint_type', $data['fingerprint_type']);
        $this->db->where('in_out', $data['in_out']);
        $this->db->where('schedule_id', $scheduleId);
        $this->db->where('DATE(fingerprint_time) =', date('Y-m-d', strtotime($data['fingerprint_time'])));

        $query = $this->db->get('emp_fingerprint');

        // If no existing record is found, insert the data
        if ($query->num_rows() > 0) {
            return $this->sendErrorResponse("الموظف بصم دخول بالفعل");
        }

        $this->db->insert('emp_fingerprint', $data);
        $data['fingerprint_id'] = $this->db->insert_id();

        if ($data['in_out'] == "out") {
            $updatedId = $this->UserModel->handleFingerprintScheduling($data);
            return ['status' => 'success', 'data' => $updatedId];
        }
        $this->UserModel->handleFingerprintScheduling($data);
        $insertedId = $this->db->insert_id();

        return ['status' => 'success', 'data' => $insertedId];
    }

    public function deleteAttend($hash)
    {

        $this->db->where('hash', $hash)
            ->delete('emp_fingerprint');
        return $this->db->affected_rows();
    }

    private function hasActualSchedule($data, $date)
    {
        $user_hash = $data['user_hash'];
        $fingerprint_time = $data['fingerprint_time'];
        $scheduleData = $this->db->select('schedule_data')->from('actual_schedules')
            ->where('user_hash', $user_hash)->where('date', $date)
            ->get()->row_array();

        if ($this->db->affected_rows() > 0) {
            $data = json_decode($scheduleData['schedule_data'], true);

            if (!empty($data) && count($data) > 0) {
                return [
                    'status' => 'success',
                    'data' => $data,
                ];
            }

            return [
                'status' => 'error',
                'message' => 'الموظف ليس لديه جدول فى هذا التاريخ',
            ];
        };

        $hasVacation = $this->isEmployeeHasVacation($user_hash, $fingerprint_time);
        if ($hasVacation) {
            $vacationDetails = $this->getEmployeeVacationDetails($user_hash, $fingerprint_time);
            if ($vacationDetails) {
                $dateStart = (new DateTime($vacationDetails['date_start']))->format('Y-m-d');
                $dateEnd = (new DateTime($vacationDetails['date_end']))->format('Y-m-d');

                return [
                    'status' => 'error',
                    'message' => 'الموظف في إجازة ' . $vacationDetails['type'] . ' من ' . $dateStart . ' إلى ' . $dateEnd,
                ];
            }
        }

        return [
            'status' => 'error',
            'message' => 'الموظف ليس لديه جدول فى هذا التاريخ',
        ];
    }

    private function isEmployeeHasVacation($user_hash, $date)
    {
        $query = $this->db->select('id')->from('vacations')->where('user_hash', $user_hash)->where('date_start <=', $date)->where('date_end >=', $date)->get();
        if ($query->num_rows() > 0) {
            return true;
        }
        return false;
    }

    private function getEmployeeVacationDetails($user_hash, $date)
    {
        $query = $this->db->select('v.date_start, v.date_end, vt.type')
            ->from('vacations v')
            ->join('vacations_type vt', 'v.vication_type_hash = vt.hash')
            ->where('v.user_hash', $user_hash)
            ->where('v.date_start <=', $date)
            ->where('v.date_end >=', $date)
            ->get();
        if ($query->num_rows() > 0) {
            return $query->row_array();
        }
        return null;
    }

    private function isMoreOneSchedule($data)
    {
        if (count($data) == 1) {
            return false;
        } else {
            return true;
        }

    }
    private function getUserHasScheduleAndIsOnlyOne($user_hash, $date)
    {
        // Retrieve schedule data for the given user and date
        $scheduleData = $this->db->select('schedule_data')->from('actual_schedules')
            ->where('user_hash', $user_hash)->where('date', $date)
            ->get()->row_array();

        // Check if any rows are returned
        if ($this->db->affected_rows() > 0) {
            // Decode the schedule data
            $data = json_decode($scheduleData['schedule_data'], true);

            // Check if there's exactly one schedule
            if (count($data) == 1) {
                // Return the ID of the schedule
                return $data[0]['id'];
            }
        }

        // Return false if no schedule is found or there is more than one schedule
        return false;
    }

    public function updateAttend($data, $hash)
    {
        $this->db->where('hash', $hash);
        $this->db->update('emp_fingerprint', $data);
        return $this->db->affected_rows();
    }

    
    public function getFingerprint($hash)
    {
        $this->db->select('fingerprint_time, in_out, system_users.name AS user_name, system_users.hash AS user_hash');
        $this->db->from('emp_fingerprint');
        $this->db->join('system_users', 'emp_fingerprint.user_hash = system_users.hash ');

        $this->db->where('emp_fingerprint.hash', $hash);

        $query = $this->db->get();
        $result = $query->result_array();
        return $result;
    }
    private function sendErrorResponse($message, $statusCode = 400)
    {
        return ['status' => 'error', 'message' => $message];
    }

}
