<?php

class BookModel extends CI_Model
{
    public function __construct()
    {
        parent::__construct();
        $this->load->database();
    }

    public function get_count()
    {
        $this->db->select('count');
        $this->db->from('book_count');
        $this->db->where('id', 1);
        $query = $this->db->get()->row();
        return $query->count;
    }

    public function update_count()
    {
        $this->db->set('count', $this->get_count() +1 );
        $this->db->where('id', 1);
        $this->db->update('book_count');
        return $this->db->affected_rows();
    }

    public function create_book_transaction($data)
    {
        $this->db->insert('book_transaction', $data);
        return $this->db->insert_id();
    }
    public function set_count($count)
    {
        $this->db->set('count', $count );
        $this->db->where('id', 1);
        $this->db->update('book_count');
        return $this->db->affected_rows();
    }

    public function add_side_company($name)
    {
        $data = array(
            'name' => $name
        );
        $this->db->insert('side_company', $data);
        return $this->db->insert_id();
    }

    public function get_transactions($data)
    {
        /*
        SELECT
            book_transaction.id,
            count,
            hash,
            subject,
            details,
            date,
            (SELECT name FROM side_company WHERE id = book_transaction.export_side) AS export,
            (SELECT name FROM side_company WHERE id = book_transaction.import_side) AS import,
        FROM
	        book_transaction
        JOIN
	        side_company
            ON
    	        book_transaction.export_side = side_company.id
        */
        $this->db->select(' book_transaction.id,
                            is_exported,
                            book_transaction.attachment,
                            book_date,
                            book_number,
                            count,
                            hash,
                            subject,
                            details,
                            date,
                            side_export.name AS export,
                            side_import.name AS import,
                            book_transaction.book_code,
                            book_transaction.main_book,
                            book_transaction.is_attachment
                            ');
        $this->db->from('book_transaction');
        $this->db->join('side_company AS side_export', 'book_transaction.export_side = side_export.id', 'left');
        $this->db->join('side_company AS side_import', 'book_transaction.import_side = side_import.id', 'left');

        if (!empty($data['book_number']) && !empty($data['book_number'])) {
            $this->db->where('book_transaction.book_number',$data['book_number']);
        }
        if (!empty($data['hash']) && !empty($data['hash'])) {
            $this->db->like('book_transaction.hash',$data['hash']);
        }
        if (!empty($data['start_date']) && !empty($data['end_date'])) {
            $this->db->where('DATE(book_transaction.date) BETWEEN "'.$data['start_date'].'" AND "'.$data['end_date'].'"');
        }
        if (!empty($data['export_side'])) {
            $this->db->where('book_transaction.export_side', $data['export_side']);
        }
        if (!empty($data['is_exported']) || $data['is_exported'] == "0" ) {
            $this->db->where('book_transaction.is_exported',$data['is_exported']);
        }
        if (!empty($data['subject'])) {
            $this->db->like('book_transaction.subject', $data['subject']);
        }
        if (!empty($data['description'])) {
            $this->db->like('book_transaction.description', $data['description']);
        }
        if (!empty($data['import_side'])) {
            $this->db->where('book_transaction.import_side', $data['import_side']);
        }
        if (!empty($data['per_page']) && !empty($data['page'])) {
            $this->db->limit($data['per_page'], ($data['page'] - 1) * $data['per_page']);
        }
        if (!empty($data['book_date']) && !empty($data['book_date'])) {
            $this->db->where('book_transaction.book_date', $data['book_date']);
        }

        $this->db->order_by('book_transaction.id', 'DESC');
        $query = $this->db->get();
        $results = $query->result();
        foreach ($results as &$result) {
            $this->db->select('*, side_import.name');
            $this->db->from('transaction_to_side_companies');
            $this->db->join('side_company AS side_import', 'transaction_to_side_companies.company_id = side_import.id', 'left');
            $this->db->where('transaction_id', $result->id);
            $imported_sides = $this->db->get()->result();
            $result->imported_sides = $imported_sides; // Assign the imported sides to the result
        }
        return $results;
    }

    public function get_transactions_count($data)
    {
        $this->db->select('COUNT(*) as count');
        $this->db->from('book_transaction');
        $this->db->join('side_company AS side_export', 'book_transaction.export_side = side_export.id', 'left');
        $this->db->join('side_company AS side_import', 'book_transaction.import_side = side_import.id', 'left');
        if (!empty($data['book_number']) && !empty($data['book_number'])) {
            $this->db->where('book_transaction.book_number',$data['book_number']);
        }
        if (!empty($data['hash']) && !empty($data['hash'])) {
            $this->db->like('book_transaction.hash',$data['hash']);
        }
        if (!empty($data['description']) && !empty($data['description'])) {
            $this->db->like('book_transaction.description',$data['description']);
        }
        if (!empty($data['start_date']) && !empty($data['end_date'])) {
            $this->db->where('DATE(book_transaction.date) BETWEEN "'.$data['start_date'].'" AND "'.$data['end_date'].'"');
        }
        if (!empty($data['export_side'])) {
            $this->db->where('book_transaction.export_side', $data['export_side']);
        }
        if (!empty($data['import_side'])) {
            $this->db->where('book_transaction.import_side', $data['import_side']);
        }
        if (!empty($data['is_exported'])) {
            $this->db->where('book_transaction.is_exported',$data['is_exported']);
        }
        if (!empty($data['book_date']) && !empty($data['book_date'])) {
            $this->db->where('book_transaction.book_date', $data['book_date']);
        }

        $query = $this->db->get();
        $result = $query->row();
        return $result->count;
    }

    public function get_sides()
    {
        $this->db->select('id,name');
        $this->db->from('side_company');
        $query = $this->db->get();
        return $query->result();
    }

    public function get_attachments($book_id)
    {
        $this->db->select('*');
        $this->db->from('book_transaction');
        $this->db->where('main_book',$book_id);
        $this->db->where('is_attachment',1);
        $query = $this->db->get();
        return $query->result();
    }

    public function create_attachments($data)
    {
        $this->db->insert('books_attachment', $data);
        return $this->db->insert_id();
    }

    public function delete_attachments($id)
    {
        $this->db->where('id', $id);
        $this->db->delete('book_transaction');
        $affected_rows = $this->db->affected_rows();
        return $affected_rows;
    }

    public function get_books()
    {
        $this->db->select('id,subject');
        $this->db->from('book_transaction');
        $query = $this->db->get();
        return $query->result();
    }

    public function insert_import_sides($data)
    {
        foreach($data as $side)
            {
                $this->db->insert('transaction_to_side_companies', $side);
            }
    }

    public function edit_book_transaction($data)
    {
        $this->db->set('subject', $data['subject']);
        $this->db->set('details', $data['details']);
        $this->db->set('book_number', $data['book_number']);
        $this->db->set('is_exported', $data['is_exported']);
        $this->db->set('export_side', $data['export_side']);
        $this->db->set('hash', $data['hash']);
        $this->db->set('attachment', $data['attachment']);
        $this->db->where('id', $data['id']);
        $this->db->update('book_transaction');


        $affected_rows = $this->db->affected_rows();
        return $affected_rows;
    }

    public function get_book_code($book_id)
    {
        $this->db->select('book_code');
        $this->db->from('book_transaction');
        $this->db->where('id', $book_id);
        $query = $this->db->get();
        return $query->result();
    }

    /*
     $this->db->set('description', $desc);
$this->db->where('hash', $hash);
$this->db->update('book_transaction');

     */

    public function set_desc($hash,$desc)
    {
        $this->db->set('description', $desc);
        $this->db->where('book_code', $hash);
        $this->db->update('book_transaction');
        $affected_rows = $this->db->affected_rows();
        return $affected_rows;
    }
    public function delete_old_import_sides($id)
    {
        $this->db->where('transaction_id', $id);
        $this->db->delete('transaction_to_side_companies');

        $affected_rows = $this->db->affected_rows();
        return $affected_rows;
    }
    public function delete_side($side_id)
    {
        $this->db->where('id', $side_id);
        $this->db->delete('side_company');
        $affected_rows = $this->db->affected_rows();
        return $affected_rows;
    }
}