<?php
defined('BASEPATH') or exit('No direct script access allowed');

class DocumentFileUtils extends CI_Model
{

    public function __construct()
    {
        parent::__construct();
        // Load the Format model that provides extension arrays.
        $this->load->model('Format_model');
    }

    /**
     * Get file name from a document URI.
     *
     * @param string $uri The document URI.
     * @return string The file name.
     */
    public function getFileName($uri)
    {
        $ind = strrpos($uri, '/');
        if ($ind === false) {
            return $uri;
        }
        return substr($uri, $ind + 1);
    }

    /**
     * Get file name without extension from the document URI.
     *
     * @param string $uri The document URI.
     * @return string The file name without extension.
     */
    public function getFileNameWithoutExt($uri)
    {
        $filename = $this->getFileName($uri);
        $ind = strrpos($filename, '.');
        if ($ind === false) {
            return $filename;
        }
        return substr($filename, 0, $ind);
    }

    /**
     * Get file extension from the document URI.
     *
     * @param string $uri The document URI.
     * @return string The file extension in lowercase (including the dot).
     */
    public function getFileExt($uri)
    {
        $filename = $this->getFileName($uri);
        $ind = strrpos($filename, '.');
        if ($ind === false) {
            return '';
        }
        return strtolower(substr($filename, $ind));
    }

    /**
     * Get the file type based on its extension.
     * Supported types: pdf, word, cell, slide.
     *
     * @param string $uri The document URI.
     * @return string The file type. Defaults to 'word' if no match found.
     */
    public function getFileType($uri)
    {
        $ext = $this->getFileExt($uri);

        if (in_array($ext, $this->Format_model->pdf_extensions())) {
            return 'pdf';
        }
        if (in_array($ext, $this->Format_model->document_extensions())) {
            return 'word';
        }
        if (in_array($ext, $this->Format_model->spreadsheet_extensions())) {
            return 'cell';
        }
        if (in_array($ext, $this->Format_model->presentation_extensions())) {
            return 'slide';
        }

        // Default file type.
        return 'word';
    }
}
