<?php
defined('BASEPATH') or exit('No direct script access allowed');

use Firebase\JWT\JWT;

class DocumentJwtManager extends CI_Model
{

    protected $configManager;

    public function __construct()
    {
        parent::__construct();
        // Load the configuration model we created earlier.
        $this->load->model('DocumentConfiguration');
        $this->configManager = $this->DocumentConfiguration;
    }

    /**
     * Check if a secret key for generating JWT tokens exists.
     *
     * @return bool
     */
    public function isEnabled()
    {
        return !empty($this->configManager->jwt_secret());
    }

    /**
     * Check if the JWT should be used for the request.
     *
     * @return bool
     */
    public function useForRequest()
    {
        return $this->configManager->jwt_use_for_request();
    }

    /**
     * Encode a payload array into a JWT token using the secret key.
     *
     * @param array $payload
     * @return string
     */
    public function encode($payload)
    {
        $secret = $this->configManager->jwt_secret();
        // Encode the payload with HS256 algorithm.
        return JWT::encode($payload, $secret, 'HS256');
    }

    /**
     * Decode a JWT token back into a payload object.
     *
     * @param string $token
     * @return object
     */
    public function decode($token)
    {
        $secret = $this->configManager->jwt_secret();
        // Decode the token with HS256 algorithm.
        return JWT::decode($token, $secret, array('HS256'));
    }
}
