<?php
defined('BASEPATH') or exit('No direct script access allowed');

class DocumentTrackerManger extends CI_Model
{

    public function __construct()
    {
        parent::__construct();
        // Load required models/libraries.
        $this->load->model('DocumentConfiguration');
        $this->load->model('DocumentJwtManager');
        $this->load->model('DocManager');
        $this->load->model('HistoryManger');
        $this->load->model('DocumentFileUtils');
        $this->load->model('DocumentServiceConventor');
    }


    public function commandRequest($method, $key, $meta = null)
    {
        $payload = array(
            'c' => $method,
            'key' => $key
        );
        if ($meta !== null) {
            $payload['meta'] = $meta;
        }

        // Setup headers.
        $headers = array(
            'Accept: application/json',
            'Content-Type: application/json'
        );


        if ($this->DocumentJwtManager->isEnabled() && $this->DocumentJwtManager->useForRequest()) {
            $headerToken = $this->DocumentJwtManager->encode(array('payload' => $payload));
            $jwtHeader = $this->DocumentConfiguration->jwt_header();
            $headers[] = $jwtHeader . ': Bearer ' . $headerToken;
            $payload['token'] = $this->DocumentJwtManager->encode($payload);
        }

        // Get the command URL from the configuration.
        $url = $this->DocumentConfiguration->document_server_command_url();
        // If $url is returned as an array (parsed URL), convert it to a string.
        if (is_array($url)) {
            $url = $this->unparse_url($url);
        }

        $ch = curl_init($url);
        $jsonPayload = json_encode($payload);
        curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
        curl_setopt($ch, CURLOPT_POST, true);
        curl_setopt($ch, CURLOPT_HTTPHEADER, $headers);
        curl_setopt($ch, CURLOPT_POSTFIELDS, $jsonPayload);
        curl_setopt($ch, CURLOPT_TIMEOUT, 5);
        curl_setopt($ch, CURLOPT_SSL_VERIFYPEER, $this->DocumentConfiguration->ssl_verify_peer_mode_enabled());

        $response = curl_exec($ch);
        if ($response === false) {
            throw new Exception("cURL error: " . curl_error($ch));
        }
        curl_close($ch);

        $jsonResponse = json_decode($response, true);
        $error = isset($jsonResponse["error"]) ? $jsonResponse["error"] : null;

        // If error is not 4 or 0, throw an exception.
        if ($error !== 4 && $error !== 0) {
            throw new Exception("Command Service Error #" . $error);
        }

        return $jsonResponse;
    }

    private function unparse_url($parsed_url)
    {
        $scheme = isset($parsed_url['scheme']) ? $parsed_url['scheme'] . '://' : '';
        $host = isset($parsed_url['host']) ? $parsed_url['host'] : '';
        $port = isset($parsed_url['port']) ? ':' . $parsed_url['port'] : '';
        $user = isset($parsed_url['user']) ? $parsed_url['user'] : '';
        $pass = isset($parsed_url['pass']) ? ':' . $parsed_url['pass'] : '';
        $pass = ($user || $pass) ? "$pass@" : '';
        $path = isset($parsed_url['path']) ? $parsed_url['path'] : '';
        $query = isset($parsed_url['query']) ? '?' . $parsed_url['query'] : '';
        $fragment = isset($parsed_url['fragment']) ? '#' . $parsed_url['fragment'] : '';
        return "$scheme$user$pass$host$port$path$query$fragment";
    }

}