<?php
/**
 * (c) Copyright Ascensio System SIA 2024
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

/**
 * Class representing a user.
 */
class DocumentUsers extends CI_Model
{
    public $id;
    public $name;
    public $email;
    public $group;
    public $reviewGroups;
    public $commentGroups;
    public $userInfoGroups;
    public $favorite;
    public $deniedPermissions;
    public $descriptions;
    public $templates;
    public $avatar;
    public $goback;

    public function __construct(
        $uid,
        $name,
        $email,
        $group,
        $reviewGroups,
        $commentGroups,
        $userInfoGroups,
        $favorite,
        $deniedPermissions,
        $descriptions,
        $templates,
        $avatar,
        $goback
    ) {
        $this->id = $uid;
        $this->name = $name;
        $this->email = $email;
        $this->group = $group;
        $this->reviewGroups = $reviewGroups;
        $this->commentGroups = $commentGroups;
        $this->userInfoGroups = $userInfoGroups;
        $this->favorite = $favorite;
        $this->deniedPermissions = $deniedPermissions;
        $this->descriptions = $descriptions;
        $this->templates = $templates;
        $this->avatar = $avatar;
        $this->goback = $goback;
    }
}

// Descriptions for users
$descr_user_1 = [
    "File author by default",
    "Doesn’t belong to any group",
    "Can review all the changes",
    "Can perform all actions with comments",
    "The file favorite state is undefined",
    "Can create files from templates using data from the editor",
    "Can see the information about all users",
    "Has an avatar",
    "Can submit forms"
];

$descr_user_2 = [
    "Belongs to Group2",
    "Can review only his own changes or changes made by users with no group",
    "Can view comments, edit his own comments and comments left by users with no group. Can remove his own comments only",
    "This file is marked as favorite",
    "Can create new files from the editor",
    "Can see the information about users from Group2 and users who don’t belong to any group",
    "Has an avatar",
    "Can’t submit forms"
];

$descr_user_3 = [
    "Belongs to Group3",
    "Can review changes made by Group2 users",
    "Can view comments left by Group2 and Group3 users. Can edit comments left by the Group2 users",
    "This file isn’t marked as favorite",
    "Can’t copy data from the file to clipboard",
    "Can’t download the file",
    "Can’t print the file",
    "Can create new files from the editor",
    "Can see the information about Group2 users",
    "Can’t submit forms",
    "Can't close history",
    "Can't restore the file version"
];

$descr_user_0 = [
    "The name is requested when the editor is opened",
    "Doesn’t belong to any group",
    "Can review all the changes",
    "Can perform all actions with comments",
    "The file favorite state is undefined",
    "Can't mention others in comments",
    "Can't create new files from the editor",
    "Can’t see anyone’s information",
    "Can't rename files from the editor",
    "Can't view chat",
    "Can't protect file",
    "View file without collaboration",
    "Can’t submit forms"
];

// Global array of users
$USERS = [
    new DocumentUsers(
        'uid-1',
        'John Smith',
        'smith@example.com',
        '',
        null,
        [],
        null,
        true,
        [],
        $descr_user_1,
        true,
        true,
        ['blank' => false]
    ),
    new DocumentUsers(
        'uid-2',
        'Mark Pottato',
        'pottato@example.com',
        'group-2',
        ['group-2', ''],
        [
            'view' => "",
            'edit' => ["group-2", ""],
            'remove' => ["group-2"]
        ],
        ['group-2', ''],
        true,
        [],
        $descr_user_2,
        false,
        true,
        ['text' => "Go to Documents"]
    ),
    new DocumentUsers(
        'uid-3',
        'Hamish Mitchell',
        null,
        'group-3',
        ['group-2'],
        [
            'view' => ["group-3", "group-2"],
            'edit' => ["group-2"],
            'remove' => []
        ],
        ['group-2'],
        false,
        ["copy", "download", "print"],
        $descr_user_3,
        false,
        false,
        null
    ),
    new DocumentUsers(
        'uid-0',
        null,
        null,
        '',
        null,
        [],
        [],
        null,
        ["protect"],
        $descr_user_0,
        false,
        false,
        null
    )
];

$DEFAULT_USER = $USERS[0];

/**
 * Get all users.
 *
 * @return array Array of DocumentUsers objects.
 */
function getAllUsers()
{
    global $USERS;
    return $USERS;
}

/**
 * Get user information from the request.
 *
 * Expects an associative array with a 'COOKIES' key.
 *
 * @param array $req Request data.
 * @return DocumentUsers
 */

// TODO : refactor this function to use a class
function getUserFromReq($userHash)
{
    global $USERS, $DEFAULT_USER;
    // Assuming $req['COOKIES'] is an associative array containing cookie data.
    $uid = $userHash;

    foreach ($USERS as $user) {
        if ($user->id === $uid) {
            return $user;
        }
    }
    return $DEFAULT_USER;
}

/**
 * Get users data for mentions.
 *
 * Returns users (except the current user) with a non-null name and email.
 *
 * @param string $uid The ID of the current user.
 * @return array Array of associative arrays with keys 'name' and 'email'.
 */
function getUsersForMentions($uid)
{
    global $USERS;
    $usersData = [];
    foreach ($USERS as $user) {
        if ($user->id !== $uid && $user->name !== null && $user->email !== null) {
            $usersData[] = [
                'name' => $user->name,
                'email' => $user->email
            ];
        }
    }
    return $usersData;
}

/**
 * Get users data for protect.
 *
 * Returns users (except the current user) with a non-null name.
 *
 * @param string $uid The ID of the current user.
 * @return array Array of associative arrays with keys 'id', 'name', and 'email'.
 */
function getUsersForProtect($uid)
{
    global $USERS;
    $usersData = [];
    foreach ($USERS as $user) {
        if ($user->id !== $uid && $user->name !== null) {
            $usersData[] = [
                'id' => $user->id,
                'name' => $user->name,
                'email' => $user->email
            ];
        }
    }
    return $usersData;
}

/**
 * Find a user by ID.
 *
 * @param string|null $searchId
 * @return DocumentUsers
 */
function find_user($searchId)
{
    global $USERS, $DEFAULT_USER;
    if ($searchId === null) {
        return $DEFAULT_USER;
    }
    foreach ($USERS as $user) {
        if ($user->id === $searchId) {
            return $user;
        }
    }
    return $DEFAULT_USER;
}
?>