<?php
defined('BASEPATH') or exit('No direct script access allowed');

class Document_model extends CI_Model
{
    public function __construct()
    {
        parent::__construct();
        $this->load->database();
    }

    public function getWorkSpaceForUser($userHash) {
        $this->db->select('id')
                 ->from('document_workspaces')
                 ->where('user_id', $userHash);
                 
        $query = $this->db->get();
        $workSpace = $query->row_array();
    
        if (!$workSpace) {
            // No workspace found, create a new one
            $newWorkSpace = [
                'user_id' => $userHash,
            ];
    
            $this->db->insert('document_workspaces', $newWorkSpace);
            return $this->db->insert_id();
        }
    
        return $workSpace['id'];
    }
    

    public function getWorkSpaces()
    {
        $this->db->select('*')->from('document_workspace');
        $query = $this->db->get();
        return $query->result_array();
    }

    public function createWorkSpace($data)
    {
        $this->db->insert('document_workspace', $data);
        return $this->db->insert_id();
    }

    public function createDocument($data)
    {
        try {

            $dataDocumentsToInsert = [
                'document_workspace_id' => $data['document_workspace_id'],
                'file_name' => $data['file_name'],
                'title' => $data['title'],
                'file_key' => $data['file_key'],
                'doc_type' => $data['doc_type'],
                'created_at' => $data['created_at'],
                'updated_at' => $data['updated_at'],
                'user_id' => $data['user_id']
            ];

            $this->db->insert('documents', $dataDocumentsToInsert);

            $documentId = $this->db->insert_id();

            $permissionDataToInsert = [
                'user_id' => $data['user_id'],
                'doc_id' => $documentId,
                'can_edit' => 1,
                'can_download' => 1,
                // 'can_comment' => 1,
                'can_review' => 1,
                'created_at' => $data['created_at'],
                'updated_at' => $data['updated_at']
            ];

            $this->db->insert('document_permissions', $permissionDataToInsert);

            return $documentId;

        } catch (Exception $e) {
            throw new Exception("Database error: " . $e->getMessage());
        }
    }
    public function createDocumentTemplate($data)
    {
        try {

            $dataDocumentsToInsert = [
                'title' => $data['title'],
                'file_name' => $data['file_name'],
                'doc_type' => $data['doc_type'],
                'file_key' => $data['file_key'],
                'owner_id' => $data['owner_id'],
                'created_at' => $data['created_at'],
                'updated_at' => $data['updated_at'],
            ];

            $this->db->insert('document_templates', $dataDocumentsToInsert);

            return $this->db->insert_id();
        } catch (Exception $e) {
            throw new Exception("Database error: " . $e->getMessage());
        }
    }

    public function getDocumentsTemplate()
    {
        try {

            $this->db->select('id ,file_name, title , doc_type')->from('document_templates');
            $query = $this->db->get();
            return $query->result_array();

        } catch (Exception $e) {
            throw new Exception("Database error: " . $e->getMessage());
        }
    }

    public function getDocumentTemplate($templateId) {
        try {
            $this->db->select('id ,file_name, title , doc_type')->from('document_templates')->where('id', $templateId);
            $query = $this->db->get();
            return $query->row_array();
        } catch (Exception $e) {
            throw new Exception("Database error: " . $e->getMessage());
        }
    }

    public function getDocs($userHash)
    {
        try {
            $this->db->select("
            d.id, 
            d.title,
            d.doc_type,
            d.file_name,
            d.file_key,
            d.created_at,
            d.updated_at,
            d_workspace_document.id AS workspace_id,
            d_workspace_document.title AS workspace_title,
            d_permissions.user_id AS user_id,
            d_permissions.can_edit AS edit,
            d_permissions.can_download AS download,
            d_permissions.can_review AS review
        ");
        $this->db->from('documents d');
        $this->db->join('document_workspaces d_workspace_document', 'd.document_workspace_id = d_workspace_document.id');
        $this->db->join('document_permissions d_permissions', 'd.id = d_permissions.doc_id');
        $this->db->where('d_permissions.user_id', $userHash);

            $query = $this->db->get();
            $results = $query->result_array();

            $documents = array_map(function ($row) {
                return [
                    'id' => $row['id'],
                    'title' => $row['title'],
                    'doc_type' => $row['doc_type'],
                    'file_name' => $row['file_name'],
                    'file_key' => $row['file_key'],
                    'user_id' => $row['user_id'],
                    'created_at' => $row['created_at'],
                    'updated_at' => $row['updated_at'],
                    'workspace' => [
                        'id' => $row['workspace_id'],
                        'title' => $row['workspace_title']
                    ],
                    'permissions' => [
                        'edit' => (bool) $row['edit'],
                        'download' => (bool) $row['download'],
                        'review' => (bool) $row['review']
                    ]
                ];
            }, $results);

            return $documents;


        } catch (Exception $e) {
            throw new Exception("Database error: " . $e->getMessage());
        }
    }

    public function getDocument($documentId, $userHash)
    {


        try {
            $this->db->select("
            d.id, 
            d.title,
            d.doc_type,
            d.file_name,
            d.file_key,
            d.created_at,
            d.updated_at,
            d_workspace_document.id AS workspace_id,
            d_workspace_document.title AS workspace_title,
            d_permissions.user_id AS user_id,
            d_permissions.can_edit AS edit,
            d_permissions.can_download AS download,
            d_permissions.can_review AS review
        ");
        $this->db->from('documents d');
        $this->db->join('document_workspaces d_workspace_document', 'd.document_workspace_id = d_workspace_document.id');
        $this->db->join('document_permissions d_permissions', 'd.id = d_permissions.doc_id');
        $this->db->where('d_permissions.user_id', $userHash);
        $this->db->where('d.id', $documentId);

            $query = $this->db->get();
            $results = $query->result_array();

            $documents = array_map(function ($row) {
                return [
                    'id' => $row['id'],
                    'title' => $row['title'],
                    'doc_type' => $row['doc_type'],
                    'file_name' => $row['file_name'],
                    'file_key' => $row['file_key'],
                    'user_id' => $row['user_id'],
                    'created_at' => $row['created_at'],
                    'updated_at' => $row['updated_at'],
                    'workspace' => [
                        'id' => $row['workspace_id'],
                        'title' => $row['workspace_title']
                    ],
                    'permissions' => [
                        'edit' => (bool) $row['edit'],
                        'download' => (bool) $row['download'],
                        'review' => (bool) $row['review']
                    ]
                ];
            }, $results);

            return $documents[0];


        } catch (Exception $e) {
            throw new Exception("Database error: " . $e->getMessage());
        }
    }
}
