<?php
class GlobalRebortModel extends CI_Model
{
    public function __construct()
    {
        parent::__construct();
        $this->load->database();
        $this->load->model('EmployeesWorkedHoursModel');
    }

    public function get_all()
    {
        $employeeName = $this->input->get('employeName') ?? '';
        $employeeNameCondition = !empty($employeeName) ? "AND system_users.name LIKE '%" . $this->db->escape_like_str($employeeName) . "%'" : '';

        $query = $this->db->query("
        SELECT
            TIME_FORMAT(rebort.time, '%h:%i %p') AS time,
            rebort.title, system_users.name, system_users.image_profile, rebort.user_hash, rebort.date,
            (SELECT name FROM system_group_name WHERE system_group_name.hash = system_users.user_type) AS jop,
            (SELECT name FROM partments WHERE hash = (SELECT partment FROM emp_information WHERE emp_information.user_hash = rebort.user_hash)) AS department,
            (SELECT Name FROM Positions WHERE Positions.ID = emp_information.position) AS position
        FROM (
            (
            SELECT
                fingerprint_time AS time,
                fingerprint_time AS date,
                CASE
                    WHEN in_out = 'in' THEN 'دخول'
                    WHEN in_out = 'out' THEN 'خروج'
                END AS title,
                user_hash
            FROM
                emp_fingerprint
            ) UNION
            (
            SELECT
                date_entry AS time,
                date_entry AS date,
                CASE
                    WHEN status = '111' THEN 'موافقة علي اجازة'
                    WHEN status = '222' THEN 'رفض اجازة'
                    WHEN status = '333' THEN 'تمرير اجازة'
                    WHEN status = '444' THEN 'طلب اجازة'
                END AS title,
                sender AS user_hash
            FROM
                vacations_agreement
            ) UNION
            (
            SELECT
                deadline AS time,
                DATE(deadline) AS date,
                'موعد التسليم' AS title,
                receiver AS user_hash
            FROM task
            )
        ) AS rebort
        JOIN system_users ON rebort.user_hash = system_users.hash
        LEFT JOIN emp_information ON rebort.user_hash = emp_information.user_hash
        WHERE
            DATE(rebort.date) = DATE(NOW())
            $employeeNameCondition
        ORDER BY
            rebort.time DESC
        ");

        $data = $query->result_array();
        $scheduleData = []; // Move this line outside the loop

        foreach ($data as $key => $value) {
            $user_hash = $value['user_hash'];
            $date = $value['date'];
            $title = $value['title'];
            list($datePart, $timePart) = explode(' ', $date);
            $schedule = $this->getCurrentSchedule($user_hash, $datePart, $timePart);

            if ($title == "دخول") {
                if ($schedule != null && !empty($schedule)) {
                    if (isset($schedule['data']) && $schedule['type'] == 'schedule') {
                        // Fixed schedule logic
                        $start = $schedule['data']['start'];
                        $end = $schedule['data']['end'];
                        $allowance = $schedule['data']['allowance'];

                        // Convert times to 12-hour format with AM/PM
                        $start = date('h:i A', strtotime($start));
                        $end = date('h:i A', strtotime($end));
                        $allowance = date('h:i A', strtotime($allowance));

                        $data[$key]['start'] = $start;
                        $data[$key]['end'] = $end;
                        $data[$key]['allowance'] = $allowance;

                        // Calculate late minutes
                        $fingerprint_time = strtotime($timePart);
                        $allowance_time = strtotime($allowance);
                        $late_minutes = 0;

                        if ($fingerprint_time > $allowance_time) {
                            $late_minutes = round(($fingerprint_time - $allowance_time) / 60);
                        }

                        $data[$key]['late_minutes'] = $late_minutes;
                        $scheduleData[$user_hash] = [
                            'start' => $start,
                            'end' => $end,
                            'allowance' => $allowance,
                            'late_minutes' => $late_minutes,
                        ];
                    } elseif (isset($schedule['data']) && $schedule['type'] == 'flexible') {
                        // Flexible schedule logic
                        $target_hours = $schedule['data']['target_hours'];
                        $worked_hours = $schedule['data']['worked_hours'];
                        $data[$key]['target_hours'] = $target_hours;
                        $data[$key]['worked_hours'] = $worked_hours;

                        // Store the target hours to use for the "خروج" logic later
                        $scheduleData[$user_hash] = [
                            'target_hours' => $target_hours,
                            'worked_hours' => $worked_hours,
                        ];
                    }
                }
            }
        }

        // Second pass: Handle all "خروج" records using stored schedule data
        foreach ($data as $key => $value) {
            $user_hash = $value['user_hash'];
            $date = $value['date'];
            $title = $value['title'];

            if ($title == "خروج") {
                // Check if we have 'in' data in today's schedule
                if (!isset($scheduleData[$user_hash]['start'])) {
                    // Try to retrieve 'in' data from the previous day
                    list($datePart, $timePart) = explode(' ', $date);
                    $previousIn = $this->getPreviousInTimestamp($user_hash, $datePart, $timePart);
                    $previousDate = $previousIn ? date('Y-m-d', strtotime($previousIn)) : null;
                    $worked_hours_global = 0;
                    $result = $this->EmployeesWorkedHoursModel->getWorkedHours($user_hash, date('Y-m-d', strtotime($date)));
                    if (($previousDate && isset($result) && $result['total_worked_hours'] == 0) || $result === null) {
                        $result = $this->EmployeesWorkedHoursModel->getWorkedHours($user_hash, $previousDate);
                    }

                    $worked_hours_global = $result['total_worked_hours'];
                    // var_dump($worked_hours_global);

                    // var_dump($previousIn);
                    if ($previousIn) {
                        // If a previous 'in' record is found, use it
                        $data[$key]['start'] = date('h:i A', strtotime($previousIn));
                        $fingerprint_time_in = strtotime($previousIn);
                        // Assuming that the same schedule applies (e.g., fixed or flexible)
                        // Retrieve the schedule based on the previous 'in' date
                        $schedule = $this->getCurrentSchedule($user_hash, date('Y-m-d', strtotime($previousIn)), date('H:i:s', strtotime($previousIn)));

                        if ($schedule && isset($schedule['data'])) {
                            if ($schedule['type'] == 'schedule') {
                                // Fixed schedule logic
                                $start = $schedule['data']['start'];
                                $end = $schedule['data']['end'];
                                $allowance = $schedule['data']['allowance'];

                                // Convert times to 12-hour format with AM/PM
                                $start = date('h:i A', strtotime($start));
                                $end = date('h:i A', strtotime($end));
                                $allowance = date('h:i A', strtotime($allowance));

                                // Assign retrieved schedule values
                                $data[$key]['start'] = $start;
                                $data[$key]['end'] = $end;
                                $data[$key]['allowance'] = $allowance;

                                $fingerprint_datetime = new DateTime($timePart);
                                $fingerprint_datetime->modify('+1 day');
                                $fingerprint_time = $fingerprint_datetime->format('Y-m-d H:i');
                                // $end_time = date(format: 'Y-m-d H:i', strtotime($end));
                                $end_datetime = new DateTime($end);

                                if ($fingerprint_datetime < $end_datetime) {
                                    // Calculate the difference in minutes
                                    $interval = $fingerprint_datetime->diff($end_datetime);
                                    $early_departure_minutes = $interval->h * 60 + $interval->i; // Convert hours to minutes and add minutes
                                }

                                if ($fingerprint_datetime > $end_datetime) {
                                    $interval = $fingerprint_datetime->diff($end_datetime);
                                    $late_minutes = $interval->h * 60 + $interval->i;
                                }
                                $late_minutes = 0;

                                $early_departure_minutes = 0;

                                // Set the result
                                $data[$key]['early_departure_minutes'] = $early_departure_minutes;
                                $data[$key]['late_minutes'] = $late_minutes;

                            } elseif ($schedule['type'] == 'flexible') {
                                // Flexible schedule logic
                                $target_hours = $schedule['data']['target_hours'];
                                $worked_hours = $schedule['data']['worked_hours'];

                                // Set flexible schedule fields
                                $data[$key]['target_hours'] = $target_hours;

                                if ($worked_hours_global > 0) {
                                    $data[$key]['worked_hours'] = $worked_hours_global;
                                } else {
                                    $data[$key]['worked_hours'] = $worked_hours;
                                }

                                // No need for early departure or late logic in flexible schedules
                                $data[$key]['early_departure_minutes'] = null;
                                $data[$key]['late_minutes'] = null;
                            }
                        } else {
                            // No valid schedule found for previous 'in'
                            $data[$key]['start'] = date('h:i A', strtotime($previousIn));
                            $data[$key]['end'] = null;
                            $data[$key]['allowance'] = null;
                            $data[$key]['late_minutes'] = null;
                            $data[$key]['early_departure_minutes'] = null;
                        }
                    } else {
                        // No 'in' record found, set default values
                        $data[$key]['start'] = null;
                        $data[$key]['end'] = null;
                        $data[$key]['allowance'] = null;
                        $data[$key]['late_minutes'] = null;
                        $data[$key]['early_departure_minutes'] = null;
                    }
                } else {
                    // Normal processing if 'in' data is available in today's schedule
                    $data[$key]['start'] = $scheduleData[$user_hash]['start'];
                    $data[$key]['end'] = $scheduleData[$user_hash]['end'];
                    $data[$key]['allowance'] = $scheduleData[$user_hash]['allowance'];
                    $data[$key]['late_minutes'] = $scheduleData[$user_hash]['late_minutes'];

                    // Calculate early departure minutes for خروج
                    list($datePart, $timePart) = explode(' ', $date);
                    $fingerprint_time = strtotime($timePart); // Out time
                    $end_time = strtotime($scheduleData[$user_hash]['end']);
                    $early_departure_minutes = 0;

                    if ($fingerprint_time < $end_time) {
                        $early_departure_minutes = round(($end_time - $fingerprint_time) / 60);
                    }

                    $data[$key]['early_departure_minutes'] = $early_departure_minutes;
                }
            }
        }
        return $data;

    }

    private function getPreviousInTimestamp($user_hash, $datePart, $timePart)
    {
        // Query to get the previous "in" record
        $query = "SELECT fingerprint_time FROM emp_fingerprint
                  WHERE user_hash = ? AND in_out = 'in' AND DATE(fingerprint_time) < ?
                  ORDER BY fingerprint_time DESC LIMIT 1";
        $result = $this->db->query($query, [$user_hash, "$datePart"])->row_array();

        return $result ? $result['fingerprint_time'] : null;
    }

    private function getTodayInTimestamp($user_hash, $datePart)
    {
        // Query to get today's "in" record
        $query = "SELECT fingerprint_time FROM emp_fingerprint
              WHERE user_hash = ? AND in_out = 'in' AND DATE(fingerprint_time) = ?
              ORDER BY fingerprint_time ASC LIMIT 1";
        $result = $this->db->query($query, [$user_hash, $datePart])->row_array();

        return $result ? $result['fingerprint_time'] : null;
    }

    private function getCurrentSchedule($user_hash, $date, $fingerprint_time)
    {

        $isFlexible = $this->isFlexible($user_hash);

        // Query the database to get the schedule data for the given user and date
        $this->db->select('schedule_data, allowance_time')
            ->from('actual_schedules')
            ->where('user_hash', $user_hash)
            ->where('date', $date);
        $data = $this->db->get()->row();

        if ($data && !empty($data->schedule_data)) {
            // Convert fingerprint time to Unix timestamp
            $fingerprint_time = strtotime($fingerprint_time);
            $allowance_minutes = intval($data->allowance_time);
            // Decode schedule data
            $schedule_array = json_decode($data->schedule_data, true);
            if (empty($schedule_array) || (is_array($schedule_array) && count(array_filter($schedule_array, 'count')) === 0)) {
                return null;
            }

            // Iterate over the schedule data to find a matching schedule
            foreach ($schedule_array as $schedule) {
                // Handle hour-only format by appending ':00' for minutes
                $scheduleStart = strtotime($this->convertDecimalToTime($schedule['start']));
                $scheduleEnd = strtotime($this->convertDecimalToTime($schedule['end']));

                // Allow check-in up to 1 hour before the schedule start
                $allowedEarlyCheckIn = strtotime('-1 hour', $scheduleStart);

                if ($fingerprint_time <= $scheduleEnd) {
                    $allowance_time = strtotime("+{$allowance_minutes} minutes", $scheduleStart);

                    return [
                        'status' => 'success',
                        'type' => 'schedule',
                        'data' => [
                            'start' => date('H:i', $scheduleStart),
                            'end' => date('H:i', $scheduleEnd),
                            'allowance' => date('H:i', $allowance_time),

                        ],
                    ];
                }
            }

            return null;
            // No matching schedule found, construct a detailed message
            $startTimes = array_map(fn($s) => strtotime($this->convertDecimalToTime($s['start'])), $schedule_array);
            $endTimes = array_map(fn($s) => strtotime($this->convertDecimalToTime($s['end'])), $schedule_array);

            $earliestStart = min($startTimes);
            $latestEnd = max($endTimes);

            $startTime = date('h:i A', strtotime('-1 hour', $earliestStart));
            $endTime = date('h:i A', $latestEnd);

            return [
                'status' => 'error',
                'message' => "عفوا، لا توجد فترة في جدولك تتطابق مع الوقت الحالي. يجب تسجيل بصمة دخولك بين الساعة $startTime والساعة $endTime حسب جدولك اليومي. يرجى مراجعة جدولك وضمان تسجيل بصمة الدخول ضمن الأوقات المقررة.",
            ];
        }
        if ($isFlexible) {

            $this->db->select('target_hours')
                ->from('daily_flexible_schedule_targets')
                ->where('user_hash', $user_hash)
                ->where('date', $date);
            $data = $this->db->get()->row();

            if ($data && !empty($data)) {
                $targetHours = $data->target_hours;
                return [
                    'status' => 'success',
                    'type' => 'flexible',
                    'data' => [
                        'target_hours' => $targetHours,
                        'worked_hours' => $this->getWorkedHours($user_hash) ?? 0,
                    ],
                ];
            }
            return [
                'status' => 'error',
                'message' => 'لم يتم العثور على بيانات الجدول لهذا المستخدم في التاريخ المحدد.',
            ];
        }
        return [
            'status' => 'error',
            'message' => 'لم يتم العثور على بيانات الجدول لهذا المستخدم في التاريخ المحدد.',
        ];
    }
    private function getWorkedHours($user_hash)
    {
        // Initialize total worked hours in minutes
        $totalWorkedMinutes = 0;

        // Query to get 'in' and 'out' times for the current day and the previous day
        $this->db->select('fingerprint_time, in_out');
        $this->db->from('emp_fingerprint');
        $this->db->where('user_hash', $user_hash);
        $this->db->where('(DATE(fingerprint_time) = CURDATE() OR DATE(fingerprint_time) = DATE_SUB(CURDATE(), INTERVAL 4 HOUR))');
        $this->db->order_by('fingerprint_time', 'ASC');
        $query = $this->db->get();

        $fingerprints = $query->result_array();

        $inTime = null;

        // Loop through the fingerprint records
        foreach ($fingerprints as $fingerprint) {
            $fingerprintTime = strtotime($fingerprint['fingerprint_time']);
            if ($fingerprint['in_out'] == 'in') {
                // Set the 'in' time
                $inTime = $fingerprintTime;
            } elseif ($fingerprint['in_out'] == 'out' && $inTime !== null) {
                // Handle crossing midnight
                if (date('Y-m-d', $inTime) != date('Y-m-d', $fingerprintTime)) {
                    // Calculate worked minutes before midnight
                    $midnight = strtotime('tomorrow 00:00:00', $inTime);
                    $workedBeforeMidnight = round(($midnight - $inTime) / 60);
                    $totalWorkedMinutes += $workedBeforeMidnight;

                    // Calculate worked minutes after midnight
                    $workedAfterMidnight = round(($fingerprintTime - $midnight) / 60);
                    $totalWorkedMinutes += $workedAfterMidnight;
                } else {
                    // Calculate the duration between 'in' and 'out'
                    $workedMinutes = round(($fingerprintTime - $inTime) / 60); // Convert seconds to minutes
                    $totalWorkedMinutes += $workedMinutes;
                }

                $inTime = null; // Reset 'in' time after each 'out'
            }
        }

        // If there's an 'in' time without a corresponding 'out', calculate the time until now
        if ($inTime !== null) {
            $now = time(); // Current time
            if (date('Y-m-d', $inTime) != date('Y-m-d', $now)) {
                // Handle crossing midnight
                $midnight = strtotime('tomorrow 00:00:00', $inTime);
                $workedBeforeMidnight = round(($midnight - $inTime) / 60);
                $totalWorkedMinutes += $workedBeforeMidnight;

                $workedAfterMidnight = round(($now - $midnight) / 60);
                $totalWorkedMinutes += $workedAfterMidnight;
            } else {
                $workedMinutes = round(($now - $inTime) / 60); // Convert seconds to minutes
                $totalWorkedMinutes += $workedMinutes;
            }
        }

        // Convert total worked minutes into hours and minutes
        $workedHours = floor($totalWorkedMinutes / 60);
        $workedMinutes = $totalWorkedMinutes % 60;

        return sprintf("%02d:%02d", $workedHours, $workedMinutes) ?? '00:00';
    }

    // Example helper function to fetch target hours from emp_information table
    private function getTargetHours($user_hash)
    {
        // Load the database in CodeIgniter

        $this->db->select('target_hours');
        $this->db->from('emp_information');
        $this->db->where('user_hash', $user_hash);
        $query = $this->db->get();

        $result = $query->row_array();

        return isset($result['target_hours']) ? $result['target_hours'] : '00:00'; // Default to '00:00' if not found
    }

    private function isFlexible($user_hash)
    {
        $isFlexible = $this->db->select('flexible_schedule')
            ->from('emp_information')
            ->where('user_hash', $user_hash)
            ->get()
            ->row()
            ->flexible_schedule;

        return (bool) $isFlexible;
    }
    private function convertDecimalToTime($decimalTime)
    {
        $hours = floor($decimalTime);
        $minutes = floor(($decimalTime - $hours) * 60);
        $seconds = round((($decimalTime - $hours) * 60 - $minutes) * 60);
        return sprintf('%02d:%02d:%02d', $hours, $minutes, $seconds);
    }

    // public function chartPie()
    // {
    //     $query = $this->db->query("
    //         SELECT
    //             status, COUNT(*) AS count
    //         FROM
    //             emp_today_status
    //         group by status
    //     ");
    //     return $query->result_array();
    // }

    public function chartPie()
    {
        $this->db->select("
        MAX(CASE WHEN v.date_start IS NOT NULL AND v.date_start = CURDATE() THEN 1 ELSE 0 END) AS المجازون,
        MAX(CASE WHEN v.date_start IS NULL AND (asched.user_hash IS NOT NULL OR dft.user_hash IS NOT NULL) AND (ef.user_hash IS NULL OR ef.fingerprint_time IS NULL) THEN 1 ELSE 0 END) AS لم_يحضر_حتى_الان,
        MAX(CASE
            WHEN v.date_start IS NULL
            AND ef.user_hash IS NOT NULL AND ef.fingerprint_time IS NOT NULL
            AND EXISTS (
                SELECT 1
                FROM emp_fingerprint ef_sub
                WHERE ef_sub.user_hash = su.hash
                AND ef_sub.in_out = 'in'
                AND DATE(ef_sub.fingerprint_time) = CURDATE()
            )
            THEN 1
            ELSE 0
        END) AS الحضور
        ", false);

        $this->db->select('su.name');
        $this->db->from('system_users su');
        $this->db->join('emp_information ei', 'su.hash = ei.user_hash AND ei.is_work = 1', 'inner');
        $this->db->join('emp_fingerprint ef', 'ef.user_hash = su.hash AND DATE(ef.fingerprint_time) = CURDATE()', 'left');
        $this->db->join('vacations v', 'v.user_hash = su.hash AND (v.date_start <= CURDATE() AND v.date_end > CURDATE() OR     (v.date_start = CURDATE() AND v.date_end = CURDATE())) AND v.status = 111', 'left');
        $this->db->join('actual_schedules asched', 'asched.user_hash = su.hash AND asched.date = CURDATE()', 'left');
        $this->db->join('daily_flexible_schedule_targets dft', 'dft.user_hash = su.hash AND dft.date = CURDATE()', 'left');

        $this->db->group_by('su.hash');

        $query = $this->db->get();
        $result = $query->result_array();

        $data = [
            ['status' => 'المجازون', 'count' => array_sum(array_column($result, 'المجازون'))],
            ['status' => 'لم يحضر حتى الان', 'count' => array_sum(array_column($result, 'لم_يحضر_حتى_الان'))],
            ['status' => 'الحضور', 'count' => array_sum(array_column($result, 'الحضور'))],
        ];

        return $data;
    }
    public function getEmployeesByStatus($status)
    {
        // Sanitize the status to avoid SQL injection
        $status = $this->db->escape_str($status);

        // Build the query
        $this->db->select('su.name, su.hash, MAX(ef.fingerprint_time) as fingerprint_time, v.date_start, v.date_end');

        $this->db->from('system_users su');
        $this->db->join('emp_information ei', 'su.hash = ei.user_hash AND ei.is_work = 1', 'inner');
        $this->db->join('emp_fingerprint ef', 'ef.user_hash = su.hash AND DATE(ef.fingerprint_time) = CURDATE()', 'left');
        $this->db->join('vacations v', 'v.user_hash = su.hash AND (v.date_start <= CURDATE() AND v.date_end > CURDATE() OR     (v.date_start = CURDATE() AND v.date_end = CURDATE())) AND v.status = "111"', 'left');
        $this->db->join('actual_schedules asched', 'asched.user_hash = su.hash AND asched.date = CURDATE()', 'left');
        $this->db->join('daily_flexible_schedule_targets dft', 'dft.user_hash = su.hash AND dft.date = CURDATE()', 'left');
        $this->db->group_by('su.name, su.hash, v.date_start, v.date_end');

        // Apply the condition based on status
        if ($status === 'المجازون') {
            $this->db->where('v.date_start IS NOT NULL AND v.date_start = CURDATE() AND v.status = 111');
        } elseif ($status === 'لم يحضر حتى الان') {
            $this->db->where('v.date_start IS NULL');
            $this->db->where('(ef.user_hash IS NULL OR (ef.user_hash IS NOT NULL AND ef.fingerprint_time IS NULL))');
            $this->db->where('(asched.user_hash IS NOT NULL OR dft.user_hash IS NOT NULL)');
        } elseif ($status === 'الحضور') {
            $this->db->where('v.date_start IS NULL');
            $this->db->where('ef.user_hash IS NOT NULL AND ef.fingerprint_time IS NOT NULL');
            $this->db->where('EXISTS (SELECT 1 FROM emp_fingerprint ef_sub WHERE ef_sub.user_hash = su.hash AND ef_sub.in_out = "in" AND DATE(ef_sub.fingerprint_time) = CURDATE())');
            // $this->db->where('(asched.user_hash IS NOT NULL OR dft.user_hash IS NOT NULL)');
        }

        // Execute query
        $query = $this->db->get();

        // Fetch results
        $results = $query->result_array();

        return $results;
    }
    // private function isEmployeeHasVacation($user_hash, $date)
    // {
    //     $query = $this->db->select('id')->from('vacations')->where('user_hash', $user_hash)->where('date_start <=', $date)->where('date_end >=', $date)->get();
    //     if ($query->num_rows() > 0) {
    //         return true;
    //     }
    //     return false;
    // }
}
