<?php
class GroupScheduleModel extends CI_Model
{

    public function __construct()
    {
        parent::__construct();
        $this->load->database();
    }

    public function getGroupScheduleCount()
    {
        $this->db->from('schedule_groups');
        return $this->db->count_all_results();
    }

    public function getGroupSchedulyTable()
    {
        $page = $this->input->get('page') ?? 1;
        $perPage = $this->input->get('perPage') ?? 10;
        $offset = ($page - 1) * $perPage;
        $search = $this->input->get('q') ?? '';
        $this->db->select('id, name , allowance_time');
        $this->db->from('schedule_groups');
        $this->db->limit($perPage, $offset);
        if (!empty($search)) {
            $this->db->like('name', $search);
        }
        return $this->db->get()->result_array();
    }
    public function create_group($data)
    {
        $this->db->insert('schedule_groups', $data);
        return $this->db->insert_id();
    }

    public function get_all_groups()
    {
        return $this->db->get('schedule_groups')->result_array();
    }

    public function get_group_by_id($id)
    {
        return $this->db->select('id, name, allowance_time')->where('id', $id)->from('schedule_groups')->get()->row();
    }

    public function getGroupDataById($id)
    {
        return $this->db->select('schedule_data, name, allowance_time')
            ->where('id', $id)
            ->from('schedule_groups')
            ->get()
            ->row();
    }

    public function addScheduleToGroup($id, $data)
    {
        $this->db->where('id', $id);
        $this->db->update('schedule_groups', $data);
        return $this->db->affected_rows();
    }

    public function getScheduleData($user_hash, $date)
    {
        $dayname = date('l', strtotime($date)); // Get the day name from the date

        $sql = 'SELECT ei.user_hash, ? AS date,
                       JSON_UNQUOTE(JSON_EXTRACT(sg.schedule_data, CONCAT(\'$.\', ?))) AS schedule_data,
                       sg.allowance_time
                FROM emp_information ei
                JOIN schedule_groups sg ON ei.work_schedule = sg.id
                WHERE JSON_UNQUOTE(JSON_EXTRACT(sg.schedule_data, CONCAT(\'$.\', ?))) IS NOT NULL
                AND ei.user_hash = ?';

        // Binding parameters
        $query = $this->db->query($sql, [$date, $dayname, $dayname, $user_hash]);

        return $query->row_array();
    }

    public function getScheduleName($user_hash)
    {
        $this->db->select('sg.name');
        $this->db->from('emp_information ei');
        $this->db->join('schedule_groups sg', 'ei.work_schedule = sg.id');
        $this->db->where('ei.user_hash', $user_hash);

        $query = $this->db->get();
        return $query->row_array();
    }

    public function getDailyFlexibleData($user_hash, $date)
    {
        $this->db->select('target_hours, date')->from('daily_flexible_schedule_targets')->where('user_hash', $user_hash)->where('date', $date);

        return $this->db->get()->row_array();
    }

    // this can get actual or daily_fleixble
    public function getScheudleToday($user_hash)
    {
        $today = date('Y-m-d');
        $result = $this->getActualSchedule($user_hash, $today);
        if ($result !== null) {
            return $result['schedule_data'];
        }

        $result = $this->getDailyFlexibleData($user_hash, $today);
        return $result;
    }

    public function updateActualSchedules()
    {
        $this->db->query("
        INSERT INTO actual_schedules (user_hash, date, schedule_data, allowance_time)
        SELECT
            ei.user_hash,
            CURDATE() AS date,
            IFNULL(
                JSON_UNQUOTE(JSON_EXTRACT(sg.schedule_data, CONCAT('$.', DAYNAME(CURDATE())))),
                '[{}]'
            ) AS schedule_data,
            sg.allowance_time
        FROM
            emp_information ei
        JOIN schedule_groups sg ON ei.work_schedule = sg.id
        LEFT JOIN user_vacations uv ON ei.user_hash = uv.user_hash
            AND uv.day_of_week_id = DAYOFWEEK(CURDATE())
        LEFT JOIN vacations v ON ei.user_hash = v.user_hash
            AND (
                CURDATE() BETWEEN v.date_start AND DATE(v.date_end - INTERVAL 1 DAY)
                OR (v.date_start = CURDATE() AND v.date_end = CURDATE())
            )
            AND v.status = '111'
        LEFT JOIN daily_flexible_schedule_targets dfts ON ei.user_hash = dfts.user_hash
            AND CURDATE() = dfts.date
         WHERE
                uv.user_hash IS NULL
               AND v.user_hash IS NULL
               AND ei.flexible_schedule = '0'
               AND JSON_UNQUOTE(JSON_EXTRACT(sg.schedule_data, CONCAT('$.', DAYNAME(CURDATE())))) IS NOT NULL
                     AND NOT EXISTS (
      SELECT 1
      FROM daily_flexible_schedule_targets
      WHERE user_hash = ei.user_hash
      AND date = CURDATE()
  )
        ON DUPLICATE KEY UPDATE
            schedule_data = VALUES(schedule_data),
            allowance_time = VALUES(allowance_time),
            updated_at = CURRENT_TIMESTAMP
    ");

        $this->db->query("
        DELETE FROM actual_schedules
        WHERE schedule_data IS NULL OR schedule_data LIKE '[{}]'
    ");
    }

    public function updateEmployeeActualSchedule($user_hash)
    {
        $this->db->query("
            INSERT INTO actual_schedules (user_hash, date, schedule_data, allowance_time)
            SELECT
                ei.user_hash,
                CURDATE() AS date,
                IFNULL(
                    JSON_UNQUOTE(JSON_EXTRACT(sg.schedule_data, CONCAT('$.', DAYNAME(CURDATE())))),
                    '[{}]'
                ) AS schedule_data,
                sg.allowance_time
            FROM
                emp_information ei
            JOIN schedule_groups sg ON ei.work_schedule = sg.id
            LEFT JOIN user_vacations uv ON ei.user_hash = uv.user_hash
                AND uv.day_of_week_id = DAYOFWEEK(CURDATE())
            LEFT JOIN vacations v ON ei.user_hash = v.user_hash
                AND (
                    CURDATE() BETWEEN v.date_start AND DATE(v.date_end - INTERVAL 1 DAY)
                    OR (v.date_start = CURDATE() AND v.date_end = CURDATE())
                )
                AND v.status = '111'
            LEFT JOIN daily_flexible_schedule_targets dfts ON ei.user_hash = dfts.user_hash
                AND CURDATE() = dfts.date
            WHERE
                ei.user_hash = '$user_hash'
                AND uv.user_hash IS NULL
                AND v.user_hash IS NULL
                AND ei.flexible_schedule = '0'
                AND JSON_UNQUOTE(JSON_EXTRACT(sg.schedule_data, CONCAT('$.', DAYNAME(CURDATE())))) IS NOT NULL
                AND NOT EXISTS (
                    SELECT 1
                    FROM daily_flexible_schedule_targets
                    WHERE user_hash = ei.user_hash
                    AND date = CURDATE()
                )
            ON DUPLICATE KEY UPDATE
                schedule_data = VALUES(schedule_data),
                allowance_time = VALUES(allowance_time),
                updated_at = CURRENT_TIMESTAMP
        ");

        $this->db->query("
            DELETE FROM actual_schedules
            WHERE schedule_data IS NULL OR schedule_data LIKE '[{}]'
        ");
    }

    public function deleteActualSchedule($user_hash, $date)
    {
        $this->db->where('user_hash', $user_hash);
        $this->db->where('date', $date);
        return $this->db->delete('actual_schedules');
    }
    public function deleteFlexibleDailySchedule($user_hash, $date)
    {
        $this->db->update('daily_flexible_schedule_targets', array('is_deleted' => 1), array('user_hash' => $user_hash, 'date' => $date));
    }
    public function update_group($id, $data)
    {
        $this->db->where('id', $id);
        $this->db->update('schedule_groups', $data);
        return $this->db->affected_rows();
    }

    public function updateActualSchedule($user_hash, $date, $new_schedules)
    {
        // Convert new schedules to JSON format correctly
        $new_schedules_json = json_encode($new_schedules);

        // Prepare the data array
        $data = [
            'schedule_data' => $new_schedules_json,
        ];

        // Set the where condition
        $this->db->where('user_hash', $user_hash);
        $this->db->where('date', $date);

        // Update the data in the actual_schedules table
        return $this->db->update('actual_schedules', $data);
    }

    public function getEmploeedWorkedData($user_hash, $date)
    {
        $this->db->select("JSON_OBJECT(
            'user_hash', ei.user_hash,
            'schedule_for_day', asch.schedule_data,
            'allowance_time', asch.allowance_time,
            'in_fingerprint_time', CONCAT('[', GROUP_CONCAT(CASE WHEN ef.in_out = 'in' THEN CONCAT('\"', DATE_FORMAT(ef.fingerprint_time, '%Y-%m-%d %H:%i:%s'), '\"') END ORDER BY ef.fingerprint_time SEPARATOR ','), ']'),
            'out_fingerprint_time', CONCAT('[', GROUP_CONCAT(CASE WHEN ef.in_out = 'out' THEN CONCAT('\"', DATE_FORMAT(ef.fingerprint_time, '%Y-%m-%d %H:%i:%s'), '\"') END ORDER BY ef.fingerprint_time DESC SEPARATOR ','), ']'),
            'date', asch.date
        ) AS employee_data", false)
            ->from('emp_information ei')
            ->join('actual_schedules asch', 'ei.user_hash = asch.user_hash AND asch.date = ' . $this->db->escape($date))
            ->join('emp_fingerprint ef', 'ei.user_hash = ef.user_hash AND DATE(ef.fingerprint_time) = ' . $this->db->escape($date) . ' AND ef.is_flexible = \'0\'', 'left')
            ->where('asch.schedule_data IS NOT NULL')
            ->where('TRIM(asch.schedule_data) <>', '')
            ->where('ei.user_hash', $user_hash)
            ->group_by(['ei.user_hash', 'asch.date']);

        // Execute the query
        $query = $this->db->get();

        // Return the result
        return $query->row();
    }
    public function insertActualSchedule($user_hash, $date, $schedule_data)
    {
        // Convert schedule data to JSON format correctly
        $schedule_data_json = json_encode(json_decode($schedule_data, true));

        // Prepare the data array
        $data = [
            'user_hash' => $user_hash,
            'date' => $date,
            'schedule_data' => $schedule_data_json,
        ];

        // Insert the data into the actual_schedules table
        return $this->db->insert('actual_schedules', $data);
    }
    public function getActualSchedule($user_hash, $date)
    {
        $this->db->select('*')->from('actual_schedules')->where('user_hash', $user_hash)->where('date', $date);
        return $this->db->get()->row_array();
    }

    public function delete_group($id)
    {
        $this->db->where('id', $id);
        $this->db->delete('schedule_groups');
        return $this->db->affected_rows();
    }

    public function getworkSchedulesOptions()
    {
        return $this->db->select('id as value , name as label')->from('schedule_groups')->get()->result_array();
    }

    // functino see the vaction time and update the schedule or insert based on this time vaction

    // first we need to make a privtate function to get the vaction time that accepted for user
    // then we need to make a public function that get the vacation time and update the schedule or insert based on this time vaction

    public function insertOrUpdateActualScheduleIfNeeded($user_hash, $date)
    {
        return $this->updateActualSchedulesForUser($user_hash, $date);
    }

    public function insertOrUpdateDailyFleixbleScheduleIfNeeded($user_hash, $date)
    {

        return $this->updateDailyFleixbleSchedulesForUser($user_hash, $date);

    }

    private function updateDailyFleixbleSchedulesForUser($user_hash, $date)
    {
        $formatted_date = date('Y-m-d', strtotime($date));

        $result = $this->db->query("
            INSERT INTO daily_flexible_schedule_targets (user_hash, date, target_hours)
            SELECT ei.user_hash, '$formatted_date' AS date, ei.target_hours
            FROM emp_information ei
            LEFT JOIN user_vacations uv ON ei.user_hash = uv.user_hash
                                        AND uv.day_of_week_id = DAYOFWEEK('$formatted_date')
            LEFT JOIN vacations v ON ei.user_hash = v.user_hash
                                   AND (
                                       '$formatted_date' BETWEEN v.date_start AND DATE(v.date_end - INTERVAL 1 DAY)
                                       OR (v.date_start = '$formatted_date' AND v.date_end = '$formatted_date')
                                   )
                                   AND v.status = '111'
            LEFT JOIN holiday h ON '$formatted_date' = h.date
            WHERE
                ei.user_hash = '$user_hash'
                AND uv.user_hash IS NULL
                AND v.user_hash IS NULL
                AND ei.flexible_schedule = '1'
                AND ei.target_hours IS NOT NULL
                AND h.date IS NULL
                AND NOT EXISTS (
                    SELECT 1
                    FROM actual_schedules
                    WHERE user_hash = ei.user_hash
                    AND date = '$formatted_date'
                )
                AND NOT EXISTS (
                    SELECT 1
                    FROM daily_flexible_schedule_targets
                    WHERE user_hash = ei.user_hash
                    AND date = '$formatted_date'
                )
        ");

        return $result ? true : false;
    }
    private function getVacationTime($user_hash, $date)
    {
        $this->db->select("DATE_FORMAT(date_start, '%H:%i:%s') as date_start_time, DATE_FORMAT(date_end, '%H:%i:%s') as date_end_time")
            ->from('vacations')
            ->where('status', '111')
            ->where('user_hash', $user_hash)
            ->where_in('vication_type_hash', ['333', '444'])
            ->where('DATE(date_start)', $date)
            ->where('DATE(date_end)', $date);

        $query = $this->db->get();
        return $query->row();
    }

    private function updateActualSchedulesForUser($user_hash, $date)
    {
        // Ensure the date is in the correct format
        $formatted_date = date('Y-m-d', strtotime($date));

        // Insert or update the actual schedules
        $result = $this->db->query("
        INSERT INTO actual_schedules (user_hash, date, schedule_data, allowance_time)
        SELECT
            ei.user_hash,
            '$formatted_date' AS date,
            IFNULL(
                JSON_UNQUOTE(JSON_EXTRACT(sg.schedule_data, CONCAT('$.', DAYNAME('$formatted_date')))),
                '[{}]'
            ) AS schedule_data,
            sg.allowance_time
        FROM
            emp_information ei
        JOIN schedule_groups sg ON ei.work_schedule = sg.id
        LEFT JOIN user_vacations uv ON ei.user_hash = uv.user_hash
            AND uv.day_of_week_id = DAYOFWEEK('$formatted_date')
        LEFT JOIN vacations v ON ei.user_hash = v.user_hash
            AND (
                '$formatted_date' BETWEEN v.date_start AND DATE(v.date_end - INTERVAL 1 DAY)
                OR (v.date_start = '$formatted_date' AND v.date_end = '$formatted_date')
            )
            AND v.status = '111'
        LEFT JOIN daily_flexible_schedule_targets dfts ON ei.user_hash = dfts.user_hash
            AND '$formatted_date' = dfts.date
        WHERE
            ei.user_hash = '$user_hash'
            AND uv.user_hash IS NULL
            AND v.user_hash IS NULL
            AND ei.flexible_schedule = '0'
            AND JSON_UNQUOTE(JSON_EXTRACT(sg.schedule_data, CONCAT('$.', DAYNAME('$formatted_date')))) IS NOT NULL
            AND NOT EXISTS (
                SELECT 1
                FROM daily_flexible_schedule_targets
                WHERE user_hash = ei.user_hash
                AND date = '$formatted_date'
            )
           AND NOT EXISTS (
                SELECT 1
                FROM actual_schedules
                WHERE user_hash = ei.user_hash
                AND date = '$formatted_date'
           )
        ON DUPLICATE KEY UPDATE
            schedule_data = VALUES(schedule_data),
            allowance_time = VALUES(allowance_time),
            updated_at = CURRENT_TIMESTAMP
    ");

        return $result ? true : false;
    }

    private function decimalToTime($decimal)
    {
        $hours = floor($decimal);
        $minutes = ($decimal - $hours) * 60;
        return sprintf('%02d:%02d', $hours, $minutes);
    }

    private function timeToDecimal($datetime)
    {
        $hours = $datetime->format('H');
        $minutes = $datetime->format('i');
        return $hours + ($minutes / 60);
    }
}
