<?php
class MovementsModel extends CI_Model
{
    public function __construct()
    {
        parent::__construct();
        $this->load->database();
        $this->load->library('Middleware');
    }


    public function get($id)
    {
        $this->db->select('it.item_name, it.item_id, iit.id, iit.name');
        $this->db->from('inventory_items AS it');
        $this->db->where('it.item_id', $id);
        $this->db->join('inventory_items_types AS iit', 'it.item_type = iit.id');
        $query = $this->db->get();
        return $query->result_array();
    }


    public function getMovements()
    {
        $query = $this->db->query("
        WITH RankedEmployeeMovements AS (
    SELECT *, 
           ROW_NUMBER() OVER(PARTITION BY user_hash ORDER BY id DESC) AS rn,
           DATE_FORMAT(timestamp, '%Y-%m-%d %r')
    FROM employee_movements
)
SELECT *
FROM RankedEmployeeMovements
WHERE rn = 1;
    ");

        return $query->result();
    }

    public function getMovementReport($data)
    {
        $startDate = $data['start_date'] ?? '';
        $endDate = $data['end_date'] ?? '';
        $userHash = $data['user_hash'] ?? '';
        $page = $data['page'] ?? 1;
        $perPage = $data['per_page'] ?? 10;
        $offset = ($page - 1) * $perPage;
        $fingerPrintValue = $data['finger_print'] ?? '';
        $name = $data['name'] ?? '';

        $startDate = $data['start_date'] ?? '';
        $endDate = $data['end_date'] ?? '';
        $userHash = $data['user_hash'] ?? '';
        $page = $data['page'] ?? 1;
        $perPage = $data['per_page'] ?? 10;
        $offset = ($page - 1) * $perPage;
        $fingerPrintValue = $data['finger_print'] ?? '';
        $name = $data['name'] ?? '';

        $this->db->select('em.user_hash, em.lat, em.lng, DATE_FORMAT(em.timestamp, "%Y-%m-%d %r") as timestamp, su.image_profile as imageProfile, su.name');
        $this->db->from('employee_movements em');
        $this->db->join('system_users su', 'su.hash = em.user_hash');

        if (!empty($startDate)) {
            $this->db->where('DATE(em.timestamp) >=', $startDate);
        }
        if (!empty($endDate)) {
            $this->db->where('DATE(em.timestamp) <=', $endDate);
        }

        if ($userHash) {
            $this->db->where('user_hash', $userHash);
        }
        if ($fingerPrintValue) {
            $this->db->where('finger_print', $fingerPrintValue);
        }
        if ($name) {
            $this->db->like('su.name', $name);
        }
        // Subquery to filter by minimum time difference
        $subquery = "(SELECT em.user_hash, em.timestamp
  FROM employee_movements em
  WHERE em.user_hash = {$this->db->escape($userHash)}
    AND DATE(em.timestamp) BETWEEN {$this->db->escape($startDate)} AND {$this->db->escape($endDate)}
  ORDER BY em.timestamp ASC)";

        $this->db->where("NOT EXISTS (
         SELECT 1 FROM ($subquery) AS next_filtered
         WHERE next_filtered.timestamp > em.timestamp
           AND TIMESTAMPDIFF(MINUTE, em.timestamp, next_filtered.timestamp) < 10
     )");


        $this->db->limit($perPage, $offset);

        $query = $this->db->get();
        $results = $query->result_array();

        return $results;



    }

    private function isInEgypt($lat, $lng)
    {
        return ($lat >= 22.0 && $lat <= 31.5) && ($lng >= 25.0 && $lng <= 35.0);
    }
    public function insertMovement($data)
    {
        $currentTime = new DateTime();
        $currentMinute = (int) $currentTime->format('i'); // Get current minute

        // Check if the current minute is a multiple of 5
        if ($currentMinute % 5 !== 0) {
            return false; // Not allowed to insert
        }

        if ($this->isInEgypt($data['lat'], $data['lng'])) {
            return false;
        }

        $this->db->insert('employee_movements', $data);
        return $this->db->insert_id();
    }

    public function updateMovement($data, $id)
    {
        $this->db->where('item_id', $id);
        $this->db->update('inventory_items', $data);
        return $this->db->affected_rows();
    }

    public function getEmployeeMovementsConfig() {
        $query = $this->db->get_where('employee_movements_config', ['id' => 1]);
        return $query->row();
    }

    public function updateEmployeeMovementsConfig($data) {
        $this->db->where('id', 1);
        $this->db->update('employee_movements_config', $data);
        return $this->db->affected_rows();
    }


}