<?php
class OrderModel extends CI_Model
{
    public function __construct()
    {
        parent::__construct();
        $this->load->database();
    }

    // get all Groups
    public function count_records()
    {
        $headers = apache_request_headers();
        $search = $headers['q'] ?? '';
        $department = $this->input->get('department');

        $this->db->select('count(*) as count');
        $this->db->from('administrative_orders');
        $this->db->join('system_users', 'system_users.hash = administrative_orders.user_hash');
        $this->db->join('emp_information', 'system_users.hash = emp_information.user_hash');
        $this->db->join('Positions', 'Positions.ID = emp_information.position', 'left');

        if (isset($department) && $department != "null") {
            $this->db->like('Positions.Name', $department);
        }

        if ($search) {
            $this->db->like('user_hash', $search);
        }
        $query = $this->db->get();
        return $query->result_array()[0]['count'];
    }

    public function getOrders()
    {
        // get page and limit and search from header
        $headers = apache_request_headers();
        $page = $headers['page'] ?? 1;
        $perPage = $headers['perPage'] ?? 10;
        $search = $this->input->get('q') ?? '';
        $department = $this->input->get('department');

        $offset = ($page - 1) * $perPage;
        // -- partments.name as department,
        $this->db->select(
            'system_users.name as user_name, image_profile,administrative_orders.user_hash,

            (select name from system_group_name where system_group_name.hash=user_type) as job_name,
            (select title from administrative_order_type where hash=administrative_orders.order_title_hash) as order_title_name,

            order_title_hash,
            number,
            date,
            attachment,
            amount,
            administrative_orders.hash'
        );
        $this->db->from('administrative_orders');
        $this->db->join('system_users', 'system_users.hash = administrative_orders.user_hash');
        $this->db->join('emp_information', 'system_users.hash = emp_information.user_hash');
        $this->db->join('Positions', 'Positions.ID = emp_information.position', 'left');
        // search
        if ($search) {
            $this->db->like('administrative_orders.user_hash', $search); // I don't know how you search with user hash !!
        }
        if (isset($department) && $department != "null") {
            $this->db->like('Positions.Name', $department);
        }
        // order by administartive order id then  partment name
        $this->db->order_by('administrative_orders.id', 'DESC');
        // $this->db->order_by('partments.name', 'ASC');
        // limit
        $this->db->limit($perPage, $offset);
        $query = $this->db->get();
        return $query->result_array();
    }

    public function getOrder($hash)
    {
        $this->db->select(
            '(select name from system_users where hash=administrative_orders.user_hash) as user_name,
            (select title from administrative_order_type where hash=administrative_orders.order_title_hash) as order_title_name,
            user_hash,
            order_title_hash,
            number,
            details,
            date,
            attachment,
            amount,
            hash'
        );
        $this->db->from('administrative_orders');
        $this->db->where('hash =', $hash);
        $query = $this->db->get();

        return $query->result_array();
    }

    public function addOrder($data)
    {
        $this->db->insert('administrative_orders', $data);
        return $this->db->insert_id();
    }

    public function updateOrder($hash, $data)
    {
        $this->db->where('hash', $hash);
        $this->db->update('administrative_orders', $data);
        return $this->db->affected_rows();
    }

    public function deleteOrder($hash)
    {
        $this->db->where('hash', $hash);
        $this->db->delete('administrative_orders');
        return $this->db->affected_rows();
    }

    public function getMyOrder($user_hash)
    {
        $this->db->select('administrative_orders.*, administrative_order_type.title');
        $this->db->from('administrative_orders');
        $this->db->join('administrative_order_type', 'administrative_order_type.hash = administrative_orders.order_title_hash');
        $this->db->where('administrative_orders.user_hash', $user_hash);
        // order by id
        $this->db->order_by('administrative_orders.id', 'DESC');
        $query = $this->db->get();
        $result = $query->result();
        return $result;
    }
}
