<?php
defined('BASEPATH') OR exit('No direct script access allowed');

class PlacesMarkers_model extends CI_Model {

    public function __construct() {
        $this->load->database();
    }

    
        public function create_marker($data) {
            if ($this->db->insert('places_markers', $data)) {
                return ['status' => true, 'message' => 'Marker created successfully.', 'data' => $data];
            } else {
                return ['status' => false, 'message' => $this->db->error()['message'], 'data' => null];
            }
        }
    
        public function get_markers() {
            // Get the current page number from the request, default to 1 if not provided
            $page = $this->input->get('page') ?? 1;
            $perPage = $this->input->get('perPage') ?? 10;
            $search = $this->input->get('q') ?? '';
            // Calculate the offset
            $offset = ($page - 1) * $perPage;
            
            // Apply search filter if provided
            if ($search != '') {
                $this->db->like('name', $search);
            }
        
            // Get the total number of records
            $total_query = $this->db->count_all_results('places_markers', FALSE);
            
            // Apply limit and offset for pagination
            $this->db->limit($perPage, $offset);
            $query = $this->db->get();
        
            return [
                'status' => true,
                'data' => $query->result(),
                'total' => $total_query
            ];
        }    
        public function get_marker_by_id($id) {
            $query = $this->db->get_where('places_markers', array('id' => $id));
            if ($query) {
                $result = $query->row_array();
                if ($result) {
                    return ['status' => true, 'message' => 'Marker retrieved successfully.', 'data' => $result];
                } else {
                    return ['status' => false, 'message' => 'Marker not found.', 'data' => null];
                }
            } else {
                return ['status' => false, 'message' => $this->db->error()['message'], 'data' => null];
            }
        }
    
        public function update_marker($id, $data) {
            $this->db->where('id', $id);
            $this->db->update('places_markers', $data);
            if ($this->db->affected_rows() > 0) {
                return ['status' => true, 'message' => 'Marker updated successfully.', 'data' => $data];
            } else {
                if ($this->db->error()['code'] == 0) {
                    return ['status' => false, 'message' => 'Marker not found or no changes made.', 'data' => null];
                } else {
                    return ['status' => false, 'message' => $this->db->error()['message'], 'data' => null];
                }
            }
        }
    
        public function delete_marker($id) {
            $this->db->where('id', $id);
            $this->db->delete('places_markers');
            if ($this->db->affected_rows() > 0) {
                return ['status' => true, 'message' => 'Marker deleted successfully.', 'data' => null];
            } else {
                if ($this->db->error()['code'] == 0) {
                    return ['status' => false, 'message' => 'Marker not found.', 'data' => null];
                } else {
                    return ['status' => false, 'message' => $this->db->error()['message'], 'data' => null];
                }
            }
        }
    }

