<?php

class PositionTreeModel extends CI_Model
{
    public function __construct()
    {
        parent::__construct();
        $this->load->database();
    }

    public function count_records()
    {
        $search = $this->input->get('q') ?? '';
        $this->db->select('count(*) as count');
        $this->db->from('Positions');
        if ($search) {
            $this->db->like('Name', $search);
        }
        $query = $this->db->get();
        return $query->result_array()[0]['count'];
    }

    public function getPositionTypeSelect()
    {
        $this->db->select('ID AS value, Name AS label');
        $this->db->from("PositionTypes");
        $query = $this->db->get();
        return $query->result_array();
    }

    public function getPositionNameSelect()
    {
        $this->db->select("ID AS value, Name AS label");
        $this->db->from("Positions");
        $query = $this->db->get();
        return $query->result_array();
    }



    
    public function getPositionById($id)
    {

        $this->db->select("CONCAT(IFNULL(JSON_OBJECT('label', pt.Name, 'value', pt.ID), NULL)) AS positionType", false);

        $this->db->select("IF(p.ParentId IS NOT NULL, CONCAT( JSON_OBJECT('label', parent.Name, 'value', p.ParentId)), NULL) AS parentId");

        $this->db->select("p.Name, ei.user_hash as user_hash");

        $this->db->select("p.Todos, p.Permissions, p.KPI, p.Specifications");
        $this->db->select("p.Percentage, p.item");

        $this->db->select("CONCAT(JSON_ARRAYAGG(
    CASE
        WHEN pw.Name IS NOT NULL OR pr.work_with_who_id IS NOT NULL THEN JSON_OBJECT('label', pw.Name, 'value', pr.work_with_who_id)
        ELSE NULL
    END
)) AS workWihtWhoId", false);

        $this->db->from("Positions p");
        $this->db->join("PositionTypes pt", "p.PositionTypeID = pt.ID");
        $this->db->join("PositionRelationship pr", "p.ID = pr.position_id", "left");
        $this->db->join("Positions pw", "pr.work_with_who_id = pw.ID", "left");

        $this->db->join("Positions parent", "p.ParentId = parent.ID", "LEFT");
        $this->db->join("emp_information ei", "p.lead_hash = ei.user_hash", "left");

        $this->db->where("p.ID", $id);

        return $this->db->get()->row();

    }

    public function insertPosition($data)
    {

        $this->db->insert('Positions', $data);

        return $this->db->insert_id();
    }

    public function updatePosition($positionId, $data)
    {
        $this->db->where('ID', $positionId);
        $this->db->update('Positions', $data);

        return true;
    }

    public function getCurrentRelations($positionId)
    {
        $this->db->select('work_with_who_id');
        $this->db->from('PositionRelationship');
        $this->db->where('position_id', $positionId);

        $query = $this->db->get();

        if ($query->num_rows() > 0) {
            // Extract work_with_who_id values from the result set
            $result = $query->result_array();
            $relations = array_column($result, 'work_with_who_id');

            return $relations;
        } else {
            // No relations found for the given position
            return [];
        }
    }

    public function insertPositionRelation($data)
    {

        $this->db->insert('PositionRelationship', $data);
    }

    public function deletePosition($positionId)
    {
        // Sanitize input
        $positionId = $this->db->escape_str($positionId);

        // Check if the record exists
        $this->db->where('ID', $positionId);
        $query = $this->db->get('Positions');

        if ($query->num_rows() > 0) {
            // Record exists, proceed with deletion
            $this->db->where('ID', $positionId);
            $this->db->delete('Positions');

            // Check if the deletion was successful
            if ($this->db->affected_rows() > 0) {
                return true;
            } else {
                // Deletion failed
                return false;
            }
        } else {
            // Record not found
            return false;
        }
    }

    public function updatePositionRelation($positionId, $workWithWhoId, $data)
    {

        // Assuming 'PositionRelationship' is the table name
        $this->db->where('position_id', $positionId);
        $this->db->where('work_with_who_id', $workWithWhoId);
        $this->db->update('PositionRelationship', $data);
    }

    public function deletePositionRelation($positionId, $workWithWhoId)
    {
        $this->db->where('position_id', $positionId);
        $this->db->where('work_with_who_id', $workWithWhoId);
        $this->db->delete('PositionRelationship');
    }


    public function getAllPositions()
    {

        $headers = apache_request_headers();
        $page = $this->input->get('page') ?? '1';

        $perPage = $this->input->get('perPage') ?? '10';

        $search = $this->input->get('q') ?? '';

        $offset = ($page - 1) * $perPage;

        $this->db->select("Name AS name ,ID AS hash");
        $this->db->from("Positions");
        // search
        if ($search) {
            // $search = urldecode($search);
            $this->db->like('name', $search);
        }
        // order by
        $this->db->order_by('ID', 'ASC');
        // limit
        $this->db->limit($perPage, $offset);
        $query = $this->db->get();
        return $query->result_array();
    }



}