<?php
class ProjectModel extends CI_Model
{
    public function __construct()
    {
        parent::__construct();
        $this->load->database();
    }

    private function has_required_role($user_hash, $required_role_id = 5)
    {
        $this->db->select('role_id');
        $this->db->from('UserRoles');
        $this->db->where('user_hash', $user_hash);
        $this->db->where('role_id', $required_role_id);
        $query = $this->db->get();

        return $query->num_rows() > 0;
    }

    public function getEmployeeDetailsProject($hash)
    {
        $employeeQuery = "SELECT
        e.user_hash,
		su.name,
        e.position,
        su.image_profile
        FROM emp_information e
        JOIN system_users su ON su.hash = e.user_hash
        WHERE e.user_hash = '$hash'";

        $projectsQuery = "SELECT
        p.id AS project_id,
        p.title AS project_name,
        -- p.status AS project_status,
        r.id AS role_id,
        r.name AS role_name,
        pr.id AS project_role_id,
        pr.employee_id,
        su.name
        FROM project_employees pr
         JOIN project p ON pr.project_id = p.id
         JOIN project_roles r ON pr.role_id = r.id
         JOIN system_users su ON su.hash = employee_id
           WHERE pr.employee_id = '$hash'
        ORDER BY p.title";

        $employeeResult = $this->db->query($employeeQuery)->row_array();
        $projectsResult = $this->db->query($projectsQuery)->result_array();

        return [
            'employee' => $employeeResult,
            'projects' => $projectsResult,
        ];
    }

    // إضافة مشروع
    public function add($data)
    {

        $this->db->insert('project', $data);
        return $this->db->insert_id();
    }

    // تحديث مشروع
    public function update($hash, $data)
    {
        $this->db->where('hash', $hash);
        $this->db->update('project', $data);
        return $this->db->affected_rows();
    }

    // حذف مشروع
    public function delete($hash, $user)
    {

        if (!$this->has_required_role($user)) {
            return "ليس ليدك صلاحيه";
        }
        $this->db->where('hash', $hash);
        $this->db->delete('project');
        return $this->db->affected_rows();
    }

    // استرداد مشروع بالهاش
    public function get($hash)
    {

        // Fetch project details

        $project_query = $this->db->get_where('project', array('hash' => $hash));
        $project = $project_query->row_array();

        if ($project) {
            // Fetch associated project employees
            $this->db->select('project_employees.*, system_users.name');
            $this->db->from('project_employees');
            $this->db->join('system_users', 'project_employees.employee_id = system_users.hash');
            $this->db->where('project_employees.project_id', $project['id']);
            $employee_query = $this->db->get();
            // $employee_query = $this->db->get_where('project_employees', array('project_id' => $project['id']));

            $project['employees'] = $employee_query->result_array();
        }

        return $project;

        // return $query->row_array();
    }

    // استرداد كل المشاريع
    public function get_all($page = 1, $perpage = 10, $search = '')
    {
        $offset = ($page - 1) * $perpage;

        $this->db->select('
        client_hash,
        (SELECT name FROM clients WHERE clients.hash = project.client_hash) AS client_name,
        deadline,
        description,
        hash,
        price,
        project_type,
        start_date,
        title
        ');
        $this->db->from('project');

        if ($search != '') {
            $this->db->like('title', $search);
            $this->db->or_like('description', $search);
        }

        $this->db->limit($perpage, $offset);

        $query = $this->db->get();
        return $query->result_array();
    }

    public function mobile_all()
    {
        $this->db->select('
        hash,
        title
        ');
        $this->db->from('project');
        $query = $this->db->get();
        return $query->result_array();
    }

    public function count_records($search = '')
    {
        $this->db->select('count(*) as count');
        $this->db->from('project');
        if ($search != '') {
            $this->db->like('title', $search);
            $this->db->or_like('description', $search);
        }
        $query = $this->db->get();
        return $query->result_array()[0]['count'];
    }

    // detail project
    public function detail($hash)
    {
        $query = $this->db->query("
            SELECT
                 (select name from system_users where system_users.hash = task.receiver) as user_name,
                 (select name from system_users where system_users.hash = task.sender) as sender_name,
                 score,
                 task.hash,
                 deadline,
                 start_task,
                 title,
                 status,
                 GROUP_CONCAT(task_attachment.link, task_attachment.id) AS attachments
            FROM task
            LEFT JOIN task_attachment  ON task.hash = task_attachment.task_hash
            WHERE task.project_hash = '$hash'
            GROUP BY task.id
        ");
        $tasks = $query->result_array();
        $project = $this->get($hash);
        $project['tasks'] = $tasks;
        return $project;
    }

    public function insert_project_with_employees($title, $admin, $subordinate, $employees, $officials, $managers, $user)
    {

        if (!$this->has_required_role($user)) {
            return "ليس ليدك صلاحيه";
        }
        // Start transaction
        $this->db->trans_start();

        try {
            // Insert project
            $data = array(
                'hash' => $this->jwtmiddleware->createHash(),
                'title' => $this->input->post('title'),
                'project_type' => $this->input->post('project_type'),
                'client_hash' => $this->input->post('client_hash'),
                'description' => $this->input->post('description'),
                'created_at' => date('Y-m-d H:i:s'),
                'updated_at' => date('Y-m-d H:i:s'),
            );
            $this->db->insert('project', $data);
            $project_id = $this->db->insert_id();

            // Insert employees for each role category
            $this->insert_project_employees($project_id, $managers);
            $this->insert_project_employees($project_id, $admin);
            $this->insert_project_employees($project_id, $subordinate);
            $this->insert_project_employees($project_id, $employees);
            $this->insert_project_employees($project_id, $officials);

            // Commit transaction
            $this->db->trans_commit();
            return true;
        } catch (Exception $e) {
            // Rollback transaction on error
            $this->db->trans_rollback();
            return false;
        }
    }

    private function insert_project_employees($project_id, $employees)
    {
        foreach ($employees as $employee) {
            $data = array(
                'project_id' => $project_id,
                'employee_id' => $employee['value'],
                'role_id' => $employee['role_id'],
            );
            $this->db->insert('project_employees', $data);
        }
    }

    public function update_project_with_employees($project_hash, $title, $admin, $subordinate, $employees, $officials, $managers, $user)
    {
        if (!$this->has_required_role($user)) {
            return "ليس ليدك صلاحيه";
        }
        // Start transaction
        $this->db->trans_start();

        try {
            // Update project details
            $data = array(
                'title' => $this->input->post('title'),
                'project_type' => $this->input->post('project_type'),
                'client_hash' => $this->input->post('client_hash'),
                'description' => $this->input->post('description'),
                'updated_at' => date('Y-m-d H:i:s'),
            );
            $this->db->where('hash', $project_hash);
            $this->db->update('project', $data);

            // Get project ID from hash
            $this->db->select('id');
            $this->db->from('project');
            $this->db->where('hash', $project_hash);
            $project_query = $this->db->get();
            $project = $project_query->row_array();

            if (!$project) {
                throw new Exception("Project not found.");
            }

            $project_id = $project['id'];

            // Gather all employees across categories
            $all_employees = array_merge($managers, $admin, $subordinate, $employees, $officials);

            // Update employees for the project
            $this->update_project_employees($project_id, $all_employees);

            // Commit transaction
            $this->db->trans_commit();
            return true;
        } catch (Exception $e) {
            // Rollback transaction on error
            $this->db->trans_rollback();
            log_message('error', $e->getMessage()); // Log the error message
            return false;
        }
    }

    public function getProjectsOptionsForUser($user)
    {
        $employee_id = $user;

        $query = $this->db->select('p.id, p.hash as value, p.title as label')
            ->from('project p')
            ->join('project_employees pe', 'p.id = pe.project_id')
            ->where('pe.employee_id', $employee_id)
            ->get();

        $result = $query->result_array();

        return (!empty($result)) ? $result : [];
    }

    public function getEmployeesByProject($projectHash)
    {

        $this->db->select('project.id');
        $this->db->from('project');
        $this->db->where('project.hash', $projectHash);
        $query = $this->db->get();
        $project = $query->row_array();
        if (!$project) {
            return [];
        }
        $project_id = $project['id'];

        $query = $this->db->select('se.hash, se.name,se.image_profile, pe.role_id')
            ->from('project_employees pe')
            ->join('system_users se', 'pe.employee_id = se.hash')
            ->where('pe.project_id', $project_id)
            ->where('pe.role_id', 5)
            ->get();

        $result = $query->result_array();

        return (!empty($result)) ? $result : [];
    }

    public function checkUserHaveActionsToAnyProject($user)
    {
        $projects = $this->getProjectsOptionsForUser($user);
        foreach ($projects as $project) {
            if ($this->checkUserRoleForProject($user, $project['id'])) {
                return true;
            }
        }
        return false;
    }

    public function checkUserRoleForProject($user, $projectId)
    {
        $userRole = $this->getUserRoleForProject($user, $projectId);
        return ($userRole == 1 || $userRole == 2);
    }

    public function getUserRoleForProject($user, $projectId)
    {
        $query = $this->db->select('role_id')
            ->from('project_employees')
            ->where('employee_id', $user)
            ->where('project_id', $projectId)
            ->get();

        if ($query->num_rows() > 0) {
            $result = $query->row_array();
            return $result['role_id'];
        } else {
            return null;
        }
    }

    private function update_project_employees($project_id, $employees)
    {

        // Get current employees for the project
        $this->db->select('id, employee_id, role_id');
        $this->db->from('project_employees');
        $this->db->where('project_id', $project_id);
        $current_employees_query = $this->db->get();
        $current_employees = $current_employees_query->result_array();

        // Create arrays for easier comparison
        $current_employee_ids = array_column($current_employees, 'employee_id');
        $new_employee_ids = array_column($employees, 'value');

        // Track employees that should be retained
        $retained_employees = [];

        // Insert or update employees
        foreach ($employees as $employee) {
            $employee_id = $employee['value'];
            $role_id = $employee['role_id'];
            $retained_employees[] = $employee_id;

            // If employee already exists, update role_id if necessary
            if (in_array($employee_id, $current_employee_ids)) {
                foreach ($current_employees as $current_employee) {
                    if ($current_employee['employee_id'] == $employee_id) {
                        if ($current_employee['role_id'] != $role_id) {
                            $this->db->where('id', $current_employee['id']);
                            $this->db->update('project_employees', array('role_id' => $role_id));
                        }
                    }
                }
            } else {
                // Insert new employee
                $data = array(
                    'project_id' => $project_id,
                    'employee_id' => $employee_id,
                    'role_id' => $role_id,
                );
                $this->db->insert('project_employees', $data);
            }
        }

        // Adjusted removal logic
        // Instead of removing based on presence in $retained_employees, we now check if the employee is not in the new list at all
        foreach ($current_employees as $current_employee) {
            if (!in_array($current_employee['employee_id'], $new_employee_ids)) {
                $this->db->where('id', $current_employee['id']);
                $this->db->delete('project_employees');
            }
        }
    }
}
