<?php
class Roles_permissions_model extends CI_Model
{

    // Constructor to load the database
    public function __construct()
    {
        parent::__construct();
        $this->load->database();
    }

    // Get permissions for a specific role
    public function get_permissions_by_role($role_id)
    {
        $query = $this->db->get_where('RolesPermissions', array('role_id' => $role_id));
        return $query->result();
    }



    // for select box
    public function get_permissions_label_by_role($role_id)
    {
        $this->db->select('permissions_for_roles.name as label, permissions_for_roles.id as value');
        $this->db->from('RolesPermissions');
        $this->db->join('permissions_for_roles', 'RolesPermissions.permission_id = permissions_for_roles.id');
        $this->db->where('RolesPermissions.role_id', $role_id);
        $query = $this->db->get();
        $results = $query->result();
        return $results;
    }

    // Insert new permission for a role
    public function insert_permission($data)
    {
        if (empty($data['permissionIds'])) {
            // Delete all rows for the role
            $this->db->where('role_id', $data['role_id']);
            $this->db->delete('RolesPermissions');
            return true;
        } else {
            // First, delete any existing permissions for the role
            $this->db->where('role_id', $data['role_id']);
            $this->db->delete('RolesPermissions');

            // Then, insert the new permissions
            $result = array();
            foreach ($data['permissionIds'] as $permissionId) {
                $insertData = array(
                    'role_id' => $data['role_id'],
                    'permission_id' => $permissionId,
                    'is_allowed' => $data['is_allowed']
                );
                $result[] = $this->db->insert('RolesPermissions', $insertData);
            }
            return $result;
        }
    }

    // Update an existing permission for a role
    public function update_permission($role_id, $permission_id, $data)
    {
        $this->db->where('role_id', $role_id);
        $this->db->where('permission_id', $permission_id);
        return $this->db->update('RolesPermissions', $data);
    }

    // Delete a permission for a role
    public function delete_permission($role_id, $permission_id)
    {
        $this->db->where('role_id', $role_id);
        $this->db->where('permission_id', $permission_id);
        return $this->db->delete('RolesPermissions');
    }
}
