<?php
class ScheduleModel extends CI_Model
{

    public function __construct()
    {
        parent::__construct();
        $this->load->database();
        $this->load->model('GroupScheduleModel');
        $this->load->model('Scheduled_tasks_model');

    }

    public function decimalToTime($decimal)
    {
        $hours = floor($decimal);
        $minutes = ($decimal - $hours) * 60;
        return sprintf('%02d:%02d', $hours, $minutes);
    }

    public function calculateLateArrivalAndEarlyDeparture($data)
    {
        $totalLateArrivalMinutes = 0;
        $totalEarlyDepartureMinutes = 0;
        $totalScheduledMinutes = 0;
        $totalWorkedMinutes = 0;
        $totalTargetMinutes = 0;

        // Check if schedules are provided
        if (!isset($data['schedule_for_day']) || !is_array($data['schedule_for_day']) || empty($data['schedule_for_day'])) {
            return [
                'error' => 'Missing or invalid schedule_for_day data.',
                'status' => 'error',
            ];
        }

        $allowanceTime = isset($data['allowance_time']) ? (int) $data['allowance_time'] : null;

        foreach ($data['schedule_for_day'] as $index => $schedule) {
            $startWork = $schedule['start'];
            $endWork = $schedule['end'];
            // $allowanceTime = isset($schedule['allowance_time']) ? (int) $schedule['allowance_time'] : null;

            // Calculate scheduled work duration
            $startWorkFormatted = $this->decimalToTime($startWork);
            $endWorkFormatted = $this->decimalToTime($endWork);
            $scheduledStartTime = new DateTime($data['date'] . ' ' . $startWorkFormatted);
            $scheduledEndTime = new DateTime($data['date'] . ' ' . $endWorkFormatted);
            $scheduledDuration = $scheduledStartTime->diff($scheduledEndTime);
            $scheduledMinutes = $scheduledDuration->h * 60 + $scheduledDuration->i;

            $totalScheduledMinutes += $scheduledMinutes;
            $totalTargetMinutes += $scheduledMinutes;

            // Check if in_fingerprint_time exists for this schedule index
            if (isset($data['in_fingerprint_time'])) {
                $inFingerprintTimes = json_decode($data['in_fingerprint_time'], true);
                if (isset($inFingerprintTimes[$index])) {
                    $inFingerprintTime = new DateTime($inFingerprintTimes[$index]);

                    // Calculate late arrival
                    if ($inFingerprintTime > $scheduledStartTime) {
                        $lateArrival = $scheduledStartTime->diff($inFingerprintTime);
                        $lateArrivalMinutes = $lateArrival->h * 60 + $lateArrival->i;
                        if (is_numeric($allowanceTime)) {
                            if ($lateArrivalMinutes <= $allowanceTime) {
                                $lateArrivalMinutes = 0;
                            }
                        }
                        $totalLateArrivalMinutes += $lateArrivalMinutes;
                    }
                }
            }

            // Check if out_fingerprint_time exists for this schedule index
            if (isset($data['out_fingerprint_time'])) {
                $outFingerprintTimes = json_decode($data['out_fingerprint_time'], true);
                if (isset($outFingerprintTimes[$index])) {
                    $outFingerprintTime = new DateTime($outFingerprintTimes[$index]);

                    // Calculate early departure
                    if ($outFingerprintTime < $scheduledEndTime) {
                        $earlyDeparture = $outFingerprintTime->diff($scheduledEndTime);
                        $earlyDepartureMinutes = $earlyDeparture->h * 60 + $earlyDeparture->i;
                        $totalEarlyDepartureMinutes += $earlyDepartureMinutes;
                    }
                }
            }

            // Check if both in_fingerprint_time and out_fingerprint_time exist for this schedule index
            if (isset($data['in_fingerprint_time'], $data['out_fingerprint_time'])) {
                $inFingerprintTimes = json_decode($data['in_fingerprint_time'], true);
                $outFingerprintTimes = json_decode($data['out_fingerprint_time'], true);
                if (isset($inFingerprintTimes[$index], $outFingerprintTimes[$index])) {
                    $inFingerprintTime = new DateTime($inFingerprintTimes[$index]);
                    $outFingerprintTime = new DateTime($outFingerprintTimes[$index]);

                    // Calculate worked duration
                    $workedDuration = $inFingerprintTime->diff($outFingerprintTime);
                    $workedMinutes = $workedDuration->h * 60 + $workedDuration->i;
                    if ($workedMinutes >= 15) {
                        $totalWorkedMinutes += $workedMinutes;
                    }
                    // $totalWorkedMinutes += $workedMinutes;
                }
            }
        }

        $totalMissedMinutes = max(round($totalScheduledMinutes - $totalWorkedMinutes), 0);
        $overtimeMinutes = $totalWorkedMinutes - $totalScheduledMinutes;
        $overtimeHours = max(round($overtimeMinutes / 60, 2), 0);
        $targetHours = round($totalTargetMinutes / 60, 2);
        $workedHours = round($totalWorkedMinutes / 60, 2);

        return [
            'totalLateArrivalMinutes' => $totalLateArrivalMinutes,
            'totalEarlyDepartureMinutes' => $totalEarlyDepartureMinutes,
            'totalMissedHours' => round($totalMissedMinutes / 60, 2),
            'totalWorkedHours' => $workedHours,
            'targetHours' => $targetHours,
            'overtimeHours' => $overtimeHours,
            'date' => $data['date'],
            'status' => "success",
            'user_hash' => $data['user_hash'],
        ];
    }

    public function extractHour($datetime)
    {
        $date = new DateTime($datetime);
        return $date->format('H'); // Format the hour as two digits (01, 02, ..., 24)
    }

    public function handleNormalVacation($user_hash, $vacation_start_date, $vacation_end_date)
    {

        $start_date = new DateTime($vacation_start_date);
        $end_date = new DateTime($vacation_end_date);

        if ($start_date->format('Y-m-d') == $end_date->format('Y-m-d')) {
            return ['status' => 'error', 'message' => "خطأ: لا يمكن أن يكون تاريخ بدء الإجازة هو نفسه تاريخ انتهاء الإجازة."];
        }
        $end_date->modify('-1 day');

        $current_date = clone $start_date;

        while ($current_date <= $end_date) {
            $date = $current_date->format('Y-m-d');

            $this->GroupScheduleModel->deleteActualSchedule($user_hash, $date);
            $this->GroupScheduleModel->deleteFlexibleDailySchedule($user_hash, $date);
            // TODO: SINGELTON TOO employees_worked_hours
            $this->db->delete('employees_worked_hours', array('user_hash' => $user_hash, 'date' => $date));

            $current_date->modify('+1 day');
        }

        return ['status' => 'succuess', 'message' => 'تم بنجاح'];
    }

    public function updateScheduleForTimeVacation($user_hash, $vacation_start_time, $vacation_end_time)
    {
        try {
            // Convert vacation times to DateTime objects
            $vacation_start = new DateTime($vacation_start_time);
            $vacation_end = new DateTime($vacation_end_time);

            // Use the date from vacation_start_time
            $date = $vacation_start->format('Y-m-d');

            // Fetch the actual schedule for the day
            $schedule_result = $this->GroupScheduleModel->getActualSchedule($user_hash, $date);

            if (!$schedule_result) {

                $current_date = date('Y-m-d');
                if ($date <= $current_date) {
                    return ['status' => 'error', 'message' => 'لا يوجد جدول للموظف فى هذا التاريخ'];
                }

                // No schedule found, fetch the user's schedule from their work schedule

                $fetch_schedule_result = $this->GroupScheduleModel->getScheduleData($user_hash, $date);

                if ($fetch_schedule_result && $fetch_schedule_result['schedule_data']) {
                    // Validate if the vacation request can be adjusted correctly
                    $original_schedules = json_decode($fetch_schedule_result['schedule_data'], true);
                    foreach ($original_schedules as $schedule_group) {
                        // Ensure $schedule_group is an array
                        if (!is_array($schedule_group)) {
                            throw new Exception('Invalid schedule group format');
                        }

                        // Ensure required keys are present
                        if (
                            !isset($schedule_group['start']) || !isset($schedule_group['end']) || !isset($schedule_group['id']) ||
                            $schedule_group['id'] === null ||
                            $schedule_group['start'] <= 0 || $schedule_group['end'] <= 0 ||
                            $schedule_group['start'] >= $schedule_group['end']// Check if start is equal to or greater than end
                        ) {
                            throw new Exception('Invalid schedule item: start time must be less than end time');
                        }

                        // Convert start and end times to DateTime objects
                        $schedule_start = new DateTime($date . ' ' . $this->decimalToTime($schedule_group['start']));
                        $schedule_end = new DateTime($date . ' ' . $this->decimalToTime($schedule_group['end']));

                        // Handle midnight crossing for schedule end time
                        if ($schedule_group['end'] < $schedule_group['start']) {
                            $schedule_end->modify('+1 day');
                        }

                        // Check if the vacation request is bigger than the schedule
                        if ($vacation_start <= $schedule_start && $vacation_end >= $schedule_end) {
                            throw new Exception('لا يمكن طلب اجازه زمنيه او واجب تغطى جدول الدوام كامل');
                        }
                    }

                    // Insert the fetched schedule into actual_schedules

                    $insert_result = $this->GroupScheduleModel->insertActualSchedule($user_hash, $date, json_encode(json_decode($fetch_schedule_result['schedule_data'])));

                    if (!$insert_result) {
                        // Failed to insert schedule data, return an error message
                        return ['status' => 'error', 'message' => 'Failed to insert schedule data.'];
                    }

                    // Refetch the inserted schedule to proceed with vacation adjustment
                    $schedule_result = $this->GroupScheduleModel->getScheduleData($user_hash, $date);

                    if (!$schedule_result) {
                        // Inserted schedule not found, return an error message
                        return ['status' => 'error', 'message' => 'Failed to retrieve the inserted schedule data.'];
                    }
                } else {
                    // No schedule data found, return a message
                    return ['status' => 'error', 'message' => 'لا يوجد جدول للموظف فى هذا التاريخ'];
                }
            }

            // Proceed with the schedule adjustment based on time vacation
            $original_schedules = json_decode($schedule_result['schedule_data'], true);
            $new_schedules = [];

            foreach ($original_schedules as $schedule_group) {
                // Ensure $schedule_group is an array
                if (!is_array($schedule_group)) {
                    throw new Exception('Invalid schedule group format');
                }

                // Ensure required keys are present
                if (
                    !isset($schedule_group['start']) || !isset($schedule_group['end']) || !isset($schedule_group['id']) ||
                    $schedule_group['id'] === null ||
                    $schedule_group['start'] <= 0 || $schedule_group['end'] <= 0 ||
                    $schedule_group['start'] >= $schedule_group['end']// Check if start is equal to or greater than end
                ) {
                    throw new Exception('Invalid schedule item: start time must be less than end time');
                }

                // Convert start and end times to DateTime objects
                $schedule_start = new DateTime($date . ' ' . $this->decimalToTime($schedule_group['start']));
                $schedule_end = new DateTime($date . ' ' . $this->decimalToTime($schedule_group['end']));

                // Handle midnight crossing for schedule end time
                if ($schedule_group['end'] < $schedule_group['start']) {
                    $schedule_end->modify('+1 day');
                }

                $adjusted = false;

                // Check if the vacation request is bigger than the schedule
                if ($vacation_start <= $schedule_start && $vacation_end >= $schedule_end) {
                    throw new Exception('لا يمكن طلب اجازه زمنيه او واجب تغطى جدول الدوام كامل');
                }

                // Adjust the start time if vacation overlaps the beginning
                if ($vacation_start <= $schedule_start && $vacation_end > $schedule_start) {
                    $schedule_start = clone $vacation_end;
                }
                // Adjust the end time if vacation overlaps the end
                else if ($vacation_start < $schedule_end && $vacation_end >= $schedule_end) {
                    $schedule_end = clone $vacation_start;
                } else if ($vacation_start > $schedule_start && $vacation_end < $schedule_end) {
                    $now = new DateTime();
                    $today = $now->format('Y-m-d');

                    if ($vacation_start->format('Y-m-d') == $today && $now > $vacation_start) {
                        throw new Exception('الرجاء مراجعه مواعيد هذا الطلب، لا يمكن طلب إجازة جزئية لوقت مضى.');
                    }

                    // Check if the vacation date is before the current date
                    if ($vacation_start->format('Y-m-d') < $today) {
                        throw new Exception('الرجاء مراجعه مواعيد هذا الطلب، لا يمكن طلب إجازة جزئية لتاريخ سابق.');
                    }

                    $new_schedules[] = [
                        'id' => $schedule_group['id'],
                        'start' => $this->timeToDecimal($schedule_start),
                        'end' => $this->timeToDecimal($vacation_start),
                    ];
                    $scheduleGroupId = $schedule_group['id'];
                    $splittedScheduledId = $this->generateUuidV4();
                    $new_schedules[] = [
                        'id' => $splittedScheduledId,
                        'start' => $this->timeToDecimal($vacation_end),
                        'end' => $this->timeToDecimal($schedule_end),
                    ];


                    $task_data_start = array('user_id' => $user_hash, 'start_time' => $vacation_start->format('Y-m-d H:i:s'), 'schedule_id' => $scheduleGroupId, 'firstScheduleId' => $scheduleGroupId, 'secondScheduleId' => $splittedScheduledId);
                    $excute_at_start = $vacation_start->format('Y-m-d H:i:s');

                    $task_data_end = array('user_id' => $user_hash, 'end_time' => $vacation_end->format('Y-m-d H:i:s'), 'schedule_id' => $splittedScheduledId, 'firstScheduleId' => $scheduleGroupId, 'secondScheduleId' => $splittedScheduledId);
                    $excute_at_end = $vacation_end->format('Y-m-d H:i:s');

                    $this->Scheduled_tasks_model->insert_task('partial_vacation_OUT', $task_data_start, $excute_at_start);
                    $this->Scheduled_tasks_model->insert_task('partial_vacation_IN', $task_data_end, $excute_at_end);

                    $adjusted = true;
                } else {
                    throw new Exception('الرجاء مراجعة مواعيد هذا الطلب. لا يمكن طلب اجازه بين بدايه الدوام ونهايته  . تأكد من أن الاجازه تبدأ من البداية وتكون قبل النهايه او بعد بدايه الدوام لنهايه الدوام.');
                }

                // Add the adjusted schedule to the new schedules array
                if (!$adjusted) {
                    $new_schedules[] = [
                        'id' => $schedule_group['id'],
                        'start' => $this->timeToDecimal($schedule_start),
                        'end' => $this->timeToDecimal($schedule_end),
                    ];
                }

            }

            // Validate new_schedules to ensure no start time is equal to or greater than end time
            foreach ($new_schedules as $schedule) {
                if ($schedule['start'] >= $schedule['end']) {
                    throw new Exception('خطأ : لا يمكن ان يكون توقيت بدايه الاجازه اكبر من توقيت نهايه الاجازه');
                }
            }
            // Update the schedule in the database
            $update_response = $this->GroupScheduleModel->updateActualSchedule($user_hash, $date, $new_schedules);

            if (!$update_response) {
                return ['status' => 'error', 'message' => 'فشل في تحديث بيانات الجدول الزمني.'];
            }

            // Update employee worked hours
            $this->updateEmployeeWorkedHours($user_hash, $date);

            return ['status' => 'success', 'message' => 'تم تحديث الجدول الزمني بنجاح.'];
        } catch (Exception $e) {
            // Handle any exceptions in the overall function
            return ['status' => 'error', 'message' => 'حدث خطأ: ' . $e->getMessage()];
        }
    }

    private function generateUuidV4()
    {
        $data = random_bytes(16);
        $data[6] = chr(ord($data[6]) & 0x0f | 0x40); // set version to 0100
        $data[8] = chr(ord($data[8]) & 0x3f | 0x80); // set bits 6-7 to 10
        return vsprintf('%s%s-%s-%s-%s-%s%s%s', str_split(bin2hex($data), 4));
    }

    private function timeToDecimal($datetime)
    {
        $hours = $datetime->format('H');
        $minutes = $datetime->format('i');
        return $hours + ($minutes / 60);
    }

    // this currently disabled
    public function update_half_day_vacation_range($user_hash, $start_date, $end_date)
    {
        $current_date = new DateTime($start_date);
        $end = new DateTime($end_date);
        $updated_count = 0;

        while ($current_date <= $end) {
            $date = $current_date->format('Y-m-d');

            // Check if a record exists for this specific day
            $check_sql = "SELECT COUNT(*) as count FROM actual_schedules
                          WHERE user_hash = ? AND date = ?";
            $result = $this->db->query($check_sql, array($user_hash, $date))->row();

            if ($result->count > 0) {

                // Record exists, so we update it
                $update_sql = "UPDATE actual_schedules AS a_s
                               JOIN (
                                   SELECT
                                       ei.user_hash,
                                       JSON_ARRAY(
                                           JSON_OBJECT(
                                               'id', JSON_EXTRACT(JSON_UNQUOTE(JSON_EXTRACT(sg.schedule_data, CONCAT('$.', DAYNAME(?)))), '$[0].id'),
                                               'start', JSON_EXTRACT(JSON_UNQUOTE(JSON_EXTRACT(sg.schedule_data, CONCAT('$.', DAYNAME(?)))), '$[0].start'),
                                               'end', (JSON_EXTRACT(JSON_UNQUOTE(JSON_EXTRACT(sg.schedule_data, CONCAT('$.', DAYNAME(?)))), '$[0].start') +
                                                       (JSON_EXTRACT(JSON_UNQUOTE(JSON_EXTRACT(sg.schedule_data, CONCAT('$.', DAYNAME(?)))), '$[0].end') -
                                                        JSON_EXTRACT(JSON_UNQUOTE(JSON_EXTRACT(sg.schedule_data, CONCAT('$.', DAYNAME(?)))), '$[0].start')) / 2)
                                           )
                                       ) AS new_schedule_data,
                                       sg.allowance_time / 2 AS new_allowance_time
                                   FROM emp_information ei
                                   JOIN schedule_groups sg ON ei.work_schedule = sg.id
                                   WHERE ei.user_hash = ?
                               ) AS temp ON a_s.user_hash = temp.user_hash
                               SET
                                   a_s.schedule_data = temp.new_schedule_data,
                                   a_s.allowance_time = temp.new_allowance_time
                               WHERE
                                   a_s.user_hash = ? AND a_s.date = ?";

                $this->db->query(
                    $update_sql,
                    array(
                        $date,
                        $date,
                        $date,
                        $date,
                        $date,
                        $user_hash,
                        $user_hash,
                        $date,
                    )
                );

                $updated_count += $this->db->affected_rows();

                $this->updateEmployeeWorkedHours($user_hash, $date);
            }

            // Move to the next day
            $current_date->modify('+1 day');
        }

        return $updated_count > 0;
    }

    public function updateEmployeeWorkedHours($user_hash, $date)
    {
        $result = $this->GroupScheduleModel->getEmploeedWorkedData($user_hash, $date);
        if ($result) {
            $employee_data = json_decode($result->employee_data, true);
            $calculation = $this->calculateLateArrivalAndEarlyDeparture($employee_data);
            // var_dump($calculation);
            $this->insertCalculatedData($calculation);
        }
    }

    public function insertCalculatedData($data)
    {
        $update_data = array(
            'total_late_arrival_minutes' => $data['totalLateArrivalMinutes'],
            'total_early_departure_minutes' => $data['totalEarlyDepartureMinutes'],
            'total_missed_hours' => $data['totalMissedHours'],
            'total_worked_hours' => $data['totalWorkedHours'],
            'target_hours' => $data['targetHours'],
            'overtime_hours' => $data['overtimeHours'],
        );

        $this->db->where('user_hash', $data['user_hash']);
        $this->db->where('date', $data['date']);
        $this->db->update('employees_worked_hours', $update_data);
    }

    public function getEmployeeSchedule($user_hash, $date)
    {
        // Query the database to get the schedule data
        $JSON_schedule_data = $this->db->select('schedule_data')
            ->from('actual_schedules')
            ->where('user_hash', $user_hash)
            ->where('date', $date)
            ->get()
            ->row_array();

        if ($JSON_schedule_data && isset($JSON_schedule_data['schedule_data'])) {
            $DECODED_schedule_data = json_decode($JSON_schedule_data['schedule_data'], true);
            return ['status' => 200, 'message' => 'جدول الموظف', 'data' => $DECODED_schedule_data, 'isAuth' => true];
        } else {
            return ['status' => 200, 'message' => 'لا يوجد جدول لهذا الموظف', 'data' => [], 'isAuth' => true, 'action' => false];
        }
    }

    public function getEmployeeFlexible($user_hash, $date)
    {
        $flexibleData = $this->db->select('*')->from('daily_flexible_schedule_targets')->where('user_hash', $user_hash)->where('date', $date)->get()->row_array();
        if ($flexibleData && !empty($flexibleData)) {
            return ['status' => 200, 'message' => 'بيانات الموظف', 'data' => $flexibleData, 'isAuth' => true];
        } else {
            return ['status' => 200, 'message' => 'لا يوجد جدول لهذا الموظف', 'data' => [], 'isAuth' => true, 'action' => false];

        }
    }

}
