<?php
defined('BASEPATH') or exit('No direct script access allowed');

class SubscribersCustomersFTTHModel extends CI_Model
{

  public function __construct()
  {
    parent::__construct();
    $this->load->database();
    $this->load->model('UserModel');
    $this->load->library('Middleware');
  }

  public function testInsertMutedNot()
  {
    $this->jwtmiddleware->insertMutedNotification();

  }

  private function insertMutedNot()
  {
    $this->jwtmiddleware->insertMutedNotification();

  }

  public function insertCustomerData($data)
  {
    $array = array(
      'id' => 1,
      'data' => $data,
      'updated_at' => date('Y-m-d H:i:s')
    );
    $this->db->where('id', 1);
    $query = $this->db->get('subscribers_customers_ftth');

    if ($query->num_rows() > 0) {
      $this->db->where('id', 1);
      $this->db->update('subscribers_customers_ftth', $array);
      if ($this->db->affected_rows() > 0) {
        $this->insertFtthPrimaryInformation();
      }
    } else {
      $this->db->insert('subscribers_customers_ftth', $array);
      if ($this->db->affected_rows() > 0) {
        $this->insertFtthPrimaryInformation();
      }
    }
  }
  public function insertCustomerDataExpired($data)
  {
    $array = array(
      'id' => 1,
      'data' => $data,
      'updated_at' => date('Y-m-d H:i:s')
    );
    $this->db->where('id', 1);
    $query = $this->db->get('subscribers_customers_ftth_all_expired');

    if ($query->num_rows() > 0) {
      $this->db->where('id', 1);
      $this->db->update('subscribers_customers_ftth_all_expired', $array);
      if ($this->db->affected_rows() > 0) {
        $this->insertFtthCustomerExpired();
      }
    } else {
      $this->db->insert('subscribers_customers_ftth_all_expired', $array);
      if ($this->db->affected_rows() > 0) {
        $this->insertFtthCustomerExpired();
      }
    }
  }


  // this for all customers
  public function insertFtthCustomerExpired()
  {
    $this->db->query('TRUNCATE TABLE ftth_customers_expired_table');
    $sql = "
        INSERT INTO `ftth_customers_expired_table` (
            `updated_at`, `username`, `zone`, `fat`, `board`, `hasActiveSession`, `customer_displayValue`, 
            `customer_id`, `expires`, `ipAddress`
        )
        SELECT 
            `subscribers_customers_ftth_all_expired`.`updated_at`,
            `jt`.`username`,
            SUBSTRING_INDEX(SUBSTRING_INDEX(`jt`.`username`, '-', 1), '-', -(1)) AS `zone`,
            SUBSTRING_INDEX(SUBSTRING_INDEX(`jt`.`username`, '-', 2), '-', -(1)) AS `fat`,
            SUBSTRING_INDEX(`jt`.`username`, '-', -(1)) AS `board`,
            `jt`.`hasActiveSession`,
            `jt`.`customer_displayValue`,
            `jt`.`customer_id`,
            `jt`.`expires`,
            `jt`.`ipAddress`
        FROM 
            `subscribers_customers_ftth_all_expired`
        JOIN 
            JSON_TABLE(
                `subscribers_customers_ftth_all_expired`.`data`, 
                '$[*]' 
                COLUMNS (
                    `username` VARCHAR(255) CHARACTER SET utf8mb4 PATH '$.username',
                    `hasActiveSession` TINYINT(1) PATH '$.hasActiveSession',
                    `customer_displayValue` VARCHAR(255) CHARACTER SET utf8mb4 PATH '$.customer.displayValue',
                    `customer_id` VARCHAR(255) CHARACTER SET utf8mb4 PATH '$.customer.id',
                    `expires` DATETIME PATH '$.expires',
                    `ipAddress` VARCHAR(255) CHARACTER SET utf8mb4 PATH '$.ipAddress'
                )
            ) AS `jt`;
    ";

    $this->db->query($sql);

    $this->insertMutedNot();
  }

  public function getFtthCustomersExpired()
  {
    $page = $this->input->get('page') ?? 1;
    $limit = $this->input->get('perPage') ?? 10;
    $zoneId = $this->input->get('zoneId') ?? null;
    $hasActiveSession = $this->input->get('hasActiveSession') ?? null;
    $search = $this->input->get('q') ?? null;
    $mobile = $this->input->get('mobile') ?? null;
    $startDate = $this->input->get('startDate') ?? null;
    $endDate = $this->input->get('endDate') ?? null;

    $searchParams = [
      'search' => $search,
      'mobile' => $mobile,
      'startDate' => $startDate,
      'endDate' => $endDate
    ];

    $offset = ($page - 1) * $limit;

    $query = $this->db->select("ftth_customers_expired_table.customer_id ,customer_displayValue , username, hasActiveSession , updated_at,subscriber_info_table.mobile, 
        CONVERT_TZ(ftth_customers_expired_table.expires, '+00:00', '+03:00') AS converted_expires,
        ftth_customers_expired_table.expires,
")
      ->from('ftth_customers_expired_table')
      ->join('subscriber_info_table', 'subscriber_info_table.customer_id = ftth_customers_expired_table.customer_id', 'left')
      ->limit($limit, $offset);

    if ($zoneId) {
      $this->db->where('ftth_customers_expired_table.zone', $zoneId);
    }

    if ($search) {
      $this->db->like("CONVERT(customer_displayValue USING utf8mb4)", $search, 'both');
      $this->db->or_like('username', $search);
    }

    if ($hasActiveSession !== null) {
      $this->db->where('ftth_customers_expired_table.hasActiveSession', $hasActiveSession);
    }

    if ($mobile) {
      $this->db->where('subscriber_info_table.mobile', $mobile);
    }
    if ($startDate !== null && $endDate !== null) {
      $this->db->where("DATE(ftth_customers_expired_table.expires) BETWEEN '$startDate' AND '$endDate'");
    }

    $query = $this->db->get();
    $data = $query->result();
    $total = $this->getFtthCustomersExpiredCount($zoneId, $searchParams, $hasActiveSession);

    $updated_at = $query->row() ? $query->row()->updated_at : null;

    return ['data' => $data, 'total' => $total, 'updated_at' => $updated_at];

  }

  public function getFtthCustomersExpiredCount($zoneId, $searchParams, $hasActiveSession = null)
  {
    $query = $this->db->select('COUNT(*) as count')
      ->from('ftth_customers_expired_table')
      ->join('subscriber_info_table', 'subscriber_info_table.customer_id = ftth_customers_expired_table.customer_id', 'left');

    $search = $searchParams['search'] ?? null;
    $mobile = $searchParams['mobile'] ?? null;
    $startDate = $searchParams['startDate'] ?? null;
    $endDate = $searchParams['endDate'] ?? null;

    if ($zoneId) {
      $this->db->where('ftth_customers_expired_table.zone', $zoneId);
    }

    if ($search) {
      $this->db->group_start();
      $this->db->like("ftth_customers_expired_table.customer_displayValue", $search);
      $this->db->or_like('ftth_customers_expired_table.username', $search);
      $this->db->group_end();
    }

    if ($mobile) {
      $this->db->where('subscriber_info_table.mobile', $mobile);
    }

    if ($hasActiveSession !== null) {
      $this->db->where('ftth_customers_expired_table.hasActiveSession', (int) $hasActiveSession);
    }

    if ($startDate !== null && $endDate !== null) {
      $this->db->where("DATE(ftth_customers_expired_table.expires) BETWEEN '$startDate' AND '$endDate'");

    }

    $result = $this->db->get()->row();
    return $result ? $result->count : 0;

  }

  public function insertFtthPrimaryInformation()
  {
    $this->db->query('TRUNCATE TABLE ftth_primaty_informations_table');
    $sql = "
        INSERT INTO `ftth_primaty_informations_table` (
            `updated_at`, `username`, `zone`, `fat`, `board`, `hasActiveSession`, `customer_displayValue`, 
            `customer_id`, `expires`, `ipAddress`
        )
        SELECT 
            `subscribers_customers_ftth`.`updated_at`,
            `jt`.`username`,
            SUBSTRING_INDEX(SUBSTRING_INDEX(`jt`.`username`, '-', 1), '-', -(1)) AS `zone`,
            SUBSTRING_INDEX(SUBSTRING_INDEX(`jt`.`username`, '-', 2), '-', -(1)) AS `fat`,
            SUBSTRING_INDEX(`jt`.`username`, '-', -(1)) AS `board`,
            `jt`.`hasActiveSession`,
            `jt`.`customer_displayValue`,
            `jt`.`customer_id`,
            `jt`.`expires`,
            `jt`.`ipAddress`
        FROM 
            `subscribers_customers_ftth`
        JOIN 
            JSON_TABLE(
                `subscribers_customers_ftth`.`data`, 
                '$[*]' 
                COLUMNS (
                    `username` VARCHAR(255) CHARACTER SET utf8mb4 PATH '$.username',
                    `hasActiveSession` TINYINT(1) PATH '$.hasActiveSession',
                    `customer_displayValue` VARCHAR(255) CHARACTER SET utf8mb4 PATH '$.customer.displayValue',
                    `customer_id` VARCHAR(255) CHARACTER SET utf8mb4 PATH '$.customer.id',
                    `expires` DATETIME PATH '$.expires',
                    `ipAddress` VARCHAR(255) CHARACTER SET utf8mb4 PATH '$.ipAddress'
                )
            ) AS `jt`;
    ";

    $this->db->query($sql);

    $this->db->query("        INSERT INTO `ftth_primaty_informations_table` (
            `updated_at`, `username`, `zone`, `fat`, `board`, `hasActiveSession`, `customer_displayValue`, 
            `customer_id`, `expires`, `ipAddress`
        )
        VALUES (
            NOW(), 'test-dummy-01', 'TestZone', 'TestFAT', 'TestBoard', 0, 'test_dummy_01', 
            '54871900', NOW() + INTERVAL 2 HOUR, '192.168.0.1'
        )");

    $this->insertMutedNot();
  }

  public function getZonesSelect()
  {
    $query = $this->db->query("
        
     SELECT 
    SUBSTRING_INDEX(jt.zoneId, '-', 1) AS value,
    SUBSTRING_INDEX(jt.zoneId, '-', 1) AS label
FROM 
    zones_ftth
JOIN 
    JSON_TABLE(
        zones_ftth.data, 
        '$[*]' 
        COLUMNS (
            `zoneId` VARCHAR(50) CHARACTER SET utf8mb4 PATH '$.displayValue'
        )
    ) AS jt;
            
            
            ");

    return $query->result();
  }

  public function getDataByZoneId($zoneId)
  {
    $query = $this->db->query("
        SELECT 
        fat as FAT,
        COUNT(*) as totalCustomers,
        SUM(IF(hasActiveSession = 1, 1, 0)) as activeCustomers
        FROM ftth_primaty_informations 
        WHERE zone = '$zoneId'
    ");

    return $query->result();
  }

  public function getCustomerByFAT($FAT)
  {

    $query = $this->db->query("
        SELECT * FROM ftth_primaty_informations 
        WHERE fat = '$FAT'
        ");
    return $query->result();
  }

  public function getCustomerInformation($id)
  {
    $query = $this->db->select('mobile')->from('view_subscriber_info')->where('id', $id)->get();
    return $query->row(); // Correct method to fetch a single row
  }


  public function getZonesNotWorking()
  {
    $page = $this->input->get('page') ?? 1;
    $limit = $this->input->get('perPage') ?? 10;
    $zoneId = $this->input->get('zoneId') ?? null;
    $FAT = $this->input->get('FAT') ?? null;

    $offset = ($page - 1) * $limit;
    $this->db->select('
    
updated_at,
    zone,
    fat AS FAT,
    COUNT(*) AS total_customers,
       COUNT(*) AS customers_not_working_count,
    GROUP_CONCAT(username) AS username
    ');

    $this->db->from('spaceLight.ftth_primaty_informations');
    $this->db->group_by(['zone', 'fat']);

    // Add HAVING clause equivalent
    $this->db->having('SUM(hasActiveSession) = 0');

    if ($zoneId) {
      $this->db->where('zone', $zoneId);
    }

    if ($FAT) {
      $this->db->where('fat', $FAT);
    }

    $query = $this->db->limit($limit, $offset)->get();

    $data = $query->result();
    $updated_at = $data[0]->updated_at;


    return ['data' => $query->result(), 'updated_at' => $updated_at];

  }

  public function getCustomersNotWorkingCuzZone()
  {
    $page = $this->input->get('page') ?? 1;
    $limit = $this->input->get('perPage') ?? 10;
    $zoneId = $this->input->get('zoneId') ?? null;
    $FAT = $this->input->get('FAT') ?? null;
    $search = $this->input->get('q') ?? null;

    $offset = ($page - 1) * $limit;

    // Subquery for inactive FATs
    $subquery = $this->db->select('zone, fat')
      ->from('ftth_primaty_informations_table AS sub')
      ->group_by(['zone', 'fat'])
      ->having('SUM(hasActiveSession) = 0')
      ->get_compiled_select();

    // Main query
    $this->db->select('zone, fat, customer_displayValue, username, updated_at, main.customer_id ,subscriber_info_table.mobile')
      ->from('ftth_primaty_informations_table AS main')
      ->join('subscriber_info_table', 'subscriber_info_table.customer_id = main.customer_id', 'left')
      ->where("(main.zone, main.fat) IN ($subquery)", null, false);

    if ($zoneId) {
      $this->db->where('main.zone', $zoneId);
    }

    if ($FAT) {
      $this->db->where('main.fat', $FAT);
    }

    if ($search) {
      $this->db->like("CONVERT(main.customer_displayValue USING utf8mb4)", $search, 'both');
      $this->db->or_like('main.username', $search);
    }

    $this->db->limit($limit, $offset);
    $data = $this->db->get()->result();

    return [
      'data' => $data,
      'total' => $this->getCustomersNotWorkingCuzZoneCount($zoneId, $search, $FAT),
      'updated_at' => $data[0]->updated_at ?? null
    ];
  }

  public function getCustomersNotWorkingCuzZoneCount($zoneId = null, $search = null, $FAT = null)
  {
    $subquery = $this->db->select('zone, fat')
      ->from('ftth_primaty_informations_table AS sub')
      ->group_by(['zone', 'fat'])
      ->having('SUM(hasActiveSession) = 0')
      ->get_compiled_select();

    $this->db->from('ftth_primaty_informations_table AS main')
      ->where("(main.zone, main.fat) IN ($subquery)", null, false);

    if ($zoneId) {
      $this->db->where('main.zone', $zoneId);
    }

    if ($FAT) {
      $this->db->where('main.fat', $FAT);
    }

    if ($search) {
      $this->db->like("CONVERT(main.customer_displayValue USING utf8mb4)", $search, 'both');
      $this->db->or_like('main.username', $search);
    }

    return $this->db->count_all_results();
  }

  public function getZonesOptionsNotWorking()
  {

    $this->db->select('    
        zone AS label,
        zone AS value
        ');

    $this->db->from('ftth_primaty_informations');
    $this->db->group_by(['zone', 'fat']);

    // Add HAVING clause equivalent
    $this->db->having('SUM(hasActiveSession) = 0');

    $query = $this->db->get();

    return $query->result();
  }





  public function getZonesNotWorkingCount()
  {
    $zoneId = $this->input->get('zoneId') ?? null;
    $FAT = $this->input->get('FAT') ?? null;

    $totalQuery = clone $this->db;
    $totalQuery->select('        
        updated_at,
    zone,
    fat AS FAT,
    COUNT(*) AS total_customers,
       COUNT(*) AS customers_not_working_count,
    GROUP_CONCAT(username) AS username
')
      ->from('spaceLight.ftth_primaty_informations')
      ->group_by(['zone', 'fat'])
      ->having('SUM(hasActiveSession) = 0');
    if ($zoneId) {
      $totalQuery->where('zone', $zoneId);
    }

    if ($FAT) {
      $totalQuery->where('fat', $FAT);
    }

    $total = $totalQuery->get()->num_rows();
    return $total;
  }

  public function insertCustomerInforamtionData($data)
  {
    $array = array(
      'id' => 1,
      'data' => $data,
      'updated_at' => date('Y-m-d H:i:s'),
    );
    $this->db->where('id', 1);
    $query = $this->db->get('subscribers_customers_information_ftth');

    if ($query->num_rows() > 0) {
      $this->db->where('id', 1);
      $this->db->update('subscribers_customers_information_ftth', $array);
      if ($this->db->affected_rows() > 0) {
        $this->insertFtthCustomerInformation();
      }
    } else {
      $this->db->insert('subscribers_customers_information_ftth', $array);
      if ($this->db->affected_rows() > 0) {
        $this->insertFtthCustomerInformation();
      }
    }
  }

  public function insertFtthCustomerInformation()
  {
    $this->db->query('TRUNCATE TABLE subscriber_info_table');

    $sql = "
INSERT IGNORE INTO `subscriber_info_table` (`id`, `user_name`, `mobile`, `customer_id`)
SELECT 
    `jt`.`id` AS `id`,
    `jt`.`user_name` AS `user_name`,
    `jt`.`mobile` AS `mobile`,
    `jt`.`customer_id` AS `customer_id`
FROM 
    `subscribers_customers_information_ftth`
JOIN 
    JSON_TABLE(
        `subscribers_customers_information_ftth`.`data`, 
        '$[*]' 
        COLUMNS (
            `id` VARCHAR(255) CHARACTER SET utf8mb4 PATH '$.self.id',
            `user_name` VARCHAR(255) CHARACTER SET utf8mb4 PATH '$.self.displayValue',
            `mobile` VARCHAR(255) CHARACTER SET utf8mb4 PATH '$.primaryContact.mobile',
            `customer_id` VARCHAR(55) CHARACTER SET utf8mb4 PATH '$.self.id'
        )
    ) `jt`;

    ";

    $this->db->query($sql);

    $dummy_sql = "
 INSERT INTO `subscriber_info_table` ( `user_name`, `mobile`, `customer_id`)
        VALUES ('test_dummy_01', '+9647703460461', '54871900');
";
    $this->db->query($dummy_sql);

    $this->insertMutedNot();
  }

  public function insertZonesData($data)
  {
    $array = array(
      'id' => 1,
      'data' => $data
    );
    $this->db->where('id', 1);
    $query = $this->db->get('zones_ftth');

    if ($query->num_rows() > 0) {
      $this->db->where('id', 1);
      $this->db->update('zones_ftth', $array);
      if ($this->db->affected_rows() > 0) {
        $this->insertMutedNot();
      }
    } else {
      $this->db->insert('zones_ftth', $array);
      if ($this->db->affected_rows() > 0) {
        $this->insertMutedNot();
      }
    }
  }

  public function getCustomerDataActiveFalse()
  {
    // Build the query using CodeIgniter's query builder or raw query
    $query = $this->db->query("
        SELECT 
            jt.customer_displayValue,
            jt.username,
            jt.zone_displayValue,
            jt.expires,
            subscribers_customers_ftth.updated_at
        FROM subscribers_customers_ftth,
        JSON_TABLE(subscribers_customers_ftth.data, '$[*]' 
            COLUMNS (
                customer_displayValue VARCHAR(255) PATH '$.customer.displayValue',
                username VARCHAR(255) PATH '$.username',
                zone_displayValue VARCHAR(255) PATH '$.zone.displayValue',
                expires VARCHAR(255) PATH '$.expires',
                hasActiveSession BOOLEAN PATH '$.hasActiveSession'
            )
        ) AS jt
        WHERE jt.hasActiveSession = true
        AND subscribers_customers_ftth.id = 1;
    ");

    // Get results
    return $query->result();
  }
  public function getCustomerDataExpiringSoon()
  {
    $page = $this->input->get('page') ?? 1;
    $limit = $this->input->get('perPage') ?? 10;
    $offset = ($page - 1) * $limit;
    $isMessageSent = $this->input->get('isMessageSent'); // Get filter from the request

    // Start building the base query
    $baseQuery = "
          SELECT  
              jt.customer_displayValue, 
              jt.username, 
              jt.zone AS zone_displayValue, 
              jt.expires, 
              jt.updated_at, 
              CONVERT_TZ(jt.expires, '+00:00', '+03:00') AS converted_expires, 
              subscriber_info_table.mobile,
              CASE 
                  WHEN EXISTS (
                      SELECT 1 
                      FROM subscribers_whatsapp_message_send_log AS log
                      WHERE log.customer_id = jt.customer_id
                      AND log.send_date BETWEEN jt.expires AND jt.expires + INTERVAL 24 HOUR
                      AND MONTH(log.send_date) = MONTH(jt.expires)
                      AND YEAR(log.send_date) = YEAR(jt.expires)
                  ) THEN 'Message Sent'
                  ELSE 'No Message Sent'
              END AS message_status,
              CASE 
                  WHEN EXISTS (
                      SELECT 1 
                      FROM subscribers_whatsapp_message_send_log AS log
                      WHERE log.customer_id = jt.customer_id
                      AND DAY(log.expires) = DAY(jt.expires)
                      AND MONTH(log.expires) = MONTH(jt.expires)
                      AND YEAR(log.expires) = YEAR(jt.expires)
                      AND log.status = 'success'
                  ) THEN 1
                  ELSE 0
              END AS is_message_sent
          FROM  
              ftth_primaty_informations_table AS jt 
          LEFT JOIN subscriber_info_table ON jt.customer_id = subscriber_info_table.customer_id 
          WHERE  
              CONVERT_TZ(jt.expires, '+00:00', '+03:00') BETWEEN NOW() AND NOW() + INTERVAL 24 HOUR 
      ";

    // Add dynamic filter for is_message_sent
    if ($isMessageSent !== null) {
      $baseQuery .= " AND (";
      if ($isMessageSent == 1) {
        $baseQuery .= "
                  EXISTS (
                      SELECT 1 
                      FROM subscribers_whatsapp_message_send_log AS log
                      WHERE log.customer_id = jt.customer_id
                      AND DAY(log.expires) = DAY(jt.expires)
                      AND MONTH(log.expires) = MONTH(jt.expires)
                      AND YEAR(log.expires) = YEAR(jt.expires)
                      AND log.status = 'success'
                  )
              ";
      } else {
        $baseQuery .= "
                  NOT EXISTS (
                      SELECT 1 
                      FROM subscribers_whatsapp_message_send_log AS log
                      WHERE log.customer_id = jt.customer_id
                      AND DAY(log.expires) = DAY(jt.expires)
                      AND MONTH(log.expires) = MONTH(jt.expires)
                      AND YEAR(log.expires) = YEAR(jt.expires)
                      AND log.status = 'success'
                  )
              ";
      }
      $baseQuery .= ")";
    }

    // Build the count query
    $countQuery = "SELECT COUNT(*) AS count FROM ($baseQuery) AS count_query";
    $countResult = $this->db->query($countQuery);
    $totalCount = $countResult->row()->count;

    // Add ordering and pagination to the base query
    $dataQuery = $baseQuery . "
          ORDER BY jt.expires DESC
          LIMIT $limit OFFSET $offset
      ";

    // Execute the data query
    $dataResult = $this->db->query($dataQuery);
    $data = $dataResult->result();

    // Get the updated_at timestamp
    $updated_at = !empty($data) ? $data[0]->updated_at : null;

    // Return both the data and the total count
    return [
      'data' => $data,
      'updated_at' => $updated_at,
      'total' => $totalCount
    ];
  }


  public function getCustomerDataExpiringSoonWithInFourHours()
  {
    $page = $this->input->get('page') ?? 1;
    $limit = $this->input->get('perPage') ?? 10;
    $offset = ($page - 1) * $limit;
    $isMessageSent = $this->input->get('isMessageSent'); // Get filter from the request

    // Start building the base query
    $baseQuery = "
          SELECT  
              jt.customer_displayValue, 
              jt.username, 
              jt.zone AS zone_displayValue, 
              jt.expires, 
              jt.updated_at, 
              CONVERT_TZ(jt.expires, '+00:00', '+03:00') AS converted_expires, 
              subscriber_info_table.mobile,
              CASE 
                  WHEN EXISTS (
                      SELECT 1 
                      FROM subscribers_whatsapp_message_send_log AS log
                      WHERE log.customer_id = jt.customer_id
                      AND log.send_date BETWEEN jt.expires AND jt.expires + INTERVAL 4 HOUR
                      AND MONTH(log.send_date) = MONTH(jt.expires)
                      AND YEAR(log.send_date) = YEAR(jt.expires)
                  ) THEN 'Message Sent'
                  ELSE 'No Message Sent'
              END AS message_status,
              CASE 
                  WHEN EXISTS (
                      SELECT 1 
                      FROM subscribers_whatsapp_message_send_log AS log
                      WHERE log.customer_id = jt.customer_id
                      AND DAY(log.expires) = DAY(jt.expires)
                      AND MONTH(log.expires) = MONTH(jt.expires)
                      AND YEAR(log.expires) = YEAR(jt.expires)
                      AND log.status = 'success'
                  ) THEN 1
                  ELSE 0
              END AS is_message_sent
          FROM  
              ftth_primaty_informations_table AS jt 
          LEFT JOIN subscriber_info_table ON jt.customer_id = subscriber_info_table.customer_id 
          WHERE  
              CONVERT_TZ(jt.expires, '+00:00', '+03:00') BETWEEN NOW() AND NOW() + INTERVAL 4 HOUR 
      ";

    // Add dynamic filter for is_message_sent
    if ($isMessageSent !== null) {
      $baseQuery .= " AND (";
      if ($isMessageSent == 1) {
        $baseQuery .= "
                  EXISTS (
                      SELECT 1 
                      FROM subscribers_whatsapp_message_send_log AS log
                      WHERE log.customer_id = jt.customer_id
                      AND DAY(log.expires) = DAY(jt.expires)
                      AND MONTH(log.expires) = MONTH(jt.expires)
                     
                      AND log.status = 'success'
                  )
              ";
      } else {
        $baseQuery .= "
                  NOT EXISTS (
                      SELECT 1 
                      FROM subscribers_whatsapp_message_send_log AS log
                      WHERE log.customer_id = jt.customer_id
                      AND DAY(log.expires) = DAY(jt.expires)
                      AND MONTH(log.expires) = MONTH(jt.expires)
                      AND YEAR(log.expires) = YEAR(jt.expires)
                      AND log.status = 'success'
                  )
              ";
      }
      $baseQuery .= ")";
    }

    // Build the count query
    $countQuery = "SELECT COUNT(*) AS count FROM ($baseQuery) AS count_query";
    $countResult = $this->db->query($countQuery);
    $totalCount = $countResult->row()->count;

    // Add ordering and pagination to the base query
    $dataQuery = $baseQuery . "
          ORDER BY jt.expires DESC
          LIMIT $limit OFFSET $offset
      ";

    // Execute the data query
    $dataResult = $this->db->query($dataQuery);
    $data = $dataResult->result();

    // Get the updated_at timestamp
    $updated_at = !empty($data) ? $data[0]->updated_at : null;

    // Return both the data and the total count
    return [
      'data' => $data,
      'updated_at' => $updated_at,
      'total' => $totalCount
    ];
  }






  // customers will expire within 24 hours
  public function getExpiringCustomerDataWithoutRecentMessage()
  {
    // Build the query to filter by expiration time (expires within 24 hours or less)
    $query = $this->db->query("
 SELECT 
    jt.customer_displayValue,
    jt.username,
    jt.zone AS zone_displayValue,
    jt.expires,
    jt.updated_at,
    jt.customer_id,
    CONVERT_TZ(jt.expires, '+00:00', '+03:00') AS converted_expires,
    subscriber_info_table.mobile,
    IFNULL(log.message_status, 'No Message Sent') AS message_status,
    IFNULL(log.is_message_sent, 0) AS is_message_sent
FROM
    ftth_primaty_informations_table AS jt
LEFT JOIN
    subscriber_info_table ON jt.customer_id = subscriber_info_table.customer_id
LEFT JOIN (
    SELECT 
        log.customer_id,
        1 AS is_message_sent,
        'Message Sent' AS message_status,
        DAY(log.expires) AS expires_day,
        log.expires AS expires,
        MONTH(log.send_date) AS send_month,
        YEAR(log.send_date) AS send_year
    FROM 
        subscribers_whatsapp_message_send_log AS log
        JOIN notify_whatsapp_type AS notify ON log.notify_type = notify.id
    WHERE 
        notify.hash_type = 111
) AS log ON log.customer_id = jt.customer_id 
  AND log.expires = jt.expires
  
WHERE
    jt.expires BETWEEN NOW() AND NOW() + INTERVAL 24 HOUR
    AND log.customer_id IS NULL
ORDER BY jt.expires DESC;


        ");

    // Get the result
    $data = $query->result();
    return $data;
  }
  // customers will expire within 24 hours and sendMessagewhatsApp failed
  public function get24hExpiringCustomersWithFailedMessages()
  {
    // Build the query to filter by expiration time (expires within 24 hours or less)
    $query = $this->db->query("
SELECT 
    jt.customer_displayValue,
    jt.username,
    jt.zone AS zone_displayValue,
    jt.expires,
    jt.updated_at,
    jt.customer_id,
    CONVERT_TZ(jt.expires, '+00:00', '+03:00') AS converted_expires,
    subscriber_info_table.mobile,
    IFNULL(log.message_status, 'No Message Sent') AS message_status,
    IFNULL(log.is_message_sent, 0) AS is_message_sent
FROM
    ftth_primaty_informations_table AS jt
LEFT JOIN
    subscriber_info_table ON jt.customer_id = subscriber_info_table.customer_id
LEFT JOIN (
    SELECT 
        log.customer_id,
        1 AS is_message_sent,
        'Message Sent' AS message_status,
        DAY(log.expires) AS expires_day,
        log.expires AS expires,
        MONTH(log.send_date) AS send_month,
        YEAR(log.send_date) AS send_year
    FROM 
        subscribers_whatsapp_message_send_log AS log
    JOIN 
        notify_whatsapp_type AS notify ON log.notify_type = notify.id
    WHERE 
        notify.hash_type = 111
        AND log.status IN ('failed', 'unknown_error')
) AS log ON log.customer_id = jt.customer_id 
   AND log.expires = jt.expires

LEFT JOIN (

SELECT 
        sms_log.customer_id,
        1 AS is_message_sent,
        'Message Sent' AS message_status,
        DAY(sms_log.expires) AS expires_day,
        sms_log.expires AS expires,
        MONTH(sms_log.send_date) AS send_month,
        YEAR(sms_log.send_date) AS send_year
    FROM 
        subscribers_sms_message_send_log AS sms_log
    JOIN 
        notify_sms_type AS sms_notify ON sms_log.notify_type = sms_notify.id
    WHERE 
        sms_notify.hash_type = 111
        
) AS sms_log ON sms_log.customer_id = jt.customer_id 
   AND sms_log.expires = jt.expires

WHERE
    jt.expires BETWEEN NOW() AND NOW() + INTERVAL 24 HOUR
    AND log.customer_id IS NOT NULL
    AND sms_log.customer_id IS NULL
        GROUP BY subscriber_info_table.mobile

ORDER BY 
    jt.expires DESC
    ;

            
        ");

    // Get the result
    $data = $query->result();
    return $data;
  }

  // customers will expire within 4 hours
  public function getExpiringWithinFourHoursCustomerDataWithoutRecentMessage()
  {
    // Build the query to filter by expiration time (expires within 24 hours or less)
    $query = $this->db->query("
 SELECT 
    jt.customer_displayValue,
    jt.username,
    jt.zone AS zone_displayValue,
    jt.expires,
    jt.updated_at,
    jt.customer_id,
    CONVERT_TZ(jt.expires, '+00:00', '+03:00') AS converted_expires,
    subscriber_info_table.mobile,
    IFNULL(log.message_status, 'No Message Sent') AS message_status,
    IFNULL(log.is_message_sent, 0) AS is_message_sent
FROM
    ftth_primaty_informations_table AS jt
LEFT JOIN
    subscriber_info_table ON jt.customer_id = subscriber_info_table.customer_id
LEFT JOIN (
    SELECT 
        log.customer_id,
        1 AS is_message_sent,
        'Message Sent' AS message_status,
        DAY(log.expires) AS expires_day,
        log.expires AS expires,
        MONTH(log.send_date) AS send_month,
        YEAR(log.send_date) AS send_year
    FROM 
        subscribers_whatsapp_message_send_log AS log
        JOIN notify_whatsapp_type AS notify ON log.notify_type = notify.id
    WHERE 
        notify.hash_type = 222
) AS log ON log.customer_id = jt.customer_id 
   AND log.expires = jt.expires
WHERE
    jt.expires BETWEEN NOW() AND NOW() + INTERVAL 4 HOUR
    AND log.customer_id IS NULL
    GROUP BY subscriber_info_table.mobile
ORDER BY jt.expires DESC;             

");

    // Get the result
    $data = $query->result();
    return $data;
  }

  // 4 hours
  public function get4hExpiringCustomersWithFailedMessages()
  {
    // Build the query to filter by expiration time (expires within 4 hours or less)
    $query = $this->db->query("
SELECT 
    jt.customer_displayValue,
    jt.username,
    jt.zone AS zone_displayValue,
    jt.expires,
    jt.updated_at,
    jt.customer_id,
    CONVERT_TZ(jt.expires, '+00:00', '+03:00') AS converted_expires,
    subscriber_info_table.mobile,
    IFNULL(log.message_status, 'No Message Sent') AS message_status,
    IFNULL(log.is_message_sent, 0) AS is_message_sent
FROM
    ftth_primaty_informations_table AS jt
LEFT JOIN
    subscriber_info_table ON jt.customer_id = subscriber_info_table.customer_id
LEFT JOIN (
    SELECT 
        log.customer_id,
        1 AS is_message_sent,
        'Message Sent' AS message_status,
        log.expires AS expires,
        DAY(log.expires) AS expires_day,

        MONTH(log.send_date) AS send_month,
        YEAR(log.send_date) AS send_year
    FROM 
        subscribers_whatsapp_message_send_log AS log
    JOIN 
        notify_whatsapp_type AS notify ON log.notify_type = notify.id
    WHERE 
        notify.hash_type = 222
        AND log.status IN ('failed', 'unknown_error')
) AS log ON log.customer_id = jt.customer_id 
    AND log.expires = jt.expires
WHERE
    jt.expires BETWEEN NOW() AND NOW() + INTERVAL 4 HOUR
    AND log.customer_id IS NOT NULL -- Remove this if you want to include all records
ORDER BY 
    jt.expires DESC;

            
        ");

    // Get the result
    $data = $query->result();
    return $data;
  }

  public function getCustomersExpired()
  {
    $page = $this->input->get('page') ?? 1;
    $limit = $this->input->get('perPage') ?? 10;
    $offset = ($page - 1) * $limit;

    $query = $this->db->query("
            SELECT 
                jt.customer_displayValue,
                jt.customer_id,
                jt.username,
                jt.zone AS zone_displayValue,
                jt.expires,
              CONVERT_TZ(jt.expires, '+00:00', '+03:00') AS converted_expires, 
                jt.updated_at,
                subscriber_info_table.mobile

                FROM ftth_primaty_informations_table AS jt
     LEFT JOIN subscriber_info_table  ON jt.customer_id = subscriber_info_table.customer_id

WHERE 
    CONVERT_TZ(jt.expires, '+00:00', '+03:00') < NOW()
                        LIMIT $limit OFFSET $offset

            ;
        ");
    $data = $query->result();
    if (!empty($data)) {
      $updated_at = $data[0]->updated_at ?? null; // Add a null coalescing operator to avoid undefined property notice
      return ['data' => $data, 'updated_at' => $updated_at];
    }

    return ['data' => [], 'updated_at' => null];

  }
  public function getCusotmerExpiredCount()
  {
    $query = $this->db->query("
        SELECT COUNT(*) AS count
       FROM ftth_primaty_informations_table AS jt
     LEFT JOIN subscriber_info_table  ON jt.customer_id = subscriber_info_table.customer_id

WHERE 
    CONVERT_TZ(jt.expires, '+00:00', '+03:00') < NOW()
        ");
    return $query->row()->count;
  }

  public function getAllCustomersNotWorking()
  {
    $page = $this->input->get('page') ?? 1;
    $limit = $this->input->get('perPage') ?? 10;
    $zoneId = $this->input->get('zoneId') ?? null;
    $search = $this->input->get('q') ?? null;

    $offset = ($page - 1) * $limit;

    $query = $this->db->select('customer_FTTH_not_working.customer_id ,customer_displayValue , username, updated_at, subscriber_info_table.mobile')->from('customer_FTTH_not_working')
      ->join('subscriber_info_table', 'subscriber_info_table.customer_id = customer_FTTH_not_working.customer_id', 'left')
      ->limit($limit, $offset);


    if ($zoneId) {
      $this->db->where('customer_FTTH_not_working.zone', $zoneId);
    }

    if ($search) {
      $this->db->like("CONVERT(customer_displayValue USING utf8mb4)", $search, 'both');
      $this->db->or_like('username', $search);
    }

    $query = $this->db->get();
    $data = $query->result();
    $total = $this->getCustomersNotWorkingCount($zoneId, $search);
    $updated_at = $query->row()->updated_at;

    return ['data' => $data, 'total' => $total, 'updated_at' => $updated_at];
  }

  private function getCustomersNotWorkingCount($zoneId = null, $search = null)
  {
    $query = $this->db->select('COUNT(*) as count')->from('customer_FTTH_not_working');

    if ($zoneId) {
      $this->db->where('customer_FTTH_not_working.zone', $zoneId);
    }

    if ($search) {
      $this->db->like("CONVERT(customer_displayValue USING utf8mb4)", $search, 'both');
      $this->db->or_like('username', $search);
    }

    $query = $this->db->get();
    $total = $query->row()->count;

    return $total;
  }

  public function getCustomersNotWorkingZonesOptions()
  {

    $this->db->select('customer_FTTH_not_working.zone as label , customer_FTTH_not_working.zone as value')->from('customer_FTTH_not_working')
      ->join('subscriber_info_table', 'subscriber_info_table.customer_id = customer_FTTH_not_working.customer_id', 'left');


    $query = $this->db->get();
    $data = $query->result();

    return $data;
  }

  public function getAllCustomersWorkingAndNotWorking()
  {
    $page = $this->input->get('page') ?? 1;
    $limit = $this->input->get('perPage') ?? 10;
    $zoneId = $this->input->get('zoneId') ?? null;
    $fatId = $this->input->get('FAT') ?? null;
    $hasActiveSession = $this->input->get('hasActiveSession') ?? null;
    $search = $this->input->get('q') ?? null;
    $mobile = $this->input->get('mobile') ?? null;

    $searchParams = [
      'search' => $search,
      'mobile' => $mobile,
      'fatId' => $fatId,
    ];

    $offset = ($page - 1) * $limit;

    $query = $this->db->select('ftth_primaty_informations_table.customer_id ,customer_displayValue , username, hasActiveSession , updated_at,subscriber_info_table.mobile ')
      ->from('ftth_primaty_informations_table')
      ->join('subscriber_info_table', 'subscriber_info_table.customer_id = ftth_primaty_informations_table.customer_id', 'left')
      ->limit($limit, $offset);

    if ($zoneId) {
      $this->db->where('ftth_primaty_informations_table.zone', $zoneId);
    }

    if ($fatId) {
      $this->db->where('ftth_primaty_informations_table.fat', $fatId);
    }

    if ($search) {
      $this->db->like("CONVERT(customer_displayValue USING utf8mb4)", $search, 'both');
      $this->db->or_like('username', $search);
    }

    if ($hasActiveSession !== null) {
      $this->db->where('ftth_primaty_informations_table.hasActiveSession', $hasActiveSession);
    }

    if ($mobile) {
      $this->db->where('subscriber_info_table.mobile', $mobile);
    }

    $query = $this->db->get();
    $data = $query->result();
    $total = $this->getCustomersWorkingAndNotWorkingCount($zoneId, $searchParams, $hasActiveSession);

    $updated_at = $query->row() ? $query->row()->updated_at : null;

    return ['data' => $data, 'total' => $total, 'updated_at' => $updated_at];
  }



  public function getCustomersWorkingAndNotWorkingCount($zoneId = null, $searchParams = [], $hasActiveSession = null)
  {
    $query = $this->db->select('COUNT(*) as count')
      ->from('ftth_primaty_informations_table')
      ->join('subscriber_info_table', 'subscriber_info_table.customer_id = ftth_primaty_informations_table.customer_id', 'left');

    $search = $searchParams['search'] ?? null;
    $mobile = $searchParams['mobile'] ?? null;
    $fatId = $searchParams['fatId'] ?? null;

    if ($zoneId) {
      $this->db->where('ftth_primaty_informations_table.zone', $zoneId);
    }
    if ($fatId) {
      $this->db->where('ftth_primaty_informations_table.fat', $fatId);
    }


    if ($search) {
      $this->db->group_start();
      $this->db->like("ftth_primaty_informations_table.customer_displayValue", $search);
      $this->db->or_like('ftth_primaty_informations_table.username', $search);
      $this->db->group_end();
    }

    if ($mobile) {
      $this->db->where('subscriber_info_table.mobile', $mobile);
    }

    if ($hasActiveSession !== null) {
      $this->db->where('ftth_primaty_informations_table.hasActiveSession', (int) $hasActiveSession);
    }

    $result = $this->db->get()->row();
    return $result ? $result->count : 0;
  }


  public function getCustomerPowerAndStatusById($username)
  {
    $token = $this->getTokenFTTH();
    $response = $this->makeGetPowerAndStatusRequest($username, $token);
    return $response;

  }


  public function getCustomerTickets($params)
  {
    $token = $this->getTokenFTTH();
    $response = $this->makeGetTicketsRequest($params, $token);
    return $response;
  }

  public function getTokenFTTH()
  {
    $this->db->select("token")->from('ftth_auth')->where('id', 1);
    $query = $this->db->get();
    return $query->row()->token;

  }

  public function checkUpdatedWithinTime($table, $id, $timeLimitInSeconds, $column = 'updated_at')
  {
    $this->db->select($column)->from($table)->where('id', $id);
    $query = $this->db->get();

    if ($query->num_rows() > 0) {
      $row = $query->row();
      $updatedAt = strtotime($row->$column); // Convert to timestamp
      $now = time(); // Current timestamp

      return ($now - $updatedAt) <= $timeLimitInSeconds;
    }

    return false;
  }





  private function makeGetPowerAndStatusRequest($username, $token)
  {
    $ch = curl_init();

    $url = "https://api.ftth.iq/api/subscriptions/device/ont?username=$username";

    $headers = [
      "accept: application/json, text/plain, */*",
      "accept-language: ar",
      "authorization: Bearer $token",
      "origin: https://admin.ftth.iq",
      "priority: u=1, i",
      "referer: https://admin.ftth.iq/",
      "sec-ch-ua: \"Google Chrome\";v=\"131\", \"Chromium\";v=\"131\", \"Not_A Brand\";v=\"24\"",
      "sec-ch-ua-mobile: ?0",
      "sec-ch-ua-platform: \"Windows\"",
      "sec-fetch-dest: empty",
      "sec-fetch-mode: cors",
      "sec-fetch-site: same-site",
      "user-agent: Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/131.0.0.0 Safari/537.36",
      "x-accept-fakes: false",
      "x-client-app: 53d57a7f-3f89-4e9d-873b-3d071bc6dd9f",
      "x-user-role: 0"
    ];

    curl_setopt_array($ch, [
      CURLOPT_URL => $url,
      CURLOPT_RETURNTRANSFER => true,
      CURLOPT_HTTPHEADER => $headers,
      CURLOPT_FOLLOWLOCATION => true,
      CURLOPT_CUSTOMREQUEST => "GET",  // You can change this if you need other methods (POST, PUT, etc.)
      CURLOPT_TIMEOUT => 30,  // Timeout for the entire request in seconds
      CURLOPT_CONNECTTIMEOUT => 10,  // Timeout for establishing the connection in seconds
    ]);

    // Execute cURL request
    $response = curl_exec($ch);

    // Check for errors
    if (curl_errno($ch)) {
      // Log the error and return false
      log_message('error', 'CURL error: ' . curl_error($ch));
      var_dump(curl_error($ch));
      curl_close($ch);
      return false;
    }

    // Close cURL session
    curl_close($ch);

    $json_response = json_decode($response, true);

    return $json_response;
  }

  private function makeGetTicketsRequest($params, $token)
  {
    $pageNumber = $params['pageNumber'] ?? 1;
    $pageSize = $params['pageSize'] ?? 10;
    $sortCriteria = $params['sortCriteria'] ?? ['property' => 'createdAt', 'direction' => 'desc'];
    $status = $params['status'] ?? 0;
    $hierarchyLevel = $params['hierarchyLevel'] ?? 0;

    $ch = curl_init();

    $url = "https://api.ftth.iq/api/support/tickets?pageNumber=$pageNumber&pageSize=$pageSize&sortCriteria[property]=$sortCriteria[property]&sortCriteria[direction]=$sortCriteria[direction]&status=$status&hierarchyLevel=$hierarchyLevel";

    $headers = [
      "accept: application/json, text/plain, */*",
      "accept-language: ar",
      "authorization: Bearer $token",
      "origin: https://admin.ftth.iq",
      "priority: u=1, i",
      "referer: https://admin.ftth.iq/",
      "sec-ch-ua: \"Google Chrome\";v=\"131\", \"Chromium\";v=\"131\", \"Not_A Brand\";v=\"24\"",
      "sec-ch-ua-mobile: ?0",
      "sec-ch-ua-platform: \"Windows\"",
      "sec-fetch-dest: empty",
      "sec-fetch-mode: cors",
      "sec-fetch-site: same-site",
      "user-agent: Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/131.0.0.0 Safari/537.36",
      "x-accept-fakes: false",
      "x-client-app: 53d57a7f-3f89-4e9d-873b-3d071bc6dd9f",
      "x-user-role: 0"
    ];

    curl_setopt_array($ch, [
      CURLOPT_URL => $url,
      CURLOPT_RETURNTRANSFER => true,
      CURLOPT_HTTPHEADER => $headers,
      CURLOPT_FOLLOWLOCATION => true,
      CURLOPT_CUSTOMREQUEST => "GET",  // You can change this if you need other methods (POST, PUT, etc.)
      CURLOPT_TIMEOUT => 30,  // Timeout for the entire request in seconds
      CURLOPT_CONNECTTIMEOUT => 10,  // Timeout for establishing the connection in seconds
    ]);

    // Execute cURL request
    $response = curl_exec($ch);

    // Check for errors
    if (curl_errno($ch)) {
      // Log the error and return false
      log_message('error', 'CURL error: ' . curl_error($ch));
      var_dump(curl_error($ch));
      curl_close($ch);
      return false;
    }

    // Close cURL session
    curl_close($ch);
    $json_response = json_decode($response, true);

    return $json_response;

  }










}
