<?php
defined('BASEPATH') or exit('No direct script access allowed');

class TaskChartModel extends CI_Model
{

    public function __construct()
    {
        parent::__construct();
        $this->load->database();
        $this->load->library('Middleware');
    }

    public function getTaskCompletionDetails($user)
    {
        $this->db->select("su.name as user_name, ts.title, TIMESTAMPDIFF(MINUTE, t.accept_date, t.action_date) AS task_completion_minutes, t.accept_date as accept_time, t.action_date as deliver_time")
            ->from('task t')
            ->join('task_status ts', 't.status = ts.hash', 'left')
            ->join('system_users su', 't.accept_user = su.hash')
            ->where('t.isaccept', '1')
            ->where_in('t.status', ['222', '333']);

        $query = $this->db->get();

        return $query->result_array();
    }

    public function getTaskChartPie($project_hash, $user_hash = '', $start_date = '', $end_date = '', $user)
    {
        // Select tasks based on the provided parameters
        $this->db->select('ts.title as status, COUNT(t.id) as count')
            ->from('task t')
            ->join('task_status ts', 't.status = ts.hash', 'left')
            ->join('system_users su', 't.accept_user = su.hash', 'left', false)
            ->where_in('t.status', ['333', '222']); // Filter by status as needed

        // Filter by user hash if provided
        if ($user_hash != '') {
            $this->db->where('t.accept_user', $user_hash);
        }

        // Filter by project hash if provided
        if (!empty($project_hash)) {
            $this->db->where('t.project_hash', $project_hash);
        }

        // Filter by date range if provided
        if (!empty($start_date) && !empty($end_date)) {
            $this->db->where("DATE(t.created_at) BETWEEN '$start_date' AND '$end_date'");
        }

        // Group by task status
        $this->db->group_by('ts.title');

        // Get the task status data
        $query = $this->db->get();
        $result = $query->result_array();

        // Prepare and return the structured result
        return $result;
    }

    // public function getTaskChartData($project_hash, $user_hash = '', $start_date = '', $end_date = '', $user)
    // {
    //     // First query
    //     $this->db->select('t.*')
    //         ->from('project_employees pe')
    //         ->join('project p', 'pe.project_id = p.id')
    //         ->join('task t', 't.project_hash = p.hash')
    //         ->where('pe.employee_id', $user)
    //         ->where_in('pe.role_id', ['1', '2', '3', '4']);

    //     $query1 = $this->db->get();

    //     // Second query
    //     $this->db->select('t.*')
    //         ->from('project_employees pe')
    //         ->join('project p', 'pe.project_id = p.id')
    //         ->join('task t', 't.project_hash = p.hash')
    //         ->where('pe.employee_id', $user)
    //         ->where('pe.role_id', '5');

    //     $query2 = $this->db->get();

    //     // Combine results
    //     $result = array_merge($query1->result_array(), $query2->result_array());

    //     // Structure the result
    //     $structuredResult = [];
    //     foreach ($result as $row) {
    //         // Start building the detailed query for each task
    //         $this->db->select("
    //     TIME_FORMAT(t.action_date, '%h:%i %p') AS time,
    //     ts.title as task_status,
    //     t.title as task_name,
    //     t.start_task,
    //     t.deadline,
    //      GROUP_CONCAT(CASE
    //     WHEN t.is_simple = '1' THEN su2.name
    //     ELSE su.name
    // END SEPARATOR ', ') as name,
    // CASE
    //     WHEN t.is_simple = '1' THEN su2.image_profile
    //     ELSE su.image_profile
    // END as image_profile,
    // CASE
    //     WHEN t.is_simple = '1' THEN su2.hash
    //     ELSE su.hash
    // END as user_hash,
    //     t.accept_date,
    //     t.created_at,
    //     t.is_simple
    // ")
    //             ->from('task t')
    //             ->join('task_status ts', 't.status = ts.hash', 'left')
    //             ->join('task_users tu', 't.id = tu.task_id', 'left')
    //             ->join('system_users su', 't.is_simple = "0" AND t.accept_user = su.hash', 'left', false)
    //             ->join('system_users su2', 't.is_simple != "0" AND tu.user_hash = su2.hash', 'left', false)
    //             ->where('t.hash', $row['hash'])
    //             ->group_by('t.id');

    //         // Apply filters
    //         if (!empty($project_hash)) {
    //             $this->db->where('t.project_hash', $project_hash);
    //         }

    //         // Conditionally apply user hash filters based on task status
    //         if (!empty($user_hash)) {
    //             if ($row['status'] == '111') {
    //                 $this->db->where('tu.user_hash', $user_hash);
    //             } elseif (!in_array($row['status'], ['222', '333'])) {
    //                 $this->db->where('t.accept_user', $user_hash);
    //             }
    //         }

    //         // Apply date range filter if provided
    //         if (!empty($start_date) && !empty($end_date)) {
    //             $this->db->where("DATE(t.created_at) BETWEEN '$start_date' AND '$end_date'");
    //         }

    //         // Group by the task ID for proper aggregation with GROUP_CONCAT
    //         $this->db->group_by('t.id');

    //         $query = $this->db->get();
    //         $taskData = $query->row_array();

    //         // Only add to result if taskData is not null
    //         if ($taskData) {
    //             $structuredResult[] = $taskData;
    //         }
    //     }

    //     return $structuredResult;
    // }

    public function getTaskChartData($project_hash, $user_hash = '', $start_date = '', $end_date = '', $user)
    {

        $this->db->select("
        TIME_FORMAT(t.action_date, '%h:%i %p') AS time,
        ts.title as task_status,
        t.title as task_name,
        t.start_task,
        su.name as name,
        su.image_profile,
        su.hash as user_hash,
        t.hash as hash,
        t.accept_date,
        t.created_at,
        t.is_simple")
            ->from('task t')
            ->join('task_status ts', 't.status = ts.hash', 'left')
            ->join('system_users su', 't.accept_user = su.hash', 'left', false)
            ->where_in('t.status', ['333', '222']);

        if ($user_hash != '') {
            $this->db->where('t.accept_user', $user_hash);
        }

        if (!empty($project_hash)) {
            $this->db->where('t.project_hash', $project_hash);
        }
        if (!empty($start_date) && !empty($end_date)) {
            $this->db->where("DATE(t.created_at) BETWEEN '$start_date' AND '$end_date'");
        }

        $query = $this->db->get();
        return $query->result_array();

        //     // First query
        //     $this->db->select('t.*')
        //         ->from('project_employees pe')
        //         ->join('project p', 'pe.project_id = p.id')
        //         ->join('task t', 't.project_hash = p.hash')
        //         ->where('pe.employee_id', $user)
        //         ->where_in('pe.role_id', ['1', '2', '3', '4']);

        //     $query1 = $this->db->get();

        //     // Second query
        //     $this->db->select('t.*')
        //         ->from('project_employees pe')
        //         ->join('project p', 'pe.project_id = p.id')
        //         ->join('task t', 't.project_hash = p.hash')
        //         ->where('pe.employee_id', $user)
        //         ->where('pe.role_id', '5');

        //     $query2 = $this->db->get();

        //     // Combine results
        //     $result = array_merge($query1->result_array(), $query2->result_array());

        //     // Structure the result
        //     $structuredResult = [];
        //     foreach ($result as $row) {
        //         // Start building the detailed query for each task
        //     $this->db->select("
        //     TIME_FORMAT(t.action_date, '%h:%i %p') AS time,
        //     ts.title as task_status,
        //     t.title as task_name,
        //     t.start_task,
        //     t.deadline,
        //      GROUP_CONCAT(CASE
        //     WHEN t.is_simple = '1' THEN su2.name
        //     ELSE su.name
        // END SEPARATOR ', ') as name,
        // CASE
        //     WHEN t.is_simple = '1' THEN su2.image_profile
        //     ELSE su.image_profile
        // END as image_profile,
        // CASE
        //     WHEN t.is_simple = '1' THEN su2.hash
        //     ELSE su.hash
        // END as user_hash,
        //     t.accept_date,
        //     t.created_at,
        //     t.is_simple
        // ")
        //         ->from('task t')
        //         ->join('task_status ts', 't.status = ts.hash', 'left')
        //         ->join('task_users tu', 't.id = tu.task_id', 'left')
        //         ->join('system_users su', 't.is_simple = "0" AND t.accept_user = su.hash', 'left', false)
        //         ->join('system_users su2', 't.is_simple != "0" AND tu.user_hash = su2.hash', 'left', false)
        //         ->where('t.hash', $row['hash'])
        //         ->group_by('t.id');

        //         // Apply filters
        //         if (!empty($project_hash)) {
        //             $this->db->where('t.project_hash', $project_hash);
        //         }

        //         // Conditionally apply user hash filters based on task status
        //         if (!empty($user_hash)) {
        //             if ($row['status'] == '111') {
        //                 $this->db->where('tu.user_hash', $user_hash);
        //             } elseif (!in_array($row['status'], ['222', '333'])) {
        //                 $this->db->where('t.accept_user', $user_hash);
        //             }
        //         }

        //         // Apply date range filter if provided
        //         if (!empty($start_date) && !empty($end_date)) {
        //             $this->db->where("DATE(t.created_at) BETWEEN '$start_date' AND '$end_date'");
        //         }

        //         // Group by the task ID for proper aggregation with GROUP_CONCAT
        //         $this->db->group_by('t.id');

        //         $query = $this->db->get();
        //         $taskData = $query->row_array();

        //         // Only add to result if taskData is not null
        //         if ($taskData) {
        //             $structuredResult[] = $taskData;
        //         }
        //     }

        //     return $structuredResult;
    }
}
