<?php
class TaskModel extends CI_Model
{
    public function __construct()
    {
        parent::__construct();
        $this->load->database();
    }

    public function add($data)
    {


        $roleCheckPassed = $data["is_simple"] == 0 ?
            $this->checkTaskAgreement($data['sender']) :
            $this->checkSenderRole($data['sender'], $data['project_hash']);
        if (!$roleCheckPassed) {
            return 'ليس لديك صلاحيه لإضافة مهمه';
        }

        $attachments = $data['attachments'];
        $note = $data['note'];
        unset($data['attachments']);
        unset($data['note']);
        $recivers = $data['receiver'];

        // this becuase we change logic and removed column reciever
        unset($data['receiver']);
        $this->db->insert('task', $data);

        $id = $this->db->insert_id();

        foreach ($recivers as $receiver) {

            $this->db->insert('task_users', [
                'task_id' => $id,
                'user_hash' => $receiver,
            ]);
        }

        // add task attachments
        if (empty($attachments)) {
            return $id;
        } else {
            $hash = $this->jwtmiddleware->createHash();
            foreach ($attachments as $file) {
                // delete any space in the file name
                $file = str_replace(' ', '', $file);
                $this->db->insert('task_attachment', [
                    'link' => $file,
                    'task_hash' => $data['hash'],
                    'create_record' => date('Y-m-d H:i:s'),
                    'user_hash' => $data['sender'],
                    'hash' => $hash,
                    'note' => $note,
                ]);
            }
        }

        return $id;
    }


    private function checkTaskAgreement($hash)
    {

        $this->db->select('*');
        $this->db->from('system_users');
        $this->db->where('hash', $hash);
        $this->db->where('task_agreement', '1');
        // Add an OR condition to check for role 1 or role 2
        $query = $this->db->get();

        if ($query->num_rows() > 0) {
            return true;
        }
        return false;
    }
    public function insertScheduler($data)
    {
        $array = array(
            'task_name' => $data['title'],
            'due_date' => $data['start_task'],
            'task_hash' => $data['hash'],
        );
        $this->db->insert('task_scheduler', $array);
    }

    public function getTasksDueWithin24Hours()
    {
        $query = $this->db->query("
                           SELECT * FROM task_scheduler
                           LEFT JOIN task ON task.hash = task_hash
                                   WHERE due_date BETWEEN NOW() AND NOW() + INTERVAL 1 DAY
                                   AND notification_sent = FALSE

                                   AND task.is_simple = 0
                                   AND task.accept_user IS NOT NULL

                                   ");

        return $query->result_array();
    }

    public function markTaskAsNotified($taskHash)
    {
        $data = array('notification_sent' => true);
        $this->db->where('task_hash', $taskHash);
        $this->db->update('task_scheduler', $data);
    }

    public function getProjectIdByHash($projectHash)
    {
        $this->db->select('id');
        $this->db->from('project');
        $this->db->where('hash', $projectHash);
        $query = $this->db->get();

        if ($query->num_rows() > 0) {
            $result = $query->row();
            return $result->id;
        }
        return false;
    }

    private function checkSenderRole($senderHash, $projectHash, $role_id = 2)
    {
        $projectId = $this->getProjectIdByHash($projectHash);

        if (!$projectId) {
            return false;
        }

        // Then, check if the sender has the required role in the project
        $this->db->select('*');
        $this->db->from('project_employees');
        $this->db->where('employee_id', $senderHash);
        $this->db->where('project_id', $projectId);
        // Add an OR condition to check for role 1 or role 2
        $this->db->where("(role_id = $role_id OR role_id = 1 OR role_id = 4 OR role_id = 3)");
        $query = $this->db->get();

        if ($query->num_rows() > 0) {
            foreach ($query->result() as $result) {
                return true; // Found a matching role, return true
            }
        }
        return false; // No matching role found
    }

    // تحديث مشروع
    public function update($hash, $data)
    {

        $roleCheckPassed = $data["is_simple"] == 0 ?
            true :
            $this->checkSenderRole($data['sender'], $data['project_hash']);
        if (!$roleCheckPassed) {
            return 'ليس لديك صلاحيه لإضافة مهمه';
        }
        // Start a database transaction
        $this->db->trans_start();

        // Extract and remove receivers from data
        $recivers = isset($data['receiver']) ? $data['receiver'] : [];
        unset($data['receiver']);

        // Update the task data
        $this->db->where('hash', $hash);
        $this->db->update('task', $data);
        $affected_rows = $this->db->affected_rows();

        // Get task id by hash
        $task_id_query = $this->db->select('id')->where('hash', $hash)->get('task');
        if ($task_id_query->num_rows() > 0) {
            $task_id = $task_id_query->row()->id;

            // Remove existing receivers
            $this->db->where('task_id', $task_id);
            $this->db->delete('task_users');

            // Insert new receivers
            foreach ($recivers as $receiver) {
                $this->db->insert('task_users', [
                    'task_id' => $task_id,
                    'user_hash' => $receiver,
                ]);
            }
        }

        // Complete the transaction
        $this->db->trans_complete();

        if ($this->db->trans_status() === false) {
            // Transaction failed, handle the error
            return 0;
        }

        return $affected_rows;
    }

    public function delete($hash, $sender, $is_simple)
    {
        $this->db->select('project_hash');
        $this->db->from('task');
        $this->db->where('hash', $hash);
        $query = $this->db->get();
        $project_hash = $query->row()->project_hash;

        $roleCheckPassed = $is_simple == 0 ?
            true :
            $this->checkSenderRole($sender, $project_hash);
        if (!$roleCheckPassed) {
            return 'ليس لديك صلاحيه لمسح الواجب';
        }
        $isAdmin = $this->isAdmin($sender);
        if (!$isAdmin) {
            return 'ليس لديك صلاحيه لمسح الواجب';
        }
        $this->db->where('hash', $hash);
        $this->db->delete('task');
        return $this->db->affected_rows();
    }

    private function isAdmin($userHash)
    {
        $this->db->select('R.name')
            ->from('Roles R')
            ->join('UserRoles UR', 'UR.role_id = R.id')
            ->where('R.name', 'أدمن نظام')
            ->where('UR.user_hash', $userHash);
        $query = $this->db->get();

        return $query->num_rows() > 0;
    }

    public function getTasksStatusOptions()
    {
        $this->db->select('title as label , hash as value')->from('task_status')->where('hash !=', '1660547175234184');
        $query = $this->db->get();
        return $query->result_array();
    }

    // استرداد مشروع بالهاش
    public function get($hash)
    {

        $this->db->select('task.id as id ,task.title,receiver, task.hash, discription, score, start_task, task.deadline,task.title,project.title as project,project_hash,emergency,sender , task.minutes_deadline, task.accept_user,  task.phone_num as phone_num, task.task_name_subscriber, task.is_simple, task.lat, task.lng');
        $this->db->select("GROUP_CONCAT(DISTINCT task_attachment.note SEPARATOR '^^') as note");
        $this->db->select("GROUP_CONCAT(DISTINCT link) as link, status");

        // croup_concat attachement {link:link,hash:hash}
        $this->db->select("GROUP_CONCAT(DISTINCT task_attachment.link) as attachment");
        $this->db->select("(select name from system_users where hash=task.sender) as name_sender");
        // $this->db->select("(select name from system_users where hash=task.receiver) as name_receiver");
        $this->db->select("(select title from task_status where hash=task.status) as title_status");

        $this->db->select("
 (
                SELECT COUNT(mr.id)
                FROM message_reads mr
                WHERE mr.user_hash = '$hash' AND mr.task_hash = task.hash
                AND mr.seen_at IS NULL
            ) AS unSeenMessages
    ");

        $this->db->from('task');
        $this->db->join('task_attachment', 'task.hash = task_attachment.task_hash', 'left');
        $this->db->join('project', 'project.hash = task.project_hash', 'left');
        $this->db->join('task_users', 'task_users.task_id = task.id', 'left');
        $this->db->where('task.hash', $hash);
        $this->db->group_by('task.hash');
        $query = $this->db->get();

        $users = $this->db->select('user_hash as value')->get_where('task_users', ['task_id' => $query->row()->id])->result_array();
        $user_values = array_column($users, 'value');
        $query->row()->receiver = $user_values;

        $result = $query->result();
        return $result[0];
    }

    public function getTaskDetails($hash)
    {
        $this->db->select('task.id as id, task.accept_date , task.action_date, TIMESTAMPDIFF(MINUTE, accept_date, action_date) AS task_completion_minutes, task.title, task.hash, discription, score, start_task, task.deadline, task.title, project.title as project, project_hash, emergency, sender, task.minutes_deadline, task.accept_user, status');
        $this->db->select("GROUP_CONCAT(DISTINCT task_attachment.note SEPARATOR '^^') as note");
        $this->db->select("GROUP_CONCAT(DISTINCT task_attachment.link) as link");
        $this->db->select("GROUP_CONCAT(DISTINCT task_attachment.link) as attachment");
        $this->db->select("(select name from system_users where hash=task.sender) as name_sender");
        $this->db->select("(select name from system_users where hash=task.accept_user) as name_receiver");
        $this->db->select("(select title from task_status where hash=task.status) as title_status");

        $this->db->from('task');
        $this->db->join('task_attachment', 'task.hash = task_attachment.task_hash AND task_attachment.attachment_type = "status_change"', 'left');
        $this->db->join('project', 'project.hash = task.project_hash', 'left');
        $this->db->join('task_users', 'task_users.task_id = task.id', 'left');
        $this->db->where('task.hash', $hash);
        $this->db->group_by('task.hash');
        $query = $this->db->get();

        $users = $this->db->select('user_hash as value')->get_where('task_users', ['task_id' => $query->row()->id])->result_array();
        $user_values = array_column($users, 'value');
        $query->row()->receiver = $user_values;

        $result = $query->result();
        return $result[0];
    }

    public function getRejectedTaskDetails($hash)
    {
        $this->db->select('task.id as id, task.acceptnote as note,task.rejected_date as accept_date , task.action_date, TIMESTAMPDIFF(MINUTE, rejected_date, action_date) AS task_completion_minutes, task.title, task.hash, discription, score, start_task, task.deadline, task.title, project.title as project, project_hash, emergency, sender, task.minutes_deadline, task.accept_user, status');

        // $this->db->select("GROUP_CONCAT(DISTINCT task_attachment.note SEPARATOR '^^') as note");
        $this->db->select("GROUP_CONCAT(DISTINCT task_attachment.link) as link");
        $this->db->select("GROUP_CONCAT(DISTINCT task_attachment.link) as attachment");
        $this->db->select("(select name from system_users where hash=task.sender) as name_sender");
        $this->db->select("(select name from system_users where hash=task.rejected_user) as name_receiver");
        $this->db->select("(select title from task_status where hash=task.status) as title_status");

        $this->db->from('task');
        $this->db->join('task_attachment', 'task.hash = task_attachment.task_hash AND task_attachment.attachment_type = "status_change"', 'left');
        $this->db->join('project', 'project.hash = task.project_hash', 'left');
        $this->db->join('task_users', 'task_users.task_id = task.id', 'left');
        $this->db->where('task.hash', $hash);
        $this->db->group_by('task.hash');
        $query = $this->db->get();

        $users = $this->db->select('user_hash as value')->get_where('task_users', ['task_id' => $query->row()->id])->result_array();
        $user_values = array_column($users, 'value');
        $query->row()->receiver = $user_values;

        $result = $query->result();
        return $result[0];
    }

    public function get_all($page = 1, $perpage = 10, $search = '', $today = false, $department = '', $name = '', $projectHash = '', $is_simple_task = '', $task_status = '', $user)
    {
        $offset = ($page - 1) * $perpage;
        $is_accpeted = $this->input->get('isAccept') ?? '';
        $task_status = $this->input->get('task_status') ?? '';
        $startDate = $this->input->get('startDate') ?? null;
        $endDate = $this->input->get('endDate') ?? null;
        $params = [
            'is_accpeted' => $is_accpeted,
            'task_status' => $task_status,
            'startDate' => $startDate,
            'endDate' => $endDate,
        ];


        // Get total rows without pagination
        $totalRows = $this->get_total_rows($search, $today, $department, $name, $projectHash, $is_simple_task, $task_status, $user, $params);
        $task_name_subscriber = $this->input->get('task_name_subscriber') ?? '';
        $phone_num = $this->input->get('phone_num') ?? '';
        // for filter accepted or not accepted



        // Define the common part for the query
        $this->db->select("t.title, discription, t.deadline, score, acceptnote, t.updated_at, isaccept, t.task_name_subscriber, t.phone_num,
            t.status, t.hash, t.sender, t.accept_user, receiver, project_hash, emergency, t.rejected_user, t.rejected_date, t.send_date,
            (select title from task_status where hash = t.status) as status_title,
              CASE 
                WHEN t.rejected_user IS NULL AND t.accept_user IS NULL AND t.status = '111' THEN 'معلقه'
                WHEN t.rejected_user IS NOT NULL AND t.accept_user IS NULL AND t.status = '111' THEN 'مرفوضه'
                WHEN t.rejected_user IS NULL AND t.accept_user IS NOT NULL AND t.status = '111' THEN 'قيد العمل'
                WHEN t.rejected_user IS NULL AND t.accept_user IS NOT NULL AND t.status = '222' THEN 'منجزه'
                WHEN t.rejected_user IS NULL AND t.accept_user IS NOT NULL AND t.status = '333' THEN 'غير منجزه'
                ELSE 'Not Rejected'
              END AS rejection_status,
            info_reciever.partment AS reciever_department_hash,
            info_sender.partment AS sender_department_hash,
            (select name from Positions where id = info_sender.position) as sender_position,
            (select name from Positions where id = info_reciever.position) as reciever_position,
            u_sender.name AS sender_name, u_reciever.name AS receiver_name,
            partments_reciever.name AS reciever_department,
            partments_sender.name AS sender_department,
            u_sender.image_profile AS sender_image,
            u_reciever.image_profile AS reciever_image,
            t.accept_user as accept_user,
            minutes_deadline,
            accept_date, action_date, TIMESTAMPDIFF(MINUTE, accept_date, action_date) AS task_completion_minutes,
            u_sender.hash as sender_hash, u_reciever.hash as receiver_hash,
            CASE 
                WHEN t.rejected_user IS NOT NULL AND t.accept_user IS NULL THEN 'Rejected'
                ELSE 'Not Rejected'
            END AS rejection_flag
            ");

        $this->db->select("
            (
                SELECT COUNT(mr.id)
                FROM message_reads mr
                WHERE mr.user_hash = '$user' AND mr.task_hash = t.hash
                AND mr.seen_at IS NULL
            ) AS unSeenMessages
        ");

        // Case for simple tasks (is_simple_task = 0)
        if ($is_simple_task == '0') {
            // Joining directly with the 'tasks' table and user related info
            $this->db->from('task t');
            $this->db->join('system_users u_sender', 'u_sender.hash = t.sender', 'left');
            $this->db->join('system_users u_reciever', 'u_reciever.hash = t.accept_user', 'left');
            $this->db->join('emp_information info_sender', 'info_sender.user_hash = u_sender.hash', 'left');
            $this->db->join('emp_information info_reciever', 'info_reciever.user_hash = u_reciever.hash', 'left');
            $this->db->join('partments partments_sender', 'info_sender.partment = partments_sender.hash', 'left');
            $this->db->join('partments partments_reciever', 'info_reciever.partment = partments_reciever.hash', 'left');
            $this->db->where('t.is_simple', 0);
            $this->db->join('UserRoles ur', 'ur.user_hash = "' . $user . '"', 'left');

            $this->db->where("
                     CASE 
                         WHEN t.direct_task = 1 THEN 
                     ( t.sender = '$user' OR ur.role_id = 4)
                         ELSE 1 = 1
                     END
                 ");
            $this->db->where_in('t.status', ['222', '333']);
            $this->db->group_start();
            $this->db->or_where('t.accept_user', $user);
            $this->db->or_where('t.sender', $user);
            $this->db->group_end();

        } else {
            // Existing logic for non-simple tasks, roles 1, 2, 3, 4
            $this->db->from('project_employees pe');
            $this->db->join('project p', 'pe.project_id = p.id');
            $this->db->join('task t', 't.project_hash = p.hash');
            $this->db->join('system_users u_sender', 'u_sender.hash = t.sender', 'left');
            $this->db->join('system_users u_reciever', 'u_reciever.hash = t.accept_user OR u_reciever.hash = t.rejected_user', 'left');
            $this->db->join('emp_information info_sender', 'info_sender.user_hash = u_sender.hash', 'left');
            $this->db->join('emp_information info_reciever', 'info_reciever.user_hash = u_reciever.hash', 'left');
            $this->db->join('partments partments_sender', 'info_sender.partment = partments_sender.hash', 'left');
            $this->db->join('partments partments_reciever', 'info_reciever.partment = partments_reciever.hash', 'left');
            $this->db->join('project', 'project.hash = t.project_hash', 'left');
            $this->db->join('UserRoles ur', 'ur.user_hash = "' . $user . '"', 'left');

            $this->db->where("
                     CASE 
                         WHEN t.direct_task = 1 THEN 
                     ( t.sender = '$user' OR ur.role_id = 4)
                         ELSE 1 = 1
                     END
                 ");
            $this->db->where('pe.employee_id', $user);
            $this->db->where_in('pe.role_id', ['1', '2', '3', '4']);
        }

        // Common conditions for both queries
        if ($today) {
            $this->db->where("start_task <= '" . date('Y-m-d') . "' and t.deadline >= '" . date('Y-m-d') . "'");
            $this->db->order_by('reciever_department');
        }

        if ($search != '') {
            $this->db->group_start();
            $this->db->like('t.title', $search);
            $this->db->or_like('t.discription', $search);
            $this->db->group_end();
        }
        if ($department != '') {
            $this->db->like('info_reciever.position', $department);
        }
        if ($name != '') {
            $this->db->like('u_reciever.name', $name);
        }
        if ($projectHash != '') {
            $this->db->where('t.project_hash', $projectHash);
        }
        if ($task_status != '') {
            if ($task_status == '111') {
                $this->db->where('t.status', $task_status)
                    ->where('t.rejected_user IS NULL');
                // ->where('t.accept_user IS NULL')
            } else if ($task_status == '222') {
                $this->db->where('t.status', $task_status)
                    ->where('t.accept_user IS NOT NULL')
                    ->where('t.rejected_user IS NULL');
            } else if ($task_status == '333') {
                $this->db->where('t.status', $task_status)
                    ->where('t.accept_user IS NOT NULL')
                    ->where('t.rejected_user IS NULL');
            } else {
                $this->db->where('t.status !=', $task_status);
            }
        }

        if ($startDate != null && $endDate != null) {
            $this->db->where("DATE(t.send_date) BETWEEN '$startDate' AND '$endDate'");
        }

        if ($task_name_subscriber != '') {
            $this->db->like('t.task_name_subscriber', $task_name_subscriber);
        }

        if ($phone_num != '') {
            $this->db->like('t.phone_num', $phone_num);
        }

        if ($is_accpeted == '1') {
            $this->db->where('t.accept_user !=', null);
        }

        if ($is_accpeted == '0') {
            $this->db->where('t.rejected_user !=', null);
            $this->db->where('t.accept_user =', null);
        }

        $this->db->group_by('t.hash');

        $this->db->order_by('t.id', 'desc');
        $this->db->limit($perpage, $offset); // Apply pagination

        // Execute the first query
        $query1 = $this->db->get();
        $arr1 = $query1->result_array(); // Get the result as an array

        if ($is_simple_task == '0') {
            // No need for the second query as we're fetching simple tasks only
            return ['totalRows' => $totalRows, 'data' => $arr1];
        }

        // Reset and prepare for the second query for role 5 if not simple tasks
        $this->db->reset_query();

        // Apply the same common part for the second query
        $this->db->select("t.title, discription, t.deadline, score, acceptnote, t.updated_at, isaccept,
            t.status, t.hash, sender, receiver, project_hash, emergency,
            (select title from task_status where hash = t.status) as status_title,
            CASE 
                WHEN t.rejected_user IS NOT NULL AND t.accept_user IS NULL THEN 'Rejected'
                WHEN t.rejected_user IS NULL AND t.accept_user IS NOT NULL AND t.status = '111' THEN 'Accepted'
                ELSE 'Not Rejected'
            END AS rejection_status,
            info_reciever.partment AS reciever_department_hash,
            info_sender.partment AS sender_department_hash,
            (select name from Positions where id = info_sender.position) as sender_position,
            (select name from Positions where id = info_reciever.position) as reciever_position,
            u_sender.name AS sender_name, u_reciever.name AS receiver_name,
            partments_reciever.name AS reciever_department,
            partments_sender.name AS sender_department,
            u_sender.image_profile AS sender_image,
            u_reciever.image_profile AS reciever_image,
            minutes_deadline,
            accept_date, action_date, TIMESTAMPDIFF(MINUTE, accept_date, action_date) AS task_completion_minutes,
            u_sender.hash as sender_hash, u_reciever.hash as receiver_hash,
            CASE 
                WHEN t.rejected_user IS NOT NULL AND t.accept_user IS NULL THEN 'Rejected'
                ELSE 'Not Rejected'
            END AS rejection_flag");
        $this->db->select("
            (
                SELECT COUNT(mr.id)
                FROM message_reads mr
                WHERE mr.user_hash = '$user' AND mr.task_hash = t.hash
                AND mr.seen_at IS NULL
            ) AS unSeenMessages
        ");

        $this->db->from('project_employees pe');
        $this->db->join('project p', 'pe.project_id = p.id');
        $this->db->join('task t', 't.project_hash = p.hash');
        $this->db->join('system_users u_sender', 'u_sender.hash = t.sender', 'left');
        $this->db->join('system_users u_reciever', 'u_reciever.hash = t.accept_user', 'left');
        $this->db->join('emp_information info_sender', 'info_sender.user_hash = u_sender.hash', 'left');
        $this->db->join('emp_information info_reciever', 'info_reciever.user_hash = u_reciever.hash', 'left');
        $this->db->join('partments partments_sender', 'info_sender.partment = partments_sender.hash', 'left');
        $this->db->join('partments partments_reciever', 'info_reciever.partment = partments_reciever.hash', 'left');
        $this->db->join('project', 'project.hash = t.project_hash', 'left');
        $this->db->where('pe.employee_id', $user);
        $this->db->where('pe.role_id', '5');

        if ($today) {
            $this->db->where("start_task <= '" . date('Y-m-d') . "' and t.deadline >= '" . date('Y-m-d') . "'");
            $this->db->order_by('reciever_department');
        }

        if ($search != '') {
            $this->db->group_start();
            $this->db->like('t.title', $search);
            $this->db->or_like('t.discription', $search);
            $this->db->group_end();
        }
        if ($department != '') {
            $this->db->like('info_reciever.position', $department);
        }
        if ($name != '') {
            $this->db->like('u_reciever.name', $name);
        }
        if ($projectHash != '') {
            $this->db->where('t.project_hash', $projectHash);
        }
        if ($task_status != '') {
            $this->db->where('t.status', $task_status);
        }

        if ($is_accpeted == '1') {
            $this->db->where('t.accept_user !=', null);
        }

        if ($is_accpeted == '0') {
            $this->db->where('t.rejected_user !=', null);
            $this->db->where('t.accept_user =', null);
        }


        $this->db->order_by('t.id', 'desc');
        $this->db->limit($perpage, $offset); // Apply pagination

        // Execute the second query for role 5
        $query2 = $this->db->get();
        $arr2 = $query2->result_array(); // Get the result as an array

        // Combine the results
        return ['totalRows' => $totalRows, 'data' => array_merge($arr1, $arr2)];
    }
    public function get_all_custom($user)
    {
        // $page = $this->input->get('page') ?? 1;
        // $perpage = $this->input->get('perpage') ?? 10;
        // $today = $this->input->get('today') ?? false;
        // $department = $this->input->get('department') ?? '';
        // $name = $this->input->get('name') ?? '';
        // $projectHash = $this->input->get('projectHash') ?? '';
        // $is_simple_task = $this->input->get('is_simple_task') ?? '';
        // $task_status = $this->input->get('task_status') ?? '';
        // $is_accpeted = $this->input->get('isAccept') ?? '';
        // $search = $this->input->get('q') ?? '';

        $page = $this->input->get('page') ?? 1;
        $perpage = $this->input->get('perPage') ?? 10;
        $search = $this->input->get('q') ?? '';
        $department = $this->input->get('department') ?? '';
        $name = $this->input->get('name') ?? '';
        $today = $this->input->get('today') ?? false;
        $projectHash = $this->input->get('projectValue') ?? '';
        $is_simple_task = $this->input->get('isSimpleTask') ?? '';
        $task_status = $this->input->get('task_status') ?? '';
        $is_accpeted = $this->input->get('isAccept') ?? '';
        $task_name_subscriber = $this->input->get('task_name_subscriber') ?? '';
        $phone_num = $this->input->get('phone_num') ?? '';

        $offset = ($page - 1) * $perpage;
        $params = [
            'is_accpeted' => $is_accpeted,
            'task_status' => $task_status
        ];

        // Get total rows without pagination
        $totalRows = $this->get_total_rows($search, $today, $department, $name, $projectHash, $is_simple_task, $task_status, $user, $params);

        // for filter accepted or not accepted



        // Define the common part for the query
        $this->db->select("t.title, discription, t.deadline, score, acceptnote, t.updated_at, isaccept, t.task_name_subscriber, t.phone_num,
            t.status, t.hash, t.sender, t.accept_user, receiver, project_hash, emergency,

            (select title from task_status where hash = t.status) as status_title,
            info_reciever.partment AS reciever_department_hash,
            info_sender.partment AS sender_department_hash,
            (select name from Positions where id = info_sender.position) as sender_position,
            (select name from Positions where id = info_reciever.position) as reciever_position,
            u_sender.name AS sender_name, u_reciever.name AS receiver_name,
            partments_reciever.name AS reciever_department,
            partments_sender.name AS sender_department,
            u_sender.image_profile AS sender_image,
            u_reciever.image_profile AS reciever_image,
            t.accept_user as accept_user,
            minutes_deadline,
            accept_date, action_date, TIMESTAMPDIFF(MINUTE, accept_date, action_date) AS task_completion_minutes,
            u_sender.hash as sender_hash, u_reciever.hash as receiver_hash,
            CASE 
                WHEN t.rejected_user IS NOT NULL AND t.accept_user IS NULL THEN 'Rejected'
                ELSE 'Not Rejected'
            END AS rejection_flag
            ");

        $this->db->select("
            (
                SELECT COUNT(mr.id)
                FROM message_reads mr
                WHERE mr.user_hash = '$user' AND mr.task_hash = t.hash
                AND mr.seen_at IS NULL
            ) AS unSeenMessages
        ");

        // Case for simple tasks (is_simple_task = 0)
        if ($is_simple_task == '0') {
            // Joining directly with the 'tasks' table and user related info
            $this->db->from('task t');
            $this->db->join('system_users u_sender', 'u_sender.hash = t.sender', 'left');
            $this->db->join('system_users u_reciever', 'u_reciever.hash = t.accept_user', 'left');
            $this->db->join('emp_information info_sender', 'info_sender.user_hash = u_sender.hash', 'left');
            $this->db->join('emp_information info_reciever', 'info_reciever.user_hash = u_reciever.hash', 'left');
            $this->db->join('partments partments_sender', 'info_sender.partment = partments_sender.hash', 'left');
            $this->db->join('partments partments_reciever', 'info_reciever.partment = partments_reciever.hash', 'left');
            $this->db->where('t.is_simple', 0);
            $this->db->join('UserRoles ur', 'ur.user_hash = "' . $user . '"', 'left');

            $this->db->where("
                     CASE 
                         WHEN t.direct_task = 1 THEN 
                     ( t.sender = '$user' OR ur.role_id = 4)
                         ELSE 1 = 1
                     END
                 ");
            $this->db->where_in('t.status', ['222', '333']);
            $this->db->group_start();
            $this->db->or_where('t.accept_user', $user);
            $this->db->or_where('t.sender', $user);
            $this->db->group_end();

        } else {
            // Existing logic for non-simple tasks, roles 1, 2, 3, 4
            $this->db->from('project_employees pe');
            $this->db->join('project p', 'pe.project_id = p.id');
            $this->db->join('task t', 't.project_hash = p.hash');
            $this->db->join('system_users u_sender', 'u_sender.hash = t.sender', 'left');
            $this->db->join('system_users u_reciever', 'u_reciever.hash = t.accept_user OR u_reciever.hash = t.rejected_user', 'left');
            $this->db->join('emp_information info_sender', 'info_sender.user_hash = u_sender.hash', 'left');
            $this->db->join('emp_information info_reciever', 'info_reciever.user_hash = u_reciever.hash', 'left');
            $this->db->join('partments partments_sender', 'info_sender.partment = partments_sender.hash', 'left');
            $this->db->join('partments partments_reciever', 'info_reciever.partment = partments_reciever.hash', 'left');
            $this->db->join('project', 'project.hash = t.project_hash', 'left');
            $this->db->join('UserRoles ur', 'ur.user_hash = "' . $user . '"', 'left');

            $this->db->where("
                     CASE 
                         WHEN t.direct_task = 1 THEN 
                     ( t.sender = '$user' OR ur.role_id = 4)
                         ELSE 1 = 1
                     END
                 ");
            $this->db->where('pe.employee_id', $user);
            $this->db->where_in('pe.role_id', ['1', '2', '3', '4']);
        }

        // Common conditions for both queries
        if ($today) {
            $this->db->where("start_task <= '" . date('Y-m-d') . "' and t.deadline >= '" . date('Y-m-d') . "'");
            $this->db->order_by('reciever_department');
        }

        if ($search != '') {
            $this->db->group_start();
            $this->db->like('t.title', $search);
            $this->db->or_like('t.discription', $search);
            $this->db->group_end();
        }
        if ($department != '') {
            $this->db->like('info_reciever.position', $department);
        }
        if ($name != '') {
            $this->db->like('u_reciever.name', $name);
        }
        if ($projectHash != '') {
            $this->db->where('t.project_hash', $projectHash);
        }
        if ($task_status != '') {
            $this->db->where('t.status', $task_status);
        }

        if ($task_name_subscriber != '') {
            $this->db->like('t.task_name_subscriber', $task_name_subscriber);
        }

        if ($phone_num != '') {
            $this->db->like('t.phone_num', $phone_num);
        }

        if ($is_accpeted == '1') {
            $this->db->where('t.accept_user !=', null);
        }

        if ($is_accpeted == '0') {
            $this->db->where('t.rejected_user !=', null);
            $this->db->where('t.accept_user =', null);
        }

        $this->db->group_by('t.hash');

        $this->db->order_by('t.id', 'desc');
        $this->db->limit($perpage, $offset); // Apply pagination

        // Execute the first query
        $query1 = $this->db->get();
        $arr1 = $query1->result_array(); // Get the result as an array

        if ($is_simple_task == '0') {
            // No need for the second query as we're fetching simple tasks only
            return ['totalRows' => $totalRows, 'data' => $arr1];
        }

        // Reset and prepare for the second query for role 5 if not simple tasks
        $this->db->reset_query();

        // Apply the same common part for the second query
        $this->db->select("t.title, discription, t.deadline, score, acceptnote, t.updated_at, isaccept,
            t.status, t.hash, sender, receiver, project_hash, emergency,
            (select title from task_status where hash = t.status) as status_title,
            info_reciever.partment AS reciever_department_hash,
            info_sender.partment AS sender_department_hash,
            (select name from Positions where id = info_sender.position) as sender_position,
            (select name from Positions where id = info_reciever.position) as reciever_position,
            u_sender.name AS sender_name, u_reciever.name AS receiver_name,
            partments_reciever.name AS reciever_department,
            partments_sender.name AS sender_department,
            u_sender.image_profile AS sender_image,
            u_reciever.image_profile AS reciever_image,
            minutes_deadline,
            accept_date, action_date, TIMESTAMPDIFF(MINUTE, accept_date, action_date) AS task_completion_minutes,
            u_sender.hash as sender_hash, u_reciever.hash as receiver_hash,
            CASE 
                WHEN t.rejected_user IS NOT NULL AND t.accept_user IS NULL THEN 'Rejected'
                ELSE 'Not Rejected'
            END AS rejection_flag");
        $this->db->select("
            (
                SELECT COUNT(mr.id)
                FROM message_reads mr
                WHERE mr.user_hash = '$user' AND mr.task_hash = t.hash
                AND mr.seen_at IS NULL
            ) AS unSeenMessages
        ");

        $this->db->from('project_employees pe');
        $this->db->join('project p', 'pe.project_id = p.id');
        $this->db->join('task t', 't.project_hash = p.hash');
        $this->db->join('system_users u_sender', 'u_sender.hash = t.sender', 'left');
        $this->db->join('system_users u_reciever', 'u_reciever.hash = t.accept_user', 'left');
        $this->db->join('emp_information info_sender', 'info_sender.user_hash = u_sender.hash', 'left');
        $this->db->join('emp_information info_reciever', 'info_reciever.user_hash = u_reciever.hash', 'left');
        $this->db->join('partments partments_sender', 'info_sender.partment = partments_sender.hash', 'left');
        $this->db->join('partments partments_reciever', 'info_reciever.partment = partments_reciever.hash', 'left');
        $this->db->join('project', 'project.hash = t.project_hash', 'left');
        $this->db->where('pe.employee_id', $user);
        $this->db->where('pe.role_id', '5');

        if ($today) {
            $this->db->where("start_task <= '" . date('Y-m-d') . "' and t.deadline >= '" . date('Y-m-d') . "'");
            $this->db->order_by('reciever_department');
        }

        if ($search != '') {
            $this->db->group_start();
            $this->db->like('t.title', $search);
            $this->db->or_like('t.discription', $search);
            $this->db->group_end();
        }
        if ($department != '') {
            $this->db->like('info_reciever.position', $department);
        }
        if ($name != '') {
            $this->db->like('u_reciever.name', $name);
        }
        if ($projectHash != '') {
            $this->db->where('t.project_hash', $projectHash);
        }
        if ($task_status != '') {
            $this->db->where('t.status', $task_status);
        }

        if ($is_accpeted == '1') {
            $this->db->where('t.accept_user !=', null);
        }

        if ($is_accpeted == '0') {
            $this->db->where('t.rejected_user !=', null);
            $this->db->where('t.accept_user =', null);
        }


        $this->db->order_by('t.id', 'desc');
        $this->db->limit($perpage, $offset); // Apply pagination

        // Execute the second query for role 5
        $query2 = $this->db->get();
        $arr2 = $query2->result_array(); // Get the result as an array

        // Combine the results
        return ['totalRows' => $totalRows, 'data' => array_merge($arr1, $arr2)];
    }

    private function get_total_rows($search, $today, $department, $name, $projectHash, $is_simple_task, $task_status, $user, $params)
    {

        $is_accpeted = isset($params['is_accpeted']) ? $params['is_accpeted'] : null;
        $startDate = isset($params['startDate']) ? $params['startDate'] : null;
        $endDate = isset($params['endDate']) ? $params['endDate'] : null;

        $this->db->select("COUNT(*) as total");

        if ($is_simple_task == '0') {
            $this->db->from('task t');
            $this->db->join('system_users u_sender', 'u_sender.hash = t.sender', 'left');
            $this->db->join('system_users u_reciever', 'u_reciever.hash = t.accept_user', 'left');
            $this->db->join('emp_information info_sender', 'info_sender.user_hash = u_sender.hash', 'left');
            $this->db->join('emp_information info_reciever', 'info_reciever.user_hash = u_reciever.hash', 'left');
            $this->db->where('t.is_simple', 0);
            $this->db->group_start();
            $this->db->or_where('t.accept_user', $user);
            $this->db->or_where('t.sender', $user);
            $this->db->group_end();
        } else {
            $this->db->from('project_employees pe');
            $this->db->join('project p', 'pe.project_id = p.id');
            $this->db->join('task t', 't.project_hash = p.hash');
            $this->db->join('system_users u_sender', 'u_sender.hash = t.sender', 'left');
            $this->db->join('system_users u_reciever', 'u_reciever.hash = t.accept_user', 'left');
            $this->db->where('pe.employee_id', $user);
            $this->db->where_in('pe.role_id', ['1', '2', '3', '4']);
        }

        if ($today) {
            $this->db->where("start_task <= '" . date('Y-m-d') . "' and t.deadline >= '" . date('Y-m-d') . "'");
        }

        if ($search != '') {
            $this->db->group_start();
            $this->db->like('t.title', $search);
            $this->db->or_like('t.discription', $search);
            $this->db->group_end();
        }

        if ($department != '') {
            $this->db->like('info_reciever.position', $department);
        }

        if ($name != '') {
            $this->db->like('u_reciever.name', $name);
        }

        if ($projectHash != '') {
            $this->db->where('t.project_hash', $projectHash);
        }

        if ($task_status != '') {
            $this->db->where('t.status', $task_status);
        }

        if ($is_accpeted == '1') {
            $this->db->where('t.accept_user !=', null);
        }

        if ($is_accpeted == '0') {
            $this->db->where('t.rejected_user !=', null);
            $this->db->where('t.accept_user =', null);
        }

        $task_name_subscriber = $this->input->get('task_name_subscriber') ?? '';
        $phone_num = $this->input->get('phone_num') ?? '';

        if ($startDate != null && $endDate != null) {
            $this->db->where("DATE(t.send_date) BETWEEN '$startDate' AND '$endDate'");
        }

        if ($task_name_subscriber != '') {
            $this->db->like('t.task_name_subscriber', $task_name_subscriber);
        }

        if ($phone_num != '') {
            $this->db->like('t.phone_num', $phone_num);
        }

        $query = $this->db->get();
        $result = $query->row_array();
        return $result['total'];
    }

    private function get_total_rows_for_today($search, $today, $department, $name, $projectHash, $task_status)
    {
        // Define the common part for the query
        $this->db->select("COUNT(*) as total");

        // Case for simple tasks (is_simple_task = 0)
        $this->db->from('task t');
        $this->db->join('system_users u_sender', 'u_sender.hash = t.sender', 'left');
        $this->db->join('system_users u_reciever', 'u_reciever.hash = t.accept_user', 'left');
        $this->db->join('emp_information info_sender', 'info_sender.user_hash = u_sender.hash', 'left');
        $this->db->join('emp_information info_reciever', 'info_reciever.user_hash = u_reciever.hash', 'left');
        $this->db->join('partments partments_sender', 'info_sender.partment = partments_sender.hash', 'left');
        $this->db->join('partments partments_reciever', 'info_reciever.partment = partments_reciever.hash', 'left');

        // Common conditions for the query
        if ($today) {
            $this->db->where("DATE(t.created_at) = '" . date('Y-m-d') . "'");
        }

        if ($search != '') {
            $this->db->group_start();
            $this->db->like('t.title', $search);
            $this->db->or_like('t.discription', $search);
            $this->db->group_end();
        }
        if ($department != '') {
            $this->db->like('info_reciever.position', $department);
        }
        if ($name != '') {
            $this->db->like('u_reciever.name', $name);
        }
        if ($projectHash != '') {
            $this->db->where('t.project_hash', $projectHash);
        }
        if ($task_status != '') {
            $this->db->where('t.status', $task_status);
        }

        // Execute the query and return the total rows
        $query = $this->db->get();
        $result = $query->row_array();
        return $result['total'];
    }

    public function getUserTasks($requestData)
    {
        $page = $requestData['page'];
        $perpage = $requestData['perpage'];
        $is_simple_task = $requestData['is_simple_task'];
        $today = $requestData['today'];
        $search = $requestData['search'];
        $department = $requestData['department'];
        $name = $requestData['name'];
        $projectHash = $requestData['projectValue'];
        $task_status = $requestData['task_status'];
        $user = $requestData['user'];

        $offset = ($page - 1) * $perpage;

        $data = $this->getAllUserTasks($user, $today, $search, $department, $name, $projectHash, $task_status, $perpage, $offset);

        return $data;
    }

    public function getAllTasksToday($page = 1, $perpage = 10, $search = '', $today = true, $department = '', $name = '', $projectHash = '', $task_status = '')
    {
        $offset = ($page - 1) * $perpage;

        // Get total rows without pagination
        $totalRows = $this->get_total_rows_for_today($search, $today, $department, $name, $projectHash, $task_status);

        // Define the common part for the query
        $this->db->select("t.title, t.discription, t.deadline, t.score, t.acceptnote, t.updated_at, t.isaccept,
            t.status, t.hash, t.sender, t.accept_user, t.receiver, t.project_hash, t.emergency,
            (select title from task_status where hash = t.status) as status_title,
            info_reciever.partment AS reciever_department_hash,
            info_sender.partment AS sender_department_hash,
            (select name from Positions where id = info_sender.position) as sender_position,
            (select name from Positions where id = info_reciever.position) as reciever_position,
            u_sender.name AS sender_name, u_reciever.name AS receiver_name,
            partments_reciever.name AS reciever_department,
            partments_sender.name AS sender_department,
            u_sender.image_profile AS sender_image,
            u_reciever.image_profile AS reciever_image,
            t.minutes_deadline,
            t.accept_date, t.action_date, TIMESTAMPDIFF(MINUTE, t.accept_date, t.action_date) AS task_completion_minutes,
            u_sender.hash as sender_hash, u_reciever.hash as receiver_hash");

        $this->db->from('task t');
        $this->db->join('system_users u_sender', 'u_sender.hash = t.sender', 'left');
        $this->db->join('system_users u_reciever', 'u_reciever.hash = t.accept_user', 'left');
        $this->db->join('emp_information info_sender', 'info_sender.user_hash = u_sender.hash', 'left');
        $this->db->join('emp_information info_reciever', 'info_reciever.user_hash = u_reciever.hash', 'left');
        $this->db->join('partments partments_sender', 'info_sender.partment = partments_sender.hash', 'left');
        $this->db->join('partments partments_reciever', 'info_reciever.partment = partments_reciever.hash', 'left');

        // Common conditions for the query
        if ($today) {
            $this->db->where("DATE(t.created_at) = '" . date('Y-m-d') . "'");
        }

        if ($search != '') {
            $this->db->group_start();
            $this->db->like('t.title', $search);
            $this->db->or_like('t.discription', $search);
            $this->db->group_end();
        }
        if ($department != '') {
            $this->db->like('info_reciever.position', $department);
        }
        if ($name != '') {
            $this->db->like('u_reciever.name', $name);
        }
        if ($projectHash != '') {
            $this->db->where('t.project_hash', $projectHash);
        }
        if ($task_status != '') {
            $this->db->where('t.status', $task_status);
        }

        $this->db->order_by('t.id', 'desc');
        $this->db->limit($perpage, $offset); // Apply pagination

        // Execute the query
        $query = $this->db->get();
        $result = $query->result_array(); // Get the result as an array

        return ['totalRows' => $totalRows, 'data' => $result];
    }

    private function getSimpleTasks($user, $today, $search, $department, $name, $projectHash, $task_status, $perpage, $offset)
    {
        $this->db->select("t.title, discription, t.deadline, score, acceptnote, t.updated_at, isaccept,
            t.status, t.hash, t.sender, t.accept_user, receiver, project_hash, emergency,
            (select title from task_status where hash = t.status) as status_title,
            info_reciever.partment AS reciever_department_hash,
            info_sender.partment AS sender_department_hash,
            (select name from Positions where id = info_sender.position) as sender_position,
            (select name from Positions where id = info_reciever.position) as reciever_position,
            u_sender.name AS sender_name, u_reciever.name AS receiver_name,
            partments_reciever.name AS reciever_department,
            partments_sender.name AS sender_department,
            u_sender.image_profile AS sender_image,
            u_reciever.image_profile AS reciever_image,
            t.accept_user as accept_user,
            minutes_deadline,
            accept_date, action_date, TIMESTAMPDIFF(MINUTE, accept_date, action_date) AS task_completion_minutes,
            u_sender.hash as sender_hash, u_reciever.hash as receiver_hash");

        $this->db->from('task t');
        $this->db->join('system_users u_sender', 'u_sender.hash = t.sender', 'left');
        $this->db->join('system_users u_reciever', 'u_reciever.hash = t.accept_user', 'left');
        $this->db->join('emp_information info_sender', 'info_sender.user_hash = u_sender.hash', 'left');
        $this->db->join('emp_information info_reciever', 'info_reciever.user_hash = u_reciever.hash', 'left');
        $this->db->join('partments partments_sender', 'info_sender.partment = partments_sender.hash', 'left');
        $this->db->join('partments partments_reciever', 'info_reciever.partment = partments_reciever.hash', 'left');
        $this->db->where('t.is_simple', 0);
        $this->db->where_in('t.status', ['222', '333']);
        $this->db->group_start();
        $this->db->or_where('t.accept_user', $user);
        $this->db->or_where('t.sender', $user);
        $this->db->group_end();

        // Common conditions
        if ($today) {
            $this->db->where("start_task <= '" . date('Y-m-d') . "' and deadline >= '" . date('Y-m-d') . "'");
            $this->db->order_by('reciever_department');
        }

        if ($search != '') {
            $this->db->group_start();
            $this->db->like('t.title', $search);
            $this->db->or_like('t.discription', $search);
            $this->db->group_end();
        }
        if ($department != '') {
            $this->db->like('info_reciever.position', $department);
        }
        if ($name != '') {
            $this->db->like('u_reciever.name', $name);
        }
        if ($projectHash != '') {
            $this->db->where('t.project_hash', $projectHash);
        }
        if ($task_status != '') {
            $this->db->where('t.status', $task_status);
        }
        $this->db->order_by('t.id', 'desc');
        $this->db->limit($perpage, $offset);

        $query = $this->db->get();
        return $query->result_array();
    }

    private function getNonSimpleTasks($user, $today, $search, $department, $name, $projectHash, $task_status, $perpage, $offset)
    {
        $this->db->select("t.title, discription, t.deadline, score, acceptnote, t.updated_at, isaccept,
            t.status, t.hash, sender, receiver, project_hash, emergency,
            (select title from task_status where hash = t.status) as status_title,
            info_reciever.partment AS reciever_department_hash,
            info_sender.partment AS sender_department_hash,
            (select name from Positions where id = info_sender.position) as sender_position,
            (select name from Positions where id = info_reciever.position) as reciever_position,
            u_sender.name AS sender_name, u_reciever.name AS receiver_name,
            partments_reciever.name AS reciever_department,
            partments_sender.name AS sender_department,
            u_sender.image_profile AS sender_image,
            u_reciever.image_profile AS reciever_image,
            minutes_deadline,
            accept_date, action_date, TIMESTAMPDIFF(MINUTE, accept_date, action_date) AS task_completion_minutes,
            u_sender.hash as sender_hash, u_reciever.hash as receiver_hash");

        $this->db->from('project_employees pe');
        $this->db->join('project p', 'pe.project_id = p.id');
        $this->db->join('task t', 't.project_hash = p.hash');
        $this->db->join('system_users u_sender', 'u_sender.hash = t.sender', 'left');
        $this->db->join('system_users u_reciever', 'u_reciever.hash = t.accept_user', 'left');
        $this->db->join('emp_information info_sender', 'info_sender.user_hash = u_sender.hash', 'left');
        $this->db->join('emp_information info_reciever', 'info_reciever.user_hash = u_reciever.hash', 'left');
        $this->db->join('partments partments_sender', 'info_sender.partment = partments_sender.hash', 'left');
        $this->db->join('partments partments_reciever', 'info_reciever.partment = partments_reciever.hash', 'left');
        $this->db->join('project', 'project.hash = t.project_hash', 'left');
        $this->db->where_in('t.status', ['222', '333']);
        $this->db->where('pe.employee_id', $user);
        $this->db->where('t.accept_user', $user);

        $this->db->where_in('pe.role_id', ['1', '2', '3', '4']);

        // Common conditions
        if ($today) {
            $this->db->where("start_task <= '" . date('Y-m-d') . "' and deadline >= '" . date('Y-m-d') . "'");
            $this->db->order_by('reciever_department');
        }

        if ($search != '') {
            $this->db->group_start();
            $this->db->like('t.title', $search);
            $this->db->or_like('t.discription', $search);
            $this->db->group_end();
        }
        if ($department != '') {
            $this->db->like('info_reciever.position', $department);
        }
        if ($name != '') {
            $this->db->like('u_reciever.name', $name);
        }
        if ($projectHash != '') {
            $this->db->where('t.project_hash', $projectHash);
        }
        if ($task_status != '') {
            $this->db->where('t.status', $task_status);
        }
        $this->db->order_by('t.id', 'desc');
        $this->db->limit($perpage, $offset);

        $query = $this->db->get();
        return $query->result_array();
    }

    private function getAllUserTasks($user, $today, $search, $department, $name, $projectHash, $task_status, $perpage, $offset)
    {
        $selectStatement = "t.title, discription, t.deadline, score, acceptnote, t.updated_at, isaccept,
            t.status, t.hash, t.sender, t.accept_user, receiver, project_hash, emergency,
            (select title from task_status where hash = t.status) as status_title,
            info_reciever.partment AS reciever_department_hash,
            info_sender.partment AS sender_department_hash,
            (select name from Positions where id = info_sender.position) as sender_position,
            (select name from Positions where id = info_reciever.position) as reciever_position,
            u_sender.name AS sender_name, u_reciever.name AS receiver_name,
            partments_reciever.name AS reciever_department,
            partments_sender.name AS sender_department,
            u_sender.image_profile AS sender_image,
            u_reciever.image_profile AS reciever_image,
            t.accept_user as accept_user,
            minutes_deadline,
            accept_date, action_date, TIMESTAMPDIFF(MINUTE, accept_date, action_date) AS task_completion_minutes,
            u_sender.hash as sender_hash, u_reciever.hash as receiver_hash,
            ";

        $this->db->from('task t');
        $this->db->join('system_users u_sender', 'u_sender.hash = t.sender', 'left');
        $this->db->join('system_users u_reciever', 'u_reciever.hash = t.accept_user', 'left');
        $this->db->join('emp_information info_sender', 'info_sender.user_hash = u_sender.hash', 'left');
        $this->db->join('emp_information info_reciever', 'info_reciever.user_hash = u_reciever.hash', 'left');
        $this->db->join('partments partments_sender', 'info_sender.partment = partments_sender.hash', 'left');
        $this->db->join('partments partments_reciever', 'info_reciever.partment = partments_reciever.hash', 'left');
        $this->db->where_in('t.status', ['222', '333']);
        $this->db->where('t.accept_user', $user);

        // Common conditions
        if ($today) {
            $this->db->where("start_task <= '" . date('Y-m-d') . "' and deadline >= '" . date('Y-m-d') . "'");
            $this->db->order_by('reciever_department');
        }

        if ($search != '') {
            $this->db->group_start();
            $this->db->like('t.title', $search);
            $this->db->or_like('t.discription', $search);
            $this->db->group_end();
        }
        if ($department != '') {
            $this->db->like('info_reciever.position', $department);
        }
        if ($name != '') {
            $this->db->like('u_reciever.name', $name);
        }
        if ($projectHash != '') {
            $this->db->where('t.project_hash', $projectHash);
        }
        if ($task_status != '') {
            $this->db->where('t.status', $task_status);
        }

        // Get total rows
        $totalRowsQuery = clone $this->db;
        $totalRows = $totalRowsQuery->count_all_results();

        // Get paginated data
        $this->db->select($selectStatement);
        $this->db->order_by('t.id', 'desc');
        $this->db->limit($perpage, $offset);
        $query = $this->db->get();
        $data = $query->result_array();

        return ['total' => $totalRows, 'data' => $data];
    }

    public function get_pending_tasks_count($params = [])
    {
        $search = $params['search'] ?? '';
        $today = $params['today'] ?? false;
        $department = $params['department'] ?? '';
        $name = $params['name'] ?? '';
        $projectHash = $params['projectHash'] ?? '';
        $is_simple_task = $params['is_simple_task'] ?? '';
        $task_status = $params['task_status'] ?? '';
        $user = $params['user'] ?? null;
        $phone_num = $params['phone_num'] ?? '';
        $sender_name = $params['sender_name'] ?? '';

        if ($is_simple_task == "1") {
            // First query for roles 1,2,3,4
            $this->db->select('COUNT(DISTINCT t.id) as count');
            $this->db->from('project_employees pe');
            $this->db->join('project p', 'pe.project_id = p.id');
            $this->db->join('task t', 't.project_hash = p.hash');
            $this->db->join('system_users u_sender', 'u_sender.hash = t.sender', 'left');
            $this->db->join('system_users u_reciever', 'u_reciever.hash = t.accept_user', 'left');
            $this->db->join('emp_information info_sender', 'info_sender.user_hash = u_sender.hash', 'left');
            $this->db->join('emp_information info_reciever', 'info_reciever.user_hash = u_reciever.hash', 'left');

            $this->db->where('t.status', '111');
            $this->db->where('t.rejected_user', null);
            $this->db->where('pe.employee_id', $user);

            if ($today) {
                $this->db->where("start_task <= '" . date('Y-m-d') . "' and deadline >= '" . date('Y-m-d') . "'");
            }

            if ($search != '') {
                $this->db->group_start();
                $this->db->like('t.title', $search);
                $this->db->or_like('t.discription', $search);
                $this->db->group_end();
            }
            if ($department != '') {
                $this->db->like('info_reciever.position', $department);
            }
            if ($name != '') {
                $this->db->like('u_reciever.name', $name);
            }
            if ($projectHash != '') {
                $this->db->where('t.project_hash', $projectHash);
            }
            if ($is_simple_task != '') {
                $this->db->where('t.is_simple', $is_simple_task);
            }
            if ($task_status != '') {
                $this->db->where('t.status', $task_status);
            }
            if ($phone_num != '') {
                $this->db->where('t.phone_num', $phone_num);
            }

            if ($sender_name != '') {
                $this->db->like('u_sender.name', $sender_name);
            }



            $this->db->where_in('pe.role_id', ['1', '2', '3', '4']);

        } else {
            $this->db->select('COUNT(DISTINCT t.id) as count');
            $this->db->from('task t');
            $this->db->join('system_users u_sender', 'u_sender.hash = t.sender', 'left');
            $this->db->join('system_users u_reciever', 'u_reciever.hash = t.accept_user', 'left');
            $this->db->join('task_users tu', 'tu.task_id = t.id', 'left');
            $this->db->join('system_users u_all_recievers', 'u_all_recievers.hash = tu.user_hash', 'left');
            $this->db->where('t.status', '111');
            $this->db->where('t.rejected_user', null);
            $this->db->where('t.is_simple', 0); // Only simple tasks

            if ($search != '') {
                $this->db->group_start();
                $this->db->like('t.title', $search);
                $this->db->or_like('t.discription', $search);
                $this->db->group_end();
            }
            if ($projectHash != '') {
                $this->db->where('t.project_hash', $projectHash);
            }
            if ($task_status != '') {
                $this->db->where('t.status', $task_status);
            }
        }

        $totalCount = $this->db->get()->row()->count;

        // Return total count
        return $totalCount;
    }

    public function get_pending_tasks(
        $page = 1,
        $perpage = 10,
        $search = '',
        $today = false,
        $department = '',
        $name = '',
        $projectHash = '',
        $is_simple_task = '',
        $task_status = '',
        $user
    ) {
        $offset = ($page - 1) * $perpage;
        $phone_num = $this->input->get('phone_num') ?? '';
        $sender_name = $this->input->get('sender_name') ?? '';

        if ($is_simple_task == '1') {
            $this->db->select("t.title, t.discription, t.deadline, t.score, t.acceptnote, t.updated_at, t.isaccept,
t.task_name_subscriber , t.phone_num,
t.status, t.hash, t.sender, t.project_hash, t.emergency,
(SELECT title FROM task_status WHERE hash = t.status) as status_title,
info_reciever.partment AS reciever_department_hash,
info_sender.partment AS sender_department_hash,
(SELECT name FROM Positions WHERE id = info_sender.position) as sender_position,
(SELECT name FROM Positions WHERE id = info_reciever.position) as reciever_position,
u_sender.name AS sender_name, u_reciever.name AS receiver_name,
partments_reciever.name AS reciever_department,
partments_sender.name AS sender_department,
u_sender.image_profile AS sender_image,
u_reciever.image_profile AS reciever_image,
t.accept_user as accept_user,
t.minutes_deadline,
t.accept_date, t.action_date, TIMESTAMPDIFF(MINUTE, t.accept_date, t.action_date) AS task_completion_minutes,
t.is_simple,t.direct_task,
u_sender.hash as sender_hash, u_reciever.hash as receiver_hash,
 GROUP_CONCAT(DISTINCT u_all_recievers.name SEPARATOR ', ') AS all_receivers_names,
    GROUP_CONCAT(DISTINCT u_all_recievers.hash SEPARATOR ', ') AS all_receivers_hashes");
            $this->db->from('project_employees pe');
            $this->db->join('project p', 'pe.project_id = p.id');
            $this->db->join('task t', 't.project_hash = p.hash');
            $this->db->join('system_users u_sender', 'u_sender.hash = t.sender', 'left');
            $this->db->join('system_users u_reciever', 'u_reciever.hash = t.accept_user', 'left');
            $this->db->join('emp_information info_sender', 'info_sender.user_hash = u_sender.hash', 'left');
            $this->db->join('emp_information info_reciever', 'info_reciever.user_hash = u_reciever.hash', 'left');
            $this->db->join('partments partments_sender', 'info_sender.partment = partments_sender.hash', 'left');
            $this->db->join('partments partments_reciever', 'info_reciever.partment = partments_reciever.hash', 'left');
            $this->db->join('project', 'project.hash = t.project_hash', 'left');
            $this->db->join('task_users tu', 'tu.task_id = t.id', 'left');
            $this->db->join('UserRoles ur', 'ur.user_hash = "' . $user . '"', 'left');

            $this->db->join('system_users u_all_recievers', 'u_all_recievers.hash = tu.user_hash', 'left');
            $this->db->where('t.status', '111');
            $this->db->where('t.rejected_user', null);
            $this->db->where("
            CASE 
                WHEN t.direct_task = 1 THEN 
                    (tu.user_hash = '$user' OR t.sender = '$user' OR ur.role_id = 4)
                ELSE 1 = 1
            END
        ");
            $this->db->where('pe.employee_id', $user);

            $this->db->group_by('t.id'); // Group by the unique task ID to enable GROUP_CONCAT

            if ($today) {
                $this->db->where("start_task <= '" . date(' Y-m-d') . "' and deadline >= '" . date('Y-m-d') . "'");
                $this->
                    db->order_by('reciever_department');
            } else {
                $this->db->order_by('t.created_at', 'desc');
            }

            if ($search != '') {
                $this->db->group_start();
                $this->db->like('t.title', $search);
                $this->db->or_like('t.discription', $search);
                $this->db->group_end();
            }
            if ($department != '') {
                $this->db->like('info_reciever.position', $department);
            }
            if ($name != '') {
                $this->db->like('u_reciever.name', $name);
            }
            if ($projectHash != '') {
                $this->db->where('t.project_hash', $projectHash);
            }
            if ($is_simple_task != '') {
                $this->db->where('t.is_simple', $is_simple_task);
            }
            if ($task_status != '') {
                $this->db->where('t.status', $task_status);
            }

            if ($phone_num != '') {
                $this->db->where('t.phone_num', $phone_num);
            }

            if ($sender_name != '') {
                $this->db->like('u_sender.name', $sender_name);
            }

            $this->db->limit($perpage, $offset); // Apply pagination

            // Query for roles 1, 2, 3, 4
            $this->db->where_in('pe.role_id', ['1', '2', '3', '4']);

        } else {

            // Apply the same common part for the second query
            $this->db->select("t.title, t.discription, t.deadline, t.score, t.acceptnote, t.updated_at, t.isaccept,
    t.task_name_subscriber , t.phone_num,
    t.status, t.hash, t.sender, t.project_hash, t.emergency,
    t.is_simple,t.direct_task,
    (SELECT title FROM task_status WHERE hash = t.status) as status_title,
    u_sender.name AS sender_name, u_reciever.name AS receiver_name,
    u_sender.image_profile AS sender_image,
    u_reciever.image_profile AS reciever_image,
    t.accept_user as accept_user,
    t.minutes_deadline,

    t.accept_date, t.action_date, TIMESTAMPDIFF(MINUTE, t.accept_date, t.action_date) AS task_completion_minutes,
    u_sender.hash as sender_hash, u_reciever.hash as receiver_hash,
    GROUP_CONCAT(DISTINCT u_all_recievers.name SEPARATOR ', ') AS all_receivers_names,
    GROUP_CONCAT(DISTINCT u_all_recievers.hash SEPARATOR ', ') AS all_receivers_hashes");
            $this->db->from('task t');
            $this->db->join('system_users u_sender', 'u_sender.hash = t.sender', 'left');
            $this->db->join('system_users u_reciever', 'u_reciever.hash = t.accept_user', 'left');
            $this->db->join('task_users tu', 'tu.task_id = t.id', 'left');
            $this->db->join('UserRoles ur', 'ur.user_hash = "' . $user . '"', 'left');


            $this->db->join('system_users u_all_recievers', 'u_all_recievers.hash = tu.user_hash', 'left');
            $this->db->where('t.status', '111');
            $this->db->where('t.rejected_user', null);
            $this->db->where('t.is_simple', 0); // Only simple tasks



            $this->db->where("
            CASE 
                WHEN t.direct_task = 1 THEN 
            (tu.user_hash = '$user' OR t.sender = '$user' OR ur.role_id = 4)
                ELSE 1 = 1
            END
        ");

            $this->db->group_by('t.id');
            $this->db->limit($perpage, $offset);
            $this->db->order_by('t.created_at', 'desc');

        }

        $query = $this->db->get();
        $arr = $query->result_array(); // Get the result as an array
        return $arr;
    }


    public function getTasksOptions()
    {
        $this->db->select('hash as value, title as label');
        $this->db->from('task_status');
        $this->db->where_in('hash', ['222', '333']);
        $query = $this->db->get();
        return $query->result_array();
    }

    public function getPendingTasksOptions()
    {
        $this->db->select('hash as value, title as label');
        $this->db->from('task_status');
        $this->db->where('hash', '111');
        $query = $this->db->get();
        return $query->result_array();
    }

    public function count_records($search = '', $today = false, $department = '', $name = '', $projectHash = '', $is_simple_task = '', $user)
    {
        // Define a common part for the query
        $this->db->select("COUNT(*) as count");
        $this->db->from('project_employees pe');
        $this->db->join('project p', 'pe.project_id = p.id');
        $this->db->join('task t', 't.project_hash = p.hash');
        $this->db->join('system_users u_sender', 'u_sender.hash = t.sender', 'left');
        $this->db->join('system_users u_reciever', 'u_reciever.hash = t.accept_user', 'left');
        $this->db->join('emp_information info_sender', 'info_sender.user_hash = u_sender.hash', 'left');
        $this->db->join('emp_information info_reciever', 'info_reciever.user_hash = u_reciever.hash', 'left');
        $this->db->join('partments partments_sender', 'info_sender.partment = partments_sender.hash', 'left');
        $this->db->join('partments partments_reciever', 'info_reciever.partment = partments_reciever.hash', 'left');
        $this->db->join('project', 'project.hash = t.project_hash', 'left');
        $this->db->where('pe.employee_id', $user);
        $this->db->where_in('t.status', ['222', '333']);
        if ($today) {
            $this->db->where("start_task <= '" . date('Y-m-d') . "' and deadline >= '" . date('Y-m-d') . "'");
        }

        if ($search != '') {
            $this->db->group_start();
            $this->db->like('t.title', $search);
            $this->db->or_like('t.discription', $search);
            $this->db->group_end();
        }
        if ($department != '') {
            $this->db->like('info_reciever.position', $department);
        }
        if ($name != '') {
            $this->db->like('u_reciever.name', $name);
        }
        if ($projectHash != '') {
            $this->db->where('t.project_hash', $projectHash);
        }
        if ($is_simple_task != '') {
            $this->db->where('t.is_simple', $is_simple_task);
        }

        // Include the role_id condition
        $this->db->where_in('pe.role_id', ['1', '2', '3', '4', '5']);

        // Get the count
        $query = $this->db->get();
        return $query->result_array()[0]['count'];

    }

    public function count_records_overDue_deadline($search = '', $today = false, $department = '', $name = '')
    {
        $this->db->select('count(*) as count');
        $this->db->from('task');
        $this->db->join('system_users u_reciever', 'u_reciever.hash = task.receiver', 'left');
        $this->db->join('emp_information info_reciever', 'info_reciever.user_hash = u_reciever.hash', 'left');
        $this->db->join('partments partments_reciever', 'info_reciever.partment = partments_reciever.hash', 'left');

        $this->db->where("task.status != '222'")->where("task.isaccept", '1')->where("task.status !=", "222")->where("task.deadline <=", date('Y-m-d H:i:s'));

        if ($today) {
            $this->db->where("start_task <= '" . date('Y-m-d') . "' and deadline >= '" . date('Y-m-d') . "'");
            $this->db->order_by('reciever_department');
        }

        if ($search != '') {
            $this->db->like('title', $search);
            $this->db->or_like('discription', $search);
        }

        if ($department != '') {
            $this->db->like('info_reciever.position', $department);
        }
        if ($name != '') {
            $this->db->like('u_reciever.name', $name);
        }
        $query = $this->db->get();
        return $query->result_array()[0]['count'];
    }

    public function changeStatus($taskHash, $status, $action_note, $attachments)
    {
        $task = $this->db->get_where('task', array('hash' => $taskHash))->row();
        if (!$task) {
            return [
                'status' => 'error',
                'message' => 'Task not found',
            ];
        }
        $sender = $task->accept_user;

        if ($action_note !== "") {
            $this->db->set('action_note', $action_note);
        }
        $this->db->set('status', $status);

        // completed or not completed
        if ($status == "111" || $status == "222" || $status == "333") {
            $currentDateTime = date('Y-m-d H:i:s');
            $this->db->set('action_date', $currentDateTime);
        }

        $this->db->where('hash', $taskHash);
        $this->db->update('task');

        if (!empty($attachments)) {
            $hash = $this->jwtmiddleware->createHash();
            foreach ($attachments as $file) {
                // delete any space in the file name
                $file = str_replace(' ', '', $file);
                $this->db->insert('task_attachment', [
                    'link' => $file,
                    'task_hash' => $taskHash,
                    'create_record' => date('Y-m-d H:i:s'),
                    'user_hash' => $sender,
                    'hash' => $hash,
                    'note' => $action_note,
                    'attachment_type' => 'status_change',
                ]);
            }
        }

        $query = $this->db->get_where('task', array('hash' => $taskHash));
        $updated_record = $query->row();
        return $updated_record;
    }

    public function getRejectedUserOrAcceptByHash($taskHash)
    {
        $query = $this->db->get_where('task', array('hash' => $taskHash));
        return $query->row();
    }

    public function resetTaskStatus($data)
    {
        $taskHash = $data['taskHash'];
        $project_hash = $data['projectHash'];
        $user = $data['user'];
        $is_simple = $data['is_simple'];
        $recievers = $data['recievers'];

        $roleCheckPassed = $is_simple == 0 ?
            true :
            $this->checkSenderRole($user, $project_hash);
        if (!$roleCheckPassed) {
            return 'ليس لديك صلاحيه لإضافة مهمه';
        }

        $currentDate = date('Y-m-d H:i:s');

        $this->db->set('isaccept', '2');
        $this->db->set('status', '111');
        $this->db->set('action_date', null);
        $this->db->set('accept_date', null);
        $this->db->set('action_note', '');
        $this->db->set('rejected_user', null);
        $this->db->set('accept_user', null);
        $this->db->set('rejected_date', null);
        $this->db->set('send_date', $currentDate);
        $this->db->where('hash', $taskHash);
        $this->db->update('task');
        $query = $this->db->get_where('task', array('hash' => $taskHash));
        $updated_record = $query->row();

        if ($updated_record) {
            $taskId = $updated_record->id;
            $insert_data = [];

            foreach ($recievers as $receiver) {
                $existingTaskUser = $this->db->get_where('task_users', array(
                    'task_id' => $taskId,
                    'user_hash' => $receiver
                ))->row();

                if (!$existingTaskUser) {
                    $insert_data[] = array(
                        'task_id' => $taskId,
                        'user_hash' => $receiver,
                    );
                }
            }

            if (!empty($insert_data)) {

                $this->db->insert_batch('task_users', $insert_data);
            }

        }

        return $updated_record;


    }

    public function deleteAttachment($hash)
    {
        $this->db->where('hash', $hash);
        return $this->db->affected_rows();
    }

    public function getCurrentUserTasks($hash)
    {
        $this->db->select('task.title, task.hash, discription, task.status, score, start_task, task.deadline,task.title,project.title as project'); //emergency was deleted
        $this->db->select("GROUP_CONCAT(task_attachment.note SEPARATOR '^^') as note");
        $this->db->select("GROUP_CONCAT(link) as link");
        $this->db->select("(select name from system_users where hash=task.sender) as name_sender");
        $this->db->select("(select name from system_users where hash=task.receiver) as name_receiver");
        $this->db->select("(select title from task_status where hash=task.status) as title_status");
        $this->db->from('task');
        $this->db->join('task_attachment', 'task.hash = task_attachment.task_hash', 'left');
        $this->db->join('project', 'project.hash = task.project_hash', 'left');
        $this->db->where("(receiver = $hash and sender = $hash and task.status = '111')");
        $this->db->group_by('task.hash');
        $this->db->order_by('task.id', 'desc');
        $query = $this->db->get();
        $result = $query->result();
        return $result;
    }
    public function getCurrentUserTasksHistory($hash)
    {
        $this->db->select('task.lat , task.lng, task.hash, task.status, task.title, discription, score, start_task, task.deadline,task.title,project.title as project,emergency , task.phone_num as phone_num, task.task_name_subscriber');
        $this->db->select("GROUP_CONCAT(task_attachment.note SEPARATOR '^^') as note");
        $this->db->select("GROUP_CONCAT(link) as link");
        $this->db->select("
            case
                when task.status = '222' then '1'
                when task.status = '333' then '1'
                else '0'
            end as status
        ");
        $this->db->select("su.name AS name_sender, su.hash as sender_hash , su.image_profile as sender_image");
        $this->db->select("(select name from system_users where hash=task.receiver) as name_receiver");
        $this->db->select("(select title from task_status where hash=task.status) as title_status");
        $this->db->from('task');
        $this->db->join('task_attachment', 'task.hash = task_attachment.task_hash', 'left');
        $this->db->join('project', 'project.hash = task.project_hash', 'left');
        $this->db->join('task_users', 'task_users.task_id = task.id');
        $this->db->join('UserRoles ur', 'ur.user_hash = "' . $hash . '"', 'left');

        // Sender not su but its okay
        $this->db->join('system_users su', 'su.hash = task.sender', 'left');
        $this->db->where("(task.accept_user = '$hash' AND task.isaccept = '1') OR (task.sender = '$hash' AND task.status != '111')");
        $this->db->where("
        CASE 
            WHEN task.direct_task = 1 THEN 
        (task_users.user_hash = '$hash' OR task.sender = '$hash' OR ur.role_id = 4)
            ELSE 1 = 1
        END
    ");
        // $this->db->where("(receiver= $hash and isaccept='1') or (receiver = $hash and sender = $hash and task.status != '111')");
        $this->db->group_by('task.hash');
        $this->db->order_by('task.id', 'desc');
        $this->db->limit(150);
        $query = $this->db->get();
        $result = $query->result();
        return $result;
    }

    public function receivedTasks($hash)
    {

        $this->db->select('task.lat, task.lng, task.hash, task.sender,task.phone_num, task.title, discription, score, start_task, task.deadline, task.title,project.title as project,emergency');
        $this->db->select("GROUP_CONCAT(task_attachment.note SEPARATOR '^^') as note");
        $this->db->select("GROUP_CONCAT(link) as link");
        $this->db->select("sender.name as name_sender, sender.hash as sender_hash, sender.image_profile as sender_image");
        $this->db->select("(select name from system_users where hash=task.sender) as name_sender");
        $this->db->select("(select name from system_users where hash=task.receiver) as name_receiver");
        $this->db->select("(select title from task_status where hash=task.status) as title_status");

        $this->db->select("
       (
                SELECT COUNT(mr.id)
                FROM message_reads mr
                WHERE mr.user_hash = '$hash' AND mr.task_hash = task.hash
                AND mr.seen_at IS NULL
            ) AS unSeenMessages
    ");
        $this->db->from('task');
        $this->db->join('task_attachment', 'task.hash = task_attachment.task_hash', 'left');
        $this->db->join('project', 'project.hash = task.project_hash', 'left');
        $this->db->join('task_users', 'task_users.task_id = task.id', 'left');
        $this->db->join('system_users sender', 'sender.hash = task.sender', 'left');
        $this->db->join('UserRoles ur', 'ur.user_hash = "' . $hash . '"', 'left');

        $this->db->where("
                 CASE 
                     WHEN task.direct_task = 1 THEN 
                        (task_users.user_hash = '$hash' OR task.sender = '$hash' OR ur.role_id = 4)
                     ELSE 1 = 1
                 END
             ");

        // $this->db->where('task_users.user_hash', $hash);
        $this->db->where('sender !=', $hash);
        $this->db->where('isaccept', '2');
        $this->db->group_start(); // Start a group for the conditional where clause
        // $this->db->where('task.is_simple', '1');
        $this->db->where('task_users.user_hash', $hash);
        $this->db->group_end(); // End the group
        $this->db->group_by('task.hash');
        $this->db->order_by('task.id', 'desc');
        $query = $this->db->get();
        $result = $query->result();
        return $result;
    }

    public function sendedTasks($hash)
    {
        $this->db->select('task.hash, task.sender, task.title, description, score, start_task, task.deadline, task.title, project.title as project, emergency');
        $this->db->select("GROUP_CONCAT(task_attachment.note SEPARATOR '^^') as note");
        $this->db->select("GROUP_CONCAT(task_attachment.link) as link");
        $this->db->select("(select name from system_users where hash=task.sender) as name_sender");
        $this->db->select("GROUP_CONCAT(system_users.name SEPARATOR ', ') as name_receiver"); // Updated to handle multiple receivers
        $this->db->select("(select title from task_status where hash=task.status) as title_status");
        $this->db->select("isaccept");
        $this->db->from('task');
        $this->db->join('task_attachment', 'task.hash = task_attachment.task_hash', 'left');
        $this->db->join('project', 'project.hash = task.project_hash', 'left');
        $this->db->join('task_users', 'task.id = task_users.task_id', 'left'); // Join task_users table with task.id
        $this->db->join('system_users', 'task_users.user_hash = system_users.hash', 'left'); // Join system_users table again for receivers
        $this->db->where("task.sender = $hash AND task_users.user_hash != $hash");
        $this->db->group_by('task.hash');
        $this->db->order_by('task.id', 'desc');
        $query = $this->db->get();
        $result = $query->result();
        return $result;

    }

    public function getAttachments($hash)
    {
        $this->db->select('hash, create_record as date_create');
        $this->db->select("(select name from system_users where hash=task_attachment.user_hash) as name");
        $this->db->select("GROUP_CONCAT(link) as link");
        $this->db->select('note');
        $this->db->from('task_attachment');
        $this->db->where('task_hash', $hash);
        $this->db->group_by('note');
        $this->db->order_by('task_attachment.id', 'desc');
        $query = $this->db->get();
        $result = $query->result();
        return $result;
    }

    public function getStatus()
    {
        /*
        SELECT * FROM task_status
         */
        $this->db->select('task_status.title, task_status.hash');
        $this->db->from('task_task_status');
        $query = $this->db->get();
        $result = $query->result();
        return $result;
    }

    public function changeEmergency($hash, $emergency)
    {
        $this->db->set('emergency', $emergency);
        $this->db->where('hash', $hash);
        $this->db->update('task');
        $this->db->affected_rows();
    }

    public function changeIsAccept($hash, $isaccept, $note = '', $user)
    {
        try {
            // Begin transaction
            $this->db->trans_begin();

            // Retrieve the task details
            $this->db->select('is_simple, minutes_deadline');
            $this->db->where('hash', $hash);
            $query = $this->db->get('task');
            $task = $query->row();

            if ($task) {
                // Update isaccept and optionally accept_date and acceptnote
                $this->db->set('isaccept', $isaccept);
                if ($isaccept == 1) {
                    $currentDateTime = date('Y-m-d H:i:s');
                    $this->db->set('accept_date', $currentDateTime);
                    $this->db->set('accept_user', $user);

                    // Check if task is_simple and has minutes_deadline set
                    if ($task->is_simple == 1 && !is_null($task->minutes_deadline)) {
                        $deadline = date('Y-m-d H:i:s', strtotime('+' . $task->minutes_deadline . ' minutes', strtotime($currentDateTime)));
                        $this->db->set('deadline', $deadline);
                    }
                }

                if ($isaccept == 0) {
                    $currentDateTime = date('Y-m-d H:i:s');
                    $this->db->set('rejected_user', $user);
                    $this->db->set('rejected_date', $currentDateTime);
                }
                if ($note != '') {
                    $this->db->set('acceptnote', $note);
                }
                $this->db->where('hash', $hash);
                $this->db->update('task');

                // Check for successful update
                if ($this->db->affected_rows() > 0) {
                    // Commit the transaction
                    $this->db->trans_commit();
                    return true;
                } else {
                    // Rollback the transaction in case of no affected rows
                    $this->db->trans_rollback();
                    return false;
                }
            } else {
                // Rollback the transaction if task not found
                $this->db->trans_rollback();
                return false;
            }
        } catch (Exception $e) {
            // Rollback the transaction in case of exception
            $this->db->trans_rollback();
            return $e;
        }
    }

    public function allEmployeesListIncludeCurrentUser()
    {
        $this->db->select('name,hash ');
        $this->db->from('system_users');
        $this->db->join('emp_information', 'system_users.hash = emp_information.user_hash');
        $this->db->where('is_work', '1');
        $this->db->order_by('system_users.id', 'desc');
        $query = $this->db->get();
        $result = $query->result();
        return $result;
    }

    public function userTasks($hash, $status)
    {
        $this->db->select('task.hash, task.title,task.status, discription, score, start_task, task.deadline,task.title,project.title as project');
        $this->db->select("GROUP_CONCAT(task_attachment.note SEPARATOR '^^') as note");
        $this->db->select("GROUP_CONCAT(link) as link");
        $this->db->select("(select name from system_users where hash=task.sender) as name_sender");
        $this->db->select("(select name from system_users where hash=task.receiver) as name_receiver");
        $this->db->select("(select title from task_status where hash=task.status) as title_status");
        $this->db->from('task');
        $this->db->join('task_attachment', 'task.hash = task_attachment.task_hash', 'left');
        $this->db->join('project', 'project.hash = task.project_hash', 'left');
        $this->db->where('receiver', $hash);
        $this->db->where('status', $status);
        $this->db->where('status !=', '111');
        $this->db->group_by('task.hash');
        $this->db->order_by('task.id', 'desc');
        $this->db->limit(150);
        $query = $this->db->get();
        $result = $query->result();
        return $result;
    }

    public function changeReceiver($hash, $receiver)
    {
        // change receiver and isaccept to 2
        $query = $this->db->query("UPDATE task SET receiver = '$receiver', isaccept = '2' WHERE hash = '$hash'");
        return $query;
    }

    public function retrieveEmployeesWithOverdueTasks($page = 1, $perpage = 10, $search = '', $today = false, $department = '', $name = '')
    {
        $offset = ($page - 1) * $perpage;
        $this->db->select("title, discription, deadline, score, acceptnote, updated_at, isaccept,
            task.status, task.hash, sender, receiver, project_hash, emergency,
            (select title from task_status where hash = task.status) as status_title,
            info_reciever.partment AS reciever_department_hash,
            info_sender.partment AS sender_department_hash,
            (select name from Positions where id = info_sender.position) as  sender_position,
            (select name from Positions where id = info_reciever.position) as  reciever_position,
            u_sender.name AS sender_name, u_reciever.name AS receiver_name,
            partments_reciever.name AS reciever_department,
            partments_sender.name AS sender_department,
            u_sender.image_profile AS sender_image,
            u_reciever.image_profile AS reciever_image,
            u_sender.hash as sender_hash, u_reciever.hash as receiver_hash");
        $this->db->from('task');
        $this->db->join('system_users u_sender', 'u_sender.hash = task.sender', 'left');
        $this->db->join('system_users u_reciever', 'u_reciever.hash = task.receiver', 'left');
        $this->db->join('emp_information info_sender', 'info_sender.user_hash = u_sender.hash', 'left');
        $this->db->join('emp_information info_reciever', 'info_reciever.user_hash = u_reciever.hash', 'left');
        $this->db->join('partments partments_sender', 'info_sender.partment = partments_sender.hash', 'left');
        $this->db->join('partments partments_reciever', 'info_reciever.partment = partments_reciever.hash', 'left');

        $this->db->where("task.status != '222'")->where("task.isaccept", '1')->where("task.status !=", "222")->where("task.deadline <=", date('Y-m-d H:i:s'));

        if ($today) {
            $this->db->where("start_task <= '" . date('Y-m-d') . "' and deadline >= '" . date('Y-m-d') . "'");
            $this->db->order_by('reciever_department');
        }

        if ($search != '') {
            $this->db->like('title', $search);
            $this->db->or_like('discription', $search);
        }
        if ($department != '') {
            $this->db->like('info_reciever.position', $department);
        }
        if ($name != '') {
            $this->db->like('u_reciever.name', $name);
        }
        // order by id desc
        $this->db->order_by('task.id', 'desc');
        $this->db->limit($perpage, $offset);

        $query = $this->db->get();

        return $query->result_array();
    }

    public function getSimpleTasksStillWithUser($page = 1, $perpage = 10, $search = '', $today = false, $department = '', $name = '', $projectHash = '', $is_simple_task = '', $task_status = '', $user)
    {
        $accept = $this->input->get('isAccept') ?? '1';
        $offset = ($page - 1) * $perpage;
        $phone_num = $this->input->get('phone_num') ?? '';
        $task_name_subscriber = $this->input->get('task_name_subscriber') ?? '';
        // Get total rows without pagination
        $totalRows = $this->get_total_rows_StillWithUser($page = 1, $perpage = 10, $search, $today, $department, $name, $projectHash, $is_simple_task, $task_status, $accept, $user);
        // Define the common part for the query
        $this->db->select("t.title, discription, t.deadline, score, acceptnote, t.updated_at, isaccept, t.phone_num, t.task_name_subscriber, t.is_simple,
            t.status, t.hash, t.sender, t.accept_user, receiver, project_hash, emergency,
            (select title from task_status where hash = t.status) as status_title,
            info_reciever.partment AS reciever_department_hash,
            info_sender.partment AS sender_department_hash,
            (select name from Positions where id = info_sender.position) as sender_position,
            (select name from Positions where id = info_reciever.position) as reciever_position,
            u_sender.name AS sender_name, u_reciever.name AS receiver_name,
            partments_reciever.name AS reciever_department,
            partments_sender.name AS sender_department,  
            u_sender.image_profile AS sender_image,
            u_reciever.image_profile AS reciever_image,
            t.accept_user as accept_user,
            minutes_deadline,
            accept_date, action_date, TIMESTAMPDIFF(MINUTE, accept_date, action_date) AS task_completion_minutes,
            u_sender.hash as sender_hash, u_reciever.hash as receiver_hash");

        $this->db->select(" (
                SELECT COUNT(mr.id)
                FROM message_reads mr
                WHERE mr.user_hash = '$user' AND mr.task_hash = t.hash
                AND mr.seen_at IS NULL
            ) AS unSeenMessages");
        // Case for simple tasks (is_simple_task = 0)
        if ($is_simple_task == '1') {
            // Joining directly with the 'tasks' table and user related info
            $this->db->from('project_employees pe');
            $this->db->join('project p', 'pe.project_id = p.id');
            $this->db->join('task t', 't.project_hash = p.hash');
            $this->db->join('system_users u_sender', 'u_sender.hash = t.sender', 'left');
            if ($accept == '0') {
                $this->db->join('system_users u_reciever', 'u_reciever.hash = t.rejected_user', 'left');
            } else {
                $this->db->join('system_users u_reciever', 'u_reciever.hash = t.accept_user', 'left');
            }
            $this->db->join('emp_information info_sender', 'info_sender.user_hash = u_sender.hash', 'left');
            $this->db->join('emp_information info_reciever', 'info_reciever.user_hash = u_reciever.hash', 'left');
            $this->db->join('partments partments_sender', 'info_sender.partment = partments_sender.hash', 'left');
            $this->db->join('partments partments_reciever', 'info_reciever.partment = partments_reciever.hash', 'left');
            $this->db->join('project', 'project.hash = t.project_hash', 'left');
            $this->db->join('UserRoles ur', 'ur.user_hash = "' . $user . '"', 'left');

            $this->db->where("
                     CASE 
                         WHEN t.direct_task = 1 THEN 
                     ( t.sender = '$user' OR ur.role_id = 4)
                         ELSE 1 = 1
                     END
                 ");


            $this->db->where('t.is_simple', '1');
            $this->db->where_in('t.status', ['111']);
            if ($accept == '0') {
                $this->db->where('rejected_user !=', null);
            } else {
                $this->db->where('accept_user !=', null);
            }
            $this->db->where('pe.employee_id', $user);
            $this->db->where_in('pe.role_id', ['1', '2', '3', '4']);

        } else {

            $this->db->from('task t');
            $this->db->join('system_users u_sender', 'u_sender.hash = t.sender', 'left');
            // $this->db->join('system_users u_reciever', 'u_reciever.hash = t.accept_user', 'left');

            if ($accept == '0') {
                $this->db->join('system_users u_reciever', 'u_reciever.hash = t.rejected_user', 'left');
            } else {
                $this->db->join('system_users u_reciever', 'u_reciever.hash = t.accept_user', 'left');
            }
            $this->db->join('emp_information info_sender', 'info_sender.user_hash = u_sender.hash', 'left');
            $this->db->join('emp_information info_reciever', 'info_reciever.user_hash = u_reciever.hash', 'left');
            $this->db->join('partments partments_sender', 'info_sender.partment = partments_sender.hash', 'left');
            $this->db->join('partments partments_reciever', 'info_reciever.partment = partments_reciever.hash', 'left');
            $this->db->join('UserRoles ur', 'ur.user_hash = "' . $user . '"', 'left');

            $this->db->where("
                     CASE 
                         WHEN t.direct_task = 1 THEN 
                     ( t.sender = '$user' OR ur.role_id = 4)
                         ELSE 1 = 1
                     END
                 ");

            $this->db->where('t.is_simple', 0);
            $this->db->where_in('t.status', ['111']);
            if ($accept == '0') {
                $this->db->where('rejected_user !=', null);
            } else {
                $this->db->where('accept_user !=', null);
            }
            $this->db->group_start();
            $this->db->or_where('t.accept_user', $user);
            $this->db->or_where('t.sender', $user);
            $this->db->group_end();

            // Existing logic for non-simple tasks, roles 1, 2, 3, 4
            // $this->db->from('project_employees pe');
            // $this->db->join('project p', 'pe.project_id = p.id');
            // $this->db->join('task t', 't.project_hash = p.hash');
            // $this->db->join('system_users u_sender', 'u_sender.hash = t.sender', 'left');
            // $this->db->join('system_users u_reciever', 'u_reciever.hash = t.accept_user', 'left');
            // $this->db->join('emp_information info_sender', 'info_sender.user_hash = u_sender.hash', 'left');
            // $this->db->join('emp_information info_reciever', 'info_reciever.user_hash = u_reciever.hash', 'left');
            // $this->db->join('partments partments_sender', 'info_sender.partment = partments_sender.hash', 'left');
            // $this->db->join('partments partments_reciever', 'info_reciever.partment = partments_reciever.hash', 'left');
            // $this->db->join('project', 'project.hash = t.project_hash', 'left');
            // $this->db->where('t.is_simple', 0);
            // $this->db->where_in('t.status', ['111']);
            // if ($accept == '0') {
            //     $this->db->where('rejected_user !=', null);
            // } else {
            //     $this->db->where('accept_user !=', null);
            // }
            // $this->db->where('pe.employee_id', $user);
            // $this->db->where_in('pe.role_id', ['1', '2', '3', '4']);
        }

        // Common conditions for both queries
        if ($today) {
            $this->db->where("start_task <= '" . date('Y-m-d') . "' and t.deadline >= '" . date('Y-m-d') . "'");
            $this->db->order_by('reciever_department');
        }

        if ($search != '') {
            $this->db->group_start();
            $this->db->like('t.title', $search);
            $this->db->or_like('t.discription', $search);
            $this->db->group_end();
        }
        if ($department != '') {
            $this->db->like('info_reciever.position', $department);
        }
        if ($name != '') {
            $this->db->like('u_reciever.name', $name);
        }
        if ($projectHash != '') {
            $this->db->where('t.project_hash', $projectHash);
        }
        if ($task_status != '') {
            $this->db->where('t.status', $task_status);
        }

        if ($accept) {
            $this->db->where('t.isaccept', $accept);
        }

        if ($task_name_subscriber != '') {
            $this->db->like('t.task_name_subscriber', $task_name_subscriber);
        }

        if ($phone_num != '') {
            $this->db->like('t.phone_num', $phone_num);
        }
        $this->db->group_by('t.hash');

        $this->db->order_by('t.id', 'desc');
        $this->db->limit($perpage, $offset); // Apply pagination

        // Execute the first query
        $query1 = $this->db->get();
        $arr1 = $query1->result_array(); // Get the result as an array

        if ($is_simple_task == '1') {
            // No need for the second query as we're fetching simple tasks only
            return ['totalRows' => $totalRows, 'data' => $arr1];
        }

        // Reset and prepare for the second query for role 5 if not simple tasks
        $this->db->reset_query();

        // Apply the same common part for the second query
        $this->db->select("t.title, discription, t.deadline, score, acceptnote, t.updated_at, isaccept,
            t.status, t.hash, sender, receiver, project_hash, emergency,
            (select title from task_status where hash = t.status) as status_title,
            info_reciever.partment AS reciever_department_hash,
            info_sender.partment AS sender_department_hash,
            (select name from Positions where id = info_sender.position) as sender_position,
            (select name from Positions where id = info_reciever.position) as reciever_position,
            u_sender.name AS sender_name, u_reciever.name AS receiver_name,
            partments_reciever.name AS reciever_department,
            partments_sender.name AS sender_department,
            u_sender.image_profile AS sender_image,
            u_reciever.image_profile AS reciever_image,
            minutes_deadline,
            accept_date, action_date, TIMESTAMPDIFF(MINUTE, accept_date, action_date) AS task_completion_minutes,
            u_sender.hash as sender_hash, u_reciever.hash as receiver_hash");
        $this->db->from('project_employees pe');
        $this->db->join('project p', 'pe.project_id = p.id');
        $this->db->join('task t', 't.project_hash = p.hash');
        $this->db->join('system_users u_sender', 'u_sender.hash = t.sender', 'left');
        $this->db->join('system_users u_reciever', 'u_reciever.hash = t.accept_user', 'left');
        $this->db->join('emp_information info_sender', 'info_sender.user_hash = u_sender.hash', 'left');
        $this->db->join('emp_information info_reciever', 'info_reciever.user_hash = u_reciever.hash', 'left');
        $this->db->join('partments partments_sender', 'info_sender.partment = partments_sender.hash', 'left');
        $this->db->join('partments partments_reciever', 'info_reciever.partment = partments_reciever.hash', 'left');
        $this->db->join('project', 'project.hash = t.project_hash', 'left');
        $this->db->join('UserRoles ur', 'ur.user_hash = "' . $user . '"', 'left');

        $this->db->where("
                 CASE 
                     WHEN t.direct_task = 1 THEN 
                 ( t.sender = '$user' OR ur.role_id = 4)
                     ELSE 1 = 1
                 END
             ");

        $this->db->where_in('t.status', ['111']);
        $this->db->where('accept_user !=', null);

        $this->db->where('pe.employee_id', $user);
        $this->db->where('pe.role_id', '5');

        if ($today) {
            $this->db->where("start_task <= '" . date('Y-m-d') . "' and t.deadline >= '" . date('Y-m-d') . "'");
            $this->db->order_by('reciever_department');
        }

        if ($search != '') {
            $this->db->group_start();
            $this->db->like('t.title', $search);
            $this->db->or_like('t.discription', $search);
            $this->db->group_end();
        }
        if ($department != '') {
            $this->db->like('info_reciever.position', $department);
        }
        if ($name != '') {
            $this->db->like('u_reciever.name', $name);
        }
        if ($projectHash != '') {
            $this->db->where('t.project_hash', $projectHash);
        }
        if ($task_status != '') {
            $this->db->where('t.status', $task_status);
        }

        $this->db->group_by('t.hash');
        $this->db->order_by('t.id', 'desc');
        $this->db->limit($perpage, $offset); // Apply pagination


        // Execute the second query for role 5
        $query2 = $this->db->get();
        $arr2 = $query2->result_array(); // Get the result as an array

        // Combine the results
        return ['totalRows' => $totalRows, 'data' => array_merge($arr1, $arr2)];
    }

    private function get_total_rows_StillWithUser($page = 1, $perpage = 10, $search = '', $today = false, $department = '', $name = '', $projectHash = '', $is_simple_task = '', $task_status = '', $accept, $user)
    {
        // Build the query
        $this->db->select("COUNT(*) as total");

        if ($is_simple_task == '1') {
            $this->db->from('project_employees pe');
            $this->db->join('project p', 'pe.project_id = p.id');
            $this->db->join('task t', 't.project_hash = p.hash');
            $this->db->join('system_users u_sender', 'u_sender.hash = t.sender', 'left');
            if ($accept == '0') {
                $this->db->join('system_users u_reciever', 'u_reciever.hash = t.rejected_user', 'left');
            } else {
                $this->db->join('system_users u_reciever', 'u_reciever.hash = t.accept_user', 'left');
            }
            $this->db->join('emp_information info_sender', 'info_sender.user_hash = u_sender.hash', 'left');
            $this->db->join('emp_information info_reciever', 'info_reciever.user_hash = u_reciever.hash', 'left');
            $this->db->join('partments partments_sender', 'info_sender.partment = partments_sender.hash', 'left');
            $this->db->join('partments partments_reciever', 'info_reciever.partment = partments_reciever.hash', 'left');
            $this->db->join('project', 'project.hash = t.project_hash', 'left');
            $this->db->where('t.is_simple', '1');
            $this->db->where_in('t.status', ['111']);
            if ($accept == '0') {
                $this->db->where('rejected_user !=', null);
            } else {
                $this->db->where('accept_user !=', null);
            }
            $this->db->where('pe.employee_id', $user);
            $this->db->where_in('pe.role_id', ['1', '2', '3', '4']);
        } else {
            $this->db->from('task t');
            $this->db->join('system_users u_sender', 'u_sender.hash = t.sender', 'left');
            // $this->db->join('system_users u_reciever', 'u_reciever.hash = t.accept_user', 'left');
            if ($accept == '0') {
                $this->db->join('system_users u_reciever', 'u_reciever.hash = t.rejected_user', 'left');
            } else {
                $this->db->join('system_users u_reciever', 'u_reciever.hash = t.accept_user', 'left');
            }
            $this->db->join('emp_information info_sender', 'info_sender.user_hash = u_sender.hash', 'left');
            $this->db->join('emp_information info_reciever', 'info_reciever.user_hash = u_reciever.hash', 'left');
            $this->db->join('partments partments_sender', 'info_sender.partment = partments_sender.hash', 'left');
            $this->db->join('partments partments_reciever', 'info_reciever.partment = partments_reciever.hash', 'left');

            $this->db->where('t.is_simple', 0);
            $this->db->where_in('t.status', ['111']);
            if ($accept == '0') {
                $this->db->where('rejected_user !=', null);
            } else {
                $this->db->where('accept_user !=', null);
            }
            $this->db->group_start();
            $this->db->or_where('t.accept_user', $user);
            $this->db->or_where('t.sender', $user);
            $this->db->group_end();

        }

        // Join the necessary tables

        // Apply the accept filter
        // Common conditions for both queries
        if ($today) {
            $this->db->where("start_task <= '" . date('Y-m-d') . "' and t.deadline >= '" . date('Y-m-d') . "'");
            $this->db->order_by('reciever_department');
        }

        if ($search != '') {
            $this->db->group_start();
            $this->db->like('t.title', $search);
            $this->db->or_like('t.discription', $search);
            $this->db->group_end();
        }
        if ($department != '') {
            $this->db->like('info_reciever.position', $department);
        }
        if ($name != '') {
            $this->db->like('u_reciever.name', $name);
        }
        if ($projectHash != '') {
            $this->db->where('t.project_hash', $projectHash);
        }
        if ($task_status != '') {
            $this->db->where('t.status', $task_status);
        }

        if ($accept) {
            $this->db->where('t.isaccept', $accept);
        }

        // Execute the query and return the total count
        $query = $this->db->get();
        return $query->row()->total; // Return the total number of rows
    }

}
