<?php
class VacationModel extends CI_Model
{
    public function __construct()
    {
        parent::__construct();
        $this->load->database();
    }

    // get all Groups
    public function count_records($start_date, $end_date)
    {
        $headers = apache_request_headers();
        $search = $headers['q'] ?? '';
        $department = $this->input->get('department') ?? "";
        $this->db->select('count(*) as count');
        $this->db->from('vacations');
        $this->db->join('emp_information', 'emp_information.user_hash = vacations.user_hash', 'left');
        $this->db->where('vacations.admin IS NOT NULL');
        $this->db->where('vacations.hash IS NOT NULL');

        // where
        if (isset($start_date) && $start_date != "" && $start_date != 'null') {
            $this->db->where('DATE(date_start) >=', $start_date);
        }
        if (isset($end_date) && $end_date != "" && $end_date != 'null') {
            $this->db->where('DATE(date_end) <=', $end_date);
        }
        if (isset($search) && $search != "" && $search != 'null') {
            $this->db->where('vacations.user_hash', $search);
        }
        if (isset($department) && $department != "" && $department != 'null') {
            $this->db->where('emp_information.partment', $department);
        }
        $query = $this->db->get();
        return $query->result_array()[0]['count'];
    }

    public function getVacations($start_date, $end_date)
    {
        // Get page and limit and search from header
        $headers = apache_request_headers();
        $department = $this->input->get('department') ?? "";
        $search = isset($headers['q']) ? $headers['q'] : '';
        $page = $headers['page'] ?? $this->input->get('page') ?? 1;
        $perPage = $headers['perPage'] ?? $this->input->get('perPage') ?? 10;
        $offset = ($page - 1) * $perPage;

        $this->db->select("system_users.name as emp, system_users.image_profile as image");
        $this->db->select("vacations_type.type");
        $this->db->select("vacations_status.title as status_title");
        $this->db->select("partments.name as department");
        $this->db->select("system_group_name.name as job");
        $this->db->select("DATE(vacations.date_start) as date_s, vacations.date_start, vacations.date_end, TIME_FORMAT(TIME(vacations.date_start), '%l:%i %p') as time_start, vacations.vication_type_hash, DATE(vacations.date_end) as date_e, TIME_FORMAT(TIME(vacations.date_end), '%l:%i %p') as time_end, vacations.hash, vacations.note, vacations.user_hash");
        $this->db->from('vacations');
        $this->db->join('system_users', 'system_users.hash = vacations.user_hash', 'left');
        $this->db->join('vacations_type', 'vacations_type.hash = vacations.vication_type_hash', 'left');
        $this->db->join('vacations_status', 'vacations_status.hash = vacations.status', 'left');
        $this->db->join('emp_information', 'emp_information.user_hash = vacations.user_hash', 'left');
        $this->db->join('partments', 'partments.hash = emp_information.partment', 'left');
        $this->db->join('system_group_name', 'system_group_name.hash = system_users.user_type', 'left');
        $this->db->where('vacations.admin IS NOT NULL');
        $this->db->where('vacations.hash IS NOT NULL');
        // where
        // where
        // Where
        if (!empty($start_date) && $start_date != 'null') {
            $this->db->where('DATE(vacations.date_start) >=', $start_date);
        }
        if (!empty($end_date) && $end_date != 'null') {
            $this->db->where('DATE(vacations.date_end) <=', $end_date);
        }

        // Search
        if (!empty($search) && $search != 'null') {
            $this->db->like('vacations.user_hash', $search);
        }
        if (!empty($department) && $department != 'null') {
            $this->db->where('emp_information.partment', $department);
        }

        // Order by
        $this->db->order_by('vacations.id DESC');

        // Limit
        $this->db->limit($perPage, $offset);

        $query = $this->db->get();
        return $query->result_array();
    }

    public function getVacation($hash)
    {
        $this->db->select('(select name from system_users where hash=vacations_agreement.sender) as sender, hash, sender as sender_hash, receiver as receiver_hash');
        $this->db->select('(select name from system_users where hash=vacations_agreement.receiver) as receiver');
        $this->db->select(' (select title from vacations_status where vacations_status.hash=vacations_agreement.status) as statustitle');
        $this->db->select('time_format(date_entry, "%l:%i %p") as time,date(date_entry) as date,status,note');
        $this->db->from('vacations_agreement');
        $this->db->where('vacations_hash =', $hash);
        $query = $this->db->get();
        return $query->result_array();
    }

    public function getVacationType()
    {
        $this->db->select('type as label,hash as value');
        $this->db->from('vacations_type');
        $query = $this->db->get();
        return $query->result_array();
    }

    public function getVacationStatus()
    {
        $this->db->select('title as label,hash as value');
        $this->db->from('vacations_status');
        $query = $this->db->get();
        return $query->result_array();
    }

    public function checkIfVacationExist($user_hash, $date_start, $date_end)
    {
        $query = $this->db->query("SELECT hash FROM vacations WHERE user_hash = '" . $user_hash . "' AND date_start = '" . $date_start . "' AND date_end = '" . $date_end . "'" . " AND status = '444'");
        // retutn true if exist
        return $query->num_rows() > 0 ? true : false;
    }

    public function createVacation($data)
    {
        $validateRequest = $this->canEmployeeRequestVacation($data);
        if ($validateRequest['status'] == 'error') {
            return $validateRequest['message'];
        }

        $this->db->insert('vacations', $data);
        return $this->db->insert_id();
    }

    private function isTimeVacation($vacationTypeHash)
    {
        return in_array($vacationTypeHash, ['333', '444']);
    }
    private function isFlexibleEmployee($userHash)
    {
        $query = $this->db->select('flexible_schedule')->from('emp_information')->where("user_hash", $userHash)->get();

        if ($query->num_rows() > 0) {
            $result = $query->row();
            return (bool) $result->flexible_schedule;
        }

        return false;
    }

    private function canEmployeeRequestVacation($data)
    {
        $vactionTypeHash = $data['vication_type_hash'];
        $userHash = $data['user_hash'];
        $dateStart = date('Y-m-d', strtotime($data['date_start']));
        $dateEnd = date('Y-m-d', strtotime($data['date_end']));

        if ($this->isTimeVacation($vactionTypeHash)) {
            if ($this->isFlexibleEmployee($userHash)) {

                // what if user was on any time not was flexible schedule should handle this case
                $query = $this->db->select('id')->from('actual_schedules')->where('date', $dateStart)->where('date', $dateEnd)->get();

                if ($query->num_rows() > 0) {
                    return ['status' => 'success'];
                }

                return ['status' => 'error', 'message' => "عفوا لا يمكن طلب اجازه بوقت لانك موظف ببصمه مرنه"];
            }

            return ['status' => 'success'];
        };
        return ['status' => 'success'];
    }

    public function updateVacation($hash, $data)
    {
        $this->db->where('hash', $hash);
        $query = $this->db->get('vacations');
        $vacation = $query->row();

        if ($vacation && $vacation->status == '111') {
            return 'لا يمكن التعديل على اجازه تمت الموافقه عليها';
        }

        $this->db->where('hash', $hash);
        $this->db->update('vacations', $data);
        return $this->db->affected_rows();
    }

    public function deleteVacation($hash)
    {

        $this->db->where('hash', $hash);
        $query = $this->db->get('vacations');
        $vacation = $query->row();

        if ($vacation && $vacation->status == '111') {
            return 'لا يمكن حذف اجازه تمت الموافقه عليها';
        }

        $this->db->where('hash', $hash);
        $this->db->delete('vacations');
        // delete from vacations_agreement
        $this->db->where('vacations_hash', $hash);
        $this->db->delete('vacations_agreement');
        // return database message
        return $this->db->affected_rows();
    }

    public function getVacationTypeData($hash)
    {
        $vacationTypeData = $this->db->select('vication_type_hash as hash, user_hash, date_start, date_end')
            ->from('vacations')
            ->where('hash', $hash)
            ->get()
            ->row();
        return $vacationTypeData;
    }

    public function changeVacationStatus($hash, $agreement_details)
    {
        // Get the last status of the vacation
        $last_status = $this->db->select('status')->from('vacations')->where('hash', $hash)->get()->row()->status;

        if ($last_status == '111' || $last_status == '222') {
            return ['status' => 'error', 'message' => 'هذه الاجازة انتهت'];
        }

        // Update vacations and vacations_agreement tables
        $this->db->query("INSERT INTO vacations_agreement (vacations_hash, sender, receiver, status, note, flage, hash) VALUES('" . $hash . "', '" . $agreement_details['sender'] . "', '" . $agreement_details['receiver'] . "', '" . $agreement_details['status'] . "', '" . $agreement_details['note'] . "', 0, '" . $this->jwtmiddleware->createHash() . "')");
        $this->db->query("UPDATE vacations SET status = '" . $agreement_details['status'] . "' WHERE hash = '" . $hash . "'");
        $this->db->query("UPDATE vacations_agreement SET status = '" . $agreement_details['status'] . "' WHERE hash = '" . $agreement_details['agreeHash'] . "'");

        return ['status' => 'success', 'message' => 'Vacation status updated', 'status_code' => $agreement_details['status']];
    }

    public function createVacationAgreement($data)
    {
        $this->db->insert('vacations_agreement', $data);
        return $this->db->insert_id();
    }

    public function yearlyDetails($userHash)
    {
        $query = $this->db->query("
        SELECT
    v.id,
    v.user_hash,
    vt.hash AS vication_type_hash,
    v.date_start AS startDate,
    v.date_end AS endDate,
    SUM(DATEDIFF(v.date_end, v.date_start)) as differnece,
    (SELECT TIMESTAMPDIFF(HOUR, work_from, work_to) FROM emp_information WHERE user_hash = v.user_hash) AS number_of_hours_per_day,
    vt.type AS vacation_type,

     IFNULL(
    CASE
        WHEN vt.hash IN ('111', '222', '555', '333', '444') THEN
            CASE
                WHEN v.date_start < CONCAT(DATE(v.date_start), ' ', ei.work_from) THEN
                    -- Adjust the start time to the work start time if the vacation starts before work
                    SUM(DATEDIFF(v.date_end, v.date_start)) *
                        (SELECT TIMESTAMPDIFF(HOUR, work_from, work_to) FROM emp_information WHERE user_hash = v.user_hash)
                WHEN v.date_end > CONCAT(DATE(v.date_start), ' ', ei.work_to) THEN
                    -- Adjust the end time to the work end time if the vacation ends after work
                    SUM(DATEDIFF(v.date_end, v.date_start)) *
                        (SELECT TIMESTAMPDIFF(HOUR, work_from, work_to) FROM emp_information WHERE user_hash = v.user_hash)
                ELSE
                     ROUND(TIMESTAMPDIFF(MINUTE, v.date_start, v.date_end) / 60.0)
            END
        ELSE 0
    END,
0) AS number_of_hours_v,

   IFNULL(
    CASE
        WHEN DATEDIFF(v.date_end, v.date_start) = 0 AND vt.hash IN ('111','333','444','222', '555') THEN
            IF(TIMESTAMPDIFF(HOUR, v.date_start, v.date_end) >= (SELECT TIMESTAMPDIFF(HOUR, work_from, work_to) FROM emp_information WHERE user_hash = v.user_hash),
                1, -- If less than a day, consider it as a full day
                ROUND(SUM(DATEDIFF(v.date_end, v.date_start)) *
                     (SELECT TIMESTAMPDIFF(HOUR, work_from, work_to) FROM emp_information WHERE user_hash = v.user_hash) /
                     (SELECT TIMESTAMPDIFF(HOUR, work_from, work_to) FROM emp_information WHERE user_hash = v.user_hash)) -- Otherwise, calculate the days normally
            )
        WHEN v.vication_type_hash IN ('111','333','444','222', '555') THEN
            Round(SUM(DATEDIFF(v.date_end, v.date_start))
			 *
                 (SELECT TIMESTAMPDIFF(HOUR, work_from, work_to) FROM emp_information WHERE user_hash = v.user_hash)
                 /
                 (SELECT TIMESTAMPDIFF(HOUR, work_from, work_to) FROM emp_information WHERE user_hash = v.user_hash))

        WHEN (v.date_start IS NULL OR v.date_end IS NULL) THEN
            0
        ELSE
            'not found'
    END,
    0
) AS number_of_days
FROM
    vacations_type vt
LEFT JOIN
    vacations v ON vt.hash = v.vication_type_hash AND v.status = '111'
AND v.user_hash = '$userHash'
AND YEAR(v.date_start) = YEAR(CURRENT_DATE())
LEFT JOIN
    emp_information ei ON v.user_hash = ei.user_hash
GROUP BY
    v.user_hash,
    vt.hash;
    ");

        return $query->result_array();

    }

    public function monthlyDetails($userHash)
    {

        $query = $this->db->query("
        SELECT
    v.id,
    v.user_hash,
    vt.hash AS vication_type_hash,
    v.date_start AS startDate,
    v.date_end AS endDate,
    SUM(DATEDIFF(v.date_end, v.date_start)) as differnece,
    (SELECT TIMESTAMPDIFF(HOUR, work_from, work_to) FROM emp_information WHERE user_hash = v.user_hash) AS number_of_hours_per_day,
    vt.type AS vacation_type,

     IFNULL(
    CASE
        WHEN vt.hash IN ('111', '222', '555', '333', '444') THEN
            CASE
                WHEN v.date_start < CONCAT(DATE(v.date_start), ' ', ei.work_from) THEN
                    -- Adjust the start time to the work start time if the vacation starts before work
                    SUM(DATEDIFF(v.date_end, v.date_start)) *
                        (SELECT TIMESTAMPDIFF(HOUR, work_from, work_to) FROM emp_information WHERE user_hash = v.user_hash)
                WHEN v.date_end > CONCAT(DATE(v.date_start), ' ', ei.work_to) THEN
                    -- Adjust the end time to the work end time if the vacation ends after work
                    SUM(DATEDIFF(v.date_end, v.date_start)) *
                        (SELECT TIMESTAMPDIFF(HOUR, work_from, work_to) FROM emp_information WHERE user_hash = v.user_hash)
                ELSE
                     ROUND(TIMESTAMPDIFF(MINUTE, v.date_start, v.date_end) / 60.0)
            END
        ELSE 0
    END,
0) AS number_of_hours_v,

   IFNULL(
    CASE
        WHEN DATEDIFF(v.date_end, v.date_start) = 0 AND vt.hash IN ('111','333','444','222', '555') THEN
            IF(TIMESTAMPDIFF(HOUR, v.date_start, v.date_end) >= (SELECT TIMESTAMPDIFF(HOUR, work_from, work_to) FROM emp_information WHERE user_hash = v.user_hash),
                1, -- If less than a day, consider it as a full day
                ROUND(SUM(DATEDIFF(v.date_end, v.date_start)) *
                     (SELECT TIMESTAMPDIFF(HOUR, work_from, work_to) FROM emp_information WHERE user_hash = v.user_hash) /
                     (SELECT TIMESTAMPDIFF(HOUR, work_from, work_to) FROM emp_information WHERE user_hash = v.user_hash)) -- Otherwise, calculate the days normally
            )
        WHEN v.vication_type_hash IN ('111','333','444','222', '555') THEN
            Round(SUM(DATEDIFF(v.date_end, v.date_start))
			 *
                 (SELECT TIMESTAMPDIFF(HOUR, work_from, work_to) FROM emp_information WHERE user_hash = v.user_hash)
                 /
                 (SELECT TIMESTAMPDIFF(HOUR, work_from, work_to) FROM emp_information WHERE user_hash = v.user_hash))

        WHEN (v.date_start IS NULL OR v.date_end IS NULL) THEN
            0
        ELSE
            'not found'
    END,
    0
) AS number_of_days
    FROM
        vacations_type vt
    LEFT JOIN
        vacations v ON vt.hash = v.vication_type_hash AND v.status = '111'
    AND v.user_hash = '$userHash'
    AND YEAR(v.date_start) = YEAR(CURRENT_DATE())
    AND MONTH(v.date_start) = MONTH(CURDATE())
    LEFT JOIN
        emp_information ei ON v.user_hash = ei.user_hash
    GROUP BY
        v.user_hash,
        vt.hash;

    ");
        return $query->result_array();
    }
    public function Details($userHash)
    {
        $query = $this->db->query("
          SELECT
              (select name from system_users where hash=vacations_agreement.sender limit 1) as sender,
              (select name from system_users where hash=vacations_agreement.receiver limit 1) as receiver,
              (select title from vacations_status where hash=vacations_agreement.status limit 1) as status,
              note,
              hash
          FROM
              vacations_agreement
          where
              vacations_hash='$userHash'
                ");
        return $query->result_array();
    }
    public function getUserVacationReport($userHash, $type)
    {
        $this->db->select("
            CASE
                WHEN vication_type_hash = 111 THEN 'sick'
                WHEN vication_type_hash = 444 THEN 'task'
                ELSE 'vacation'
            END AS type_v
        ");

        $this->db->from('emp_information');
        $this->db->join('vacations', 'emp_information.user_hash = vacations.user_hash', 'left');
        $this->db->where('emp_information.user_hash', $userHash);
        $this->db->where('vacations.status', "111");
        if ($type == 'month') {
            $this->db->where('MONTH(vacations.date_start)', date('m'));
            $this->db->where('YEAR(vacations.date_start)', date('Y'));
            $this->db->select("get_vacations_current_month(vication_type_hash,emp_information.user_hash) as days");
        } else {
            $this->db->where('YEAR(vacations.date_start)', date('Y'));
            $this->db->select("get_vacations_current_year(vication_type_hash,emp_information.user_hash) as days");

        }
        $this->db->group_by('type_v , emp_information.user_hash');

        $query = $this->db->get();
        return $query->result_array();
    }

    public function getUserVacaitionLimit($userHash)
    {
        $this->db->select('vacations_limit * TIMESTAMPDIFF(HOUR,work_from,work_to) as vacation, sick_leave * TIMESTAMPDIFF(HOUR,work_from,work_to) as sick');
        $this->db->select('(select name from system_users where system_users.hash=user_hash limit 1) as name');
        $this->db->from('emp_information');
        $this->db->where('user_hash', $userHash);
        $query = $this->db->get();
        return $query->result_array();
    }
}

// vacation report to get all vacations of one user during a year and check if he is volatile his limits

/*
SELECT
DATEDIFF(`vacations`.`date_end`, `vacations`.`date_start`) +1 AS days_count,
system_users.name AS employee_name,
emp_information.vacations_limit,
emp_information.sick_leave AS sick_leave_limit,
vacations_type.type AS vacations_type

FROM `vacations`
JOIN `system_users` ON vacations.user_hash = system_users.hash
JOIN emp_information ON emp_information.user_hash = vacations.user_hash
JOIN vacations_type ON vacations.vication_type_hash = vacations_type.hash

WHERE
YEAR(vacations.date_start) = YEAR(CURRENT_DATE)
AND
system_users.hash = 16598976805413171

GROUP BY system_users.name, vacations_type.type
 */

/*
 *
 *
SELECT
DATEDIFF(`vacations`.`date_end`, `vacations`.`date_start`) +1 AS days_count,
system_users.name AS employee_name,
emp_information.vacations_limit AS vacations_limit_year,
emp_information.sick_leave AS sick_leave_limit_year,
CEIL(emp_information.vacations_limit /12) AS vacations_limit_month,
CEIL(emp_information.sick_leave/12) AS sick_leave_limit_month,
vacations_type.type AS vacations_type,
CASE
WHEN (DATEDIFF(`vacations`.`date_end`, `vacations`.`date_start`) +1) > emp_information.vacations_limit AND vacations_type.hash != 111 THEN "إجتاز العدد المسموح به"
ELSE "لم يجتز الحد المسموح به"
END AS vacations_limit_check,

CASE
WHEN (DATEDIFF(`vacations`.`date_end`, `vacations`.`date_start`) +1) > emp_information.sick_leave AND vacations_type.hash = 111 THEN "إجتاز العدد المسموح به"
ELSE "لم يجتز الحد المسموح به"
END AS sick_leave_limit_check

FROM `vacations`
JOIN `system_users` ON vacations.user_hash = system_users.hash
JOIN emp_information ON emp_information.user_hash = vacations.user_hash
JOIN vacations_type ON vacations.vication_type_hash = vacations_type.hash

WHERE
YEAR(vacations.date_start) = YEAR(CURRENT_DATE)
AND
system_users.hash = 16598976805413171

GROUP BY system_users.name, vacations_type.type
 * */

/*
SELECT
DATEDIFF(v.date_end, v.date_start) + 1 AS days_count_year,
su.name AS employee_name,
ei.vacations_limit AS vacations_limit_year,
ei.sick_leave AS sick_leave_limit_year,
CEIL(ei.vacations_limit / 12) AS vacations_limit_month,
CEIL(ei.sick_leave / 12) AS sick_leave_limit_month,
vt.type AS vacations_type,
IFNULL(vm.days_count_vacation_month, 0) AS days_count_vacation_month,
IFNULL(sm.days_count_sick_month, 0) AS days_count_sick_month,
CASE
WHEN (DATEDIFF(v.date_end, v.date_start) + 1) > ei.vacations_limit AND vt.hash != 111 THEN "إجتاز العدد المسموح به"
ELSE "خلال لم يجتز الحد المسموح به خلال السنة"
END AS vacations_limit_check,
CASE
WHEN (DATEDIFF(v.date_end, v.date_start) + 1) > ei.sick_leave AND vt.hash = 111 THEN "إجتاز العدد المسموح به"
ELSE "لم يجتز الحد المسموح به خلال السنة"
END AS sick_leave_limit_check,
IFNULL(cm.total_days_in_current_month, 0) AS total_days_in_current_month,
CASE
WHEN IFNULL(cm.total_days_in_current_month, 0) > CEIL(ei.vacations_limit / 12) THEN "إجتاز الحد الشهري المسموح به"
ELSE "لم يجتز الحد الشهري المسموح به"
END AS monthly_vacations_limit_check
FROM system_users su
JOIN emp_information ei ON su.hash = ei.user_hash
JOIN vacations v ON su.hash = v.user_hash
JOIN vacations_type vt ON v.vication_type_hash = vt.hash
LEFT JOIN (
SELECT
v.user_hash,
SUM(DATEDIFF(v.date_end, v.date_start) + 1) AS total_days_in_current_month
FROM vacations v
WHERE MONTH(v.date_start) = MONTH(CURRENT_DATE)
AND YEAR(v.date_start) = YEAR(CURRENT_DATE)
GROUP BY v.user_hash
) cm ON su.hash = cm.user_hash
LEFT JOIN (
SELECT
v.user_hash,
SUM(DATEDIFF(v.date_end, v.date_start) + 1) AS days_count_vacation_month
FROM vacations v
WHERE MONTH(v.date_start) = MONTH(CURRENT_DATE)
AND YEAR(v.date_start) = YEAR(CURRENT_DATE)
AND v.vication_type_hash != 111
GROUP BY v.user_hash
) vm ON su.hash = vm.user_hash
LEFT JOIN (
SELECT
v.user_hash,
SUM(DATEDIFF(v.date_end, v.date_start) + 1) AS days_count_sick_month
FROM vacations v
WHERE MONTH(v.date_start) = MONTH(CURRENT_DATE)
AND YEAR(v.date_start) = YEAR(CURRENT_DATE)
AND v.vication_type_hash = 111
GROUP BY v.user_hash
) sm ON su.hash = sm.user_hash
WHERE YEAR(v.date_start) = YEAR(CURRENT_DATE)
AND su.hash = 16598976805413171
GROUP BY su.name, vt.type;

 */
