<?php
defined('BASEPATH') or exit('No direct script access allowed');

class Zk_queModel extends CI_Model
{
    protected $table = 'zk_queue';

    public function __construct()
    {
        parent::__construct();
        $this->load->database();
    }

    public function create($data)
    {
        if ($this->checkIsDuplicate($data)) {
            return false; // Prevent duplicate entry
        }
        $this->db->insert($this->table, $data);
        return $this->db->insert_id(); // Return the ID of the newly created record
    }

    private function checkIsDuplicate($data)
    {
        if (!isset($data['task_type']) || empty($data['task_type'])) {
            return false; // Invalid data
        }

        $task_type = $data['task_type'];

        if ($task_type === 'get_user_finger_templates') {
            $task_data = $data['task_data']; // Keep as JSON string

            $query = $this->db->query(
                "
                SELECT COUNT(*) as count 
                FROM {$this->table} 
                WHERE task_type = ? 
                AND JSON_CONTAINS(task_data, ?) 
                AND status = 'pending'",
                [$task_type, $task_data]
            );

            $row = $query->row();
            return $row->count > 0; // True if a duplicate exists
        }

        return false;
    }



    public function getPending()
    {
        return $this->db->where_in('status', ['pending'])
            ->get($this->table)
            ->result_array();
    }


    public function returnToPending($id)
    {
        $this->db->where('id', $id);
        $this->db->update($this->table, array('status' => 'pending'));
        return $this->db->affected_rows();
    }

    public function getAll()
    {
        return $this->db->get($this->table)->result_array();
    }

    public function getById($id)
    {
        return $this->db->get_where($this->table, ['id' => $id])->row_array();
    }

    public function updateStatus($id, $data)
    {
        $this->db->where('id', $id);
        $this->db->update($this->table, $data);
        return $this->db->affected_rows(); // Return the number of affected rows
    }

    public function update($id, $data)
    {
        $this->db->where('id', $id);
        $this->db->update($this->table, $data);
        return $this->db->affected_rows(); // Return the number of affected rows
    }

    public function delete($id)
    {
        $this->db->where('id', $id);
        $this->db->delete($this->table);
        return $this->db->affected_rows(); // Return the number of affected rows
    }
}