<?php
defined('BASEPATH') or exit('No direct script access allowed');

class Zk_usersModel extends CI_Model
{

    private $table = 'zk_users';

    public function __construct()
    {
        parent::__construct();
    }

    // Fetch all users
    public function get_all_users()
    {
        $q = $this->input->get('q') ?? '';

        // Base query
        $sql = "
           SELECT 
    zk_queue.task_type,
    CASE 
        WHEN zk_users.id IS NULL THEN zk_queue.id
        ELSE zk_users.id 
    END AS id,
    CASE 
        WHEN zk_users.id IS NULL 
        THEN JSON_UNQUOTE(JSON_EXTRACT(zk_queue.task_data, '$.name')) 
        ELSE zk_users.name 
    END AS name,
    CASE 
        WHEN zk_users.id IS NULL 
        THEN JSON_UNQUOTE(JSON_EXTRACT(zk_queue.task_data, '$.user_hash')) 
        ELSE zk_users.user_hash
    END AS user_hash,
    CASE 
        WHEN zk_users.id IS NULL 
        THEN JSON_UNQUOTE(JSON_EXTRACT(zk_queue.task_data, '$.device_hash')) 
        ELSE zk_users.device_hash
    END AS device_hash,
    CASE 
        WHEN zk_users.id IS NULL THEN 'queue' 
        ELSE 'zk_users' 
    END AS source,
    CASE 
        WHEN zk_users.finger_print_template IS NULL THEN NULL
        ELSE zk_users.finger_print_template
    END AS finger_print_template,
    CASE 
        WHEN zk_users.face_finger_print_template IS NULL THEN NULL
        ELSE zk_users.face_finger_print_template
    END AS face_finger_print_template,
    zk_queue.status,
    zk_devices.name AS device_name,
    zk_devices.port AS device_port,
    zk_devices.ip AS device_ip,
    CASE 
        WHEN q_delete.user_hash IS NOT NULL 
        THEN 
            CASE 
                WHEN q_delete.status = 'in_progress' THEN 'in_progress'
                WHEN q_delete.status = 'pending' THEN 'pending'
                WHEN q_delete.status = 'completed' THEN 'completed'
                ELSE 'not_in_progress'
            END
        ELSE 'user_not_created'
    END AS que_status_deleted,
    -- New column: Check if a pending finger template request exists
    CASE 
       WHEN (
            SELECT COUNT(*) 
            FROM zk_queue AS t
            WHERE t.task_type = 'get_user_finger_templates'
              AND t.status = 'pending'
              AND t.user_hash = (
                    CASE 
                        WHEN zk_users.id IS NULL 
                        THEN JSON_UNQUOTE(JSON_EXTRACT(zk_queue.task_data, '$.user_hash'))
                        ELSE zk_users.user_hash
                    END
              )
       ) > 0 
       THEN 1 
       ELSE 0 
    END AS finger_template_requested
FROM zk_queue
LEFT JOIN zk_devices 
    ON JSON_UNQUOTE(JSON_EXTRACT(zk_queue.task_data, '$.device_hash')) = zk_devices.id
LEFT JOIN zk_users 
    ON zk_users.user_hash = zk_queue.user_hash
LEFT JOIN (
    SELECT user_hash, status 
    FROM zk_queue 
    WHERE task_type = 'delete_user'
) AS q_delete 
    ON q_delete.user_hash = zk_queue.user_hash
WHERE zk_queue.task_type = 'create_user'
GROUP BY zk_users.id;

        ";

        // Add search condition if $q is provided
        if ($q) {
            $q = $this->db->escape_like_str($q); // Escape the search term for safety
            $sql .= " HAVING zk_users.name LIKE '%$q%' OR zk_devices.name LIKE '%$q%'";
        }

        // Execute the query
        return $this->db->query($sql)->result_array();
    }

    public function getUsersCreated()
    {
        $this->db->select('zk_u.id, zk_u.name, zk_u.user_hash, zk_u.device_hash, zk_d.name as device_name, zk_d.port as device_port, zk_d.ip as device_ip, zk_u.finger_print_template, zk_u.face_finger_print_template');
        $this->db->from('zk_users as zk_u');
        $this->db->join('zk_devices as zk_d', 'zk_u.device_hash = zk_d.id');
        $query = $this->db->get();
        return $query->result_array();
    }

    // Fetch a single user by ID
    public function get_user_by_id($id)
    {
        return $this->db->get_where($this->table, ['id' => $id])->row_array();
    }

    // Insert a new user
    public function insert_user($data)
    {
        $this->db->insert($this->table, $data);
        return $this->db->insert_id();
    }

    public function updateUserFinger($data)
    {
        $userHash = $data['user_hash'];
        $deviceHash = $data['device_hash'];
        $jsonData = $data['data'];

        $this->db->where('user_hash', $userHash)
            ->where('device_hash', $deviceHash)
            ->update($this->table, ['finger_print_template' => $jsonData]);

        return $this->db->affected_rows() > 0;
    }
    public function updateUserFace($data)
    {
        $userHash = $data['user_hash'];
        $deviceHash = $data['device_hash'];
        $jsonData = $data['data'];

        $this->db->where('user_hash', $userHash)
            ->where('device_hash', $deviceHash)
            ->update($this->table, ['face_finger_print_template' => $jsonData]);

        return $this->db->affected_rows() > 0;
    }


    // Update an existing user
    public function update_user($id, $data)
    {
        $this->db->where('id', $id);
        return $this->db->update($this->table, $data);
    }

    // Delete a user
    public function delete_user($id)
    {
        $this->db->where('user_hash', $id);
        return $this->db->delete($this->table);
    }
}
