<?php

function calculateHours($startTime, $endTime)
{
    $start = new DateTime($startTime);
    $end = new DateTime($endTime);
    $interval = $start->diff($end);
    return ($interval->h * 60 + $interval->i) / 60; // Convert to hours
}

$config_path = '/var/www/html/spaceLight/app/application/config/database.php';

// Check if the CodeIgniter database configuration file exists
if (file_exists($config_path)) {
    // Include the configuration file
    include $config_path;

    // Debugging: Check if $db is actually an array
    if (!is_array($db)) {
        die("Error: \$db is not an array. The config file might not be loaded correctly.");
    }

    // Ensure active group is set
    $active_group = isset($active_group) ? $active_group : 'default';

    // Check if the active group exists in $db
    if (!isset($db[$active_group])) {
        die("Error: Active group '$active_group' does not exist in \$db.");
    }

    // Dynamically set these variables from the CodeIgniter config
    $servername = $db[$active_group]['hostname'];
    $username = $db[$active_group]['username'];
    $password = $db[$active_group]['password'];
    $dbname = $db[$active_group]['database'];

} else {
    die('Error: Configuration file not found.');
}

// Create connection using MySQLi
$conn = new mysqli($servername, $username, $password, $dbname);

// Check connection
if ($conn->connect_error) {
    die("Connection failed: " . $conn->connect_error);
}

// SQL query to get data
$sql = "SELECT
            JSON_OBJECT(
                'user_hash', ei.user_hash,
                'schedule_for_day', dft.target_hours,
                'in_fingerprint_time', CONCAT('[', GROUP_CONCAT(CASE
                    WHEN ef.in_out = 'in' THEN CONCAT('\"', DATE_FORMAT(ef.fingerprint_time, '%Y-%m-%d %H:%i:%s'), '\"')
                END ORDER BY ef.fingerprint_time SEPARATOR ','), ']'),
                'out_fingerprint_time', CONCAT('[', GROUP_CONCAT(CASE
                    WHEN ef.in_out = 'out'
                    AND (
                        DATE(ef.fingerprint_time) = DATE_SUB(CURDATE(), INTERVAL 1 DAY)
                        OR (
                            DATE(ef.fingerprint_time) = CURDATE()
                            AND TIME(ef.fingerprint_time) < '03:57:00'
                        )
                    )
                    THEN CONCAT('\"', DATE_FORMAT(ef.fingerprint_time, '%Y-%m-%d %H:%i:%s'), '\"')
                END ORDER BY ef.fingerprint_time DESC SEPARATOR ','), ']'),
                'date', dft.date
            ) AS employee_data
        FROM
            emp_information ei
        JOIN
            daily_flexible_schedule_targets dft ON ei.user_hash = dft.user_hash
            AND dft.date = DATE_SUB(CURDATE(), INTERVAL 1 DAY)
  	AND dft.is_deleted = '0'
        LEFT JOIN
            emp_fingerprint ef ON ei.user_hash = ef.user_hash
            AND (
                (DATE(ef.fingerprint_time) = DATE_SUB(CURDATE(), INTERVAL 1 DAY))
                OR (
                    DATE(ef.fingerprint_time) = CURDATE()
                    AND TIME(ef.fingerprint_time) < '03:57:00'
                )
            )
        WHERE
            dft.target_hours IS NOT NULL
            AND TRIM(dft.target_hours) <> ''
         AND ei.user_hash NOT IN (SELECT user_hash FROM employees_worked_hours WHERE date = CURDATE())

        GROUP BY
            ei.user_hash, dft.date";

$sql_fix = "SELECT
            JSON_OBJECT(
                'user_hash', ei.user_hash,
                'schedule_for_day', dft.target_hours,
                'in_fingerprint_time', CONCAT('[', GROUP_CONCAT(CASE
                    WHEN ef.in_out = 'in' THEN CONCAT('\"', DATE_FORMAT(ef.fingerprint_time, '%Y-%m-%d %H:%i:%s'), '\"')
                END ORDER BY ef.fingerprint_time SEPARATOR ','), ']'),
                'out_fingerprint_time', CONCAT('[', GROUP_CONCAT(CASE
                    WHEN ef.in_out = 'out'
                    AND (
                        DATE(ef.fingerprint_time) BETWEEN '2024-09-16' AND '2024-09-24'
                        OR (
                            DATE(ef.fingerprint_time) = '2024-09-24'
                            AND TIME(ef.fingerprint_time) < '03:57:00'
                        )
                    )
                    THEN CONCAT('\"', DATE_FORMAT(ef.fingerprint_time, '%Y-%m-%d %H:%i:%s'), '\"')
                END ORDER BY ef.fingerprint_time DESC SEPARATOR ','), ']'),
                'date', dft.date
            ) AS employee_data
        FROM
            emp_information ei
        JOIN
            daily_flexible_schedule_targets dft ON ei.user_hash = dft.user_hash
            AND dft.date BETWEEN '2024-09-16' AND '2024-09-24'
            AND dft.is_deleted = '0'
        LEFT JOIN
            emp_fingerprint ef ON ei.user_hash = ef.user_hash
            AND (
                (DATE(ef.fingerprint_time) BETWEEN '2024-09-16' AND '2024-09-24')
                OR (
                    DATE(ef.fingerprint_time) = '2024-09-24'
                    AND TIME(ef.fingerprint_time) < '03:57:00'
                )
            )
        WHERE
            dft.target_hours IS NOT NULL
            AND TRIM(dft.target_hours) <> ''
            AND ei.user_hash NOT IN (SELECT user_hash FROM employees_worked_hours WHERE date BETWEEN '2024-09-16' AND '2024-09-24')
        GROUP BY
            ei.user_hash, dft.date";

$result = $conn->query($sql_fix);

if ($result->num_rows > 0) {
    $data = [];
    while ($row = $result->fetch_assoc()) {
        $data[] = json_decode($row['employee_data'], true);
    }

    foreach ($data as $entry) {
        $user_hash = $entry['user_hash'];
        $date = $entry['date'];
        $targetHours = $entry['schedule_for_day'];
        $inFingerprintTimes = json_decode($entry['in_fingerprint_time'], true);
        $outFingerprintTimes = json_decode($entry['out_fingerprint_time'], true);

        // Calculate total worked hours
        $totalWorkedHours = 0;
        $usedOutTimes = [];

        foreach ($inFingerprintTimes as $inTime) {
            $closestOutTime = null;
            $minDiff = PHP_INT_MAX;

            foreach ($outFingerprintTimes as $outIndex => $outTime) {
                if (in_array($outIndex, $usedOutTimes)) {
                    continue; // Skip if already used
                }

                $diff = strtotime($outTime) - strtotime($inTime);

                if ($diff >= 0 && $diff < $minDiff) {
                    $minDiff = $diff;
                    $closestOutTime = $outTime;
                    $closestOutIndex = $outIndex;
                }
            }

            if ($closestOutTime !== null) { // Use !== to ensure we are checking against null and not a falsy value
                $workedHours = calculateHours($inTime, $closestOutTime);
                $totalWorkedHours += $workedHours;
                $usedOutTimes[] = $closestOutIndex; // Mark this out time as used
            }
        }

        // Calculate missed hours and overtime
        $missedHours = max(0, $targetHours - $totalWorkedHours);
        $overTimeHours = max(0, $totalWorkedHours - $targetHours);

        // Initialize additional fields
        $totalLateArrivalMinutes = 0; // Placeholder, calculation logic can be added
        $totalEarlyDepartureMinutes = 0; // Placeholder, calculation logic can be added
        $isFlexible = 1;

        // Prepare the SQL statement to insert data into employees_worked_hours
        $insert_sql = "INSERT INTO employees_worked_hours
               (user_hash, date, total_late_arrival_minutes, total_early_departure_minutes,
                total_missed_hours, total_worked_hours, target_hours, overtime_hours, is_flexible)
               VALUES (?, ?, ?, ?, ?, ?, ?, ?, ?)
               ON DUPLICATE KEY UPDATE
               total_late_arrival_minutes = VALUES(total_late_arrival_minutes),
               total_early_departure_minutes = VALUES(total_early_departure_minutes),
               total_missed_hours = VALUES(total_missed_hours),
               total_worked_hours = VALUES(total_worked_hours),
               target_hours = VALUES(target_hours),
               overtime_hours = VALUES(overtime_hours),
               is_flexible = VALUES(is_flexible)";

        // Prepare and bind
        $stmt = $conn->prepare($insert_sql);
        $stmt->bind_param("ssiiiiiii",
            $user_hash,
            $date,
            $totalLateArrivalMinutes,
            $totalEarlyDepartureMinutes,
            $missedHours,
            $totalWorkedHours,
            $targetHours,
            $overTimeHours,
            $isFlexible
        );

        // Execute the prepared statement
        if ($stmt->execute()) {
            echo "Data for User Hash: $user_hash inserted successfully.\n";
        } else {
            echo "Error inserting data for User Hash: $user_hash. " . $stmt->error . "\n";
        }

        // Close the statement
        $stmt->close();

        // Output the results
        echo "In Times: " . json_encode($inFingerprintTimes) . "\n";
        echo "Out Times: " . json_encode($outFingerprintTimes) . "\n";

        echo "User Hash: $user_hash\n";
        echo "Date: $date\n";
        echo "Target Hours: $targetHours\n";
        echo "Worked Hours: $totalWorkedHours\n";
        echo "Missed Hours: $missedHours\n";
        echo "Overtime Hours: $overTimeHours\n";
        echo "--------------------------\n";
    }
} else {
    $data = [];
}

$conn->close();
