<?php

$config_path = '/var/www/html/spaceLight/app/application/config/database.php';
// $config_path = '../config/database.php';

// Check if the CodeIgniter database configuration file exists
if (file_exists($config_path)) {
    // Include the configuration file
    include $config_path;

    // Debugging: Check if $db is actually an array
    if (!is_array($db)) {
        die("Error: \$db is not an array. The config file might not be loaded correctly.");
    }

    // Ensure active group is set
    $active_group = isset($active_group) ? $active_group : 'default';

    // Check if the active group exists in $db
    if (!isset($db[$active_group])) {
        die("Error: Active group '$active_group' does not exist in \$db.");
    }

    // Continue with database connection using $db[$active_group]
    $host = $db[$active_group]['hostname'];
    $db_name = $db[$active_group]['database'];
    $user = $db[$active_group]['username'];
    $pass = $db[$active_group]['password'];
    $charset = 'utf8mb4'; // Define your charset

    // Create DSN and set PDO options
    $dsn = "mysql:host=$host;dbname=$db_name;charset=$charset";
    $options = [
        PDO::ATTR_ERRMODE => PDO::ERRMODE_EXCEPTION,
        PDO::ATTR_DEFAULT_FETCH_MODE => PDO::FETCH_ASSOC,
        PDO::ATTR_EMULATE_PREPARES => false,
    ];

    // Try establishing a PDO connection
    try {
        $pdo = new PDO($dsn, $user, $pass, $options);
        echo "Database connection successful!";
    } catch (PDOException $e) {
        // Log connection error
        error_log("Database connection failed: " . $e->getMessage(), 3, './error_log.txt');
        die('Database connection failed.');
    }
} else {
    error_log("Error: Configuration file not found" . $e->getMessage(), 3, './error_log.txt');

    die('Error: Configuration file not found.');
}

// Insert into vacations
try {
    $sql1 = "INSERT INTO vacations (user_hash, vication_type_hash, status, date_start, date_end, duration_day)
SELECT
    uv.user_hash,
    666,
    111,
    CURDATE() + INTERVAL 1 DAY,
    CURDATE() + INTERVAL 2 DAY,
    1
FROM
    user_vacations uv
LEFT JOIN vacations v ON uv.user_hash = v.user_hash
                       AND v.date_start = CURDATE() + INTERVAL 1 DAY
                       AND v.vication_type_hash = 666
WHERE
    uv.day_of_week_id = DAYOFWEEK(CURDATE() + INTERVAL 1 DAY)
    AND v.user_hash IS NULL;
";
    $pdo->exec($sql1);
} catch (Exception $e) {
    error_log("Error in vacations insert: " . $e->getMessage(), 3, './error_log.txt');
}

try {
    $sql = "
      INSERT INTO vacations (user_hash, vication_type_hash, status, date_start, date_end, duration_day, is_work)
    SELECT ei.user_hash, '555' as vacation_type_hash, '111' as status,
    CURDATE() + INTERVAL 1 DAY as date_start, CURDATE() + INTERVAL 2 DAY as date_end, 1 as duration_day,
    '0' as is_work
    FROM emp_information ei
    WHERE ei.is_work = '2'
    AND (ei.stop_work_date IS NULL OR CURDATE() >= ei.stop_work_date)

";

    $pdo->exec($sql);
} catch (Exception $e) {
    error_log("Error in vacations stopped work insert: " . $e->getMessage(), 3, './error_log.txt');
}

// Insert into actual_schedules
try {
    $sql2 = "INSERT INTO actual_schedules (user_hash, date, schedule_data, allowance_time)
             SELECT ei.user_hash, CURDATE() + INTERVAL 1 DAY,
                    JSON_UNQUOTE(JSON_EXTRACT(sg.schedule_data, CONCAT('$.', DAYNAME(CURDATE() + INTERVAL 1 DAY)))) AS schedule_data,
                    sg.allowance_time
             FROM emp_information ei
             JOIN schedule_groups sg ON ei.work_schedule = sg.id
             LEFT JOIN user_vacations uv ON ei.user_hash = uv.user_hash
                                         AND uv.day_of_week_id = DAYOFWEEK(CURDATE() + INTERVAL 1 DAY)
             LEFT JOIN vacations v ON ei.user_hash = v.user_hash
                                    AND (
                                        CURDATE() + INTERVAL 1 DAY BETWEEN v.date_start AND DATE(v.date_end - INTERVAL 1 DAY)
                                        OR (v.date_start = CURDATE() + INTERVAL 1 DAY AND v.date_end = CURDATE() + INTERVAL 1 DAY)
                                    )
            LEFT JOIN holiday h ON CURDATE() + INTERVAL 1 DAY = h.date

             WHERE uv.user_hash IS NULL
               AND v.user_hash IS NULL
               AND h.date IS NULL
               AND ei.flexible_schedule = '0'
               AND JSON_UNQUOTE(JSON_EXTRACT(sg.schedule_data, CONCAT('$.', DAYNAME(CURDATE() + INTERVAL 1 DAY)))) IS NOT NULL
               AND NOT EXISTS (
                   SELECT 1
                   FROM actual_schedules
                   WHERE user_hash = ei.user_hash
                   AND date = CURDATE() + INTERVAL 1 DAY)";
    $pdo->exec($sql2);
} catch (Exception $e) {
    error_log("Error in actual_schedules insert: " . $e->getMessage(), 3, './error_log.txt');
}

// Insert into daily_flexible_schedule_targets
try {
    $sql3 = "INSERT INTO daily_flexible_schedule_targets (user_hash, date, target_hours)
             SELECT ei.user_hash, CURDATE() + INTERVAL 1 DAY, ei.target_hours
             FROM emp_information ei
             LEFT JOIN user_vacations uv ON ei.user_hash = uv.user_hash
                                         AND uv.day_of_week_id = DAYOFWEEK(CURDATE() + INTERVAL 1 DAY)
             LEFT JOIN vacations v ON ei.user_hash = v.user_hash
                                    AND (
                                        CURDATE() + INTERVAL 1 DAY BETWEEN v.date_start AND DATE(v.date_end - INTERVAL 1 DAY)
                                        OR (v.date_start = CURDATE() + INTERVAL 1 DAY AND v.date_end = CURDATE() + INTERVAL 1 DAY)
                                    )
                                    AND v.status = '111'
             LEFT JOIN holiday h ON CURDATE() + INTERVAL 1 DAY = h.date

             WHERE uv.user_hash IS NULL
               AND v.user_hash IS NULL
               AND h.date IS NULL
               AND ei.flexible_schedule = '1'
               AND ei.target_hours IS NOT NULL
               AND NOT EXISTS (
                   SELECT 1
                   FROM actual_schedules
                   WHERE user_hash = ei.user_hash
                   AND date = CURDATE() + INTERVAL 1 DAY)
                   AND NOT EXISTS (
      SELECT 1
      FROM daily_flexible_schedule_targets
      WHERE user_hash = ei.user_hash
      AND date = CURDATE() + INTERVAL 1 DAY
  )
                   ";
    $pdo->exec($sql3);
} catch (Exception $e) {
    error_log("Error in daily_flexible_schedule_targets insert: " . $e->getMessage(), 3, './error_log.txt');
}

echo "Cron job executed successfully.\n";
