<?php

// Path to CodeIgniter database config file
$config_path = '/var/www/html/spaceLight/app/application/config/database.php';

// $config_path = '../config/database.php';

// Check if the CodeIgniter database configuration file exists
if (file_exists($config_path)) {
    // Include the configuration file
    include $config_path;

    // Debugging: Check if $db is actually an array
    if (!is_array($db)) {
        die("Error: \$db is not an array. The config file might not be loaded correctly.");
    }

    // Ensure active group is set
    $active_group = isset($active_group) ? $active_group : 'default';

    // Check if the active group exists in $db
    if (!isset($db[$active_group])) {
        die("Error: Active group '$active_group' does not exist in \$db.");
    }

    // Continue with database connection using $db[$active_group]
    $host = $db[$active_group]['hostname'];
    $db_name = $db[$active_group]['database'];
    $user = $db[$active_group]['username'];
    $pass = $db[$active_group]['password'];
    $charset = 'utf8mb4'; // Define your charset

    // Create DSN and set PDO options
    $dsn = "mysql:host=$host;dbname=$db_name;charset=$charset";
    $options = [
        PDO::ATTR_ERRMODE => PDO::ERRMODE_EXCEPTION,
        PDO::ATTR_DEFAULT_FETCH_MODE => PDO::FETCH_ASSOC,
        PDO::ATTR_EMULATE_PREPARES => false,
    ];

    // Try establishing a PDO connection
    try {
        $pdo = new PDO($dsn, $user, $pass, $options);
        echo "Database connection successful!";
    } catch (PDOException $e) {
        // Log connection error
        error_log("Database connection failed: " . $e->getMessage(), 3, './error_log.txt');
        die('Database connection failed.');
    }
} else {
    error_log("Error: Configuration file not found" . $e->getMessage(), 3, './error_log.txt');

    die('Error: Configuration file not found.');
}

try {
    $sql1 = "SELECT
                uv.user_hash,
                666 AS vication_type_hash,
                111 AS status,
                CURDATE() AS date_start,
                CURDATE() + INTERVAL 1 DAY AS date_end,
                1 AS duration_day
             FROM
                user_vacations uv
             LEFT JOIN vacations v ON uv.user_hash = v.user_hash
                                    AND v.date_start = CURDATE()
                                    AND v.vication_type_hash = 666
             WHERE
                uv.day_of_week_id = DAYOFWEEK(CURDATE())
                AND DATE(uv.created_at) < CURDATE()
                AND DATE(uv.updated_at) < CURDATE()
                AND v.user_hash IS NULL";
    $stmt1 = $pdo->query($sql1);
    $results1 = $stmt1->fetchAll();

    if (!empty($results1)) {
        $insertSql1 = "INSERT INTO vacations (user_hash, vication_type_hash, status, date_start, date_end, duration_day) VALUES ";
        $insertValues1 = [];
        foreach ($results1 as $row) {
            $insertValues1[] = "('{$row['user_hash']}', 666, 111, CURDATE(), CURDATE() + INTERVAL 1 DAY, 1)";
        }
        $insertSql1 .= implode(', ', $insertValues1);
        $pdo->exec($insertSql1);
    }
} catch (Exception $e) {
    error_log("Error in vacations query: " . $e->getMessage(), 3, './error_log.txt');
}

// Check and insert into actual_schedules
try {
    $sql2 = "SELECT ei.user_hash, CURDATE() AS date,
                    JSON_UNQUOTE(JSON_EXTRACT(sg.schedule_data, CONCAT('$.', DAYNAME(CURDATE())))) AS schedule_data,
                    sg.allowance_time
             FROM emp_information ei
             JOIN schedule_groups sg ON ei.work_schedule = sg.id
             LEFT JOIN user_vacations uv ON ei.user_hash = uv.user_hash
                                         AND uv.day_of_week_id = DAYOFWEEK(CURDATE())
             LEFT JOIN vacations v ON ei.user_hash = v.user_hash
                                    AND (
                                        CURDATE() BETWEEN v.date_start AND DATE(v.date_end - INTERVAL 1 DAY)
                                        OR (v.date_start = CURDATE() AND v.date_end = CURDATE())
                                    )
                                    AND v.status = '111'
            LEFT JOIN holiday h ON CURDATE() = h.date

             WHERE uv.user_hash IS NULL
               AND v.user_hash IS NULL
               AND ei.flexible_schedule = '0'
               AND h.date IS NULL
               AND JSON_UNQUOTE(JSON_EXTRACT(sg.schedule_data, CONCAT('$.', DAYNAME(CURDATE())))) IS NOT NULL
               AND NOT EXISTS (
                   SELECT 1
                   FROM actual_schedules
                   WHERE user_hash = ei.user_hash
                   AND date = CURDATE())
                   AND NOT EXISTS (
      SELECT 1
      FROM daily_flexible_schedule_targets
      WHERE user_hash = ei.user_hash
      AND date = CURDATE()
  )";
    $stmt2 = $pdo->query($sql2);
    $results2 = $stmt2->fetchAll();

    if (!empty($results2)) {
        $insertSql2 = "INSERT INTO actual_schedules (user_hash, date, schedule_data, allowance_time) VALUES ";
        $insertValues2 = [];
        foreach ($results2 as $row) {
            $insertValues2[] = "('{$row['user_hash']}', CURDATE(), '{$row['schedule_data']}', '{$row['allowance_time']}')";
        }
        $insertSql2 .= implode(', ', $insertValues2);
        $pdo->exec($insertSql2);
    }
} catch (Exception $e) {
    error_log("Error in actual_schedules query: " . $e->getMessage(), 3, './error_log.txt');
}

// Check and insert into daily_flexible_schedule_targets
try {
    $sql3 = "SELECT ei.user_hash, CURDATE() AS date, ei.target_hours
             FROM emp_information ei
             LEFT JOIN user_vacations uv ON ei.user_hash = uv.user_hash
                                         AND uv.day_of_week_id = DAYOFWEEK(CURDATE())
             LEFT JOIN vacations v ON ei.user_hash = v.user_hash
                                    AND (
                                        CURDATE() BETWEEN v.date_start AND DATE(v.date_end - INTERVAL 1 DAY)
                                        OR (v.date_start = CURDATE() AND v.date_end = CURDATE())
                                    )
                                    AND v.status = '111'
             LEFT JOIN holiday h ON CURDATE() = h.date
             WHERE uv.user_hash IS NULL
               AND v.user_hash IS NULL
               AND ei.flexible_schedule = '1'
               AND ei.target_hours IS NOT NULL
               AND h.date IS NULL
               AND NOT EXISTS (
                   SELECT 1
                   FROM actual_schedules
                   WHERE user_hash = ei.user_hash
                   AND date = CURDATE())

                AND NOT EXISTS (
      SELECT 1
      FROM daily_flexible_schedule_targets
      WHERE user_hash = ei.user_hash
      AND date = CURDATE()
  )
                   ";
    $stmt3 = $pdo->query($sql3);
    $results3 = $stmt3->fetchAll();

    if (!empty($results3)) {
        $insertSql3 = "INSERT INTO daily_flexible_schedule_targets (user_hash, date, target_hours) VALUES ";
        $insertValues3 = [];
        foreach ($results3 as $row) {
            $insertValues3[] = "('{$row['user_hash']}', CURDATE(), '{$row['target_hours']}')";
        }
        $insertSql3 .= implode(', ', $insertValues3);
        $pdo->exec($insertSql3);
    }
} catch (Exception $e) {
    error_log("Error in daily_flexible_schedule_targets query: " . $e->getMessage(), 3, './error_log.txt');
}

echo "Cron job executed successfully.\n";
